/*
 * 作成日: 2006/11/24
 *
 * TODO この生成されたファイルのテンプレートを変更するには次へジャンプ:
 * ウィンドウ - 設定 - Java - コード・スタイル - コード・テンプレート
 */
package bakusen.game.throwbom;

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.zip.GZIPOutputStream;

import bakusen.common.BStage;
import bakusen.common.Body;
import bakusen.common.InvalidAIData;
import bakusen.common.ai.AIInterface;
import bakusen.common.ai.AIOutput;
import neuralnetwork.NeuralNetwork;
/*
 * -In 11
 * OnOff
 * type(man,missile,bom,softwall,wall)
 * position
 * velocity
 * armor
 * 
 * -Out 8
 * velocity
 * actiontarget
 * action1
 * action2
 * action3
 * action4
 */

public class AINeuralnetworkV1 extends AIInterface
{
    public static final int NEURON_PAR_BODY = 11;
    public static final int SITE_NUM = 40; 
    public static final int SHAKE_NUM = 3; 
    public static final int INTPUT_NEURON_NUM = NEURON_PAR_BODY * SITE_NUM + 3;
    public static final int OUTPUT_NEURON_NUM = 8;
    private NeuralNetwork network = new NeuralNetwork();
    private TreeSet<Body> sorter;
    private BodySiteComparator comparator;
    private double[] inputValues;
    private double[] desiredValues;
    private File recordFile;
    private DataOutputStream recordStream;
    private boolean recordAI = false;
    private long lastRecordTime = 0;
    
    public AINeuralnetworkV1()
    {
        super();
        network.initialize(new int[]{INTPUT_NEURON_NUM, 10, 10, OUTPUT_NEURON_NUM});
        network.setLearningRate(0.01);
        network.setLinearOutput(false);
        inputValues = new double[INTPUT_NEURON_NUM];
        desiredValues = new double[OUTPUT_NEURON_NUM];
        comparator = new BodySiteComparator();
        sorter = new TreeSet<Body>(comparator);
    }
    
    public NeuralNetwork getNetwork()
    {
        return network;
    }

    public String getAIName()
    {
        return "AINeuralnetworkVer1";
    }

    public void setNetwork(NeuralNetwork network)
    {
        if (network.getInputLayerNodeNum() != inputValues.length
            || network.getOutputLayerNodeNum() != desiredValues.length)
            throw new ArrayIndexOutOfBoundsException();
        this.network = network;
    }

    public void input(double[] input)
    {
        if (input.length != INTPUT_NEURON_NUM)
            throw new ArrayIndexOutOfBoundsException();
        for (int i=0;i<input.length;i++)
        {
            network.setInput(i,input[i]);
        }
    }
    public void input(Body mybody, BStage obstage, long currenttime, long actionlock)
    {
        BStageThrowBom bstage = (BStageThrowBom)obstage;
        //BVector2f offset = mybody.getPosition();
        
        Body[] bs = bstage.getBodys();
        sorter.clear();
        comparator.setTarget(mybody.getPosition());
        for (int i=0;i<bs.length;i++)
        {
            if (bs[i] != mybody)
                sorter.add(bs[i]);
        }
        inputValues[0] = (double)(currenttime %   1000l) /   1000d;
        inputValues[1] = (double)(currenttime %  10000l) /  10000d;
        inputValues[2] = (double)(currenttime % 100000l) / 100000d;
        
        Iterator<Body> ite = sorter.iterator();
        Body body = mybody;
        for (int i=0;i<SITE_NUM;i++)
        {
            int s = i*NEURON_PAR_BODY + SHAKE_NUM;
            if (body != null)
            {
                inputValues[s+0] = 0.7d;
                inputValues[s+1] = (body instanceof BodyMan) ? 0.7d : 0.3d;
                inputValues[s+2] = (body instanceof BodyMissile) ? 0.7d : 0.3d;
                inputValues[s+3] = (body instanceof BodyBom) ? 0.7d : 0.3d;
                inputValues[s+4] = (body instanceof BodySoftWall) ? 0.7d : 0.3d;
                inputValues[s+5] = (body instanceof BodyWall) ? 0.7d : 0.3d;
                inputValues[s+6] = body.getPosition().x;
                inputValues[s+7] = body.getPosition().y;
                inputValues[s+8] = body.getVelocity().x;
                inputValues[s+9] = body.getVelocity().y;
                inputValues[s+10] = (double)body.getArmor();
            }
            else
            {
                inputValues[s+0] = 0.3d;
                inputValues[s+1] = 0.3d;
                inputValues[s+2] = 0.3d;
                inputValues[s+3] = 0.3d;
                inputValues[s+4] = 0.3d;
                inputValues[s+5] = 0.3d;
                inputValues[s+6] = mybody.getPosition().x;
                inputValues[s+7] = mybody.getPosition().y;
                inputValues[s+8] = mybody.getVelocity().x;
                inputValues[s+9] = mybody.getVelocity().y;
                inputValues[s+10] = 0.3d;
            }
            if (ite.hasNext())
            {
                body = ite.next();
            }
            else
            {
                body = null;
            }
        }
        input(inputValues);
    }
    
    public double[] getInputValues()
    {
        return inputValues;
    }

    public AIOutput getAIOutput()
    {
        network.feedForward();
        AIOutput out = new AIOutput();
        out.getVelocity().x = (float)(logit(network.getOutput(0)));
        out.getVelocity().y = (float)(logit(network.getOutput(1)));
        out.getActiontarget().x = (float)(logit(network.getOutput(2)));
        out.getActiontarget().y = (float)(logit(network.getOutput(3)));

        boolean[] actions = new boolean[4];
        actions[0] = (network.getOutput(4) > 0.5d);
        actions[1] = (network.getOutput(5) > 0.5d);
        actions[2] = (network.getOutput(6) > 0.5d);
        actions[3] = (network.getOutput(7) > 0.5d);
        out.setActions(actions);
        return out;
    }
    public void inputDesired(long currenttime, AIOutput out)
    {
        desiredValues[0] = logistic(out.getVelocity().x);
        desiredValues[1] = logistic(out.getVelocity().y);
        desiredValues[2] = logistic(out.getActiontarget().x);
        desiredValues[3] = logistic(out.getActiontarget().y);
        boolean[] actions = new boolean[4];
        out.getActions(actions);
        desiredValues[4] = (actions[0]) ? 0.7d:0.3d;
        desiredValues[5] = (actions[1]) ? 0.7d:0.3d;
        desiredValues[6] = (actions[2]) ? 0.7d:0.3d;
        desiredValues[7] = (actions[3]) ? 0.7d:0.3d;
        
        network.setDesiredOutput(desiredValues);
        writeRecord(currenttime);
    }
//    private void inputDesired(double[] input)
//    {
//        if (input.length != OUTPUT_NEURON_NUM)
//            throw new ArrayIndexOutOfBoundsException();
//        
//        for (int i=0;i<input.length;i++)
//        {
//            network.setDesiredOutput(i,input[i]);
//        }
//    }
    public void backPropagate()
    {
        network.backPropagate();
    }
    public double[] getDesiredValues()
    {
        return desiredValues;
    }
    private static double logistic(double x)
    {
        double d = x/100;
        return 1.0/(1+Math.exp(-d));
    }
    private static double logit(double x)
    {
        return Math.log(x/(1-x)) * 100;
    }

    public File getWorkDir()
    {
        return recordFile;
    }

    public void setWorkDir(File recordFile)
    {
        this.recordFile = recordFile;
    }

    public boolean isRecordAI()
    {
        return recordAI;
    }

    public void setRecordAI(boolean recordAI)
    {
        this.recordAI = recordAI;
        if (recordAI)
        {
            if (recordFile == null)
            {
                recordAI = false;
                return;
            }
            if (recordStream != null)
                return;
            
            if (!recordFile.exists())
            {
                try
                {
                    if (!recordFile.createNewFile())
                    {
                        recordAI = false;
                        return;
                    }
                }
                catch(IOException e)
                {
                    e.printStackTrace();
                    recordAI = false;
                    return;
                }
            }
            
            try
            {
                recordStream = new DataOutputStream(new GZIPOutputStream(new FileOutputStream(recordFile),1024*1024));
                recordStream.writeInt(INTPUT_NEURON_NUM);
                recordStream.writeInt(OUTPUT_NEURON_NUM);
            }
            catch(IOException e)
            {
                recordAI = false;
                return;
            }
        }
        else
        {
            if (recordStream != null)
            {
                try
                {
                    recordStream.close();
                }
                catch(IOException e)
                {
                    e.printStackTrace();
                }
                recordStream = null;
            }
        }
    }
    public void writeRecord(long currenttime)
    {
        if (!recordAI || recordStream == null)
            return;
        if (currenttime - lastRecordTime < 100)
        {
            return;
        }
        lastRecordTime = currenttime;

        try
        {
            double[] ds;
            ds = inputValues;
            recordStream.writeInt(ds.length);
            for (int i=0;i<ds.length;i++)
            {
                recordStream.writeDouble(ds[i]);
            }
            ds = desiredValues;
            recordStream.writeInt(ds.length);
            for (int i=0;i<ds.length;i++)
            {
                recordStream.writeDouble(ds[i]);
            }
        }
        catch(IOException e)
        {
            e.printStackTrace();
            setRecordAI(false);
        }

        
    }
    public int getInputNeuronNum()
    {
        return INTPUT_NEURON_NUM;
    }
    public int getOutputNeuronNum()
    {
        return OUTPUT_NEURON_NUM;
    }
    public void loadAI(File file) throws IOException, InvalidAIData
    {
        FileInputStream in = new FileInputStream(file);
        NeuralNetwork nn = NeuralNetwork.loadFromStream(in);
        in.close();
        setNetwork(nn);
    }

    public void saveAI(File file) throws IOException
    {
        FileOutputStream out = new FileOutputStream(file);
        NeuralNetwork.dumpToStream(out, network);
        out.close();
    }
    public boolean isFileLoadable()
    {
        return true;
    }
}

