/*
 * 作成日: 2006/11/24
 *
 * TODO この生成されたファイルのテンプレートを変更するには次へジャンプ:
 * ウィンドウ - 設定 - Java - コード・スタイル - コード・テンプレート
 */
package bakusen.game.throwbom;

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.zip.GZIPOutputStream;

import bakusen.common.BStage;
import bakusen.common.Body;
import bakusen.common.InvalidAIData;
import bakusen.common.ai.AIInterface;
import bakusen.common.ai.AIOutput;
import neuralnetwork.NeuralNetwork;
/*
 * -In 11
 * OnOff
 * type(man,missile,bom,softwall,wall)
 * position
 * velocity
 * armor
 * 
 * -Out 8
 * velocity
 * actiontarget
 * action1
 * action2
 * action3
 * action4
 */

public class AINeuralnetworkV2 extends AIInterface
{
    //public static final int NEURON_PAR_BODY = 11;
    //public static final int SITE_NUM = 40; 
    public static final int SHAKE_NUM = 3; 
    public static final int MYBODY_PN = 6;
    public static final int SITE_WALL_NUM = 10;
    public static final int SITE_WALL_PN = 3;
    public static final int SITE_SOFTWALL_NUM = 10;
    public static final int SITE_SOFTWALL_PN = 4;
    public static final int SITE_ITEM_NUM = 5;
    public static final int SITE_ITEM_PN = 3;
    public static final int SITE_BODY_NUM = 5;
    public static final int SITE_BODY_PN = 6;
    public static final int SITE_MISSLE_NUM = 5;
    public static final int SITE_MISSLE_PN = 6;
    public static final int SITE_BOM_NUM = 5;
    public static final int SITE_BOM_PN = 6;
    public static final int SITE_FIRE_NUM = 5;
    public static final int SITE_FIRE_PN = 5;
    //public static final int INTPUT_NEURON_NUM = NEURON_PAR_BODY * SITE_NUM + 3;
    public static final int INTPUT_NEURON_NUM = SHAKE_NUM + MYBODY_PN
    + SITE_WALL_NUM * SITE_WALL_PN
    + SITE_SOFTWALL_NUM * SITE_SOFTWALL_PN
    + SITE_ITEM_NUM * SITE_ITEM_PN
    + SITE_BODY_NUM * SITE_BODY_PN
    + SITE_MISSLE_NUM * SITE_MISSLE_PN
    + SITE_BOM_NUM * SITE_BOM_PN
    + SITE_FIRE_NUM * SITE_FIRE_PN;
    public static final int OUTPUT_NEURON_NUM = 8;
    
    private NeuralNetwork network;
    private TreeSet<Body> wallSorter;
    private TreeSet<Body> softwallSorter;
    private TreeSet<Body> itemSorter;
    private TreeSet<Body> manSorter;
    private TreeSet<Body> missileSorter;
    private TreeSet<Body> bomSorter;
    private TreeSet<Body> fireSorter;
    private BodySiteComparator comparator;
    private double[] inputValues;
    private double[] desiredValues;
    private File workDir;
    private DataOutputStream recordStream;
    private boolean recordAI = false;
    private long lastRecordTime = 0;
    
    public AINeuralnetworkV2()
    {
        super();
        inputValues = new double[INTPUT_NEURON_NUM];
        desiredValues = new double[OUTPUT_NEURON_NUM];
        comparator = new BodySiteComparator();
        wallSorter = new TreeSet<Body>(comparator);
        softwallSorter = new TreeSet<Body>(comparator);
        itemSorter = new TreeSet<Body>(comparator);
        manSorter = new TreeSet<Body>(comparator);
        missileSorter = new TreeSet<Body>(comparator);
        bomSorter = new TreeSet<Body>(comparator);
        fireSorter = new TreeSet<Body>(comparator);
        NeuralNetwork nn = new NeuralNetwork();
        nn.initialize(new int[]{INTPUT_NEURON_NUM, 10, 10, OUTPUT_NEURON_NUM});
        nn.setLearningRate(0.01);
        nn.setLinearOutput(false);
        try
        {
            setNetwork(nn);
        }
        catch(InvalidAIData e)
        {
            throw new RuntimeException(e);
        }
    }
    
    public String getAIName()
    {
        return "AINeuralnetworkVer2";
    }
    
    public NeuralNetwork getNetwork()
    {
        return network;
    }

    public void setNetwork(NeuralNetwork network) throws InvalidAIData
    {
        if (network.getInputLayerNodeNum() != inputValues.length
            || network.getOutputLayerNodeNum() != desiredValues.length)
            throw new InvalidAIData();
        this.network = network;
    }

    public void input(double[] input)
    {
        if (input.length != INTPUT_NEURON_NUM)
            throw new ArrayIndexOutOfBoundsException();
        for (int i=0;i<input.length;i++)
        {
            network.setInput(i,input[i]);
        }
        network.feedForward();
    }
    public void input(Body mybody, BStage obstage, long currenttime, long actionlock)
    {
        BStageThrowBom bstage = (BStageThrowBom)obstage;
        //BVector2f offset = mybody.getPosition();
        
        Body[] bs = bstage.getBodys();
        wallSorter.clear();
        softwallSorter.clear();
        itemSorter.clear();
        manSorter.clear();
        missileSorter.clear();
        bomSorter.clear();
        fireSorter.clear();
        
        comparator.setTarget(mybody.getPosition());
        for (int i=0;i<bs.length;i++)
        {
            if (bs[i].getType() == BStageThrowBom.WALL1)
                wallSorter.add(bs[i]);
            else if (bs[i].getType() == BStageThrowBom.SOFT_WALL)
                softwallSorter.add(bs[i]);
            else if (bs[i].getType() == BStageThrowBom.ITEM)
                itemSorter.add(bs[i]);
            else if (bs[i].getType() == BStageThrowBom.MAN1)
            {
                if (bs[i] != mybody)
                    manSorter.add(bs[i]);
            }
            else if (bs[i].getType() == BStageThrowBom.MISSILE1)
                missileSorter.add(bs[i]);
            else if (bs[i].getType() == BStageThrowBom.SLUG1)
                missileSorter.add(bs[i]);
            else if (bs[i].getType() == BStageThrowBom.HOMING1)
                missileSorter.add(bs[i]);
            else if (bs[i].getType() == BStageThrowBom.BOM1)
                bomSorter.add(bs[i]);
            else if (bs[i].getType() == BStageThrowBom.FIRE1)
                fireSorter.add(bs[i]);
        }

        inputValues[0] = (double)(currenttime %   1000l) /   1000d;
        inputValues[1] = (double)(currenttime %  10000l) /  10000d;
        inputValues[2] = (double)(currenttime % 100000l) / 100000d;
        int offset = SHAKE_NUM;

        //mybody
        Body body = mybody;
        inputValues[offset+0] = 1d;
        inputValues[offset+1] = body.getPosition().x;
        inputValues[offset+2] = body.getPosition().y;
        inputValues[offset+3] = body.getVelocity().x;
        inputValues[offset+4] = body.getVelocity().y;
        inputValues[offset+5] = (double)body.getArmor();
        offset += MYBODY_PN;
        
        //wall
        Iterator<Body> ite;
        ite = wallSorter.iterator();
        for (int i=0;i<SITE_WALL_NUM;i++)
        {
            int s = offset + i*SITE_WALL_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
                inputValues[s+1] = body.getPosition().x;
                inputValues[s+2] = body.getPosition().y;
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = mybody.getPosition().x;
                inputValues[s+2] = mybody.getPosition().y;
            }
        }
        offset += SITE_WALL_NUM * SITE_WALL_PN;
        
        //softwall
        ite = softwallSorter.iterator();
        for (int i=0;i<SITE_SOFTWALL_NUM;i++)
        {
            int s = offset + i*SITE_SOFTWALL_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
                inputValues[s+1] = body.getPosition().x;
                inputValues[s+2] = body.getPosition().y;
                inputValues[s+3] = (double)body.getArmor();
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = mybody.getPosition().x;
                inputValues[s+2] = mybody.getPosition().y;
                inputValues[s+3] = 0d;
            }
        }
        offset += SITE_SOFTWALL_NUM * SITE_SOFTWALL_PN;

        //item
        ite = itemSorter.iterator();
        for (int i=0;i<SITE_ITEM_NUM;i++)
        {
            int s = offset + i*SITE_ITEM_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
                inputValues[s+1] = body.getPosition().x;
                inputValues[s+2] = body.getPosition().y;
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = mybody.getPosition().x;
                inputValues[s+2] = mybody.getPosition().y;
            }
        }
        offset += SITE_ITEM_NUM * SITE_ITEM_PN;
        
        //man
        ite = manSorter.iterator();
        for (int i=0;i<SITE_BODY_NUM;i++)
        {
            int s = offset + i*SITE_BODY_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
                inputValues[s+1] = body.getPosition().x;
                inputValues[s+2] = body.getPosition().y;
                inputValues[s+3] = body.getVelocity().x;
                inputValues[s+4] = body.getVelocity().y;
                inputValues[s+5] = (double)body.getArmor();
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = mybody.getPosition().x;
                inputValues[s+2] = mybody.getPosition().y;
                inputValues[s+3] = mybody.getVelocity().x;
                inputValues[s+4] = mybody.getVelocity().y;
                inputValues[s+5] = 0d;
            }
        }
        offset += SITE_BODY_NUM * SITE_BODY_PN;

        //missile
        ite = missileSorter.iterator();
        for (int i=0;i<SITE_MISSLE_NUM;i++)
        {
            int s = offset + i*SITE_MISSLE_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
                inputValues[s+1] = body.getPosition().x;
                inputValues[s+2] = body.getPosition().y;
                inputValues[s+3] = body.getVelocity().x;
                inputValues[s+4] = body.getVelocity().y;
                inputValues[s+5] = (double)body.getArmor();
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = mybody.getPosition().x;
                inputValues[s+2] = mybody.getPosition().y;
                inputValues[s+3] = mybody.getVelocity().x;
                inputValues[s+4] = mybody.getVelocity().y;
                inputValues[s+5] = 0d;
            }
        }
        offset += SITE_MISSLE_NUM * SITE_MISSLE_PN;

        //bom
        ite = bomSorter.iterator();
        for (int i=0;i<SITE_BOM_NUM;i++)
        {
            int s = offset + i*SITE_BOM_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
                inputValues[s+1] = body.getPosition().x;
                inputValues[s+2] = body.getPosition().y;
                inputValues[s+3] = body.getVelocity().x;
                inputValues[s+4] = body.getVelocity().y;
                inputValues[s+5] = (double)body.getArmor();
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = mybody.getPosition().x;
                inputValues[s+2] = mybody.getPosition().y;
                inputValues[s+3] = mybody.getVelocity().x;
                inputValues[s+4] = mybody.getVelocity().y;
                inputValues[s+5] = 0d;
            }
        }
        offset += SITE_BOM_NUM * SITE_BOM_PN;

        //fire
        ite = fireSorter.iterator();
        for (int i=0;i<SITE_FIRE_NUM;i++)
        {
            int s = offset + i*SITE_FIRE_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
                inputValues[s+1] = body.getPosition().x;
                inputValues[s+2] = body.getPosition().y;
                inputValues[s+3] = body.getVelocity().x;
                inputValues[s+4] = body.getVelocity().y;
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = mybody.getPosition().x;
                inputValues[s+2] = mybody.getPosition().y;
                inputValues[s+3] = mybody.getVelocity().x;
                inputValues[s+4] = mybody.getVelocity().y;
            }
        }
        offset += SITE_FIRE_NUM * SITE_FIRE_PN;
        assert(offset == INTPUT_NEURON_NUM);
        input(inputValues);
    }
    
    public double[] getInputValues()
    {
        return inputValues;
    }

    public AIOutput getAIOutput()
    {
        AIOutput out = new AIOutput();
        out.getVelocity().x = (float)(logit(network.getOutput(0)));
        out.getVelocity().y = (float)(logit(network.getOutput(1)));
        out.getActiontarget().x = (float)(logit(network.getOutput(2)));
        out.getActiontarget().y = (float)(logit(network.getOutput(3)));

        boolean[] actions = new boolean[4];
        actions[0] = (network.getOutput(4) > 0.5d);
        actions[1] = (network.getOutput(5) > 0.5d);
        actions[2] = (network.getOutput(6) > 0.5d);
        actions[3] = (network.getOutput(7) > 0.5d);
        out.setActions(actions);
        return out;
    }
    public void inputDesired(long currenttime, AIOutput out)
    {
        desiredValues[0] = logistic(out.getVelocity().x);
        desiredValues[1] = logistic(out.getVelocity().y);
        desiredValues[2] = logistic(out.getActiontarget().x);
        desiredValues[3] = logistic(out.getActiontarget().y);
        boolean[] actions = new boolean[4];
        out.getActions(actions);
        desiredValues[4] = (actions[0]) ? 0.7d:0.3d;
        desiredValues[5] = (actions[1]) ? 0.7d:0.3d;
        desiredValues[6] = (actions[2]) ? 0.7d:0.3d;
        desiredValues[7] = (actions[3]) ? 0.7d:0.3d;
        
        //inputDesired(desiredValues);
        network.setDesiredOutput(desiredValues);
        writeRecord(currenttime);
    }
//    private void inputDesired(double[] input)
//    {
//        if (input.length != OUTPUT_NEURON_NUM)
//            throw new ArrayIndexOutOfBoundsException();
//        
//        for (int i=0;i<input.length;i++)
//        {
//            network.setDesiredOutput(i,input[i]);
//        }
//    }
    public void backPropagate()
    {
        network.backPropagate();
    }
    public double[] getDesiredValues()
    {
        return desiredValues;
    }
    private static double logistic(double x)
    {
        double d = x/100;
        return 1.0/(1+Math.exp(-d));
    }
    private static double logit(double x)
    {
        return Math.log(x/(1-x)) * 100;
    }

    public File getWorkDir()
    {
        return workDir;
    }

    public void setWorkDir(File recordFile)
    {
        this.workDir = recordFile;
    }

    public boolean isRecordAI()
    {
        return recordAI;
    }

    public void setRecordAI(boolean recordAI)
    {
        this.recordAI = recordAI;
        if (recordAI)
        {
            if (workDir == null)
            {
                recordAI = false;
                return;
            }
            if (recordStream != null)
                return;
            SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
            File recordFile = new File( workDir + "/ver2_" + format.format(new Date()) + ".rec");
            
            if (!recordFile.exists())
            {
                try
                {
                    if (!recordFile.createNewFile())
                    {
                        recordAI = false;
                        return;
                    }
                }
                catch(IOException e)
                {
                    e.printStackTrace();
                    System.out.println(recordFile);
                    recordAI = false;
                    return;
                }
            }
            
            try
            {
                recordStream = new DataOutputStream(new GZIPOutputStream(new FileOutputStream(recordFile),1024*1024));
                recordStream.writeInt(INTPUT_NEURON_NUM);
                recordStream.writeInt(OUTPUT_NEURON_NUM);
            }
            catch(IOException e)
            {
                recordAI = false;
                return;
            }
        }
        else
        {
            if (recordStream != null)
            {
                try
                {
                    recordStream.close();
                }
                catch(IOException e)
                {
                    e.printStackTrace();
                }
                recordStream = null;
            }
        }
    }
    public void writeRecord(long currenttime)
    {
        if (!recordAI || recordStream == null)
            return;
        if (currenttime - lastRecordTime < 100)
        {
            return;
        }
        lastRecordTime = currenttime;

        try
        {
            double[] ds;
            ds = inputValues;
            recordStream.writeInt(ds.length);
            for (int i=0;i<ds.length;i++)
            {
                recordStream.writeDouble(ds[i]);
            }
            ds = desiredValues;
            recordStream.writeInt(ds.length);
            for (int i=0;i<ds.length;i++)
            {
                recordStream.writeDouble(ds[i]);
            }
        }
        catch(IOException e)
        {
            e.printStackTrace();
            setRecordAI(false);
        }

        
    }
    public int getInputNeuronNum()
    {
        return INTPUT_NEURON_NUM;
    }
    public int getOutputNeuronNum()
    {
        return OUTPUT_NEURON_NUM;
    }

    public void loadAI(File file) throws IOException, InvalidAIData
    {
        FileInputStream in = new FileInputStream(file);
        NeuralNetwork nn = NeuralNetwork.loadFromStream(in);
        in.close();
        setNetwork(nn);
    }

    public void saveAI(File file) throws IOException
    {
        FileOutputStream out = new FileOutputStream(file);
        NeuralNetwork.dumpToStream(out, network);
        out.close();
    }
    public boolean isFileLoadable()
    {
        return true;
    }
}

