/*
 * 作成日: 2005/01/19
 *
 */
package bakusen.server;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.util.ArrayList;

import bakusen.common.BStage;
import bakusen.common.BakusenClock;
import bakusen.common.Body;
import bakusen.common.CompressedInputStream;
import bakusen.common.CompressedOutputStream;
import bakusen.common.DataManager;
import bakusen.common.event.BodyActionEvent;
import bakusen.common.event.BodyCommandEvent;
import bakusen.common.event.BodyDeleteEvent;
import bakusen.common.event.BodyEvent;
import bakusen.common.event.BodyMoveEvent;
import bakusen.common.event.BodyPushEvent;
import bakusen.common.event.StageEvent;

/**
 * @author cattaka
 *
 */
abstract public class AbstractRemoteClient extends Thread
{
	private DataInputStream in;
	private DataOutputStream out;
	private Socket socket;
	private ArrayList buf = new ArrayList();
	private Body body;
	//private int score = 0;
	private BStage bstage;
	private boolean goflag;
	
	public AbstractRemoteClient(Socket _socket, BStage bstage)
	{
		super("RemoteClient Reader Thread");
		setPriority(Thread.NORM_PRIORITY+1);
		this.socket = _socket;
		this.bstage = bstage;
		try
		{
			in = new DataInputStream(new CompressedInputStream(socket.getInputStream(),32768));
			out = new DataOutputStream(new CompressedOutputStream(socket.getOutputStream(),32768));
		}catch(IOException e)
		{
			try
			{
				socket.close();
			}catch(IOException ex)
			{
				//ex.printStackTrace();
			}
			return;
		}
		goflag = true;
		Thread thread = new Thread("RemoteClient Flusher Thread")
		{
			public void run()
			{
				try
				{
					int i = 0;
					while(socket.isConnected())
					{
						byte[] bs;
						int n;
						synchronized(buf)
						{
							n = buf.size();
							if (n == 0)
							{
								if (!goflag)
									break;
								try
								{
									buf.wait();
								}catch(InterruptedException e)
								{
									e.printStackTrace();
								}
								continue;
							}else
							{
								n--;
								bs = (byte[])buf.remove(0);
							}
						}
						out.write(bs);
						if (n == 0)
							out.flush();
					}
				}catch(IOException e)
				{
					//e.printStackTrace();
				}
				try
				{
					if (socket.isConnected())
						socket.close();
				}catch(IOException e)
				{
				}
			}
		};
		thread.start();
	}
	public void run()
	{
		byte mode;
		try
		{
			out.writeUTF(DataManager.getProtocolVersion());
			out.flush();
			
			mode = in.readByte();
			if (mode != DataManager.REQUEST_LOGIN && mode != DataManager.REQUEST_BSTAGESTATE)
			{
				throw new IOException();
			}
		}catch(IOException e)
		{
			//e.printStackTrace();
			try
			{
				socket.close();
			}catch(IOException ex)
			{
				//ex.printStackTrace();
			}
			return;
		}
		if (mode == DataManager.REQUEST_LOGIN)
		{
			try
			{
				body = bstage.createRemoteClientBody();
				String id = in.readUTF();
				boolean guest = in.readBoolean();
				if (!guest)
				{
					throw new IOException();
				}
				if (id.length() > 7 && id.substring(0,7).equals("_debug_"))
				{
					String tmp = id.substring(7,id.length());
					try
					{
						int level = Integer.parseInt(tmp);
						setDebugLevel(level);
					}catch(NumberFormatException e)
					{
					}
					throw new IOException("Debug level changed");
				}
				
				body.setName(id);
				//body.setRemoteClient(this);
			}catch(IOException e)
			{
				//e.printStackTrace();
				try
				{
					socket.close();
				}catch(IOException ex)
				{
					//ex.printStackTrace();
				}
				return;
			}
			
			long l;
			l = BakusenClock.getCurrenttime();//System.currentTimeMillis();
			StageEvent se = new StageEvent(new Body[0], new BodyClientAddEvent(body, this),l,l);
			putBodyEvent(se);
			
			try
			{
				while(goflag && socket.isConnected())
				{
					BodyEvent be = DataManager.inputBodyEvent(in,bstage);
					if (be instanceof BodyActionEvent
						|| be instanceof BodyPushEvent
						|| be instanceof BodyCommandEvent
						)
					{
						l = BakusenClock.getCurrenttime();//System.currentTimeMillis();
						StageEvent sse = new StageEvent(new Body[]{body}, be, l,l);
						putBodyEvent(sse);
					}else if(be instanceof BodyMoveEvent)
					{
						BodyMoveEvent bme = (BodyMoveEvent)be;
						BodyClientMoveEvent bcme = new BodyClientMoveEvent(bme, this);
						StageEvent sse = new StageEvent(new Body[]{body}, bcme, l,l);
						putBodyEvent(sse);
					}
					else
					{
						throw new IOException("restrect BodyEvent.");
					}
						
				}
			}catch(IOException e)
			{
			}
			try
			{
				if (socket.isConnected())
					socket.close();
			}catch(IOException ex)
			{
			}
			l = BakusenClock.getCurrenttime();//System.currentTimeMillis();
			StageEvent sse = new StageEvent(new Body[]{this.body}, new BodyDeleteEvent(),l,l);
			putBodyEvent(sse);
		}else if (mode == DataManager.REQUEST_BSTAGESTATE)
		{
			long l = BakusenClock.getCurrenttime();
			StageEvent se = new StageEvent(new Body[0], new BodyClientAddEvent(this),l,l);
			putBodyEvent(se);
		}
		
	}
	public void sendPacket(byte[] packet)
	{
		if (packet == null)
		{
			throw new NullPointerException();
		}
		if (socket.isConnected())
		{
			synchronized(buf)
			{
				buf.add(packet);
				buf.notify();
			}
		}
	}
	public Body getBody()
	{
		return body;
	}
	/*
	public int getScore()
	{
		return score;
	}
	public void setScore(int score)
	{
		this.score = score;
	}
	*/
	public void dispose()
	{
		synchronized(buf)
		{
			goflag = false;
			buf.notify();
		}
	}
	public Socket getSocket()
	{
		return socket;
	}
	abstract public void putBodyEvent(StageEvent be);
	abstract public void setDebugLevel(int level);
}
