/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/04
 */
package bakutan.common.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URL;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;



/**
 * @author cattaka
 */
public class BAudio
{
	private AudioBundle audioBundle;
    private Player[] player; 
	private int next = 0;
    private int lineNum = 0;
	
	public BAudio(int lineNum)
	{
        setLineNum(lineNum);
	}

    public void setLineNum(int lineNum)
    {
        if (this.lineNum != lineNum && player != null)
        {
            this.lineNum = lineNum;
            setEnable(true);
        }
        else
        {
            this.lineNum = lineNum;
        }
    }
    
    public synchronized void setEnable(boolean enable)
    {
        if (enable)
        {
            if (player != null && player.length != lineNum)
            {
                for (int i=0;i<player.length;i++)
                {
                    if (player[i] != null)
                        player[i].myDestroy();
                }
                player = null;
            }
            try
            {
//                URL url = getClass().getClassLoader().getResource("sound/fire.wav");
//                AudioFileFormat fileFormat = AudioSystem.getAudioFileFormat(url);
//                AudioFormat format = fileFormat.getFormat();
                //PCM_SIGNED 22050.0 Hz, 16 bit, mono, 2 bytes/frame, little-endian
                AudioFormat format = new AudioFormat(22050.0f, 16, 1, true, false);
                System.out.println(format);
                DataLine.Info dataInfo = new DataLine.Info(SourceDataLine.class, format);

                player = new Player[lineNum];
                for (int i=0;i<lineNum;i++)
                {
                    SourceDataLine line;
                    line = (SourceDataLine)AudioSystem.getLine(dataInfo);
                    line.open(format);
                    line.start();
                    player[i] = new Player(line);
                    player[i].start();
                }
                next = 0;
            }catch(Exception e)
            {
                player = null;
                e.printStackTrace();
                return;
            }
        }
        else
        {
            if (player == null)
                return;
            for (int i=0;i<player.length;i++)
            {
                if (player[i] != null)
                    player[i].myDestroy();
            }
            player = null;
        }
        
    }
//	public void stop()
//	{
//		if (player == null)
//			return;
//		for (int i=0;i<player.length;i++)
//		{
//			if (player[i] != null)
//				player[i].myDestroy();
//		}
//		player = null;
//	}
    public boolean isRunning()
    {
        return lineNum > 0;
    }
	/*
	private void createSoundData()
	{
		try
		{
			soundData = new byte[SOUNDTYPE_NUM][];
			
			ByteArrayOutputStream bao = new ByteArrayOutputStream(1024);
			for (int i=0;i<soundData.length;i++)
			{
				bao.reset();
				URL url;
				AudioInputStream ais;
				url = getClass().getClassLoader().getResource("sound"+i+".wav");
				ais = AudioSystem.getAudioInputStream(url);
				byte[] bs = new byte[(int)ais.getFrameLength()];
				while(true)
				{
					int s = ais.read(bs);
					if (s == -1)
						break;
					bao.write(bs,0,s);
				}
				soundData[i] = bao.toByteArray();
				ais.close();
			}
			bao.close();
			
		}catch(IOException e)
		{
			e.printStackTrace();
		}catch(UnsupportedAudioFileException e)
		{
			e.printStackTrace();
		}
	}
	*/
	public static byte[] loadAudioFile(URL url)
	{
		try
		{
			ByteArrayOutputStream bao = new ByteArrayOutputStream(1024);
			AudioInputStream ais;
			ais = AudioSystem.getAudioInputStream(url);
			byte[] bs = new byte[(int)ais.getFrameLength()];
			while(true)
			{
				int s = ais.read(bs);
				if (s == -1)
					break;
				bao.write(bs,0,s);
			}
			bs = bao.toByteArray();
			ais.close();
			bao.close();
			return bs;
		}catch(IOException e)
		{
			e.printStackTrace();
			return null;
		}catch(UnsupportedAudioFileException e)
		{
			e.printStackTrace();
			return null;
		}
	}
	
	public synchronized void play(byte type, int rag)
	{
		if (player == null || audioBundle == null)
			return;
		
		byte[] soundData = audioBundle.getAudioData(type);
		if (soundData == null)
			return;
		for (int i=0;i<player.length;i++)
		{
			if (player[i].isActive())
				continue;
			player[i].playSound(rag, soundData);
			break;
		}
	}
	public AudioBundle getAudioBundle()
	{
		return audioBundle;
	}
	public void setAudioBundle(AudioBundle audioBundle)
	{
		this.audioBundle = audioBundle;
	}
	class Player extends Thread
	{
		private SourceDataLine line;
		private int sound = 0;
		private int rag = 0;
		private boolean active = true;
		private boolean destroy = false;
		private byte[] soundData;
		private byte[] cleaner;
		
		Player(SourceDataLine line)
		{
			super("Sound Thread");
			setPriority(Thread.MIN_PRIORITY);
			this.line = line;
			this.cleaner = new byte[line.available()];
			for (int i=0;i<cleaner.length;i++)
			{
				cleaner[i] = 0;
			}
		}
		public boolean isActive()
		{
			synchronized(this)
			{
				return active;
			}
		}
		public void playSound(int rag, byte[] soundData)
		{
			if (soundData == null)
				throw new NullPointerException();
			synchronized(this)
			{
				this.soundData = soundData;
				this.rag = rag;
				active = true;
				notify();
			}
		}
		public void run()
		{
			while (true)
			{
				int s;
				synchronized(this)
				{
					try
					{
						active = false;
						if (destroy)
							break;
						wait();
						if (destroy)
							break;
						s = sound;
						if (s <0)
							continue;
					}catch(InterruptedException e)
					{
						e.printStackTrace();
						continue;
					}
				}
				try
				{
					Thread.sleep(rag);
				}catch(InterruptedException e)
				{
					e.printStackTrace();
				}
				line.flush();
				byte[] bs = soundData;
				/*
				int i;
				for (i=0;i<bs.length/line.available();i++)
				{
					line.write(bs,i*line.available(),line.available());
					line.drain();
				}
				*/
				line.start();
				line.write(bs,0,bs.length);
				int avail = line.available();;
				//line.write(cleaner,0,avail - bs.length % avail);
				line.write(cleaner,0,avail);
				line.drain();
				line.stop();
			}
			line.close();
		}
		public void myDestroy()
		{
			synchronized(this)
			{
				destroy = true;
				notify();
			}
		}
	}
}
