/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/03/16
 */
package bakutan.common.util;

import javax.sound.midi.ControllerEventListener;
import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MetaEventListener;
import javax.sound.midi.MetaMessage;
import javax.sound.midi.MidiChannel;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Sequence;
import javax.sound.midi.Sequencer;
import javax.sound.midi.ShortMessage;
import javax.sound.midi.Synthesizer;

public class BBGM implements MetaEventListener
{
	private Sequencer sequencer;
	private BGMBundle bgmBundle;
	private float volume = 0.4f;
    //private long lastPlay = 0;
	
	public BBGM()
	{
	}
	public void setBGMEnable(boolean enable)
	{
		if (enable)
		{
			if (sequencer == null)
			{
				try
				{
					this.sequencer = MidiSystem.getSequencer();
					if (sequencer instanceof Synthesizer)
					{
						MidiChannel[] channels = ((Synthesizer)sequencer).getChannels();
						for (int i=0, n=channels.length; i<n; i++)
						{
						  channels[i].controlChange(7, (int)(volume * 127.0)); // gain between 0 and 1
						}
					    sequencer.addControllerEventListener(new ControllerEventListener() {
				            public void controlChange(ShortMessage message)
				            {
				            	MidiChannel ch = ((Synthesizer)sequencer).getChannels()[message.getChannel()];
				            	ch.controlChange(7,(int)(volume * (float)ch.getController(7)));
				            	//System.out.println(ch.getController(7));
				            }
				        },new int[]{7});
					}

				    MetaEventListener mel = new MetaEventListener()
					{
						public void meta(MetaMessage meta)
						{
							if (meta.getType() == 47)
							{
								play();
							}
						}
					};
                    this.sequencer.addMetaEventListener(mel);
				}catch(MidiUnavailableException e)
				{
					e.printStackTrace();
					this.sequencer = null;
				}
			}
		}else
		{
            if (sequencer != null)
            {
                stop();
                sequencer = null;
            }
		}
	}
	
	public void play()
	{
		if (sequencer == null || bgmBundle == null)
			return;
        stop();
        try
        {
            sequencer.open();
        }
        catch(MidiUnavailableException e)
        {
            e.printStackTrace();
            return;
        }
		try
		{
			Sequence seq = bgmBundle.getRandom();
			if (seq != null)
			{
				sequencer.setSequence(seq);
				sequencer.start();
			}
		}catch(InvalidMidiDataException e)
		{
			e.printStackTrace();
		}
	}
	public void stop()
	{
		if (sequencer == null)
			return;
        if (sequencer.isRunning())
            sequencer.stop();
        if (sequencer.isOpen())
            sequencer.close();
//        Iterator<Transmitter> ite = sequencer.getTransmitters().iterator();
//        while(ite.hasNext())
//        {
//            Transmitter trans = ite.next();
//            trans.close();
//        }
//        Iterator<Receiver> ite2 = sequencer.getReceivers().iterator();
//        while(ite.hasNext())
//        {
//            Receiver rece = ite2.next();
//            rece.close();
//        }
	}
	public boolean isRunning()
	{
		if (sequencer == null)
			return false;
		return sequencer.isRunning();
	}
	public BGMBundle getBgmBundle()
	{
		return bgmBundle;
	}
	public void setBgmBundle(BGMBundle bgmBundle)
	{
		this.bgmBundle = bgmBundle;
	}
    private Sequencer createSequencer()
    {
        try
        {
            this.sequencer = MidiSystem.getSequencer();
            if (sequencer instanceof Synthesizer)
            {
                MidiChannel[] channels = ((Synthesizer)sequencer).getChannels();
                for (int i=0, n=channels.length; i<n; i++)
                {
                  channels[i].controlChange(7, (int)(volume * 127.0)); // gain between 0 and 1
                }
                sequencer.addControllerEventListener(new ControllerEventListener() {
                    public void controlChange(ShortMessage message)
                    {
                        MidiChannel ch = ((Synthesizer)sequencer).getChannels()[message.getChannel()];
                        ch.controlChange(7,(int)(volume * (float)ch.getController(7)));
                        //System.out.println(ch.getController(7));
                    }
                },new int[]{7});
            }

            MetaEventListener mel = new MetaEventListener()
            {
            };
            this.sequencer.addMetaEventListener(mel);
        }catch(MidiUnavailableException e)
        {
            e.printStackTrace();
            this.sequencer = null;
        }
    }
    public void meta(MetaMessage meta)
    {
        if (meta.getType() == 47)
        {
            play();
        }
    }
   
}
