/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/01/14
 */
package bakusen.common.math;

/**
 * @author cattaka
 */
public class BPolygon
{
	public static final byte NONE = 0;
	public static final byte RAW = 1;
	public static final byte SQUARE = 2;
	public static final byte DIAGONAL_SQUARE = 3;
	public static final byte RECTANGLES = 4;
	public static final byte CROSS = 5;
	
	private byte type = NONE;
	private BVector2f[][] vertices;
	
	private float width = Float.NaN;
	private float[] length = null;
	private BVector2f[] size = null;
	private BVector2f[] offset = null;
	
	public BPolygon()
	{
		setVertices(new BVector2f[0][0]);
	}
	public BPolygon(byte type, float width)
	{
		BVector2f[][] vss;
		if (type == SQUARE)
		{
			vss = new BVector2f[][]{new BVector2f[]{new BVector2f( width, width),new BVector2f( width,-width),new BVector2f(-width,-width),new BVector2f(-width, width)}};
		}else if(type == DIAGONAL_SQUARE)
		{
			vss = new BVector2f[][]{new BVector2f[]{new BVector2f( 0.0f, width), new BVector2f( width, 0.0f), new BVector2f( 0.0f, -width), new BVector2f(-width, 0.0f)}};
		}else
		{
			throw new RuntimeException("invalid argument");
		}
		setVertices(vss);
		this.type = type;
		this.width = width;
	}
	public BPolygon(byte type, float width, float[] length)
	{
		if (type != CROSS)
		{
			throw new RuntimeException("invalid argument");
		}
		this.type = type;
		this.length = length;
		BVector2f[] vs1 = new BVector2f[4];
		BVector2f[] vs2 = new BVector2f[4];
		vs1[0] = new BVector2f(width+length[0], width);
		vs1[1] = new BVector2f(width+length[0],-width);
		vs1[2] = new BVector2f(-width-length[2],-width);
		vs1[3] = new BVector2f(-width-length[2], width);

		vs2[0] = new BVector2f( width, width+length[3]);
		vs2[1] = new BVector2f( width,-width-length[1]);
		vs2[2] = new BVector2f(-width,-width-length[1]);
		vs2[3] = new BVector2f(-width, width+length[3]);
		setVertices(new BVector2f[][]{vs1,vs2});
	}
	public BPolygon(byte type, BVector2f[] size, BVector2f[] offset)
	{
		if (type != RECTANGLES || size.length != offset.length)
		{
			throw new RuntimeException("invalid argument");
		}
		BVector2f[][] vss = new BVector2f[size.length][];
		for (int i=0;i<size.length;i++)
		{
			vss[i] = new BVector2f[]{new BVector2f( size[i].x+offset[i].x, size[i].y+offset[i].y),
								new BVector2f( size[i].x+offset[i].x, -size[i].y+offset[i].y),
								new BVector2f(-size[i].x+offset[i].x, -size[i].y+offset[i].y),
								new BVector2f(-size[i].x+offset[i].x,  size[i].y+offset[i].y)};
		}
		setVertices(vss);
		this.type = type;
		this.size = size;
		this.offset = offset;
	}
	
	public BPolygon(BVector2f[][] vertices)
	{
		setVertices(vertices);
	}
	public BPolygon(BPolygon polygon)
	{
		BVector2f[][] vs = (BVector2f[][])polygon.getVertices().clone();
		setVertices(vs);
	}
	public BVector2f[][] getVertices()
	{
		return vertices;
	}
	private void setVertices(BVector2f[][] vertices)
	{
		this.vertices = vertices;
		this.type = RAW;
	}
	public BVector2f[] getSize()
	{
		return size;
	}
	public byte getType()
	{
		return type;
	}
	public float getWidth()
	{
		return width;
	}
	public BVector2f[] getOffset()
	{
		return offset;
	}
	public float[] getLength()
	{
		return length;
	}
}
