/*
 * 作成日: 2007/01/26
 */
package bakutan.server;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.URL;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import bakutan.common.BUserInterface;
import bakutan.common.BUserInterfacePanel;
import bakutan.common.BotInfo;
import bakutan.common.DataManager;
import bakutan.common.LoginInfo;
import bakutan.common.util.HelpPanel;
import bakutan.game.throwbom.BStageThrowBom;

public class BakutanServerGUI extends JFrame implements ActionListener
{
    private BUserInterfacePanel userinterface;
    private ServerProcess serverProcess;
    private ServerConfigurationInfoDialg serverConfigurationInfoDialog;
    private UserDataHolder userDataHolder;
    private HelpPanel panelHelp;
    
    public BakutanServerGUI()
    {
        super("BakutanServerGUI");
        setSize(400,400);
        this.userinterface = new BUserInterfacePanel();
        this.serverConfigurationInfoDialog = new ServerConfigurationInfoDialg(this, this.userinterface);
        this.userDataHolder = new NullUserDataHolder();
        makeLayout();
    }
    private void makeLayout()
    {
        URL helpURL = getClass().getClassLoader().getResource("help_server.utf8.txt");
        if (helpURL != null)
        {
            panelHelp = new HelpPanel("Bakutan server help", helpURL);
        }
        
        JMenuBar menuBar = new JMenuBar();
        JMenu menuServer = new JMenu(userinterface.translate("server"));
        JMenuItem itemStart = new JMenuItem(userinterface.translate("start"));
        JMenuItem itemStop = new JMenuItem(userinterface.translate("stop"));
        itemStart.setActionCommand("start");
        itemStop.setActionCommand("stop");
        itemStart.addActionListener(this);
        itemStop.addActionListener(this);
        menuServer.add(itemStart);
        menuServer.add(itemStop);
        menuBar.add(menuServer);

        JMenu help = new JMenu(userinterface.translate("help"));
        JMenuItem aboutItem = new JMenuItem(userinterface.translate("help"));
        aboutItem.setActionCommand("help");
        aboutItem.addActionListener(this);
        help.add(aboutItem);
        menuBar.add(help);

        setJMenuBar(menuBar);

        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints(); 
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridwidth = 2;
        gbl.setConstraints(userinterface, gbc);
        
        gbc.anchor = GridBagConstraints.CENTER;
        getContentPane().setLayout(gbl);
        getContentPane().add(userinterface);
    }
    public void actionPerformed(ActionEvent e)
    {
        if (e.getActionCommand().equals("start"))
        {
            if (serverProcess != null)
            {
                serverProcess.stopServer();
                userinterface.outputMessage("Server stoped.");
                serverProcess = null;
            }
            
            ServerConfigurationInfo sci = serverConfigurationInfoDialog.inputServerConfigurationInfo();
            if (sci == null)
                return;
            
            BStageInfo bstageInfo = sci.getBStageInfo();
            if (bstageInfo == null)
                return;
            serverProcess = new ServerProcess(userinterface);
            try
            {
                if (bstageInfo.getMapFileName() == null)
                {
                    URL mapFileURL = getClass().getClassLoader().getResource("bakutan.map");
                    serverProcess.init(bstageInfo, mapFileURL, userDataHolder);
                }
                else
                {
                    serverProcess.init(bstageInfo, userDataHolder);
                }
            }
            catch(Exception exc)
            {
                userinterface.outputMessage(exc.getMessage());
                exc.printStackTrace();
                return;
            }
            serverProcess.setRegisterServer(sci.isRegisterServer());
            serverProcess.setUseUPNP(sci.isUseUPNP());
            serverProcess.setServerInfoManagerAddress(sci.getServerInfoManagerAddress());
            serverProcess.start();
            userinterface.outputMessage("Server started.");
            userinterface.outputMessage("Port : " + serverProcess.getPort());
        }
        else if (e.getActionCommand().equals("stop"))
        {
            if (serverProcess != null)
            {
                serverProcess.stopServer();
                userinterface.outputMessage("Server stoped.");
                serverProcess = null;
            }
            else
            {
                userinterface.outputMessage("Server did not started.");
            }
        }
        else if (e.getActionCommand().equals("help"))
        {
            if (panelHelp != null)
            {
                panelHelp.setVisible(true);
            }
        }
    }
    public static void main(String[] args)
    {
        BakutanServerGUI bstageServerGUI = new BakutanServerGUI();
        bstageServerGUI.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        bstageServerGUI.setVisible(true);
    }
}

class ServerConfigurationInfoDialg extends JDialog implements ActionListener,ChangeListener
{
//    private int port;
//    private String loginMessage = "";
//    private int timeout = DataManager.DEFAULT_TIMEOUT;
//    private long timeLimit = DataManager.DEFAULT_TIMELIMIT;
//    private int maxClients = DataManager.DEFAULT_MAXCLIENTS;
    private JCheckBox checkRegister;
    private JCheckBox checkUPNP;
    private JTextField fieldLabel;
    private JTextField fieldPort;
    private JTextField fieldLoginMessage;
    private JTextField fieldTimelimit;
    private JTextField fieldMaxClients;
    private JTextField fieldBotNum;
    private JCheckBox checkUseOuterMapFile;
    private JButton buttonOpenMapFile;
    private ServerConfigurationInfo serverConfigurationInfo = null;
    private JComboBox comboMapFile;
    private JComboBox fieldMapFile;
    private JTextField fieldOuterMapFile = new JTextField();
    private BUserInterface userInterface;
    
    public ServerConfigurationInfoDialg(JFrame parent, BUserInterface userinterface)
    {
        super(parent, true);
        this.userInterface = userinterface;
        setTitle("Bakutan ServerInfoConfigurationInfoDialog");
        setSize(400,300);
        makeLayout();
        checkRegister.setSelected(false);
        checkUPNP.setSelected(false);
        checkUPNP.setEnabled(false);
        checkUseOuterMapFile.setSelected(false);
        checkUseOuterMapFile.addChangeListener(this);
        fieldOuterMapFile.setEditable(false);
        fieldOuterMapFile.setEnabled(false);
    }
    private void makeLayout()
    {
        String[] mapfiles = new String[]{"bakutan_std1.map","bakutan_std2.map","bakutan_clear1.map","bakutan_clear2.map"};
        comboMapFile = new JComboBox(mapfiles);

        checkRegister = new JCheckBox(userInterface.translate("register_to_server_list"));
        checkUPNP = new JCheckBox(userInterface.translate("use_upnp"));
        checkUseOuterMapFile = new JCheckBox(userInterface.translate("use_outer_mapfile"));
        buttonOpenMapFile = new JButton(userInterface.translate("open"));
        fieldLabel = new JTextField("Bakutan Server");
        fieldPort = new JTextField("0");
        fieldLoginMessage = new JTextField("Welcome to Bakutan");
        fieldTimelimit = new JTextField(String.valueOf(DataManager.DEFAULT_TIMELIMIT/60000l));
        fieldMaxClients = new JTextField(String.valueOf(DataManager.DEFAULT_MAXCLIENTS));
        fieldBotNum = new JTextField("0");

        JLabel labelLabel = new JLabel(userInterface.translate("label"));
        JLabel labelPort = new JLabel(userInterface.translate("port_0_is_auto"));
        JLabel labelLoginMessage = new JLabel(userInterface.translate("loginmessage"));
        JLabel labelTimelimit = new JLabel(userInterface.translate("timelimit_minute"));
        JLabel labelMaxClients = new JLabel(userInterface.translate("max_clients"));
        JLabel labelBotNum = new JLabel(userInterface.translate("number_of_bots"));
        JLabel labelMapFile = new JLabel(userInterface.translate("map_file"));
        buttonOpenMapFile.addActionListener(this);
        buttonOpenMapFile.setActionCommand("open_mapfile");
        buttonOpenMapFile.setEnabled(false);
        checkRegister.addChangeListener(this);

        JButton buttonStart = new JButton(userInterface.translate("start"));
        JButton buttonStop = new JButton(userInterface.translate("cancel"));
        buttonStart.setActionCommand("ok");
        buttonStop.setActionCommand("cancel");
        buttonStart.addActionListener(this);
        buttonStop.addActionListener(this);
        
        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints(); 
        
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbl.setConstraints(labelLabel, gbc);
        gbc.gridy++;
        gbl.setConstraints(labelPort, gbc);
        gbc.gridy++;
        gbl.setConstraints(labelLoginMessage, gbc);
        gbc.gridy++;
        gbl.setConstraints(labelTimelimit, gbc);
        gbc.gridy++;
        gbl.setConstraints(labelMaxClients, gbc);
        gbc.gridy++;
        gbl.setConstraints(labelBotNum, gbc);
        gbc.gridy++;
        gbl.setConstraints(labelMapFile, gbc);
        
        gbc.gridwidth = 2;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1;
        gbc.gridx++;
        gbc.gridy-=6;
        gbl.setConstraints(fieldLabel, gbc);
        gbc.gridy++;
        gbl.setConstraints(fieldPort, gbc);
        gbc.gridy++;
        gbl.setConstraints(fieldLoginMessage, gbc);
        gbc.gridy++;
        gbl.setConstraints(fieldTimelimit, gbc);
        gbc.gridy++;
        gbl.setConstraints(fieldMaxClients, gbc);
        gbc.gridy++;
        gbl.setConstraints(fieldBotNum, gbc);
        gbc.gridy++;
        gbl.setConstraints(comboMapFile, gbc);
        gbc.gridy++;
        gbl.setConstraints(checkUseOuterMapFile, gbc);
        gbc.gridy++;
        gbc.gridwidth = 1;
        gbl.setConstraints(fieldOuterMapFile, gbc);
        gbc.gridx++;
        gbc.weightx = 0;
        gbl.setConstraints(buttonOpenMapFile, gbc);

        gbc.gridx--;
        gbc.gridwidth = 3;
        gbc.gridy++;
        gbl.setConstraints(checkRegister, gbc);
        gbc.gridy++;
        gbl.setConstraints(checkUPNP, gbc);
        gbc.gridwidth = 1;

        gbc.fill = GridBagConstraints.NONE;
        gbc.gridy++;
        gbl.setConstraints(buttonStart, gbc);
        gbc.gridx++;
        gbl.setConstraints(buttonStop, gbc);
        
        getContentPane().setLayout(gbl);
        getContentPane().add(labelLabel);
        getContentPane().add(labelPort);
        getContentPane().add(labelLoginMessage);
        getContentPane().add(labelTimelimit);
        getContentPane().add(labelMaxClients);
        getContentPane().add(labelBotNum);
        getContentPane().add(fieldLabel);
        getContentPane().add(fieldPort);
        getContentPane().add(fieldLoginMessage);
        getContentPane().add(fieldTimelimit);
        getContentPane().add(fieldMaxClients);
        getContentPane().add(fieldBotNum);
        getContentPane().add(buttonStart);
        getContentPane().add(buttonStop);
        getContentPane().add(checkRegister);
        getContentPane().add(checkUPNP);
        getContentPane().add(labelMapFile);
        getContentPane().add(comboMapFile);
        getContentPane().add(checkUseOuterMapFile);
        getContentPane().add(fieldOuterMapFile);
        getContentPane().add(buttonOpenMapFile);
    }
    public void stateChanged(ChangeEvent e)
    {
        if (e.getSource() == checkUseOuterMapFile)
        {
            fieldOuterMapFile.setEnabled(checkUseOuterMapFile.isSelected());
            buttonOpenMapFile.setEnabled(checkUseOuterMapFile.isSelected());
            comboMapFile.setEnabled(!checkUseOuterMapFile.isSelected());
        }
        else if (e.getSource() == checkRegister)
        {
            checkUPNP.setEnabled(checkRegister.isSelected());
        }
    }
    public void actionPerformed(ActionEvent e)
    {
        if (e.getActionCommand().equals("ok"))
        {
            String loginMessage = fieldLoginMessage.getText();
            String lable = fieldLabel.getText();
            int port;
            int timelimit;
            int maxclients;
            int botNum;
            try
            {
                port = Integer.parseInt(fieldPort.getText());
            }
            catch(NumberFormatException exc)
            {
                fieldPort.setText("-1");
                return;
            }
            try
            {
                timelimit = Integer.parseInt(fieldTimelimit.getText());
            }
            catch(NumberFormatException exc)
            {
                fieldTimelimit.setText(String.valueOf(DataManager.DEFAULT_TIMELIMIT/60000l));
                return;
            }
            try
            {
                maxclients = Integer.parseInt(fieldMaxClients.getText());
            }
            catch(NumberFormatException exc)
            {
                fieldMaxClients.setText(String.valueOf(DataManager.DEFAULT_MAXCLIENTS));
                return;
            }
            try
            {
                botNum = Integer.parseInt(fieldBotNum.getText());
            }
            catch(NumberFormatException exc)
            {
                fieldBotNum.setText("0");
                return;
            }
            BStageInfo bstageInfo = new BStageInfo(BStageThrowBom.class.getName(), null, port, "label", loginMessage, false);
            for (int i=0;i<botNum;i++)
            {
                BotInfo bo = new BotInfo();
                LoginInfo li = new LoginInfo();
                li.setMode(LoginInfo.MODE_GUEST);
                li.setNickname("Bot:"+i);
                bo.setLoginInfo(li);
                bstageInfo.getBotInfos().add(bo);
            }
            
            if (checkUseOuterMapFile.isSelected())
            {
                bstageInfo.setMapFileName(fieldOuterMapFile.getText());
            }
            else
            {
                String mapFile = (String)comboMapFile.getSelectedItem();
                bstageInfo.setMapFileName(mapFile);
            }
            bstageInfo.setLabel(lable);
            bstageInfo.setTimeLimit(timelimit*60000l);
            bstageInfo.setMaxClients(maxclients);
            serverConfigurationInfo = new ServerConfigurationInfo();
            serverConfigurationInfo.setBStageInfo(bstageInfo);

            serverConfigurationInfo.setRegisterServer(checkRegister.isSelected());
            serverConfigurationInfo.setUseUPNP(checkUPNP.isSelected());
            setVisible(false);
        }
        else if (e.getActionCommand().equals("cancel"))
        {
            this.serverConfigurationInfo = null;
            setVisible(false);
        }
        else if (e.getActionCommand().equals("open_mapfile"))
        {
            JFileChooser jfc = new JFileChooser();
            if (jfc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
            {
                File mapfile = jfc.getSelectedFile();
                fieldOuterMapFile.setText(mapfile.toString());
            }
        }
    }
    public ServerConfigurationInfo inputServerConfigurationInfo()
    {
        serverConfigurationInfo = null;
        this.setVisible(true);
        return serverConfigurationInfo;
    }
}