/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/07
 */
package bakusen.game.arena;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;

import bakusen.client.BUserInterface;
import bakusen.common.BPickIntersection;
import bakusen.common.BStage;
import bakusen.common.Body;
import bakusen.common.CharacterImage;
import bakusen.common.DataManager;
import bakusen.common.EventHandler;
import bakusen.common.event.BodyAddEvent;
import bakusen.common.event.BodyEvent;
import bakusen.common.event.BodyArmorEvent;
import bakusen.common.math.BPolygon;
import bakusen.common.math.BVector2f;
import bakusen.common.util.BAudio;

/**
 * @author cattaka
 */
public class BodyBom extends Body
{
	public static final int ACTION_BOMB = 0;
	public static final int ACTION_HOP = 1;
	public static final int OPTION_FIRE = 0;
	
	public BodyBom()
	{
		setType(BStageArena.BOM1);
		setOptions(new int[1]);
		setOption(BodyBom.OPTION_FIRE, getOption(OPTION_FIRE));
		setPolygon(new BPolygon(BPolygon.SQUARE, BStageArena.BOM_WIDTH));
		enableState(Body.COLLIDABLE);
		enableState(Body.LIFE);
	}
	public boolean hasName()
	{
		return false;
	}
	public boolean hasVariableBPolygon()
	{
		return false;
	}
	public boolean hasScore()
	{
		return false;
	}
	public void drawCharacter(Graphics g, Dimension offset, long currenttime)
	{
		if (!isState(COLLIDABLE))
		{
			int offsetx = getCharacterImage().getWidth() / 2;
			int offsety = getCharacterImage().getHeight() / 2;
			int x = -offset.width + (int) (getPosition().x * DataManager.CHARACTERSIZE - offsetx);
			int y = -offset.height + (int) (getPosition().y * DataManager.CHARACTERSIZE - offsety);
			int n = ((int)(currenttime - getLastStateChange()))/DataManager.ANIMINTERVAL;
			Image[] img = getCharacterImage().getImages(CharacterImage.DEAD);
			g.drawImage(img[n % img.length], x, y, null);
		}
		super.drawCharacter(g, offset, currenttime);
	}
	public void actionForClient(BodyEvent bodyEvent, BAudio baudio, BUserInterface ui, long currenttime)
	{
		if (bodyEvent instanceof BodyAddEvent)
		{
			baudio.play(BStageArena.SOUND_PUTBOM, 0);
		}
	}
	public void eventAction(EventHandler eventHandler, BStage bstage, BVector2f position, int action)
	{
		if (action == ACTION_BOMB)
		{
			long currenttime = bstage.getCurrentTime();
			
			BVector2f p = new BVector2f(getPosition());
			p.x = 0.5f + (int)(p.x);
			p.y = 0.5f + (int)(p.y);
			
			Body fire = new BodyFire();
			Body[] tgs = new Body[4];
			int[] length = fire.getOptions();;
			{
				BVector2f end = new BVector2f();
				BPickIntersection pi;
				int l;
				l = getOptions()[OPTION_FIRE];
				length[BodyFire.OPTION_LENGTH] = l;
				//length = new byte[]{l,l,l,l};
				end.set(p);
				end.x += l;
				pi = bstage.pickClosest(p,end,Body.FIREWALL);
				if (pi != null)
				{
					length[BodyFire.OPTION_RIGHT] = (int)pi.getDistance();
					if (pi.getBody().isState(Body.LIFE))
					{
						tgs[0] = pi.getBody();
						//eventHandler.damage(currenttime, currenttime, pi.getBody(), this, BodyFire.DAMAGE);
					}
				}else
				{
					length[BodyFire.OPTION_RIGHT] = l;
				}
				end.set(p);
				end.y -= l;
				pi = bstage.pickClosest(p,end,Body.FIREWALL);
				if (pi != null)
				{
					length[BodyFire.OPTION_UP] = (int)pi.getDistance();
					if (pi.getBody().isState(Body.LIFE))
					{
						tgs[1] = pi.getBody();
						//eventHandler.damage(currenttime, currenttime, pi.getBody(), this, BodyFire.DAMAGE);
					}
				}else
				{
					length[BodyFire.OPTION_UP] = l;
				}
				end.set(p);
				end.x -= l;
				pi = bstage.pickClosest(p,end,Body.FIREWALL);
				if (pi != null)
				{
					length[BodyFire.OPTION_LEFT] = (int)pi.getDistance();
					if (pi.getBody().isState(Body.LIFE))
					{
						tgs[2] = pi.getBody();
						//eventHandler.damage(currenttime, currenttime, pi.getBody(), this, BodyFire.DAMAGE);
					}
				}else
				{
					length[BodyFire.OPTION_LEFT] = l;
				}
				end.set(p);
				end.y += l;
				pi = bstage.pickClosest(p,end,Body.FIREWALL);
				if (pi != null)
				{
					length[BodyFire.OPTION_DOWN] = (int)pi.getDistance();
					if (pi.getBody().isState(Body.LIFE))
					{
						tgs[3] = pi.getBody();
						//eventHandler.damage(currenttime, currenttime, pi.getBody(), this, BodyFire.DAMAGE);
					}
				}else
				{
					length[BodyFire.OPTION_DOWN] = l;
				}
			}
			fire.setOptions(length);
			fire.setPolygon(createBpolygon(length));
			fire.setPosition(p);
			
			eventHandler.delete(currenttime, currenttime,this);
			//eventHandler.sound(currenttime,currenttime,BStageArena.SOUND_BOMB);
			eventHandler.add(currenttime, currenttime, fire);
			for (int i=0;i<tgs.length;i++)
			{
				if (tgs[i] != null)
				{
					eventHandler.armor(currenttime, currenttime, tgs[i], fire, BodyArmorEvent.SUB, BodyFire.DAMAGE);
				}
			}
			eventHandler.delete(currenttime, currenttime + 500, fire);
			
			/*
			BPickResult pr = bstage.pickAll(fire,0);
			Iterator ite = pr.getBPickIntersections();
			while(ite.hasNext())
			{
				BPickIntersection pi = (BPickIntersection)ite.next();
				Body body = pi.getBody();
				if (body == this)
				{
					continue;
				}
				if (body.isState(Body.LIFE))
				{
					BodyDamageEvent ibde = new BodyDamageEvent(100);
					StageEvent sse = new StageEvent(new Body[]{fire,body},ibde,currenttime);
					sse.setGenerationTime(currenttime);
					bstage.getEventThread().putStageEvent(sse);
				}
			}
			*/
		}else if (action == ACTION_HOP)
		{
			long currenttime = bstage.getCurrentTime();
			if (!getDirection().isZeroVector()
					&& bstage.isIntersectOther(this, COLLIDABLE))
			{
				BVector2f pos = new BVector2f(getDirection());
				pos.normalize();
				pos.add(getPosition());
				jump(currenttime, pos, eventHandler);
			}else
			{
				eventHandler.state(currenttime, currenttime, this, ENABLE, Body.LIFE | Body.COLLIDABLE);
				BVector2f pos = new BVector2f(getPosition());
				pos.x = 0.5f + (int)pos.x;
				pos.y = 0.5f + (int)pos.y;
				eventHandler.setPosition(currenttime, currenttime, this, pos);
				//eventHandler.move(currenttime, currenttime, this, getPosition(), new BVector2f(0,0), getDirection());
			}
		}
	}
	public void eventCollisition(EventHandler eventHandler, BStage bstage, Body target)
	{
		if (isState(COLLIDABLE) && target.isState(Body.COLLIDABLE))
		{
			long currenttime = bstage.getCurrentTime();
			BVector2f v = new BVector2f(0,0);
			BVector2f p = new BVector2f(getPosition());
			p.x = 0.5f + (int)(p.x);
			p.y = 0.5f + (int)(p.y);
			eventHandler.move(currenttime, currenttime, this, p, v, v);
		}
	}
	
	public void eventStateChange(EventHandler eventHandler, BStage bstage, byte type, int state)
	{
		if ((type == Body.SET || type == Body.ENABLE)
				&& (state & Body.DEAD) == Body.DEAD)
		{
			long currenttime = bstage.getCurrentTime();
			eventHandler.action(currenttime,currenttime,this,getPosition(),ACTION_BOMB);
		}
	}
	public static BPolygon createBpolygon(int[] length)
	{
		float[] dls = new float[4];
		for (int i=0;i<dls.length;i++)
		{
			dls[i] = length[i+1];
		}
		return new BPolygon(BPolygon.CROSS,BStageArena.FIRE_WIDTH, dls);
	}
	public boolean isCheckIntersectWith(Body target)
	{
		/*
		if (target.getType() == BStageDeathMatch.MAN1
			&& (target.getOption(BodyMan.OPTION_WEAPON) & BodyMan.WEAPON_KICK) == BodyMan.WEAPON_KICK )
		{
			return true;
		}
		*/
		if (target.getType() == BStageArena.FIRE1)
			return true;
		else
			return false;
	}
	public void jump(long currenttime, BVector2f jumpTo, EventHandler eventHandler)
	{
		BVector2f sub = new BVector2f(jumpTo);
		BVector2f pos = new BVector2f(jumpTo);
		sub.sub(getPosition());
		//sub.normalize();
		sub.scale(2f);
		eventHandler.state(currenttime, currenttime, this, DISABLE, Body.LIFE | Body.COLLIDABLE);
		eventHandler.move(currenttime, currenttime,this,getPosition(),sub,sub);
		eventHandler.action(currenttime, currenttime + 500, this, pos, ACTION_HOP);
	}
}
