/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

package bakutan.client;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import bakutan.common.BUserInterface;
import bakutan.common.serverinfo.SmallServerInfo;

public class ConfigurationPanel extends JPanel implements ActionListener
{
	public static final int CANCEL_OPTION = 0;
	public static final int APPROVE_OPTION = 1;
	public static final int ERROR_OPTION = 2;

	private JPanel genericPane;
    private KeyConfigPanel keyConfigPanel;
	private JTextField defaultUserNameF;
    private JTextField defaultNickNameF;
    private JTextField fpsLimitF;
	private JCheckBox buttonSound;
	private JCheckBox buttonBGM;
    private JCheckBox buttonForceReconnect;
    private JSlider scrollVolumeBGM;
    private JSlider scrollVolumeAudio;
    
	private JButton okB;
	private JButton cancelB;

	private SmallServerInfo serverLinkInfo;	
	
	private ConfigurationInfo config;
	private BUserInterface userinterface;
	
	public ConfigurationPanel(BUserInterface userinterface)
	{
		this.userinterface = userinterface;
		setSize(400,500);
		serverLinkInfo = new SmallServerInfo();
		
		initialize();
	}

	private void initialize()
	{
		GridBagLayout gbl;
		GridBagConstraints gbc;

		okB = new JButton(userinterface.translate("ok"));
		cancelB = new JButton(userinterface.translate("cancel"));
		
		/* generic panel */
		JLabel defaultUserNameL = new JLabel(userinterface.translate("default_user_name"));
        JLabel defaultNickNameL = new JLabel(userinterface.translate("default_nick_name"));
        JLabel fpsLimitL = new JLabel(userinterface.translate("fps_limit"));
        JLabel volumeBGML = new JLabel(userinterface.translate("bgm_volume"));
        JLabel volumeAudioL = new JLabel(userinterface.translate("se_volume"));
		defaultUserNameF = new JTextField("Bakusen");
        defaultNickNameF = new JTextField("Bakusen");
        fpsLimitF = new JTextField("60");
		buttonSound = new JCheckBox(userinterface.translate("sound"));
		buttonBGM = new JCheckBox(userinterface.translate("bgm"));
        buttonForceReconnect = new JCheckBox(userinterface.translate("force_reconnect"));
        scrollVolumeBGM = new JSlider(JSlider.HORIZONTAL, 0, 127, 64);
        scrollVolumeAudio = new JSlider(JSlider.HORIZONTAL, 0, 127, 64);
        
        JTabbedPane mainTab = new JTabbedPane();
        // generic panel
		genericPane = new JPanel();
		gbl = new GridBagLayout();
		gbc = new GridBagConstraints();
        gbc.insets = new Insets(5,5,5,5);
        
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridx=0; gbc.gridy=0;
        gbl.setConstraints(defaultNickNameL,gbc);
		gbc.gridy++;
        gbl.setConstraints(defaultUserNameL,gbc);
        gbc.gridy++;
        gbl.setConstraints(fpsLimitL,gbc);
        gbc.gridy++;
		gbl.setConstraints(buttonSound,gbc);
		gbc.gridy++;
		gbl.setConstraints(buttonBGM,gbc);
        gbc.gridy++;
        gbl.setConstraints(buttonForceReconnect,gbc);
        gbc.gridy++;
        gbl.setConstraints(volumeBGML,gbc);
        gbc.gridy++;
        gbl.setConstraints(volumeAudioL,gbc);
		gbc.gridy -= 7;
		gbc.gridx++;
		gbc.weightx=1;
		gbc.fill = GridBagConstraints.HORIZONTAL;
        gbl.setConstraints(defaultNickNameF,gbc);
        gbc.gridy++;
        gbl.setConstraints(defaultUserNameF,gbc);
        gbc.gridy++;
        gbl.setConstraints(fpsLimitF,gbc);
        gbc.gridy += 4;
        gbl.setConstraints(scrollVolumeBGM,gbc);
        gbc.gridy++;
        gbl.setConstraints(scrollVolumeAudio,gbc);
        
		genericPane.setLayout(gbl);
		genericPane.add(defaultUserNameL);
		genericPane.add(defaultUserNameF);
        genericPane.add(defaultNickNameL);
        genericPane.add(defaultNickNameF);
        genericPane.add(fpsLimitL);
        genericPane.add(fpsLimitF);
		genericPane.add(buttonSound);
		genericPane.add(buttonBGM);
        genericPane.add(buttonForceReconnect);
        genericPane.add(volumeAudioL);
        genericPane.add(volumeBGML);
        genericPane.add(scrollVolumeBGM);
        genericPane.add(scrollVolumeAudio);
        
		mainTab.add(userinterface.translate("generic"), genericPane);
		
		gbl = new GridBagLayout();
		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.NONE;
		
		gbc.gridx=0; gbc.gridy=0;
		gbc.gridwidth = 4;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx=1; gbc.weighty=1;
		gbl.setConstraints(mainTab,gbc);

		gbc.gridwidth = 1;
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx=0; gbc.weighty=0;
		gbc.gridy++;
		gbl.setConstraints(okB,gbc);
		gbc.gridx++;
		gbl.setConstraints(cancelB,gbc);

		setLayout(gbl);
		add(mainTab);
		add(okB);
		add(cancelB);
        
		okB.addActionListener(this);
		cancelB.addActionListener(this);

        //keyConfig
        keyConfigPanel = new KeyConfigPanel(userinterface);
        mainTab.add(userinterface.translate("key_config"), keyConfigPanel);
    }

	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == okB)
		{
            closeOperation(true);
		}
		else if (e.getSource() == cancelB)
		{
            closeOperation(false);
		}
	}

	public void setConfigurationInfo(ConfigurationInfo ci)
	{
		defaultUserNameF.setText(ci.getDefaultUserName());
        defaultNickNameF.setText(ci.getDefaultNickName());
        fpsLimitF.setText(String.valueOf(ci.getFpsLimit()));
		buttonSound.setSelected(ci.isSoundEffect());
		buttonBGM.setSelected(ci.isBgm());
		buttonForceReconnect.setSelected(ci.isForceReconnect());
        keyConfigPanel.setKeyConfig(ci.getKeyConfig());
        scrollVolumeAudio.setValue(ci.getVolumeSE());
        scrollVolumeBGM.setValue(ci.getVolumeBGM());
	}
	
	public ConfigurationInfo getConfigurationInfo()
	{
		ConfigurationInfo ci = new ConfigurationInfo();
		ci.setDefaultUserName(defaultUserNameF.getText());
        ci.setDefaultNickName(defaultNickNameF.getText());
        try
        {
            ci.setFpsLimit(Integer.parseInt(fpsLimitF.getText()));
        }
        catch(NumberFormatException e)
        {
        }
		ci.setSoundEffect(buttonSound.isSelected());
		ci.setBgm(buttonBGM.isSelected());
		ci.setForceReconnect(buttonForceReconnect.isSelected());
        ci.setKeyConfig(keyConfigPanel.getKeyConfig());
        ci.setVolumeSE(scrollVolumeAudio.getValue());
        ci.setVolumeBGM(scrollVolumeBGM.getValue());
		return ci;
	}
    public void closeOperation(boolean approve)
    {
    }
}
