/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/01/16
 */
package bakusen.debug;

import bakusen.common.math.BMath;
import bakusen.common.math.BVector2f;
import bakusen.common.math.BVector3f;
import bakusen.common.math.BVector4f;

/**
 * @author cattaka
 */
public class MathTest
{
	public static void test1()
	{
		BVector2f p1 = new BVector2f( 1, 1);
		BVector2f p2 = new BVector2f( 1,-1);
		BVector2f v = new BVector2f(1,1);

		BVector4f plain = new BVector4f();
		BMath.createPlane(plain,p1,p2,v);
		System.out.println("Plain:" + plain);
		BVector2f fx = new BVector2f(-1, 1);
		BVector2f fy = new BVector2f(-1,-1);
		float z = BMath.getIntersection(plain,fx,fy);
		float x = z*fx.x + fx.y;
		float y = z*fy.x + fy.y;
		BVector3f ip = new BVector3f(x,y,z);
		System.out.println("Intersect Point:" + ip);
	}
	public static void test2()
	{
		BVector2f p11 = new BVector2f( 1, 1);
		BVector2f p12 = new BVector2f( 1, 0);		
		BVector2f p21 = new BVector2f(-1, 0);
		BVector2f p22 = new BVector2f(-1,-1);		
		BVector2f v1 = new BVector2f(-1, 0);
		BVector2f v2 = new BVector2f( 1, 0);		
		float t = BMath.nextCollidPoint(p11,p12,v1,p21,p22,v2);
		System.out.println(t);
	}
	public static void test3()
	{
		BVector2f[] vs1 = new BVector2f[]{
				new BVector2f( 0.51f, 0.51f),
				new BVector2f( 0.51f,-0.51f),
				new BVector2f(-0.51f,-0.51f),
				new BVector2f(-0.51f, 0.51f)
		};
		BVector2f[] vs2 = new BVector2f[]{
				new BVector2f( 0.0f, 0.5f),
				new BVector2f( 0.5f, 0.0f),
				new BVector2f( 0.0f,-0.5f),
				new BVector2f(-0.5f, 0.0f)
		};
		BPolygon pg1 = new BPolygon(new BVector2f[][]{vs1});
		BPolygon pg2 = new BPolygon(new BVector2f[][]{vs2});
		BVector2f p1 = new BVector2f();
		BVector2f p2 = new BVector2f();
		p1.x = 0;
		p1.y = 0;
		p2.x = 5;
		p2.y = 0.0f;
		
		BVector2f v1 = new BVector2f( 0, 0);
		BVector2f v2 = new BVector2f(-1, 0);	
		BVector2f normal1 = new BVector2f();
		BVector2f normal2 = new BVector2f();
		//float t = BMath.nextCollidPoint(normal1,normal2,p1,pg1,v1,p2,pg2,v2);
		float t = BMath.nextCollidPoint(normal1,normal2,p1,vs1,v1,p2,vs2,v2);
		System.out.println(normal1);
		System.out.println(normal2);
		System.out.println(t);
	}
	public static void test4()
	{
		BVector2f p11 = new BVector2f( 0, 0);
		BVector2f p12 = new BVector2f( 2, 2);
		BVector2f p21 = new BVector2f( 2, 0);
		BVector2f p22 = new BVector2f( 0, 2);
		BVector2f dist = new BVector2f();
		BMath.getIntersection(dist,p11,p12,p21,p22);
		System.out.println(dist);
	}
	public static void test5()
	{
		BVector2f[] vs1 = new BVector2f[]{
				new BVector2f( 1, 1),
				new BVector2f( 1,-1),
				new BVector2f(-1,-1),
				new BVector2f(-1, 1)
		};
		BVector2f[] vs2 = new BVector2f[]{
				new BVector2f( 2, 2),
				new BVector2f( 2,-2),
				new BVector2f(-2,-2),
				new BVector2f(-2, 2)
		};
		BPolygon pg1 = new BPolygon(new BVector2f[][]{vs1});
		BPolygon pg2 = new BPolygon(new BVector2f[][]{vs2});
		
		BVector2f p1 = new BVector2f( 0, 0);
		BVector2f p2 = new BVector2f( 0, 0);
		boolean f = BMath.isIntersect(p1,pg1,p2,pg2);
		System.out.println(f);

		p1.set(10,0);
		p2.set(-1,0);
		f = BMath.isIntersect(p1,pg1,p2,pg2);
		System.out.println(f);
	}
	public static void test6()
	{
		BVector2f[] vs = new BVector2f[]{
				new BVector2f( 1, 1),
				new BVector2f( 1,-1),
				new BVector2f(-1,-1),
				new BVector2f(-1, 1)
		};
		BPolygon pg1 = new BPolygon(new BVector2f[][]{vs});
		
		BVector2f pos = new BVector2f( 0, 0);
		BVector2f p1 = new BVector2f( 2, 2);
		BVector2f p2 = new BVector2f( -2, 0);
		BVector2f dist = new BVector2f();
		float d = BMath.getClosestIntersection(p1,p2,pos,pg1);
		System.out.println(""+d+","+dist);
	}
	public static void test7()
	{
		BVector2f p11 = new BVector2f(-1, 1);
		BVector2f p12 = new BVector2f(-1,-1);		
		BVector2f p21 = new BVector2f( 2, 0);
		BVector2f p22 = new BVector2f( 1, 0);		
		BVector2f v1 = new BVector2f(-1, 0);
		BVector2f v2 = new BVector2f( 1, 0);		
		float t = BMath.nextCollidPoint(p11,p12,v1,p21,p22,v2);
		System.out.println(t);
	}
	public static void test8()
	{
		BVector2f p1 = new BVector2f();
		BVector2f p2 = new BVector2f();
		p1.x = 0;
		p1.y = 0;
		p2.x = 5;
		p2.y = 0.0f;

		BVector2f p11 = new BVector2f( 0.51f, 0.51f);
		BVector2f p12 = new BVector2f( 0.51f,-0.51f);
		BVector2f p21 = new BVector2f( 4.5f, 0.0f);
		BVector2f p22 = new BVector2f( 5.0f, 0.5f);
		
		BVector2f v1 = new BVector2f( 0, 0);
		BVector2f v2 = new BVector2f(-1, 0);	
		float t = BMath.nextCollidPoint(p11,p12,v1,p21,p22,v2);
		System.out.println(t);
	}
	public static void test9()
	{
		BPolygon pg = new BPolygon(BPolygon.SQUARE, 0.49f);
		BVector2f v1 = new BVector2f(7.5f, 7.5f);
		BVector2f v2 = new BVector2f(11.5f, 7.5f);		
		BVector2f p = new BVector2f(9.5f, 7.5f);
		float t = BMath.getClosestIntersection(v1,v2,p,pg);
		System.out.println(t);
	}
	public static void test10()
	{
		BPolygon pg1 = new BPolygon(BPolygon.SQUARE, 0.49f);
		BPolygon pg2 = new BPolygon(BPolygon.SQUARE, 0.49f);
		BVector2f v1 = new BVector2f(1, 0);
		BVector2f v2 = new BVector2f(0, 0);		
		BVector2f p1 = new BVector2f(0, 0);
		BVector2f p2 = new BVector2f(5, 0);
		BVector2f nd1 = new BVector2f();
		BVector2f nd2 = new BVector2f();
		float t = BMath.nextCollidPoint(nd1,nd2,p1,pg1,v1,p2,pg2,v2);
		System.out.println(t);
	}
	public static void test11()
	{
		BVector2f dist = new BVector2f();
		BVector2f p1 = new BVector2f(0, 0);
		BVector2f v1 = new BVector2f(1, -1);
		BVector2f p2 = new BVector2f(1, 0);		
		BMath.getClosestPoint(dist, p1, v1, p2);
		System.out.println(dist);
	}
	public static void test12()
	{
		BPolygon pg1 = new BPolygon(BPolygon.SQUARE, 0.5f);
		BPolygon pg2 = new BPolygon(BPolygon.DIAGONAL_SQUARE, 0.5f);
		BVector2f v = new BVector2f();
		BVector2f p1 = new BVector2f();
		BVector2f p2 = new BVector2f(1, 0);
		BVector2f d1 = new BVector2f();
		BVector2f d2 = new BVector2f();
		float dist = BMath.nextCollidPoint(d1,d2,p1, pg1, v, p2, pg2, v);
		boolean b = BMath.isIntersect(p1, pg2, p2,pg2);
		System.out.println(dist);
		System.out.println(b);
	}

	public static void main(String[] args)
	{
		test12();
	}
}
