package bakutan.client;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import bakutan.common.ai.AIInterface;
import bakutan.game.deathmatch.AIAlgorismicVer1;
import bakutan.game.deathmatch.AIAlgorismicVer2;
import bakutan.game.deathmatch.AINeuralnetworkV3;

public class AISelectDialog extends JDialog implements ActionListener
{
    private JRadioButton algorismicV1Radio = new JRadioButton("AlgorismVer1");
    private JRadioButton algorismicV2Radio = new JRadioButton("AlgorismVer2");
    private JRadioButton neuralnetworkV3Radio = new JRadioButton("NeuralNetworkVer3");
    private JRadioButton otherRadio = new JRadioButton("Other");
    private JTextField otherField = new JTextField();
    private boolean confirm = false;
    private AIInterface aiInterface = null;
    
	public AISelectDialog()
	{
		setTitle("Select AIInterface");
		setSize(300,200);
		initialize();
	}

	public void initialize()
	{
		JButton bok = new JButton("OK");
        JButton bcancel = new JButton("Cancel");
        
        ButtonGroup group = new ButtonGroup();
        group.add(algorismicV1Radio);
        group.add(algorismicV2Radio);
        group.add(neuralnetworkV3Radio);
        group.add(otherRadio);
        algorismicV1Radio.setSelected(true);
        algorismicV1Radio.addActionListener(this);
        neuralnetworkV3Radio.addActionListener(this);
        otherRadio.addActionListener(this);
        
        GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weighty=0;

		gbc.gridx=0;
        gbc.gridy=0;
        gbl.setConstraints(algorismicV1Radio,gbc);
        gbc.gridy++;
        gbl.setConstraints(algorismicV2Radio,gbc);
        gbc.gridy++;
		gbl.setConstraints(neuralnetworkV3Radio,gbc);
        gbc.gridy++;
		gbl.setConstraints(otherRadio,gbc);
        gbc.gridy++;
        gbl.setConstraints(otherField,gbc);

		gbc.weightx=0;
		gbc.fill = GridBagConstraints.NONE;
		gbc.gridx=0;
        gbc.gridy++;
		//gbc.gridwidth=2;
		gbl.setConstraints(bok,gbc);
        gbc.gridy++;
        gbl.setConstraints(bcancel,gbc);

		getContentPane().setLayout(gbl);
		getContentPane().add(algorismicV1Radio);
        getContentPane().add(algorismicV2Radio);
		getContentPane().add(neuralnetworkV3Radio);
		getContentPane().add(otherRadio);
        getContentPane().add(otherField);
		getContentPane().add(bok);
        getContentPane().add(bcancel);

        bok.setActionCommand("ok");
        bcancel.setActionCommand("cancel");
		bok.addActionListener(this);
		bcancel.addActionListener(this);
        
        update();
    }
	
	private void update()
    {
	    if(otherRadio.isSelected())
        {
            otherField.setEditable(true);
        }
        else
        {
            otherField.setEditable(false);
        }
    }
    
	public void actionPerformed(ActionEvent e)
	{
		if (e.getActionCommand().equals("ok"))
		{
            if (algorismicV1Radio.isSelected())
            {
                aiInterface = new AIAlgorismicVer1();
                confirm = true;
            }
            else if (algorismicV2Radio.isSelected())
            {
                aiInterface = new AIAlgorismicVer2();
                confirm = true;
            }
            else if (neuralnetworkV3Radio.isSelected())
            {
                aiInterface = new AINeuralnetworkV3();;
                confirm = true;
            }
            else if (otherRadio.isSelected())
            {
                try
                {
                    Class aiClass = Class.forName(otherField.getText().trim());
                    aiInterface = (AIInterface)(aiClass.newInstance());
                    confirm = true;
                }
                catch(ClassNotFoundException ex)
                {
                    JOptionPane.showMessageDialog(this,otherField.getText()+" is not found.");
                }
                catch(ClassCastException ex)
                {
                    JOptionPane.showMessageDialog(this,otherField.getText()+" is not AIInterface.");
                }
                catch(Exception ex)
                {
                    confirm = false;
                    JOptionPane.showMessageDialog(this,ex.getMessage());
                }
            }
            if (confirm)
            {
                setVisible(false);
            }
		}
        else if (e.getActionCommand().equals("cancel"))
        {
            confirm = false;
            setVisible(false);
        }
        else
        {
            update();
        }
	}

    public AIInterface showDialog()
    {
        setModal(true);
        setVisible(true);
        
        if (confirm)
            return aiInterface;
        else
            return null;
    }
    
    public void setOtherAIClassName(String arg)
    {
        otherField.setText(arg);
    }
    public String getOtherAIClassName()
    {
        if (otherRadio.isSelected())
        {
            return otherField.getText().trim();
        }
        return null;
    }
    
	public static void main(String[] args)
	{
		AISelectDialog d = new AISelectDialog();
        System.out.println(d.showDialog());
        System.exit(0);
	}
}
