package bakutan.client;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.regex.Pattern;
import java.util.zip.GZIPInputStream;

import neuralnetwork.NeuralNetwork;

/*
 * 作成日: 2006/11/24
 *
 * ウィンドウ - 設定 - Java - コード・スタイル - コード・テンプレート
 */

public class AiLearningToolCui
{
    public static final int DEFAULT_REPEAT = 1000;
    public static final int DEFAULT_LAYERS = 5;
    public static final int DEFAULT_NODES = 50;
    public static final double DEFAULT_LEANINGRATE = 0.001;
    public static final int BAR_WIDTH = 10;
    public static final int DEFAULT_GRAPH_WIDTH = 500;

    NeuralNetwork neuralNetwork;

    double[][] orgInputData = new double[0][0];
    double[][] orgDesiredData = new double[0][0];
    double[] orgDesiredMin = new double[0];
    double[] orgDesiredMax = new double[0];
    File workDir = null;
    
    int inputNeuronNum = 1;
    int outputNeuronNum = 1;
    
    LearnThread learnThread = null;
    
    public AiLearningToolCui()
    {
        super();
        reset(DEFAULT_LAYERS, DEFAULT_NODES);
    }
    public AiLearningToolCui(File workdir)
    {
        super();
        this.workDir = workdir;
        reset(DEFAULT_LAYERS, DEFAULT_NODES);
    }
    
    private void learn(int repeat, double rate)
    {
        if (learnThread != null)
        {
            return;
        }
        neuralNetwork.setLearningRate(rate);
        //neuralNetwork.setLearningWeight(1d);
        //neuralNetwork.setLearningWeight(new double[]{ 1, 1, 1, 1, 0.1, 0.1, 0.1, 0.1});
        learnThread = new LearnThread(repeat);
        learnThread.setPriority(Thread.MIN_PRIORITY);
        learnThread.start();
    }
    public void stopLearn()
    {
        if (learnThread == null)
        {
            return;
        }
        learnThread.stopLearning();
        try
        {
            learnThread.join();
        }
        catch(InterruptedException e)
        {
            e.printStackTrace();
        }
    }
    private void reset(int layernum, int nodenum)
    {
        if (learnThread != null)
        {
            return;
        }
        int[] layers = new int[2+layernum];
        for (int i=1;i<layers.length-1;i++)
        {
            layers[i] = nodenum;
        }
        layers[0] = inputNeuronNum;
        layers[layers.length - 1] = outputNeuronNum;
        
        if (neuralNetwork != null)
            neuralNetwork.cleanUp();
        neuralNetwork = new NeuralNetwork();
        neuralNetwork.initialize(layers);
        neuralNetwork.setLinearOutput(false);
        //double[] learningWeight = new double[]{4,4,4,4,1,1,1,1};
        //neuralNetwork.setLearningWeight(learningWeight);
    }
    public void loadRecord(URL url)
    {
        if (learnThread != null)
        {
            stopLearn();
        }
        ByteArrayOutputStream rawData = new ByteArrayOutputStream(1024*1024);
        try
        {
            InputStream in = new GZIPInputStream(url.openStream());
            int i;
            while((i=in.read()) != -1)
            {
                rawData.write(i);
            }
            byte[] rd = rawData.toByteArray();
            DataInputStream din = new DataInputStream(new ByteArrayInputStream(rd));
            this.inputNeuronNum = din.readInt();
            this.outputNeuronNum = din.readInt();
            int dataNum = rd.length/((Integer.SIZE/8*2) + (inputNeuronNum+outputNeuronNum)*Double.SIZE/8);
            orgInputData = new double[dataNum][inputNeuronNum];
            orgDesiredData = new double[dataNum][outputNeuronNum];
            orgDesiredMin = new double[outputNeuronNum];
            orgDesiredMax = new double[outputNeuronNum];
            for (int j=0;j<outputNeuronNum;j++)
            {
                orgDesiredMin[j] = Double.NaN;
                orgDesiredMax[j] = Double.NaN;
            }
            for (int j=0;j<dataNum;j++)
            {
                if (inputNeuronNum != din.readInt())
                    throw new ArrayIndexOutOfBoundsException();
                for (int k=0;k<inputNeuronNum;k++)
                {
                    orgInputData[j][k] = din.readDouble();
                }
                if (outputNeuronNum != din.readInt())
                    throw new ArrayIndexOutOfBoundsException();
                for (int k=0;k<outputNeuronNum;k++)
                {
                    orgDesiredData[j][k] = din.readDouble();
                    if (Double.isNaN(orgDesiredMin[k]) || orgDesiredData[j][k] < orgDesiredMin[k])
                        orgDesiredMin[k] = orgDesiredData[j][k];
                    if (Double.isNaN(orgDesiredMax[k]) || orgDesiredData[j][k] > orgDesiredMax[k])
                        orgDesiredMax[k] = orgDesiredData[j][k];
                }
            }
//            System.out.println("DataNum :" + dataNum);
//            System.out.println("InputNodes :" + inputNum);
//            System.out.println("OutputNodes :" + outputNum);
//            for (int k=0;k<outputNum;k++)
//            {
//                System.out.println(orgDesiredMin[k] + ", " + orgDesiredMax[k]);
//            }
//            for (int j=0;j<dataNum;j++)
//            {
//                for (int k=0;k<orgDesiredData[j].length;k++)
//                {
//                    System.out.print(orgDesiredData[j][k] + ", ");
//                }
//                System.out.println();
//            }
        }
        catch(IOException exc)
        {
            exc.printStackTrace();
        }
        //org = new BufferedImage(orgInputData.length, aiRapper.getOutputNeuronNum()*(BAR_WIDTH*2+1), BufferedImage.TYPE_INT_RGB);
        reset(1,1);
    }
    
    public static void main(String[] args)
    {
        AiLearningToolCui gt = new AiLearningToolCui(new File("./"));
        boolean error = false;
        if (args.length == 2)
        {
            try
            {
                gt.loadRecord(new File(args[0]).toURL());
                gt.load(new File(args[1]));
            }
            catch(IOException e)
            {
                error = true;
            }
        }
        else if (args.length != 0)
        {
            error = true;
        }
        if (error)
        {
            System.out.println(
                    "Usage1: java AILearningToolCUI\n"
                    +"Usage2: java AILearningToolCUI <record file> <neuralnetwork file>"
                    );
        }
        else
        {
            gt.uiProcess();
        }
        
    }
    public void uiProcess()
    {
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        System.out.println("Started:");
        while(true)
        {
            if (learnThread == null)
                System.out.print(">");
            String input = null;
            try
            {
                input = in.readLine();
            }
            catch(IOException e)
            {
                break;
            }
            String[] args = Pattern.compile("[ ]").split(input, 0); // = input.split("[ ]");
            boolean error = false;
            if (learnThread != null || args[0].equals("stop"))
            {
                stopLearn();
                System.out.println("Learning is Stoped.");
            }
            else if (args[0].equals("learn"))
            {
                if (args.length == 3)
                {
                    try
                    {
                        int repeat = Integer.parseInt(args[1]);
                        double rate = Double.parseDouble(args[2]);
                        System.out.println("start learning.");
                        learn(repeat, rate);
                    }
                    catch(NumberFormatException e)
                    {
                        error = true;
                    }
                }
                else
                {
                    error = true;
                }
            }
            else if (args[0].equals("reset"))
            {
                if (args.length == 3)
                {
                    try
                    {
                        int layers = Integer.parseInt(args[1]);
                        int nodes = Integer.parseInt(args[2]);
                        System.out.println("reset:"+layers + "x" + nodes);
                        reset(layers, nodes);
                    }
                    catch(NumberFormatException e)
                    {
                        error = true;
                    }
                }
                else
                {
                    error = true;
                }
            }
            else if (args[0].equals("loadrecord"))
            {
                if (args.length == 2)
                {
                    try
                    {
                        System.out.println("loading record.");
                        loadRecord(new File(args[1]).toURL());
                        System.out.println("succeed.");
                    }
                    catch(IOException e)
                    {
                        e.printStackTrace();
                        System.out.println("failed.");
                        error = true;
                    }
                }
                else
                {
                    error = true;
                }
            }
            else if (args[0].equals("load"))
            {
                if (args.length == 2)
                {
                    try
                    {
                        System.out.println("loading NeuralNetwork.");
                        load(new File(args[1]));
                        System.out.println("succeed.");
                    }
                    catch(IOException e)
                    {
                        e.printStackTrace();
                        System.out.println("failed.");
                        error = true;
                    }
                }
                else
                {
                    error = true;
                }
            }
            else if (args[0].equals("save"))
            {
                if (args.length == 2)
                {
                    try
                    {
                        System.out.println("saveing NeuralNetwork.");
                        save(new File(args[1]));
                        System.out.println("succeed.");
                    }
                    catch(IOException e)
                    {
                        e.printStackTrace();
                        System.out.println("failed.");
                        error = true;
                    }
                }
                else
                {
                    error = true;
                }
            }
            else if (args[0].equals("show"))
            {
                int[] layout = neuralNetwork.getLayerLayout();
                System.out.println("Layer Num:" + layout.length );
                System.out.print("Layer Layout:");
                for (int i = 0;i<layout.length;i++)
                    System.out.print(" " + layout[i]);
                System.out.println();
            }
            else
            {
                error = true;
            }
            if (error)
            {
                System.out.println(
                        "Commands: load <file>\n"
                        +"         save <file>\n"
                        +"         loadrecord <file>\n"
                        +"         reset <layers> <nodes>\n"
                        +"         learn <learing rate>\n"
                        +"         stop"
                        +"         show"
                );
            }
        }
    }
    public void load(File target) throws IOException
    {
        FileInputStream in = new FileInputStream(target);
        neuralNetwork = NeuralNetwork.loadFromStream(in);
        in.close();
        int[] layout = neuralNetwork.getLayerLayout();
    }
    public void save(File target) throws IOException
    {
        FileOutputStream out = new FileOutputStream(target);
        NeuralNetwork.dumpToStream(out, neuralNetwork);
        out.close();
    }

    class LearnThread extends Thread
    {
        int learnRepeat = 0;
        boolean goFlag = false;
        
        public LearnThread(int learnRepeat)
        {
            this.learnRepeat = learnRepeat;
        }
        public void stopLearning()
        {
            goFlag = false;
        }
        
        public void run()
        {
            super.run();
            FileWriter logWriter = null;
//            try
//            {
//                int[] layout = neuralNetwork.getLayerLayout();
//                File logfile = new File("learning" + layout[1] + "x" + (layout.length-2) + "_" + rateField.getText() +".log");
//                int num = 0;
//                while (logfile.exists())
//                {
//                    logfile = new File("learning" + layout[1] + "x" + (layout.length-2) + "_" + rateField.getText() + "_" + num +".log");
//                    num++;
//                }
//                logfile.createNewFile();
//                
//                logWriter = new FileWriter(logfile);
//                logWriter.write("layout, ");
//                for (int i=0;i<layout.length;i++)
//                {
//                    if (i == layout.length-1)
//                        logWriter.write(layout[i] + "\n");
//                    else
//                        logWriter.write(layout[i]+ ", ");
//                }
//                logWriter.write("rate, " + rateField.getText() + "\n");
//            }catch(IOException e)
//            {
//                e.printStackTrace();
//            }
            
            int num = orgInputData.length;
            long currenttime = System.currentTimeMillis();
            long starttime = currenttime;
                
            goFlag = true;
            int t;
            outer: for (t = 0;t<learnRepeat;t++)
            {
                //System.out.println("repeat " + t);
                double error = 0d;
                for (int i = 0;i<num;i++)
                {
                    if (!goFlag)
                        break outer;
                    
                    //int p = (int)(orgInputData.length * Math.random());
                    int p = i;
                    neuralNetwork.setInput(orgInputData[p]);

                    neuralNetwork.feedForward();

                    neuralNetwork.setDesiredOutput(orgDesiredData[p]);
                    neuralNetwork.backPropagate();
                    error += neuralNetwork.calculateError();
                }
                
                long timer = System.currentTimeMillis();
                double time = (double)(timer - currenttime)/1000d;
                currenttime = timer;
                error /= num;
                int k = t + 1;
                System.out.println("count:" + t + "/" + learnRepeat + ", " + time + "sec, error:" + error);
                
                try
                {
                    if (logWriter != null)
                        logWriter.write("" + t + ", " + Double.toString(error) + "\n");
                }
                catch(IOException e)
                {
                    e.printStackTrace();
                }
                
            }
            currenttime = System.currentTimeMillis();
            double time = (double)(currenttime - starttime) / 1000d;
            System.out.println("repeat " + t + ", time " + time + "sec");
            System.out.println( (time/t) + "sec/repeat");
            
            try
            {
                if (logWriter != null)
                    logWriter.close();
            }
            catch(IOException e)
            {
                e.printStackTrace();
            }
            
            learnThread = null;
        }
    }
}

