/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/01/18
 */
package bakutan.client;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.Image;
import java.awt.Point;
import java.awt.Transparency;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.imageio.ImageIO;
import javax.swing.JComponent;

import bakutan.common.BUserInterface;
import bakutan.common.Body;
import bakutan.common.ClientBody;
import bakutan.common.DataManager;
import bakutan.common.GameOption;
import bakutan.common.GameThread;
import bakutan.common.LoginInfo;
import bakutan.common.NetworkThread;
import bakutan.common.NullUserInterface;
import bakutan.common.ai.AIInterface;
import bakutan.common.event.BodyEvent;
import bakutan.common.event.BodySoundEvent;
import bakutan.common.graphics.CharacterImage;
import bakutan.common.graphics.CharacterImageBundle;
import bakutan.common.graphics.EffectBundle;
import bakutan.common.math.BVector2f;
import bakutan.common.util.BAudio;
import bakutan.server.NullUserDataHolder;
import bakutan.server.ServerConfigurationInfo;
import bakutan.server.ServerProcess;
import bakutan.server.util.NetworkTool;

/**
 * @author cattaka
 */

public class BStagePanel extends JComponent implements KeyListener,MouseListener,MouseMotionListener
{
	public static final long SHOW_MARKER_TIME = 2000;
	public static final int RECORD_INTERVAL = 100;
    public static final int DRAW_RATE = 1<<1;
    public static final int DRAW_GHOST = 1<<2;
    public static final int DRAW_SCORE = 1<<3;
    public static final int DRAW_STATUS = 1<<4;
    public static final int DRAW_RADAR = 1<<5;
    public static final float DEFAULT_ACTION_TARGET_DISTANCE = 10.0f;

	private BUserInterface userinterface;
	private GraphicsConfiguration graphicsConfiguration;
    
	//draw option
	private int drawOption = DRAW_STATUS | DRAW_RADAR;
    
	//markers
	private BufferedImage[] img_marker;
    private BufferedImage[] img_pointer;
//    private CharacterImage img_ghost;

	//for interface
	private boolean keyAction = false;
    private boolean[] keyActions = new boolean[32];
	private boolean key_up = false;
	private boolean key_down = false;
	private boolean key_left = false;
	private boolean key_right = false;
    private boolean[] mouseButtonPress = new boolean[]{false,false,false};
    private boolean mouseButtonMove = false;
    private boolean mouseButtonAction = false;
    private boolean[] mouseActions = new boolean[32];
	private Point mousePoint = new Point();
    private int actionSelectWeapon = -1;
    private int actionWeaponChange = 0;
    private BVector2f uiVelocity = new BVector2f();
    //private boolean directionLock = false;
    //private boolean[] mouseButtonState = new boolean[]{false,false};

    private KeyConfig keyConfig;
    private boolean screenShot = false;;
    
    private NetworkThread networkThread;
	
	private Font nameFont;
	private Font scoreFont;

	// GameThreadOption
    private GameOption gameOption = new GameOption();
//	private int fpsLimit = 60;
//    private int interval = 16;
//    private long intervalDerey = 0;
//    private boolean recordReplay = false;
    private Boolean drawable = true;
    
    //for AI
//    private AIInterface aiInterface;
//    private boolean workAI = false;
//    private boolean recordAI = false;

	public BStagePanel(GraphicsConfiguration graphicsConfiguration, BUserInterface userinterface)
	{
        this.graphicsConfiguration = graphicsConfiguration;
		this.userinterface = userinterface;
		setPreferredSize(new Dimension((int)(DataManager.BLOCKSIZE*DataManager.DEFAULT_WIDTH), (int)(DataManager.BLOCKSIZE*DataManager.DEFAULT_HEIGHT)));
		createImages();
//		baudio = new BSound(MAXSOUNDS);
//		bBGM = new BBGM();
        addMouseListener(this);
        addMouseMotionListener(this);
        keyConfig = new KeyConfig();
        //fpsTimer = new ClockTimer(weight);
	}
	private void createImages()
	{
		nameFont = new Font("Serif",Font.PLAIN,12);
		scoreFont = new Font("Serif",Font.BOLD,20);
		try
		{
			BufferedImage img_mk = ImageIO.read(getClass().getClassLoader().getResourceAsStream("image/marker.png"));
			img_marker = new BufferedImage[]{img_mk.getSubimage(0,0,64,64),img_mk.getSubimage(64,0,64,64)};
            BufferedImage img_pt = ImageIO.read(getClass().getClassLoader().getResourceAsStream("image/pointer.png"));
            img_pointer = new BufferedImage[]{img_pt.getSubimage(0,0,32,32),img_pt.getSubimage(32,0,32,32)};
//			img_ghost = new CharacterImageFD(graphicsConfiguration, Transparency.BITMASK, getClass().getClassLoader().getResourceAsStream("image/ghost.png"));
		}catch(IOException e)
		{
			throw new RuntimeException(e);
		}
		
	}

    private void processAction(int action, boolean flag, boolean mouse)
    {
        //System.out.println(""+action + flag + mouse);
        if (flag)
        {
            if (action == KeyConfig.ACTION_SELECT_ACTION_1)
            {
                actionSelectWeapon = 0;
            }
            else if (action == KeyConfig.ACTION_SELECT_ACTION_2)
            {
                actionSelectWeapon = 1;
            }
            else if (action == KeyConfig.ACTION_SELECT_ACTION_3)
            {
                actionSelectWeapon = 2;
            }
            else if (action == KeyConfig.ACTION_SELECT_ACTION_4)
            {
                actionSelectWeapon = 3;
            }
            else if (action == KeyConfig.ACTION_SELECT_ACTION_5)
            {
                actionSelectWeapon = 4;
            }
            else if (action == KeyConfig.ACTION_SELECT_ACTION_6)
            {
                actionSelectWeapon = 5;
            }
            else if (action == KeyConfig.ACTION_SELECT_ACTION_7)
            {
                actionSelectWeapon = 6;
            }
            else if (action == KeyConfig.ACTION_SELECT_ACTION_8)
            {
                actionSelectWeapon = 7;
            }
            else if (action == KeyConfig.ACTION_SELECT_ACTION_9)
            {
                actionSelectWeapon = 8;
            }
            else if (action == KeyConfig.ACTION_WEAPON_NEXT)
            {
                actionWeaponChange++;
            }
            else if (action == KeyConfig.ACTION_WEAPON_PREV)
            {
                actionWeaponChange--;
            }
            else if (action == KeyConfig.ACTION_UP)
            {
                key_up = true;
                updateUIVelocity();
            }
            else if (action == KeyConfig.ACTION_LEFT)
            {
                key_left = true;
                updateUIVelocity();
            }
            else if (action == KeyConfig.ACTION_RIGHT)
            {
                key_right = true;
                updateUIVelocity();
            }
            else if (action == KeyConfig.ACTION_DOWN)
            {
                key_down = true;
                updateUIVelocity();
            }
            else if (action == KeyConfig.ACTION_MOUSE_MOVE)
            {
                mouseButtonMove = true;
            }
            else
            {
                if (mouse)
                {
                    if (action == KeyConfig.ACTION_ATTACK)
                    {
                        mouseButtonAction = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_1)
                    {
                        mouseActions[0] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_2)
                    {
                        mouseActions[1] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_3)
                    {
                        mouseActions[2] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_4)
                    {
                        mouseActions[3] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_5)
                    {
                        mouseActions[4] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_6)
                    {
                        mouseActions[5] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_7)
                    {
                        mouseActions[6] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_8)
                    {
                        mouseActions[7] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_9)
                    {
                        mouseActions[8] = true;
                    }
                }
                else
                {
                    if (action == KeyConfig.ACTION_ATTACK)
                    {
                        keyAction = true;
                    }
                    if (action == KeyConfig.ACTION_ACTION_1)
                    {
                        keyActions[0] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_2)
                    {
                        keyActions[1] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_3)
                    {
                        keyActions[2] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_4)
                    {
                        keyActions[3] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_5)
                    {
                        keyActions[4] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_6)
                    {
                        keyActions[5] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_7)
                    {
                        keyActions[6] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_8)
                    {
                        keyActions[7] = true;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_9)
                    {
                        keyActions[8] = true;
                    }
                }
            }
        }
        else
        {
            if (action == KeyConfig.ACTION_UP)
            {
                key_up = false;
                updateUIVelocity();
            }
            else if (action == KeyConfig.ACTION_LEFT)
            {
                key_left = false;
                updateUIVelocity();
            }
            else if (action == KeyConfig.ACTION_RIGHT)
            {
                key_right = false;
                updateUIVelocity();
            }
            else if (action == KeyConfig.ACTION_DOWN)
            {
                key_down = false;
                updateUIVelocity();
            }
            else if (action == KeyConfig.ACTION_MOUSE_MOVE)
            {
                mouseButtonMove = false;
            }
            else
            {
                if (mouse)
                {
                    if (action == KeyConfig.ACTION_ATTACK)
                    {
                        mouseButtonAction = false;
                    }
                    if (action == KeyConfig.ACTION_ACTION_1)
                    {
                        mouseActions[0] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_2)
                    {
                        mouseActions[1] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_3)
                    {
                        mouseActions[2] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_4)
                    {
                        mouseActions[3] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_5)
                    {
                        mouseActions[4] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_6)
                    {
                        mouseActions[5] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_7)
                    {
                        mouseActions[6] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_8)
                    {
                        mouseActions[7] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_9)
                    {
                        mouseActions[8] = false;
                    }
                }
                else
                {
                    if (action == KeyConfig.ACTION_ATTACK)
                    {
                        keyAction = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_1)
                    {
                        keyActions[0] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_2)
                    {
                        keyActions[1] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_3)
                    {
                        keyActions[2] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_4)
                    {
                        keyActions[3] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_5)
                    {
                        keyActions[4] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_6)
                    {
                        keyActions[5] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_7)
                    {
                        keyActions[6] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_8)
                    {
                        keyActions[7] = false;
                    }
                    else if (action == KeyConfig.ACTION_ACTION_9)
                    {
                        keyActions[8] = false;
                    }
                }
            }
        }
    }
	public void keyPressed(KeyEvent e)
	{
        int keyCode = e.getKeyCode();
		if (e.getKeyCode() == KeyEvent.VK_F1)
		{
		    //networkThread.setShowRate(!networkThread.isShowRate());
            setDrawOption(DRAW_RATE, !isDrawOption(DRAW_RATE));
			return;
		}
		else if (e.getKeyCode() == KeyEvent.VK_F2)
		{
            setDrawOption(DRAW_GHOST, !isDrawOption(DRAW_GHOST));
			return;
		}
		else if (e.getKeyCode() == KeyEvent.VK_F3)
		{
            setDrawOption(DRAW_SCORE, !isDrawOption(DRAW_SCORE));
			return;
		}
		else if (e.getKeyCode() == KeyEvent.VK_F4)
		{
            setDrawOption(DRAW_STATUS, !isDrawOption(DRAW_STATUS));
			return;
		}
		else if (e.getKeyCode() == KeyEvent.VK_F5)
		{
            setDrawOption(DRAW_RADAR, !isDrawOption(DRAW_RADAR));
			return;
		}
        else if (e.getKeyCode() == KeyEvent.VK_F6)
        {
            setWorkAI(!isWorkAI());
            return;
        }
        else if (e.getKeyCode() == KeyEvent.VK_F7)
        {
            setRecordAI(!isRecordAI());
            return;
        }
        else if (e.getKeyCode() == KeyEvent.VK_F8)
        {
            setRecordReplay(!isRecordReplay());
            return;
        }
		else if (KeyEvent.VK_F9 <= e.getKeyCode() && e.getKeyCode() <= KeyEvent.VK_F12)
		{
			int i = e.getKeyCode() - KeyEvent.VK_F9 + 1;
			if (networkThread.checkDrawOption(i))
				networkThread.setDrawOptionEx(false,i);
			else
				networkThread.setDrawOptionEx(true,i);
			return;
		}
        else if (e.getKeyCode() == KeyEvent.VK_PRINTSCREEN)
        {
            screenShot = true;
            return;
        }
        else
        {
            int action = keyConfig.getAction(keyCode);
            synchronized(uiVelocity)
            {
                processAction(action, true, false);
            }
        }
	}
	public void keyReleased(KeyEvent e)
	{
        int keyCode = e.getKeyCode();
        int action = keyConfig.getAction(keyCode);
        synchronized(uiVelocity)
        {
            processAction(action, false, false);
        }
	}
	public void keyTyped(KeyEvent e)
	{
	}
    
	public void mouseClicked(MouseEvent e)
    {
    }
    public void mouseEntered(MouseEvent e)
    {
    }
    public void mouseExited(MouseEvent e)
    {
    }
    public void mousePressed(MouseEvent e)
    {
        synchronized(uiVelocity)
        {
            if (e.getButton() == MouseEvent.BUTTON1)
            {
                int action = keyConfig.getMouseAction(0);
                processAction(action, true, true);
                mouseButtonPress[0] = true;
            }
            else if (e.getButton() == MouseEvent.BUTTON2)
            {
                int action = keyConfig.getMouseAction(1);
                processAction(action, true, true);
                mouseButtonPress[1] = true;
            }
            else if (e.getButton() == MouseEvent.BUTTON3)
            {
                int action = keyConfig.getMouseAction(2);
                processAction(action, true, true);
                mouseButtonPress[2] = true;
            }
            mousePoint.setLocation(e.getPoint());
        }
    }
    public void mouseReleased(MouseEvent e)
    {
        synchronized(uiVelocity)
        {
            if (e.getButton() == MouseEvent.BUTTON1)
            {
                int action = keyConfig.getMouseAction(0);
                processAction(action, false, true);
                mouseButtonPress[0] = false;
            }
            else if (e.getButton() == MouseEvent.BUTTON2)
            {
                int action = keyConfig.getMouseAction(1);
                processAction(action, false, true);
                mouseButtonPress[1] = false;
            }
            else if (e.getButton() == MouseEvent.BUTTON3)
            {
                int action = keyConfig.getMouseAction(2);
                processAction(action, false, true);
                mouseButtonPress[2] = false;
            }
        }        
    }
    
    public void mouseDragged(MouseEvent e)
    {
        synchronized(uiVelocity)
        {
            mousePoint.setLocation(e.getPoint());
        }
    }
    public void mouseMoved(MouseEvent e)
    {
    }
    private void updateUIVelocity()
	{
        synchronized (uiVelocity)
        {
            uiVelocity.set(0,0);
            if (key_up)
                uiVelocity.y -= 1f;
            if (key_down)
                uiVelocity.y += 1f;
            if (key_right)
                uiVelocity.x += 1f;
            if (key_left)
                uiVelocity.x -= 1f;
            if (!uiVelocity.isZeroVector())
            {
                uiVelocity.normalize();
            }
        }
	}
	public void setMessage(String message)
	{
		networkThread.setMessage(message);
	}
	public void setEnableSE(boolean sound)
	{
	    userinterface.getBAudio().setSEEnable(sound);
	}
	public void setEnableBGM(boolean flag)
	{
        if (userinterface.getBAudio().isBGMEnable() && flag)
        {
            return;
        }
        userinterface.getBAudio().setBGMEnable(flag);
        if (flag && networkThread != null)
        {
            userinterface.getBAudio().playBGM();
        }
	}
    public void setVoumeSE(int volume)
    {
        userinterface.getBAudio().setSEVolume(volume);
    }
    public void setVoumeBGM(int volume)
    {
        userinterface.getBAudio().setBGMVolume(volume);
    }
	
    class DrawableNetworkThread extends NetworkThread
    {
        public DrawableNetworkThread(BUserInterface userinterface, GameOption gameOption)
        {
            super(userinterface,gameOption);
        }
        @Override
        protected GameThread createGameThread()
        {
            return new DrawableGameThread(dataManager, userinterface, gameOption);
        }
    }
    class DrawableServerNetworkThread extends NetworkThread
    {
        private ServerProcess serverProcess;
        private BakutanAIClient[] aiClients = new BakutanAIClient[0];
        
        public DrawableServerNetworkThread(BUserInterface userinterface, GameOption gameOption)
        {
            super(userinterface,gameOption);
        }
        public synchronized void startSession(LoginInfo loginInfo, ServerConfigurationInfo sci, NetworkTool networkTool)
        {
            this.serverProcess = new ServerProcess(new NullUserInterface(), networkTool);
            try
            {
                this.serverProcess.init(sci.getBStageInfo(), new NullUserDataHolder());
            }
            catch(IOException e)
            {
                throw new RuntimeException(e);
            }
            this.serverProcess.start();
            int p = serverProcess.getPort();
            super.startSession("localhost", p, loginInfo);
        }
        @Override
        public synchronized void startSession(String host, int port, LoginInfo loginInfo)
        {
            throw new RuntimeException("This method is not supported.");
        }
        @Override
        public synchronized void startSession(URL replay)
        {
            throw new RuntimeException("This method is not supported.");
        }
        @Override
        public void stopNetworkThread()
        {
            super.stopNetworkThread();
            for (int i=0;i<aiClients.length;i++)
            {
                aiClients[i].stopSession();
            }
            this.serverProcess.stopServer();
        }
        @Override
        protected GameThread createGameThread()
        {
            return new DrawableGameThread(dataManager, userinterface, gameOption);
        }
    }
    /*
	class NetworkThread extends Thread
	{
		private String host;
		private int port;
		private LoginInfo loginInfo;
        private DataManager dataManager;
		private Socket socket;
		private MesureInputStream min;
		private MesureOutputStream mout;
		private DataInputStream in;
		private DataOutputStream out;
        private DataFlusher dataFlusher;
		private URL replayURL;
        private boolean goflag = false;
        private DrawableGameThread drawThread;
        
		NetworkThread()
		{
			super("Network Thread");
		}
		public void run()
		{
			boolean wait = false;
			outer: do
			{
                //connect to server
                if (wait)
                {
                    //foolish busy wait
                    for (int i=0;i<50;i++)
                    {
                        try
                        {
                            Thread.sleep(100);
                        }catch(InterruptedException e)
                        {
                            e.printStackTrace();
                        }
                        if (!goflag)
                        {
                            break outer;
                        }
                    }
                }
                closeAll();
                wait = true;
                if (replayURL == null)
                {
                    
                    int myID;
                    try
                    {
                        userinterface.outputMessage(userinterface.translate("connecting_to", new String[]{host,String.valueOf(port)}));
                        socket = new Socket();
                        InetSocketAddress isa = new InetSocketAddress(host, port);
                        socket.connect(isa, 3000);
                    }catch(IOException e)
                    {
                        userinterface.outputMessage(userinterface.translate("connection_refused"));
                        e.printStackTrace();
                        continue;
                    }
                    try
                    {
                        min = new MesureInputStream(socket.getInputStream());
                        mout = new MesureOutputStream(socket.getOutputStream());
                        in = new DataInputStream(new CompressedInputStream(min,32768));
                        out = new DataOutputStream(new CompressedOutputStream(mout,32768));
                        userinterface.outputMessage(userinterface.translate("connected"));
                        
                        String protocol = in.readUTF();
                        dataManager = DataManager.getAvailableDataManager(protocol);
                        if (dataManager == null)
                        {
                            userinterface.outputMessage(userinterface.translate("different_protocol_version", new String[]{DataManager.getCurrentProtocolVersion(), protocol}));
                            socket.close();
                            break outer;
                        }
                        out.writeByte(DataManager.REQUEST_LOGIN);
                        if (loginInfo.getMode() == LoginInfo.MODE_LOGIN)
                        {
                            out.writeByte(DataManager.LOGINMODE_USER);
                            out.writeUTF(loginInfo.getUsername());
                            out.writeUTF(loginInfo.getPassword());
                        }
                        else if (loginInfo.getMode() == LoginInfo.MODE_NEWACOUNT)
                        {
                            out.writeByte(DataManager.LOGINMODE_NEWUSER);
                            out.writeUTF(loginInfo.getUsername());
                            out.writeUTF(loginInfo.getPassword());
                            out.writeUTF(loginInfo.getNickname());
                            loginInfo.setMode(LoginInfo.MODE_LOGIN);
                        }
                        else
                        {
                            out.writeByte(DataManager.LOGINMODE_GUEST);
                            out.writeUTF(loginInfo.getNickname());
                        }
                        out.flush();
                        dataFlusher = new DataFlusher("DataFlusher Thread", out, (Thread.NORM_PRIORITY + Thread.MAX_PRIORITY) / 2);
                        dataFlusher.start();
                    }catch(IOException e)
                    {
                        e.printStackTrace();
                        continue;
                    }
                }
                else //loadReplayFile
                {
                    mout = null;
                    out = null;
                    try
                    {
                        InputStream replayStream = replayURL.openStream();
                        min = new MesureInputStream(new TimerInputStream(new GZIPInputStream(replayStream,1024*1024)));
                        in = new DataInputStream(min);
                        String protocol = in.readUTF();
                        dataManager = DataManager.getAvailableDataManager(protocol);
                        if (dataManager == null)
                        {
                            userinterface.outputMessage(userinterface.translate("different_version_replay", new String[]{DataManager.getCurrentProtocolVersion(), protocol}));
                            break outer;
                        }
                    }
                    catch(IOException e)
                    {
                        e.printStackTrace();
                        break outer;
                    }
                    userinterface.outputMessage(userinterface.translate("replayfile_is_loaded"));
                }

                try
                {
                    while(goflag)
                    {
                        BStage bstage;
                        
                        //when replay mode ignore this block
                        if (replayURL == null)
                        {
                            int status = in.readByte();
                            if (status != DataManager.AUTH_SUCCEED)
                            {
                                if (status == DataManager.AUTH_FAILED_INVALID)
                                    userinterface.outputMessage(userinterface.translate("authorization_failed_invalid"));
                                else if (status == DataManager.AUTH_FAILED_EXIST)
                                    userinterface.outputMessage(userinterface.translate("authorization_failed_exist"));
                                else if (status == DataManager.AUTH_TOO_MANY_CLIENTS)
                                    userinterface.outputMessage(userinterface.translate("too_many_clients"));
                                else
                                    userinterface.outputMessage(userinterface.translate("authorization_failed"));
                                break outer;
                            }
                        }
                        
                        bstage = dataManager.inputBStage(in);
                        if (drawThread != null)
                        {
                            drawThread.stopDrawThread();
                            drawThread = null;
                            System.gc();
                        }
    					drawThread = new DrawableGameThread(dataManager);
    					Body[] bodys = dataManager.inputBodys(in, bstage);
    					for (int i=0;i<bodys.length;i++)
    					{
    						BodyAddEvent bae = new BodyAddEvent(bodys[i]);
    						ClientStageEvent cse = new ClientStageEvent(new int[0], bae);
    						drawThread.putStageEvent(cse);
    					}
                        if (dataFlusher != null)
                            drawThread.startDrawThread(bstage, dataFlusher);
                        else
                            drawThread.startDrawThread(bstage);
    					int l;
    					//while(socket.isConnected())
                        while((l = in.readByte()) != -1 && goflag)
    					{
                            //int l = in.readByte();
    						int[] bodysID = new int[l];
    						for (int i=0;i<l;i++)
    						{
    							bodysID[i] = in.readShort();
    						}
    						BodyEvent be = dataManager.inputBodyEvent(in,bstage);
    						ClientStageEvent cse = new ClientStageEvent(bodysID, be);
    						cse.setGenerationTime(BakutanClock.getCurrenttime());
    						drawThread.putStageEvent(cse);
    						if (be instanceof GameSetEvent)
    						{
    							GameSetEvent gse = (GameSetEvent)be;
    							if (gse.getTimeLimit() == 0)
    							{
                                    break;
    							}
    						}
    					}
                    }
				}catch(IOException e)
				{
				    e.printStackTrace();
					ClientStageEvent disconnectEvent = new ClientStageEvent(new int[0], new DisconnectEvent());
                    if (drawThread != null)
                        drawThread.putStageEvent(disconnectEvent);
                    if (replayURL == null)
                        userinterface.outputMessage(userinterface.translate("connection_disconnected"));
                    else
                        userinterface.outputMessage(userinterface.translate("end_of_replay_file"));
				}
			}
			while(gameThreadOption.isForceReconnect());

            if (drawThread != null)
            {
                drawThread.stopDrawThread();
            }
		}
		public synchronized void startSession(String host, int port, LoginInfo loginInfo)
		{
			if (this.isAlive())
			{
				stopNetworkThread();
			}
			this.host = host;
			this.port = port;
			this.loginInfo = loginInfo;
			this.goflag = true;
			this.start();
		}
        public synchronized void startSession(URL replay)
        {
            if (this.isAlive())
            {
                stopNetworkThread();
            }
            this.replayURL = replay;
            this.goflag = true;
            this.start();
        }
        private void closeAll()
        {
            if (in != null)
            {
                try
                {
                    in.close();
                }catch(IOException e)
                {
                }
            }
            if (dataFlusher != null)
            {
                dataFlusher.dispose();
            }
            if (socket != null && socket.isConnected())
            {
                try
                {
                    socket.close();
                }catch(IOException e)
                {
                }
            }
        }
		public void stopNetworkThread()
		{
            this.goflag = false;
            closeAll();
            try
            {
                join();
            }catch(InterruptedException e)
            {
            }
		}
		public int getSend()
		{
			if (mout == null)
				return 0;
			
			return mout.getSend();
		}
		public int getReceive()
		{
			if (min == null)
				return 0;

			return min.getReceive();
		}
		public void resetSend()
		{
            if (mout != null)
                mout.resetSend();
		}
		public void resetReceive()
		{
            if (min != null)
                min.resetReceive();
		}
        public void setDrawOptionEx(boolean value, int type)
        {
            drawThread.setDrawOptionEx(value, type);
        }
        public void setMessage(String message)
        {
            drawThread.setMessage(message);
        }
        public boolean checkDrawOption(int type)
        {
            return drawThread.checkDrawOption(type);
        }
        
	}
	*/
    public void startSingleGameSession(LoginInfo loginInfo, ServerConfigurationInfo  sci)
    {
        if (networkThread != null)
        {
            networkThread.stopNetworkThread();
        }
        networkThread = null;
        System.gc();
        NetworkTool networkTool = NetworkTool.getAvailableNetworkTool();
        networkThread = new DrawableServerNetworkThread(userinterface, gameOption);
        ((DrawableServerNetworkThread)networkThread).startSession(loginInfo, sci, networkTool);
    }
	public void startSession(String host, int port, LoginInfo loginInfo)
	{
		if (networkThread != null)
		{
			networkThread.stopNetworkThread();
		}
        networkThread = null;
		System.gc();
		networkThread = new DrawableNetworkThread(userinterface, gameOption);
		networkThread.startSession(host, port, loginInfo);
	}
    public void startSession(URL replay)
    {
        if (networkThread != null)
        {
            networkThread.stopNetworkThread();
        }
        networkThread = null;
        System.gc();
        networkThread = new DrawableNetworkThread(userinterface, gameOption);
        networkThread.startSession(replay);
    }
	public void stopSession(boolean kill)
	{
        if (kill)
        {
            userinterface.getBAudio().stopBGM();
        }
		if (networkThread != null)
		{
			networkThread.stopNetworkThread();
		}
	}
    class DrawableGameThread extends GameThread
    {
        //graphics
        private int sendRate = 0;
        private int receivedRate = 0;
        private int decompReceivedRate = 0;
        CharacterImageBundle characterImageBundle;
        EffectBundle effectBundle;
        Dimension bufferSize = new Dimension((int)(DataManager.BLOCKSIZE * DataManager.DEFAULT_WIDTH), (int)(DataManager.BLOCKSIZE * DataManager.DEFAULT_HEIGHT));
        Dimension bufferOffset = new Dimension();
        BVector2f bufferCenter = new BVector2f();
        Image buf;
        Graphics bg;
        long showMarker = 0;
        long nextRateUpdate = 0;
        boolean moveByMouse = false;
        BVector2f moveTargetPoint = new BVector2f();
        int fps = 0;
        int fpsCount = 0;
        ClientBody lastViewBody = null;
        
        DrawableGameThread(DataManager dataManager, BUserInterface userinterface, GameOption gameOption)
        {
            super(dataManager, userinterface, gameOption);
        }
        protected void proceedUserInput(boolean[] actions, BVector2f actionTargetPoint, BVector2f inputVelocity)
        {
            if (viewMode)
            {
                setWorkAI(false);
                if (mouseButtonMove)
                {
                    if (myBody != null)
                        lastViewBody = myBody;
                    setMyBody(null);
//                    myBody = null;
//                    myBodyGhost = null;
                    float w = getWidth()/2;
                    float h = getHeight()/2;
                    BVector2f speed = new BVector2f();
                    speed.x = ((float)mousePoint.x - w)/w;
                    speed.y = ((float)mousePoint.y - h)/h;
                    float l = speed.getLength();
                    if (l > 0 && l<0.5f)
                    {
                        speed.normalize();
                        speed.scale(0.5f);
                    }
                        
                    speed.scale((float)(currenttime - lastUpdateTime) / 100f);
                    bufferCenter.add(speed);
                }
                else if (mouseButtonAction)
                {
                    mouseButtonAction = false;
                    Body[] bodys = bstage.getBodysHasScore();
                    if (bodys.length > 0)
                    {
                        if (myBody == null)
                        {
//                            myBody = (ClientBody)bodys[0];
//                            myBodyGhost = (ClientBody)bstage.createBodyInstance(myBody.getType());
//                            myBodyGhost.set(myBody);
                            if (lastViewBody != null && lastViewBody.getBstage() != null)
                            {
                                setMyBody(lastViewBody);
                            }
                            else
                            {
                                setMyBody((ClientBody)bodys[0]);
                            }
                        }
                        else
                        {
                            Body old = myBody;
                            for (int i=0;i<bodys.length;i++)
                            {
                                if (bodys[i] == old)
                                {
//                                    myBody = (ClientBody)bodys[(i+1) % bodys.length];
//                                    myBodyGhost = (ClientBody)bstage.createBodyInstance(myBody.getType());
//                                    myBodyGhost.set(myBody);
                                    setMyBody((ClientBody)bodys[(i+1) % bodys.length]);
                                }
                            }
                        }
                    }
                    else
                    {
                        setMyBody(null);
//                        myBody = null;
//                        myBodyGhost = null;
                    }
                }
            }
            else if (myBody != null && !stopFlag)
            {
                BVector2f pos = new BVector2f(myBodyGhost.getPosition());
                actionTargetPoint.set(myBodyGhost.getDirection());
                if (actionTargetPoint.isZeroVector())
                {
                    actionTargetPoint.set(pos);
                }
                else
                {
                    actionTargetPoint.normalize();
                    actionTargetPoint.scale(DEFAULT_ACTION_TARGET_DISTANCE);
                    actionTargetPoint.add(pos);
                }
                
                //input
                for (int i=0;i<actions.length;i++)
                {
                    actions[i] = false;
                }
                synchronized(uiVelocity)
                {
                    inputVelocity.set(uiVelocity);
                    if (!(inputVelocity.isZeroVector()))
                    {
                        moveByMouse = false;
                    }
                    
                    if (actionSelectWeapon != -1)
                    {
                        myBody.setActionMode((byte) actionSelectWeapon);
                        actionSelectWeapon = -1;
                    }
                    if (keyAction)
                    {
                        actions[myBody.getActionMode()] = true;
                    }
                    for (int i=0;i<actions.length;i++)
                    {
                        if (keyActions[i])
                        {
                            actions[i] = true;
                            keyActions[i] = false;
                        }
                    }
                    if (actionWeaponChange != 0)
                    {
                        myBody.changeActionMode(actionWeaponChange);
                        actionWeaponChange = 0;
                    }
                    boolean doMouseAction = mouseButtonAction;
                    if (mouseButtonAction)
                    {
                        actions[myBody.getActionMode()] = true;
                    }
                    for (int i=0;i<mouseActions.length;i++)
                    {
                        if (mouseActions[i])
                        {
                            actions[i] = true;
                            doMouseAction = true;
                        }
                    }
                    if (mouseButtonMove || doMouseAction)
                    {
                        float org = (float) bufferSize.width / (float)bufferSize.height;
                        if (getHeight() > 0 && getWidth() > 0)
                        {
                            float dist = (float)getWidth() / (float)getHeight();
                            float scale;
                            BVector2f cp = new BVector2f();;
                            cp.x = mousePoint.x;
                            cp.y = mousePoint.y;
                            if ( org >= dist)
                            {
                                scale = (float)bufferSize.width / (float)getWidth();
                                int height = (int)((float)getWidth() / org);
                                int off = (getHeight() - height) / 2;
                                cp.y -= off;
                            }else
                            {
                                scale = (float)bufferSize.height / (float)getHeight();
                                int width = (int)((float)getHeight() * org);
                                int off = (getWidth() - width) / 2;
                                cp.x -= off;
                            }
                            cp.scale(scale);
                            cp.x +=  bufferOffset.width;
                            cp.y +=  bufferOffset.height;
                            cp.scale(1/(float)DataManager.BLOCKSIZE);
                            if (mouseButtonMove)
                            {
                                moveTargetPoint.set(cp);
                                moveByMouse = true;
                            }
                            else if (doMouseAction)
                            {
                                actionTargetPoint.set(cp);
                            }
                        }
                    }
                }
                
                if (moveByMouse)
                {
                    inputVelocity.set(moveTargetPoint);
                    inputVelocity.sub(pos);
                    float distance = inputVelocity.getLength();
                    if (distance < myBody.getMaxSpeed() * (float)(currenttime - lastUpdateTime) / 1000f )
                    {
                        inputVelocity.set(0,0);
                        moveByMouse = false;
                    }
                    else if (distance > 0)
                    {
                        inputVelocity.normalize();
                    }
//                    if (lastVelocityUpdateTime <= lastUpdateTime)
//                        lastVelocityUpdateTime = lastUpdateTime + 1;
//                    else if (lastVelocityUpdateTime > currenttime)
//                        lastVelocityUpdateTime = currenttime;
                }
                if (isWorkAI())
                {
                    moveByMouse = false;
                }

//                //about ai
//                if (aiInterface != null)
//                {
//                    if (aiInterface.isRecordAI() ^ isRecordAI())
//                        aiInterface.setRecordAI(isRecordAI());
//                    if (isRecordAI() || isWorkAI())
//                    {
//                        aiInterface.input(new BodyInfoForAI(myBodyGhost), new BStageInfoForAI(bstage), currenttime, actionLock, energy);
//                    }
//                    if (aiInterface.isRecordAI())
//                    {
//                        AIOutput aiout = new AIOutput();
//                        aiout.getVelocity().set(inputVelocity);
//                        aiout.setAction(-1);
//                        for (int i=0;i<actions.length;i++)
//                        {
//                            if (actions[i])
//                            {
//                                aiout.setAction(i);
//                                break;
//                            }
//                        }
//                        aiout.getActiontarget().set(actionTargetPoint);
//                        aiInterface.inputDesired(currenttime, aiout);
//                    }
//                    if (isWorkAI())
//                    {
//                        for (int i=0;i<actions.length;i++)
//                        {
//                            actions[i] = false;
//                        }
//                        moveByMouse = false;
//                        AIOutput out = aiInterface.getAIOutput();
//
//                        inputVelocity.set(out.getVelocity());
//                        lastVelocityUpdateTime = currenttime;
//                        
//                        actionTargetPoint.set(out.getActiontarget());
//                        byte action = out.getAction();
//                        if (0 <= action && action < actions.length)
//                            actions[action] = true;
//                    }
//                }
                
            }
        }
        protected void proceedPrepare()
        {
            characterImageBundle = bstage.createCharacterImageBundle(graphicsConfiguration);
            effectBundle = new EffectBundle();
            effectBundle.setEffectLevel(3);
            String accel;
            SecurityManager securityManager = System.getSecurityManager();
            try
            {
                if (securityManager != null)
                    securityManager.checkPropertyAccess("sun.java2d.opengl");
                accel = System.getProperty("sun.java2d.opengl");
                if (accel != null && (accel.equals("True") || accel.equals("true")))
                {
                    effectBundle.setEffectLevel(5);
                }
            }
            catch(SecurityException e)
            {
            }
            try
            {
                if (securityManager != null)
                    securityManager.checkPropertyAccess("sun.java2d.d3d");
                accel = System.getProperty("sun.java2d.d3d");
                if (accel != null && (accel.equals("True") || accel.equals("true")))
                {
                    effectBundle.setEffectLevel(5);
                }
            }
            catch(SecurityException e)
            {
            }
            BAudio baudio = userinterface.getBAudio();
            if (baudio.getSEBundle() != null)
                baudio.getSEBundle().dispose();
            baudio.setSEBundle(bstage.createSEBundle());
            baudio.stopBGM();
            if (baudio.getBgmBundle() != null)
            {
                baudio.getBgmBundle().dispose();
            }
            baudio.setBgmBundle(bstage.createBGMBundle());
            baudio.playBGM();
            if (graphicsConfiguration != null)
                buf = graphicsConfiguration.createCompatibleVolatileImage(bufferSize.width, bufferSize.height, Transparency.OPAQUE);
            else
                buf = new BufferedImage(bufferSize.width, bufferSize.height, BufferedImage.TYPE_INT_RGB);
            bg = buf.getGraphics();
        }
        protected void proceedDraw()
        {
            synchronized (drawable)
            {
                if (!drawable)
                    return;
                
                if (myBodyGhost != null)
                {
                    bufferCenter.set(myBodyGhost.getPosition());
                }
                
                bstage.setBufferCenter(bufferCenter);
                bufferOffset.setSize(bstage.getBufferOffset());
                bstage.drawBStage(bg, currenttime, bufferSize, characterImageBundle, effectBundle, myBody, myBodyGhost);
                if (moveByMouse)
                {
                    int n = (int)((currenttime / DataManager.ANIMINTERVAL) % img_pointer.length);
                    int x = -bufferOffset.width + (int) (moveTargetPoint.x * DataManager.BLOCKSIZE) - img_pointer[n].getWidth()/2;
                    int y = -bufferOffset.height + (int) (moveTargetPoint.y * DataManager.BLOCKSIZE) - img_pointer[n].getHeight()/2;
                    bg.drawImage(img_pointer[n],x,y,null);
                }
                if (myBody != null)
                {
                    if (myBody != null && myBody.isState(Body.STATE_MARKER))
                    {
                        int n = (int)((currenttime / DataManager.ANIMINTERVAL) % img_marker.length);
                        int x = -bufferOffset.width + (int) (myBodyGhost.getPosition().x * DataManager.BLOCKSIZE) - img_marker[n].getWidth()/2;
                        int y = -bufferOffset.height + (int) (myBodyGhost.getPosition().y * DataManager.BLOCKSIZE) - img_marker[n].getHeight()/2;
                        bg.drawImage(img_marker[n],x,y,null);
                    }
                    if (isDrawOption(DRAW_GHOST))
                    {
                        int r = (int)(myBody.getShape().getRadius() * DataManager.BLOCKSIZE);
                        int x = -bufferOffset.width + (int) (myBody.getPosition().x * DataManager.BLOCKSIZE) - r;
                        int y = -bufferOffset.height + (int) (myBody.getPosition().y * DataManager.BLOCKSIZE) - r;
                        
                        //myBodyGhost.drawCharacter(bg, bufferOffset, currenttime, characterImageBundle, effectBundle);
                        bg.setColor(Color.WHITE);
                        bg.drawOval(x,y,r*2,r*2);
                    }
                }
                if (isDrawOption(DRAW_STATUS))
                {
                    //draw status
                    if (myBody != null)
                        bstage.drawStatus(bg, currenttime, bufferSize, myBody, userinterface);
                    
                    //draw time limit
                    bg.setFont(scoreFont);
                    int px = (int)(DataManager.DEFAULT_WIDTH - 2) * DataManager.BLOCKSIZE;
                    int py = scoreFont.getSize();
                    int sec = (int)(timelimit/1000);
                    if (sec > 60)
                        bg.setColor(Color.GREEN);
                    else if (sec > 10)
                        bg.setColor(Color.YELLOW);
                    else
                        bg.setColor(Color.RED);
    
                    if (sec < 0)
                        sec = 0;
                    int min = sec / 60;
                    sec = sec % 60;
                    if (sec < 10)
                        bg.drawString(min + ":0" + sec, px, py);
                    else
                        bg.drawString(min + ":" + sec, px, py);
                }
                AIInterface aiInterface = gameOption.getAiInterface();
                if (aiInterface != null)
                {
                    if (aiInterface.isRecordAI())
                    {
                        bg.setFont(scoreFont);
                        int px = (int)(DataManager.DEFAULT_WIDTH - 5) * DataManager.BLOCKSIZE;
                        int py = scoreFont.getSize();
                        bg.setColor(Color.WHITE);
    
                        bg.drawString("R", px, py);
                    }
                    if (isWorkAI())
                    {
                        bg.setFont(scoreFont);
                        int px = (int)(DataManager.DEFAULT_WIDTH - 6) * DataManager.BLOCKSIZE;
                        int py = scoreFont.getSize();
                        bg.setColor(Color.WHITE);
    
                        bg.drawString("A", px, py);
                    }
                }
                if (isRecordReplay())
                {
                    bg.setFont(scoreFont);
                    int px = (int)(DataManager.DEFAULT_WIDTH - 5) * DataManager.BLOCKSIZE;
                    int py = scoreFont.getSize();
                    bg.setColor(Color.RED);
    
                    bg.drawString("Rec", px, py);
                }
                
                if (isDrawOption(DRAW_SCORE) || stopFlag || (myBody != null && myBody.isState(Body.STATE_DEAD)))
                {
                    bstage.drawScore(bg, currenttime, bufferSize, myBody);
                }
                
                if (isDrawOption(DRAW_RADAR))
                {
                    bstage.drawRadar(bg, currenttime, bufferSize, myBody, bufferCenter);
                }
                
                fpsCount++;
                if (nextRateUpdate < currenttime)
                {
                    sendRate = networkThread.getSend();
                    networkThread.resetSend();
                    receivedRate = networkThread.getReceive();
                    networkThread.resetReceive();
                    nextRateUpdate = currenttime + 1000;
                    
                    fps = fpsCount;
                    fpsCount = 0;
                }
                if (isDrawOption(DRAW_RATE))
                {
                    bg.setColor(Color.WHITE);
                    bg.setFont(nameFont);
                    bg.drawString("FPS:" + fps, DataManager.BLOCKSIZE, DataManager.BLOCKSIZE - nameFont.getSize());
                    bg.drawString("send:" + sendRate, DataManager.BLOCKSIZE, DataManager.BLOCKSIZE);
                    bg.drawString("receive:" + receivedRate, DataManager.BLOCKSIZE, DataManager.BLOCKSIZE + nameFont.getSize());
                    bg.drawString("rag:" + getRag(), DataManager.BLOCKSIZE, DataManager.BLOCKSIZE + nameFont.getSize()*2);
                    bg.drawString("objects:" + bstage.getBodysNum(), DataManager.BLOCKSIZE, DataManager.BLOCKSIZE + nameFont.getSize()*3);
                }
                
                // draw redout
                if (redout > 0)
                {
                    bg.setColor(new Color(1f,0f,0f,(float)redout/1000f));
                    bg.fillRect(0,0,bufferSize.width,bufferSize.height);
                }
                //save screenshot
                if (screenShot)
                {
                    screenShot = false;
                    if (userinterface.getWorkDir() != null)
                    {
                        File ssDir = new File(userinterface.getWorkDir() + "/screenshot/");
                        if (!ssDir.exists())
                            ssDir.mkdir();
                        if (ssDir.isDirectory())
                        {
                            BufferedImage output;
                            if (buf instanceof BufferedImage)
                            {
                                output = (BufferedImage)buf;
                            }
                            else
                            {
                                output = new BufferedImage(bufferSize.width, bufferSize.height, BufferedImage.TYPE_INT_RGB);
                                Graphics og = output.getGraphics();
                                og.drawImage(buf, 0,0,null);
                                og.dispose();
                            }
                            SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
                            File ssFile = new File(userinterface.getWorkDir()
                                    + "/screenshot/" + "ss" + format.format(new Date()) + ".png");
                            try
                            {
                                ImageIO.write(output, "png", ssFile);
                                userinterface.outputMessage(userinterface.translate("save_screenshot", new String[]{ssFile.toString()}));
                            }
                            catch(IOException e)
                            {
                                userinterface.outputMessage(e.getMessage());
                            }
                        }
                    }
                    
                }
                //flush backbuffer
                Graphics g = getGraphics();
                if (g != null)
                {
                    g.setColor(Color.BLACK);
                    float org = (float) bufferSize.width / (float)bufferSize.height;
                    if (getHeight() > 0 && getWidth() > 0)
                    {
                        float dist = (float)getWidth() / (float)getHeight();
                        if ( org >= dist)
                        {
                            int height = (int)((float)getWidth() / org);
                            int off = (getHeight() - height) / 2;
                            g.fillRect(0,0,getWidth(),off);
                            g.fillRect(0,getHeight()-off,getWidth(),off);
                            g.drawImage(buf,0,off,getWidth(),off + height,0,0,bufferSize.width,bufferSize.height,BStagePanel.this);
                        }else
                        {
                            int width = (int)((float)getHeight() * org);
                            int off = (getWidth() - width) / 2;
                            g.fillRect(0,0,off,getHeight());
                            g.fillRect(getWidth()-off,0,off,getHeight());
                            g.drawImage(buf,off,0,off + width,getHeight(),0,0,bufferSize.width,bufferSize.height,BStagePanel.this);
                        }
                    }
                    g.dispose();
                }
            }
        }
        protected void proceedDispose()
        {
            buf.flush();
            characterImageBundle.dispose();
        }
        protected void proceedActionForClient(BodyEvent event, Body body, long currenttime)
        {
            body.actionForClient(event, myBody, characterImageBundle, effectBundle, userinterface, currenttime, bufferCenter);
        }
        protected void proceedSoundEvent(BodySoundEvent bse, Body target, int rag)
        {
            if (bse.getMode() == BodySoundEvent.MODE_POSITION)
            {
                userinterface.getBAudio().playSE(bse.getSound(), rag, bufferCenter, bse.getPosition());
            }
            else if (bse.getMode() == BodySoundEvent.MODE_TARGET)
            {
                if (target != null)
                {
                    userinterface.getBAudio().playSE(bse.getSound(), rag, bufferCenter, target.getPosition());
                }
                else
                {
                    userinterface.getBAudio().playSE(bse.getSound(), rag);
                }
            }
        }
        protected void proceedShowMarker()
        {
            showMarker = currenttime + SHOW_MARKER_TIME;
        }
        protected void proceedBodyAdd(Body body)
        {
            CharacterImage ci = characterImageBundle.getCharacterImage(body.getType());
            if (ci != null)
            {
                body.setCharacterImage(ci);
            }
        }
    }
	/*
	class GameThread extends Thread
	{
        //common
		private String message;
		private DataManager dataManager;
		private BStage bstage;
		private List<ClientStageEvent> stageEvents = new LinkedList<ClientStageEvent>();
        private DataFlusher dataFlusher;
		private boolean goflag = false;
        private boolean restart = false;
        private boolean viewMode = false;

        //graphics
        CharacterImageBundle characterImageBundle;
        EffectBundle effectBundle;
        Dimension bufferSize = new Dimension((int)(DataManager.BLOCKSIZE * DataManager.DEFAULT_WIDTH), (int)(DataManager.BLOCKSIZE * DataManager.DEFAULT_HEIGHT));
        Dimension bufferOffset = new Dimension();
        BVector2f bufferCenter = new BVector2f();
        Image buf;
        Graphics bg;
        long showMarker = 0;
        long nextRateUpdate = 0;
        boolean moveByMouse = false;
        BVector2f moveTargetPoint = new BVector2f();
        int fps = 0;
        int fpsCount = 0;
        
        //common
        long currenttime;
        long timelimit;
        long lastUpdateTime;
        long lastVelocityUpdateTime;
        boolean[] actions;
        BVector2f actionTargetPoint = new BVector2f();
        int myBodyID = -1;
        Body myBody = null;
        Body myBodyGhost = null;
        BVector2f inputVelocity = new BVector2f();
        int actionLock = 0;
        float energy = 0f;
        int redout = 0;
        boolean stopFlag = false;
        long stopFlagTime = 0l;
        
		GameThread(DataManager dataManager)
		{
			super("Draw Thread");
            this.dataManager = dataManager;
		}
        private synchronized void startDrawThread(BStage bstage)
        {
            if (bstage == null)
                throw new NullPointerException();
            
            if (isAlive())
                throw new RuntimeException();

            this.bstage = bstage;
            viewMode = true;
            dataFlusher = null;

            setPriority(Thread.MIN_PRIORITY + 1);
            start();
        }
		private synchronized void startDrawThread(BStage bstage, DataFlusher flusher)
		{
            if (bstage == null || flusher == null)
                throw new NullPointerException();

            if (isAlive())
                throw new RuntimeException();
            
			this.bstage = bstage;
            if (flusher != null)
            {
                viewMode = false;
                dataFlusher = flusher;
            }
            else
            {
                viewMode = true;
                dataFlusher = null;
            }

			setPriority(Thread.MIN_PRIORITY + 1);
			start();
		}
		public void stopDrawThread()
		{
		    this.restart = false;
            goflag = false;
			try
			{
				join();
			}catch(InterruptedException e)
			{
			}
		}
        protected void proceedUserInput()
        {
            if (viewMode)
            {
                setWorkAI(false);
                if (mouseButtonMove)
                {
                    myBody = null;
                    myBodyGhost = null;
                    float w = getWidth()/2;
                    float h = getHeight()/2;
                    BVector2f speed = new BVector2f();
                    speed.x = ((float)mousePoint.x - w)/w;
                    speed.y = ((float)mousePoint.y - h)/h;
                    float l = speed.getLength();
                    if (l > 0 && l<0.5f)
                    {
                        speed.normalize();
                        speed.scale(0.5f);
                    }
                        
                    speed.scale((float)(currenttime - lastUpdateTime) / 100f);
                    bufferCenter.add(speed);
                }
                else if (mouseButtonPress[1])
                {
                    mouseButtonPress[1] = false;
                    Body[] bodys = bstage.getBodysHasScore();
                    if (bodys.length > 0)
                    {
                        if (myBody == null)
                        {
                            myBody = bodys[0];
                            myBodyGhost = bstage.createBodyInstance(myBody.getType());
                            myBodyGhost.set(myBody);
                        }
                        else
                        {
                            Body old = myBody;
                            myBody = null;
                            myBodyGhost = null;
                            for (int i=0;i<bodys.length;i++)
                            {
                                if (bodys[i] == old)
                                {
                                    myBody = bodys[(i+1) % bodys.length];
                                    myBodyGhost = bstage.createBodyInstance(myBody.getType());
                                    myBodyGhost.set(myBody);
                                }
                            }
                        }
                    }
                    else
                    {
                        myBody = null;
                        myBodyGhost = null;
                    }
                }
            }
            else if (myBody != null && !stopFlag)
            {
                BVector2f pos = new BVector2f(myBodyGhost.getPosition());
                actionTargetPoint.set(myBodyGhost.getDirection());
                if (actionTargetPoint.isZeroVector())
                {
                    actionTargetPoint.set(pos);
                }
                else
                {
                    actionTargetPoint.normalize();
                    actionTargetPoint.scale(DEFAULT_ACTION_TARGET_DISTANCE);
                    actionTargetPoint.add(pos);
                }
                
                //input
                for (int i=0;i<actions.length;i++)
                {
                    actions[i] = false;
                }
                synchronized(uiVelocity)
                {
                    if (lastVelocityUpdateTime <= lastUpdateTime)
                        lastVelocityUpdateTime = lastUpdateTime + 1;
                    else if (lastVelocityUpdateTime > currenttime)
                        lastVelocityUpdateTime = currenttime;
                    inputVelocity.set(uiVelocity);
                    if (!(inputVelocity.isZeroVector()))
                    {
                        moveByMouse = false;
                    }
                    
                    if (actionSelectWeapon != -1)
                    {
                        bstage.setActionMode((byte) actionSelectWeapon);
                        actionSelectWeapon = -1;
                    }
                    if (keyAction)
                    {
                        actions[bstage.getActionMode()] = true;
                    }
                    for (int i=0;i<actions.length;i++)
                    {
                        if (keyActions[i])
                        {
                            actions[i] = true;
                            keyActions[i] = false;
                        }
                    }
                    if (actionWeaponChange != 0)
                    {
                        bstage.changeActionMode(actionWeaponChange);
                        actionWeaponChange = 0;
                    }
                    boolean doMouseAction = mouseButtonAction;
                    if (mouseButtonAction)
                    {
                        actions[bstage.getActionMode()] = true;
                    }
                    for (int i=0;i<mouseActions.length;i++)
                    {
                        if (mouseActions[i])
                        {
                            actions[i] = true;
                            doMouseAction = true;
                        }
                    }
                    if (mouseButtonMove || doMouseAction)
                    {
                        float org = (float) bufferSize.width / (float)bufferSize.height;
                        if (getHeight() > 0 && getWidth() > 0)
                        {
                            float dist = (float)getWidth() / (float)getHeight();
                            float scale;
                            BVector2f cp = new BVector2f();;
                            cp.x = mousePoint.x;
                            cp.y = mousePoint.y;
                            if ( org >= dist)
                            {
                                scale = (float)bufferSize.width / (float)getWidth();
                                int height = (int)((float)getWidth() / org);
                                int off = (getHeight() - height) / 2;
                                cp.y -= off;
                            }else
                            {
                                scale = (float)bufferSize.height / (float)getHeight();
                                int width = (int)((float)getHeight() * org);
                                int off = (getWidth() - width) / 2;
                                cp.x -= off;
                            }
                            cp.scale(scale);
                            cp.x +=  bufferOffset.width;
                            cp.y +=  bufferOffset.height;
                            cp.scale(1/(float)DataManager.BLOCKSIZE);
                            if (mouseButtonMove)
                            {
                                moveTargetPoint.set(cp);
                                moveByMouse = true;
                            }
                            else if (doMouseAction)
                            {
                                actionTargetPoint.set(cp);
                            }
                        }
                    }
                }
                
                if (moveByMouse)
                {
                    inputVelocity.set(moveTargetPoint);
                    inputVelocity.sub(pos);
                    float distance = inputVelocity.getLength();
                    if (distance < myBody.getMaxSpeed() * (float)(currenttime - lastUpdateTime) / 1000f )
                    {
                        inputVelocity.set(0,0);
                        moveByMouse = false;
                    }
                    else if (distance > 0)
                    {
                        inputVelocity.normalize();
                    }
                    if (lastVelocityUpdateTime <= lastUpdateTime)
                        lastVelocityUpdateTime = lastUpdateTime + 1;
                    else if (lastVelocityUpdateTime > currenttime)
                        lastVelocityUpdateTime = currenttime;
                }
                
                //about ai
                if (aiInterface != null)
                {
                    if (aiInterface.isRecordAI() ^ isRecordAI())
                        aiInterface.setRecordAI(isRecordAI());
                    if (isRecordAI() || isWorkAI())
                    {
                        aiInterface.input(new BodyInfoForAI(myBodyGhost), new BStageInfoForAI(bstage), currenttime, actionLock, energy);
                    }
                    if (aiInterface.isRecordAI())
                    {
                        AIOutput aiout = new AIOutput();
                        aiout.getVelocity().set(inputVelocity);
                        aiout.setAction(-1);
                        for (int i=0;i<actions.length;i++)
                        {
                            if (actions[i])
                            {
                                aiout.setAction(i);
                                break;
                            }
                        }
                        aiout.getActiontarget().set(actionTargetPoint);
                        aiInterface.inputDesired(currenttime, aiout);
                    }
                    if (isWorkAI())
                    {
                        for (int i=0;i<actions.length;i++)
                        {
                            actions[i] = false;
                        }
                        moveByMouse = false;
                        AIOutput out = aiInterface.getAIOutput();

                        inputVelocity.set(out.getVelocity());
                        lastVelocityUpdateTime = currenttime;
                        
                        actionTargetPoint.set(out.getActiontarget());
                        byte action = out.getAction();
                        if (0 <= action && action < actions.length)
                            actions[action] = true;
                    }
                }
                
            }
            else if (stopFlag && currenttime - stopFlagTime > 5000)
            {
                goflag = false;
            }
        }
        protected void proceedPrepare()
        {
            characterImageBundle = bstage.createCharacterImageBundle(graphicsConfiguration);
            effectBundle = new EffectBundle();
            effectBundle.setEffectLevel(3);
            String accel;
            SecurityManager securityManager = System.getSecurityManager();
            try
            {
                if (securityManager != null)
                    securityManager.checkPropertyAccess("sun.java2d.opengl");
                accel = System.getProperty("sun.java2d.opengl");
                if (accel != null && (accel.equals("True") || accel.equals("true")))
                {
                    effectBundle.setEffectLevel(5);
                }
            }
            catch(SecurityException e)
            {
            }
            try
            {
                if (securityManager != null)
                    securityManager.checkPropertyAccess("sun.java2d.d3d");
                accel = System.getProperty("sun.java2d.d3d");
                if (accel != null && (accel.equals("True") || accel.equals("true")))
                {
                    effectBundle.setEffectLevel(5);
                }
            }
            catch(SecurityException e)
            {
            }
            baudio.setAudioBundle(bstage.createAudioBundle());
            bBGM.setBgmBundle(bstage.createBGMBundle());
            bBGM.play();
            if (graphicsConfiguration != null)
                buf = graphicsConfiguration.createCompatibleVolatileImage(bufferSize.width, bufferSize.height, Transparency.OPAQUE);
            else
                buf = new BufferedImage(bufferSize.width, bufferSize.height, BufferedImage.TYPE_INT_RGB);
            bg = buf.getGraphics();
        }
        protected void proceedDraw()
        {
            if (myBodyGhost != null)
            {
                bufferCenter.set(myBodyGhost.getPosition());
            }
            
            bstage.setBufferCenter(bufferCenter);
            bufferOffset.setSize(bstage.getBufferOffset());
            bstage.drawBStage(bg, currenttime, bufferSize, characterImageBundle, effectBundle, myBody, myBodyGhost);
            if (moveByMouse)
            {
                int n = (int)((currenttime / DataManager.ANIMINTERVAL) % img_pointer.length);
                int x = -bufferOffset.width + (int) (moveTargetPoint.x * DataManager.BLOCKSIZE) - img_pointer[n].getWidth()/2;
                int y = -bufferOffset.height + (int) (moveTargetPoint.y * DataManager.BLOCKSIZE) - img_pointer[n].getHeight()/2;
                bg.drawImage(img_pointer[n],x,y,null);
            }
            if (myBody != null)
            {
                if (myBody != null && myBody.isState(Body.STATE_MARKER))
                {
                    int n = (int)((currenttime / DataManager.ANIMINTERVAL) % img_marker.length);
                    int x = -bufferOffset.width + (int) (myBodyGhost.getPosition().x * DataManager.BLOCKSIZE) - img_marker[n].getWidth()/2;
                    int y = -bufferOffset.height + (int) (myBodyGhost.getPosition().y * DataManager.BLOCKSIZE) - img_marker[n].getHeight()/2;
                    bg.drawImage(img_marker[n],x,y,null);
                }
                if (isDrawOption(DRAW_GHOST))
                {
                    int x = -bufferOffset.width + (int) (myBody.getPosition().x * DataManager.BLOCKSIZE) - img_ghost.getWidth()/2;
                    int y = -bufferOffset.height + (int) (myBody.getPosition().y * DataManager.BLOCKSIZE) - img_ghost.getHeight()/2;
                    
                    //myBodyGhost.drawCharacter(bg, bufferOffset, currenttime, characterImageBundle, effectBundle);
                    bg.setColor(Color.WHITE);
                    bg.drawOval(x,y,DataManager.BLOCKSIZE, DataManager.BLOCKSIZE);
                }
            }
            if (isDrawOption(DRAW_STATUS))
            {
                //draw status
                if (myBody != null)
                    bstage.drawStatus(bg, currenttime, bufferSize, myBody, userinterface);
                
                //draw time limit
                bg.setFont(scoreFont);
                int px = (int)(DataManager.DEFAULT_WIDTH - 2) * DataManager.BLOCKSIZE;
                int py = scoreFont.getSize();
                int sec = (int)(timelimit/1000);
                if (sec > 60)
                    bg.setColor(Color.GREEN);
                else if (sec > 10)
                    bg.setColor(Color.YELLOW);
                else
                    bg.setColor(Color.RED);

                if (sec < 0)
                    sec = 0;
                int min = sec / 60;
                sec = sec % 60;
                if (sec < 10)
                    bg.drawString(min + ":0" + sec, px, py);
                else
                    bg.drawString(min + ":" + sec, px, py);
            }
            if (aiInterface != null)
            {
                if (aiInterface.isRecordAI())
                {
                    bg.setFont(scoreFont);
                    int px = (int)(DataManager.DEFAULT_WIDTH - 5) * DataManager.BLOCKSIZE;
                    int py = scoreFont.getSize();
                    bg.setColor(Color.WHITE);

                    bg.drawString("R", px, py);
                }
                if (isWorkAI())
                {
                    bg.setFont(scoreFont);
                    int px = (int)(DataManager.DEFAULT_WIDTH - 6) * DataManager.BLOCKSIZE;
                    int py = scoreFont.getSize();
                    bg.setColor(Color.WHITE);

                    bg.drawString("A", px, py);
                }
            }
            if (isRecordReplay())
            {
                bg.setFont(scoreFont);
                int px = (int)(DataManager.DEFAULT_WIDTH - 5) * DataManager.BLOCKSIZE;
                int py = scoreFont.getSize();
                bg.setColor(Color.RED);

                bg.drawString("Rec", px, py);
            }
            
            if (isDrawOption(DRAW_SCORE) || stopFlag || (myBody != null && myBody.isState(Body.STATE_DEAD)))
            {
                bstage.drawScore(bg, currenttime, bufferSize, myBody);
            }
            
            if (isDrawOption(DRAW_RADAR))
            {
                bstage.drawRadar(bg, currenttime, bufferSize, myBody, bufferCenter);
            }
            
            fpsCount++;
            if (nextRateUpdate < currenttime)
            {
                sendRate = networkThread.getSend();
                networkThread.resetSend();
                receivedRate = networkThread.getReceive();
                networkThread.resetReceive();
                nextRateUpdate = currenttime + 1000;
                
                fps = fpsCount;
                fpsCount = 0;
            }
            if (isDrawOption(DRAW_RATE))
            {
                bg.setColor(Color.WHITE);
                bg.setFont(nameFont);
                bg.drawString("FPS:" + fps, DataManager.BLOCKSIZE, DataManager.BLOCKSIZE - nameFont.getSize());
                bg.drawString("send:" + sendRate, DataManager.BLOCKSIZE, DataManager.BLOCKSIZE);
                bg.drawString("receive:" + receivedRate, DataManager.BLOCKSIZE, DataManager.BLOCKSIZE + nameFont.getSize());
                bg.drawString("objects:" + bstage.getBodysNum(), DataManager.BLOCKSIZE, DataManager.BLOCKSIZE + nameFont.getSize()*2);
            }
            
            // draw redout
            if (redout > 0)
            {
                bg.setColor(new Color(1f,0f,0f,(float)redout/1000f));
                bg.fillRect(0,0,bufferSize.width,bufferSize.height);
            }
            //save screenshot
            if (screenShot)
            {
                screenShot = false;
                if (userinterface.getWorkDir() != null)
                {
                    File ssDir = new File(userinterface.getWorkDir() + "/screenshot/");
                    if (!ssDir.exists())
                        ssDir.mkdir();
                    if (ssDir.isDirectory())
                    {
                        BufferedImage output;
                        if (buf instanceof BufferedImage)
                        {
                            output = (BufferedImage)buf;
                        }
                        else
                        {
                            output = new BufferedImage(bufferSize.width, bufferSize.height, BufferedImage.TYPE_INT_RGB);
                            Graphics og = output.getGraphics();
                            og.drawImage(buf, 0,0,null);
                            og.dispose();
                        }
                        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
                        File ssFile = new File(userinterface.getWorkDir()
                                + "/screenshot/" + "ss" + format.format(new Date()) + ".png");
                        try
                        {
                            ImageIO.write(output, "png", ssFile);
                            userinterface.outputMessage(userinterface.translate("save_screenshot", new String[]{ssFile.toString()}));
                        }
                        catch(IOException e)
                        {
                            userinterface.outputMessage(e.getMessage());
                        }
                    }
                }
                
            }
            //flush backbuffer
            Graphics g = getGraphics();
            if (g != null)
            {
                g.setColor(Color.BLACK);
                float org = (float) bufferSize.width / (float)bufferSize.height;
                if (getHeight() > 0 && getWidth() > 0)
                {
                    float dist = (float)getWidth() / (float)getHeight();
                    if ( org >= dist)
                    {
                        int height = (int)((float)getWidth() / org);
                        int off = (getHeight() - height) / 2;
                        g.fillRect(0,0,getWidth(),off);
                        g.fillRect(0,getHeight()-off,getWidth(),off);
                        g.drawImage(buf,0,off,getWidth(),off + height,0,0,bufferSize.width,bufferSize.height,BStagePanel.this);
                    }else
                    {
                        int width = (int)((float)getHeight() * org);
                        int off = (getWidth() - width) / 2;
                        g.fillRect(0,0,off,getHeight());
                        g.fillRect(getWidth()-off,0,off,getHeight());
                        g.drawImage(buf,off,0,off + width,getHeight(),0,0,bufferSize.width,bufferSize.height,BStagePanel.this);
                    }
                }
                g.dispose();
            }
        }
        protected void proceedDispose()
        {
            buf.flush();
            characterImageBundle.dispose();
        }
        protected void proceedActionForClient(BodyEvent event, Body body, long currenttime)
        {
            body.actionForClient(event, baudio, characterImageBundle, effectBundle, userinterface, currenttime);
        }
        protected void proceedSoundEvent(BodySoundEvent bse, int rag)
        {
            baudio.play(bse.getSound(), rag);
        }
        protected void proceedShowMarker()
        {
            showMarker = currenttime + SHOW_MARKER_TIME;
        }
        protected void proceedBodyAdd(Body body)
        {
            CharacterImage ci = characterImageBundle.getCharacterImage(body.getType());
            if (ci != null)
            {
                body.setCharacterImage(ci);
            }
        }
        public void run()
		{
			super.run();

			proceedPrepare();
            
			PacketManager packetManager = new PacketManager(dataManager);
			DataOutputStream recordStream = null;
            long recordTimer = 0;

            goflag = true;

            currenttime = BakutanClock.getCurrenttime();
            timelimit = currenttime;
            lastUpdateTime = currenttime;
            lastVelocityUpdateTime = currenttime;
            long lastSendTime = currenttime;

			HashSet hasString = new HashSet();
			BVector2f lastSendVelocity = new BVector2f();
			BVector2f lastSendDirection = new BVector2f();
			Body lastCollidWith = null;


            actions = new boolean[bstage.getActionNum()];
            
			while(goflag)
			{
                lastUpdateTime = currenttime;
				currenttime = BakutanClock.getCurrenttime();
                //currenttime = System.currentTimeMillis();
                
				timelimit -= currenttime - lastUpdateTime;
				
                // proccess user input
                proceedUserInput();
                
                //move and action
                if (myBody != null && !stopFlag)
                {
                    energy += myBody.getEnergyCharge() * (float)(currenttime - lastUpdateTime) / 1000f;
                    if (energy > myBody.getMaxEnergy())
                    {
                        energy = myBody.getMaxEnergy();
                    }
                    actionLock -= (int)(currenttime - lastUpdateTime);
                    if (actionLock < 0)
                        actionLock = 0;
                    //collision
                    BVector2f pos = new BVector2f(myBodyGhost.getPosition());
                    boolean isCollid = false;
                    if (myBody.isState(Body.STATE_DEAD) || myBody.isState(Body.STATE_WIN))
                    {
                        myBodyGhost.getVelocity().set(0,0);
                    }else
                    {
                        Body collidWith = null;
                        int count = 0;
                        while (currenttime > myBodyGhost.getLastUpdate())
                        {
                            if(count >= 10)
                            {
                                myBodyGhost.setCurrentTime(currenttime);
                                break;
                            }
                            //System.out.println(count);
                            count++;
                            long nextUpdateTime = currenttime;
                            
                            //BPickIntersection bpi = bstage.pickNextIntersection(myBodyGhost,Body.COLLIDABLE);
                            BPickResult bpr = bstage.estimateCollision(myBodyGhost);
                            BPickIntersection bpi = null;
                            if (bpr.getSize() > 0)
                                bpi = bpr.getFirst();
                            isCollid = false;
                            if (bpi != null)
                            {
                                long collisionTime = (long)(bpi.getDistance() * 1000d);
                                collisionTime += myBodyGhost.getLastUpdate();
                                if (collisionTime <= nextUpdateTime)
                                {
                                    isCollid = true;
                                    nextUpdateTime = collisionTime;
                                }
                            }
                            if (myBodyGhost.getLastUpdate() < lastVelocityUpdateTime
                                && lastVelocityUpdateTime <= nextUpdateTime)
                            {
                                nextUpdateTime = lastVelocityUpdateTime;
                                myBodyGhost.update(lastVelocityUpdateTime);
                                if (!inputVelocity.isZeroVector()f)
                                {
                                    if (inputVelocity.getLength() > 1f)
                                        inputVelocity.normalize();
                                    inputVelocity.scale(myBody.getMaxSpeed());
                                }
                                myBodyGhost.setVelocity(inputVelocity);
                                if (!inputVelocity.isZeroVector()) // && !directionLock)
                                    myBodyGhost.setDirection(inputVelocity);
                                continue;
                            }
                            else
                            {
                                if (isCollid)
                                {
                                    collidWith = bpi.getBody();
                                    //BVector2f m = myBodyGhost.getVelocity();
//                                  BVector2f n = new BVector2f();
//                                    n.set(myBodyGhost.getPosition());
//                                    n.sub(bpi.getBody().getPosition());
                                    BVector2f m = new BVector2f();
                                    BVector2f n = new BVector2f();
                                    BMath.calcNormal(m, n, myBodyGhost.getPosition(), myBodyGhost.getShape(), bpi.getBody().getPosition(), bpi.getBody().getShape());
                                    m.set(myBodyGhost.getVelocity());
                                    myBodyGhost.update(nextUpdateTime-10);
                                    float speed = m.getLength();
                                    if (speed > 0 && !n.isZeroVector())
                                    {
                                        float dot = BVector2f.getDistance(m,n);
                                        m.scaleAdd(-dot,n);
                                        if (!m.isZeroVector())
                                        {
                                            m.normalize();
                                            m.scale(speed);
                                        }
                                        float cos = BVector2f.getDot(m,n) / m.getLength();
                                        if (cos == 0)
                                        {
                                            cos = -1;
                                        }
                                        float scale = m.getLength();
                                        float a = (scale * cos);// * 1.00d;
                                        m.x -= a * n.x;
                                        m.y -= a * n.y;
                                        
                                        if (m.getLength() > (scale / 2))
                                        {
                                            m.normalize();
                                            m.scale(scale);
                                        }
                                        else if (m.getLength() < (scale / 1000))
                                        {
                                            m.set(0,0);
                                        }
                                        myBodyGhost.setVelocity(m);
                                    }else
                                    {
                                        myBodyGhost.getVelocity().set(0,0);
                                    }
                                }else
                                {
                                    myBodyGhost.update(nextUpdateTime);
                                }
                            }
                        }
                        if (collidWith == null || !collidWith.doPushReport())
                        {
                            lastCollidWith = null;
                        }
                        else if (collidWith != lastCollidWith)
                        {
                            lastCollidWith = collidWith;
                            BodyPushEvent bpe = new BodyPushEvent(collidWith.getID());
                            dataFlusher.sendPacket(packetManager.makePacket(bpe));
                        }
                    }
                    
                    //send move event to server
                    if ((currenttime > lastSendTime + DataManager.MIM_MOVE_REPORT_INTERVAL &&
                        (!(myBodyGhost.getVelocity().equals(lastSendVelocity))
                        || !(myBodyGhost.getDirection()).equals(lastSendDirection)
                        || isCollid))
                        || currenttime > lastSendTime + DataManager.MAX_MOVE_REPORT_INTERVAL
                        )
                    {
                        lastSendVelocity.set(myBodyGhost.getVelocity());
                        lastSendDirection.set(myBodyGhost.getDirection());
                        BodyMoveEvent bme;
                        //bme = new BodyMoveEvent(myBodyGhost.getPosition(),myBodyGhost.getVelocity(),myBodyGhost.getDirection());
                        bme = new BodyMoveEvent(pos, myBodyGhost.getVelocity(),myBodyGhost.getDirection());
                        dataFlusher.sendPacket(packetManager.makePacket(bme));
                        lastSendTime = currenttime;
                    }
                    //send action event to server
                    if (actionLock == 0)
                    {
                        for (int i=0;i<actions.length;i++)
                        {
                            if (actions[i])
                            {
                                actions[i] = false;
                                int j = bstage.getClientActionHook(myBody, i);
                                actions[j] = true;
                            }
                        }
                        for (byte i=0;i<actions.length;i++)
                        {
                            if (actions[i])
                            {
                                if (!myBody.isActionableForClient(i,energy,actionLock))
                                    continue;
                                actions[i] = false;
                                BodyActionEvent bae = new BodyActionEvent(myBody.getPosition(),actionTargetPoint, (byte)i);
                                dataFlusher.sendPacket(packetManager.makePacket(bae));
                                actionLock += myBody.getActionLockTime(i);
                                energy -= myBody.getEnergyConsumption(i);
                            }
                        }
                    }

                    if (message != null)
                    {
                        BodyCommandEvent bce = new BodyCommandEvent(message);
                        dataFlusher.sendPacket(packetManager.makePacket(bce));
                        message = null;
                    }
                    myBodyGhost.update(currenttime);
                }
                
                //update all bodys
                bstage.updateAll(currenttime);
                
                //start record replay
                if (gameThreadOption.isRecordReplay())
                {
                    
                    if (recordStream == null)
                    {
                        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
                        File recordFile = new File(userinterface.getWorkDir()
                                + "/replay/" + DataManager.getBuildNumber() + "_" + format.format(new Date()) + ".replay");
                        try
                        {
                            if (recordFile.exists() || !recordFile.createNewFile())
                            {
                                setRecordReplay(false);
                            }
                            else
                            {
                                recordStream = new DataOutputStream(new GZIPOutputStream(new FileOutputStream(recordFile),128*1024));
                                Body[] bodys = bstage.getBodys();
                                byte[] protocolPacket = packetManager.makePacket(DataManager.getCurrentProtocolVersion());
                                byte[] bstagePacket = packetManager.makePacket(bstage);
                                byte[] bodysPacket = packetManager.makePacket(bodys);
                                recordStream.writeInt(0); //rag of first packet
                                recordStream.writeInt(protocolPacket.length + bstagePacket.length + bodysPacket.length);
                                recordStream.write(protocolPacket);
                                recordStream.write(bstagePacket);
                                recordStream.write(bodysPacket);
                                
                                ClientStageEvent csegse = new ClientStageEvent(new int[0],new GameSetEvent(timelimit));
                                byte[] timelimitPacket = packetManager.makePacket(csegse);
                                recordStream.writeInt(0);
                                recordStream.writeInt(timelimitPacket.length);
                                recordStream.write(timelimitPacket);

                                if (myBody != null)
                                {
                                    ClientStageEvent csebcie = new ClientStageEvent(new int[0],new BodyClientIDEvent(myBody.getID()));
                                    byte[] clientIdPacket = packetManager.makePacket(csebcie);
                                    recordStream.writeInt(0);
                                    recordStream.writeInt(clientIdPacket.length);
                                    recordStream.write(clientIdPacket);
                                }
                                userinterface.outputMessage(userinterface.translate("recording_to", new String[]{recordFile.toString()}));
                                recordTimer = currenttime;
                            }
                        }catch(IOException e)
                        {
                            e.printStackTrace();
                            setRecordReplay(false);
                            if(recordStream != null)
                            {
                                try
                                {
                                    recordStream.close();
                                }catch(IOException e2)
                                {
                                    e2.printStackTrace();
                                }
                                recordStream = null;
                            }
                        }
                    }
                }
                else
                {
                    if (recordStream != null)
                    {
                        try
                        {
                            recordStream.close();
                        }catch(IOException e2)
                        {
                            e2.printStackTrace();
                        }
                        recordStream = null;
                    }
                }
                
				synchronized(stageEvents)
				{
					int soundRag = 0;
					while(stageEvents.size() > 0)
					{
						ClientStageEvent cse = (ClientStageEvent)stageEvents.remove(0);
                        if (recordStream != null)
                        {
                            try
                            {
                                long t = cse.getGenerationTime();
                                recordStream.writeInt((int)(t - recordTimer));
                                recordTimer = t;
                                byte[] packet = packetManager.makePacket(cse);
                                recordStream.writeInt(packet.length);
                                recordStream.write(packet);
                            }
                            catch(IOException e)
                            {
                                e.printStackTrace();
                                try
                                {
                                    recordStream.close();
                                }catch(IOException e2)
                                {
                                    e2.printStackTrace();
                                }
                                recordStream = null;
                            }
                        }
                        BodyEvent event = cse.getBodyEvent();
						Body[] cbs = new Body[cse.getBodysID().length];
						for (int i=0;i<cbs.length;i++)
						{
							cbs[i] = bstage.getBody(cse.getBodysID()[i]);
						}
						if (cbs.length > 0)
						{
							if (cbs[0] != null)
							{
                                Body body = cbs[0];
                                if (event instanceof BodySmallActionEvent)
                                {
                                    BodySmallActionEvent bsae = (BodySmallActionEvent)event;
                                    event = new BodyActionEvent(body.getPosition(), body.getPosition(), bsae.getAction());
                                }
								//body.actionForClient(event, baudio, characterImageBundle, effectBundle, userinterface, currenttime);
                                proceedActionForClient(event, body, currenttime);
                                if (event instanceof BodyActionEvent)
                                {
                                    BodyActionEvent bae = (BodyActionEvent)event;
                                    byte action = bae.getAction();
                                    body.addActionLockEnergy(-body.getEnergyConsumption(action), body.getActionLockTime(action));
                                }
							}
						}
						if (event instanceof BodyAddEvent)
						{
							BodyAddEvent bae = (BodyAddEvent)event; 
							Body sb = bae.getBody();
                            proceedBodyAdd(sb);
//							CharacterImage ci = characterImageBundle.getCharacterImage(sb.getType());
//							if (ci != null)
//							{
//                                sb.setCharacterImage(ci);
//                            }
							if (sb.getType() == bstage.getRemoteClientBodyType())
							{
								userinterface.outputMessage(userinterface.translate("others_login", new String[]{sb.getName(), String.valueOf(sb.getID())}));
							}
							sb.setLastStateChange(currenttime);
							if (sb.getID() == myBodyID)
							{
								myBody = sb;
                                proceedShowMarker();
                                //showMarker = currenttime + SHOW_MARKER_TIME;
								myBodyGhost = bstage.createBodyInstance(myBody.getType());
								myBodyGhost.set(myBody);
                                energy = myBody.getEnergy();
							}
							sb.setCurrentTime(currenttime);
							bstage.addBody(sb);
							//sb.actionForClient(event, baudio, characterImageBundle, effectBundle, userinterface, currenttime);
                            proceedActionForClient(event, sb, currenttime);
						}
                        else if (event instanceof BodyMultiAddEvent)
                        {
                            BodyMultiAddEvent bmae = (BodyMultiAddEvent)event; 
                            Body[] bodys = bmae.getBodys();
                            for (int i=0;i<bodys.length;i++)
                            {
                                Body sb = bodys[i];
                                proceedBodyAdd(sb);
//                                CharacterImage ci = characterImageBundle.getCharacterImage(sb.getType());
//                                if (ci != null)
//                                {
//                                    sb.setCharacterImage(ci);
//                                }
                                if (sb.getType() == bstage.getRemoteClientBodyType())
                                {
                                    userinterface.outputMessage(userinterface.translate("others_login", new String[]{sb.getName(), String.valueOf(sb.getID())}));
                                }
                                sb.setLastStateChange(currenttime);
                                if (sb.getID() == myBodyID)
                                {
                                    myBody = sb;
                                    proceedShowMarker();
                                    //showMarker = currenttime + SHOW_MARKER_TIME;
                                    myBodyGhost = bstage.createBodyInstance(myBody.getType());
                                    myBodyGhost.set(myBody);
                                    energy = myBody.getEnergy();
                                }
                                //System.out.println(""+ sb + sb.getID());
                                sb.setCurrentTime(currenttime);
                                bstage.addBody(sb);
                                //sb.actionForClient(event, baudio, characterImageBundle, effectBundle, userinterface, currenttime);
                                proceedActionForClient(event, sb, currenttime);
                            }
                        }
                        else if (event instanceof BodyClientIDEvent)
                        {
                            BodyClientIDEvent bcie = (BodyClientIDEvent)event;
                            myBodyID = bcie.getId();
                            Body body = bstage.getBody(myBodyID);
                            myBody = body;
                            proceedShowMarker();
                            //showMarker = currenttime + SHOW_MARKER_TIME;
                            myBodyGhost = bstage.createBodyInstance(myBody.getType());
                            myBodyGhost.set(myBody);
                            energy = myBody.getEnergy();
                        }
						else if (event instanceof BodyDeleteEvent)
						{
							BodyDeleteEvent bde = (BodyDeleteEvent)event; 
							if (cbs[0].getType() == bstage.getRemoteClientBodyType())
							{
								userinterface.outputMessage(userinterface.translate("others_logout", new String[]{cbs[0].getName()}));
							}
							bstage.removeBody(cbs[0]);
								
						}
						else if (event instanceof BodyMoveEvent)
						{
							BodyMoveEvent bme = (BodyMoveEvent)event;
							cbs[0].setPosition(bme.getPosition());
							cbs[0].setVelocity(bme.getVelocity());
							cbs[0].setDirection(bme.getDirection());
							cbs[0].setCurrentTime(cse.getGenerationTime());
						}
						else if (event instanceof BodySetPotisionEvent)
						{
							BodySetPotisionEvent bse = (BodySetPotisionEvent)event;
							cbs[0].setPosition(bse.getPosition());
							cbs[0].getVelocity().set(0,0);
							if (cbs[0] == myBody)
							{
								myBodyGhost.setPosition(bse.getPosition());
								myBodyGhost.getVelocity().set(0,0);
							}
						}
						else if (event instanceof BodyStateEvent)
						{
							BodyStateEvent bse = (BodyStateEvent)event;
							Body tg = cbs[0];
							switch(bse.getType())
							{
							case Body.SET:
								tg.setState(bse.getState());
								break;
							case Body.ENABLE:
								tg.enableState(bse.getState());
								break;
							case Body.DISABLE:
								tg.disableState(bse.getState());
								if (tg == myBody
									&& ((bse.getState() & Body.STATE_DEAD) == Body.STATE_DEAD || (bse.getState() & Body.STATE_WIN) == Body.STATE_WIN))
								{
                                    proceedShowMarker();
                                    //showMarker = currenttime + SHOW_MARKER_TIME;
								}
								break;
							}
						}else if (event instanceof BodyOptionEvent)
						{
							BodyOptionEvent boe = (BodyOptionEvent)event;
							cbs[0].setOption(boe.getType(), boe.getValue());
						}else if (event instanceof BodyArmorEvent)
						{
							BodyArmorEvent bhe = (BodyArmorEvent)event;
							Body to = cbs[0];
							if (bhe.getType() == BodyArmorEvent.SUB)
							{
								to.setArmor(to.getArmor() - bhe.getArmor());
                                if (to == myBody)
                                {
                                    redout += bhe.getArmor()*10;
                                    if (redout > 1000)
                                        redout = 1000;
                                }
							}
							else if (bhe.getType() == BodyArmorEvent.ADD)
							{
								to.setArmor(to.getArmor() + bhe.getArmor());
							}
							else if (bhe.getType() == BodyArmorEvent.SET)
							{
								to.setArmor(bhe.getArmor());
							}
						}
						else if (event instanceof BodyMessageEvent)
						{
							BodyMessageEvent bme = (BodyMessageEvent)event;
							if (bme.getType() == DataManager.MESSAGE_SYSTEM)
							{
								//userinterface.outputMessage("System message:" + bme.getMessage());
								userinterface.outputMessage(userinterface.translate("system_message", new String[]{bme.getMessage()}));
							}
							else if (bme.getType() == DataManager.MESSAGE_LOGIN)
							{
								userinterface.outputMessage(userinterface.translate("login_message", new String[]{bme.getMessage()}));
							}
							else if (bme.getType() == DataManager.MESSAGE_NOTE)
							{
								userinterface.outputMessage(userinterface.translate("note_message", new String[]{bme.getMessage()}));
							}
							else if (bme.getType() == DataManager.MESSAGE_CHAT)
							{
								cbs[0].setMessage(currenttime, bme.getMessage());
								userinterface.outputMessage(userinterface.translate("say", new String[]{cbs[0].getName(), bme.getMessage()}));
								//userinterface.outputMessage("[" + cbs[0].getName() + "] " + bme.getMessage());
							}
						}
						else if (event instanceof BodyStageMessageEvent)
						{
							BodyStageMessageEvent bme = (BodyStageMessageEvent)event;
							bstage.setStageMessage(bme.getType(), bme.getMessage());
						}
						else if (event instanceof BodyScoreEvent)
						{
							BodyScoreEvent bse = (BodyScoreEvent)event;
							for (int i=0;i<bse.getScores().length;i++)
							{
								cbs[i].setScore(bse.getScores()[i]);
							}
						}
						else if (event instanceof BodyActionEvent)
						{
						}
						else if (event instanceof BodySoundEvent)
						{
							BodySoundEvent bse = (BodySoundEvent)event;
                            proceedSoundEvent(bse, 0);
                            //baudio.play(bse.getSound(), 0);
						}
						else if (event instanceof GameSetEvent)
						{
							GameSetEvent gse = (GameSetEvent)event;
							timelimit = gse.getTimeLimit();
							if (timelimit == 0)
							{
                                userinterface.outputMessage(userinterface.translate("game_set"));
								Body[] bodys = bstage.getBodys();
								BVector2f velocity = new BVector2f(0,0);
								for (int i=0;i<bodys.length;i++)
								{
									bodys[i].setVelocity(velocity);
								}
								stopFlag = true;
                                stopFlagTime = currenttime;
                                restart = true;
							}
						}
						else if (event instanceof DisconnectEvent)
						{
							Body[] bodys = bstage.getBodys();
							BVector2f velocity = new BVector2f(0,0);
							for (int i=0;i<bodys.length;i++)
							{
								bodys[i].setVelocity(velocity);
							}
							stopFlag = true;
						}
                        else if (event instanceof BodyActionLockEnergyEvent)
                        {
                            BodyActionLockEnergyEvent balae = (BodyActionLockEnergyEvent)event;
                            actionLock += balae.getActionLock();
                            energy += balae.getEnergy();
                        }
						else
						{
							System.out.println("unknown event : "+cse+" : "+cse.getBodyEvent()
                                    + " : " + event
                                    + " : " + cse.getBodysID());
						}
					}
				}
                if (myBody != null)
                {
                    if (viewMode)
                        myBodyGhost.set(myBody);
                    else
                        myBodyGhost.setWithoutPosition(myBody);
                }
                
                if (gameThreadOption.isDrawable())
                {
                    proceedDraw();
                }

                redout -= currenttime - lastUpdateTime;
                if (redout < 0)
                {
                    redout = 0;
                }
                long intervalDerey = gameThreadOption.getIntervalDerey();
                intervalDerey += gameThreadOption.getInterval() - (currenttime - lastUpdateTime);
                if (intervalDerey > 0)
                {
                    try
                    {
                        Thread.sleep(intervalDerey);
                    }catch(InterruptedException e)
                    {
                        e.printStackTrace();
                    }
                }
                else
                {
                    gameThreadOption.setIntervalDerey(0);
                }
			}
            if (recordStream != null)
            {
                try
                {
                    recordStream.close();
                }
                catch(IOException e)
                {
                }
            }
            proceedDispose();
            if (!viewMode)
            {
                if (restart)
                {
                    GameRestartEvent gre = new GameRestartEvent();
                    dataFlusher.sendPacket(packetManager.makePacket(gre));
                }
            }
		}
        public void putStageEvent(ClientStageEvent se)
        {
            synchronized(stageEvents)
            {
                stageEvents.add(se);
            }
        }
		public void setDrawOptionEx(boolean value, int type)
		{
			if (bstage == null)
				return;
			bstage.setDrawOption(value, type);
		}
		public boolean checkDrawOption(int type)
		{
			if (bstage == null)
				return false;
			return bstage.isDrawOption(type);
		}
		public void setMessage(String message) {
			this.message = message;
		}
	}
	*/
    
    public boolean isForceReconnect()
    {
        return gameOption.isForceReconnect();
    }
    public void setForceReconnect(boolean forceReconnect)
    {
        gameOption.setForceReconnect(forceReconnect);
    }
    public boolean isRecordReplay()
    {
        return gameOption.isRecordReplay();
    }
    public void setRecordReplay(boolean recordReplay)
    {
        if (userinterface.getWorkDir() == null)
        {
            gameOption.setRecordReplay(false);
            return;
        }
        if (recordReplay)
        {
            File replayDir = new File(userinterface.getWorkDir() + "/replay");
            if ((!replayDir.exists() && !replayDir.mkdir()) || !replayDir.isDirectory())
            {
                gameOption.setRecordReplay(false);
                return;
            }
        }
        gameOption.setRecordReplay(recordReplay);
    }
    public boolean isWorkAI()
    {
        return gameOption.isWorkAI();
    }
    public void setWorkAI(boolean workAI)
    {
        gameOption.setWorkAI(workAI);
    }
    public boolean isRecordAI()
    {
        return gameOption.isRecordAI();
    }
    public void setRecordAI(boolean recordAI)
    {
        if (userinterface.getWorkDir() == null)
        {
            gameOption.setRecordAI(false);
            return;
        }
        gameOption.setRecordAI(recordAI);
    }
    public AIInterface getAIInterface()
    {
        return gameOption.getAiInterface();
    }
    public void setAIInterface(AIInterface aiRapper)
    {
        if (gameOption.getAiInterface() != null)
        {
            gameOption.getAiInterface().setRecordAI(false);
        }
        gameOption.setAiInterface(aiRapper);
    }
    
    public boolean getDrawable()
    {
        synchronized (drawable)
        {
            return drawable;
        }
    }
    public void setDrawable(boolean drawable)
    {
        synchronized (this.drawable)
        {
            this.drawable = drawable;
        }
    }
    public void switchDraw()
    {
        if (getDrawable())
        {
            setDrawable(false);
            gameOption.setIntervalDerey(50);
        }
        else
        {
            setDrawable(true);
            gameOption.setIntervalDerey((int)(1000d / (double)gameOption.getFpsLimit()));
        }
        //fpsTimer.setInterval(w);
    }
    public int getFpsLimit()
    {
        return gameOption.getFpsLimit();
    }
    public void setFpsLimit(int fpsLimit)
    {
        gameOption.setFpsLimit(fpsLimit);
    }
    public int getDrawOption()
    {
        return drawOption;
    }
    public void setDrawOption(int drawOption)
    {
        this.drawOption = drawOption;
    }
    public boolean isDrawOption(int type)
    {
        return type == (drawOption & type);
    }
    public void setDrawOption(int type, boolean flag)
    {
        if (flag)
            drawOption = drawOption | type;
        else
            drawOption = drawOption & ~type;
    }
    public KeyConfig getKeyConfig()
    {
        return keyConfig;
    }
    public void setKeyConfig(KeyConfig keyConfig)
    {
        this.keyConfig = keyConfig;
    }
}
