/*
 * 作成日: 2007/01/17
 */
package bakutan.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import bakutan.common.BUserInterface;
import bakutan.common.DataManager;
import bakutan.common.GameOption;
import bakutan.common.LoginInfo;
import bakutan.common.NetworkThread;
import bakutan.common.SimpleUserInterface;
import bakutan.common.ai.AIInterface;
import bakutan.game.deathmatch.AIAlgorismicVer1;

public class BakutanAIClient
{
    private BUserInterface userinterface;
    private GameOption gameOption;
    NetworkThread networkThread;
    
    public BakutanAIClient(BUserInterface userinterface, GameOption gameOption)
    {
        super();
        this.userinterface = userinterface; 
        this.gameOption = gameOption;
    }
    public void startSession(String host, int port, LoginInfo loginInfo)
    {
        networkThread = new NetworkThread(userinterface,gameOption);
        networkThread.startSession(host, port, loginInfo);
    }
    
    public void stopSession()
    {
        if (networkThread != null)
        {
            networkThread.stopNetworkThread();
        }
    }
    
    public GameOption getGameOption()
    {
        return gameOption;
    }
    public void setGameOption(GameOption gameOption)
    {
        this.gameOption = gameOption;
    }
    public static void main(String[] args)
    {
        String nickname = null;
        String username = null;
        String password = null;
        String host = null;
        int port = DataManager.DEFAULT_PORT;
        String aiClassName = AIAlgorismicVer1.class.getName();
        int mode = 0;
        boolean goflag = true;
        for (int i=0;i<args.length;i++)
        {
            if (mode == 0)
            {
                if (args[i].equals("-l"))
                    mode = 1;
                else if (args[i].equals("-g"))
                    mode = 2;
                else if (args[i].equals("-h"))
                    mode = 3;
                else if (args[i].equals("-p"))
                    mode = 4;
                else if (args[i].equals("-a"))
                    mode = 5;
                else
                {
                    System.out.println("Unknown option:"+args[i]);
                    goflag = false;
                    break;
                }
            }
            else if (mode == 1)
            {
                String[] tmp = args[i].split("[:]");
                username = tmp[0];
                if (tmp.length > 1)
                {
                    password = tmp[1];
                }
                mode = 0;
            }
            else if (mode == 2)
            {
                nickname = args[i];
                mode = 0;
            }
            else if (mode == 3)
            {
                host = args[i];
                mode = 0;
            }
            else if (mode == 4)
            {
                try
                {
                    port = Integer.parseInt(args[i]);
                }
                catch(NumberFormatException e)
                {
                    System.out.println("Invalid port.");
                    goflag = false;
                    break;
                }
                mode = 0;
            }
            else if (mode == 5)
            {
                aiClassName = args[i];
                mode = 0;
            }
        }
        if (nickname == null && username == null)
        {
            System.out.println("username or nickname are needed.");
            goflag = false;
        }
        if (host == null)
        {
            System.out.println("host is needed.");
        }
        if (!goflag)
        {
            printHelp();
            return;
        }
        LoginInfo loginInfo = new LoginInfo();
        if (nickname != null)
        {
            loginInfo.setMode(LoginInfo.MODE_GUEST);
            loginInfo.setNickname(nickname);
        }
        else if (username != null)
        {
            loginInfo.setMode(LoginInfo.MODE_LOGIN);
            loginInfo.setUsername(username);
            if (password == null)
            {
                try
                {
                    BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
                    System.out.print("Password:");
                    password = reader.readLine();
                }
                catch(IOException e)
                {
                    e.printStackTrace();
                    return;
                }
            }
            loginInfo.setPassword(password);
        }
        else
        {
            throw new RuntimeException();
        }

        AIInterface aiInterface;
        try
        {
            Class aiClass = Class.forName(aiClassName);
            aiInterface = (AIInterface)aiClass.newInstance();
        }
        catch(Exception e)
        {
            e.printStackTrace();
            return;
        }
        
        SimpleUserInterface userInterface = new SimpleUserInterface();
        userInterface.setOutputDate(true);
        GameOption gameOption = new GameOption();
        gameOption.setFpsLimit(20);
        gameOption.setForceReconnect(true);
        gameOption.setAiInterface(aiInterface);
        gameOption.setWorkAI(true);
        BakutanAIClient bac = new BakutanAIClient(userInterface, gameOption);
        bac.startSession(host, port, loginInfo);
    }
    private static void printHelp()
    {
        String helpString = "Usage:\n"
            + "    -l <username>:<password>\n"
            + "    -g <nickname>\n"
            + "    -h <host>\n"
            + "    -p <port>\n"
            + "    -a <AIClass>";
        System.out.println(helpString);
    }
}
