/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

package bakutan.client;

import java.awt.Dimension;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.AccessControlException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Locale;

import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import org.w3c.dom.Element;
import org.xml.sax.SAXParseException;

import bakutan.client.BStagePanel;
import bakutan.common.BUserInterface;
import bakutan.common.DataManager;
import bakutan.common.InvalidStructureException;
import bakutan.common.LoginInfo;
import bakutan.common.XMLConverter;
import bakutan.common.ai.AIInterface;
import bakutan.common.ai.InvalidAIData;
import bakutan.common.serverinfo.ServerInfo;
import bakutan.common.serverinfo.SmallServerInfo;
import bakutan.common.util.BAudio;
import bakutan.common.util.HelpPanel;
import bakutan.common.util.ImplementBAudio;
import bakutan.common.util.LanguageTranslator;
import bakutan.game.deathmatch.AIAlgorismicVer2;
import bakutan.server.ServerConfigurationInfo;
import bakutan.server.ServerConfigurationInfoPanel;

/**
 * @author cattaka
 */
public class BakutanClient extends JApplet implements ActionListener,KeyListener,BUserInterface
{
	public static final String RESOURCE_CONFIGURATIONINFO = "ConfigurationInfo.xml";
	public static final String RESOURCE_SERVERINFOLIST = "ServerInfoList.xml";
	public static final String RESOURCE_OFFICIALSERVERINFOLIST = "OfficialServerInfoList.xml";
	public static final int MESSAGE_ROWS = 10;
    
	private BStagePanel stagePanel;
    private JMenuItem loadAiItem;
	private JTextField messageField;
	private JTextArea messageArea;
	private HelpPanel panelHelp;
	private JMenu serverInfoMenu;
	private JMenu officialServerInfoMenu;
	private ServerInfoPanel serverInfoPanel;
	private ServerInfoTableModel serverInfoList;
	private ServerInfoTableModel officialServerInfoList;
	private ConfigurationPanel configurationPanel;
	private ConfigurationInfo configurationInfo;
	private LanguageTranslator languageTranslator;
    private BAudio baudio;
	private File workDir;

    private ServerConfigurationInfoPanel serverConfigurationInfoPanel;
    
    private int messageAreaRows = 0;
    
    private JFileChooser fileChooser;
    private InputLoginInfoDialog inputLoginInfoDialog;
    private AISelectDialog aiSelectDialog;
    
    private JPanel mainPanel;
    
	public BakutanClient()
	{
        workDir = null;
		constracter();
	}
	public BakutanClient(File workDir)
	{
		this.workDir = workDir;
        fileChooser = new JFileChooser(workDir);
		constracter();
        
	}
	private void constracter()
	{
	    //setIgnoreRepaint(true);
        setFocusable(true);
		this.languageTranslator = createLanguageTranslator();
		this.baudio = new ImplementBAudio();
        
		URL helpURL = getClass().getClassLoader().getResource("help.utf8.txt");
		if (helpURL != null)
		{
			panelHelp = new HelpPanel("Bakutan Help",helpURL);
		}
		
		messageField = new JTextField();
		messageArea = new JTextArea();
		messageArea.setEditable(false);
		messageArea.setLineWrap(true);
        
        GraphicsConfiguration gcon = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
        stagePanel = new BStagePanel(gcon, this);
        configurationPanel = new ConfigurationPanel(this)
        {
            @Override
            public void closeOperation(boolean approve)
            {
                if (approve)
                {
                    ConfigurationInfo ci = getConfigurationInfo();
                    BakutanClient.this.setConfigurationInfo(ci);
                    saveResource(RESOURCE_CONFIGURATIONINFO, ClientXMLConverter.createDocument(configurationInfo).getDocumentElement());
                }
                popPanel();
            }
        };
        serverConfigurationInfoPanel = new ServerConfigurationInfoPanel(this, false);
        {
            JButton buttonOK = serverConfigurationInfoPanel.getButtonOK();
            JButton buttonCancel = serverConfigurationInfoPanel.getButtonCancel();
            buttonOK.setActionCommand("ok");
            buttonCancel.setActionCommand("cancel");
            ActionListener al = new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    if (e.getActionCommand().equals("ok"))
                    {
                        ServerConfigurationInfo sci = serverConfigurationInfoPanel.getServerConfigurationInfo();
                        if (sci == null)
                            return;
                        
                        String nickname = JOptionPane.showInputDialog(BakutanClient.this, translate("input_nick_name"), configurationInfo.getDefaultNickName());
                        if (nickname == null)
                            return;
                        
                        LoginInfo loginInfo = new LoginInfo();
                        loginInfo.setMode(LoginInfo.MODE_GUEST);
                        loginInfo.setNickname(nickname);
                        
                        updateLoginInfo(loginInfo);
                        
                        stagePanel.startSingleGameSession(loginInfo, sci);
                        popPanel();
                    }
                    else if (e.getActionCommand().equals("cancel"))
                    {
                        popPanel();
                    }
                    
                }
            };
            buttonOK.addActionListener(al);
            buttonCancel.addActionListener(al);
        }

		JScrollPane messagePane = new JScrollPane(messageArea, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		Dimension dim = new Dimension(stagePanel.getPreferredSize());
        mainPanel = new JPanel();
        mainPanel.setLayout(new GridLayout());
        mainPanel.setPreferredSize(new Dimension(dim));
		dim.height /= 4;
		messagePane.setPreferredSize(dim);
		messagePane.setMinimumSize(dim);
		
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gc = new GridBagConstraints();
		gc.fill = GridBagConstraints.BOTH;
		gc.gridx=0;
		gc.gridy=0;
		gc.weightx = 1;
		gc.weighty = 1;
		gbl.setConstraints(mainPanel,gc);
		gc.gridy++;
		gc.weightx = 1;
		gc.weighty = 0;
		gbl.setConstraints(messagePane,gc);
		gc.fill = GridBagConstraints.HORIZONTAL;
		gc.gridy++;
		gc.weighty = 0;
		gbl.setConstraints(messageField,gc);
		
		getContentPane().setLayout(gbl);
		getContentPane().add(mainPanel);
		getContentPane().add(messagePane);
		getContentPane().add(messageField);

		messageField.setEditable(false);

		JPopupMenu.setDefaultLightWeightPopupEnabled(false);
		JMenuBar menubar = new JMenuBar();
		//JMenu menu = new JMenu("menu");
		JMenu menu = new JMenu(translate("menu"));
        JMenuItem singleGameItem = new JMenuItem(translate("single_game"));
        singleGameItem.setActionCommand("single_game");
		JMenuItem connectItem = new JMenuItem(translate("connect"));
		connectItem.setActionCommand("connect");
		JMenuItem disconnectItem = new JMenuItem(translate("disconnect"));
		disconnectItem.setActionCommand("disconnect");
        JMenuItem replayItem = new JMenuItem(translate("replay"));
        replayItem.setActionCommand("replay");
		JMenuItem configItem = new JMenuItem(translate("configuration"));
		configItem.setActionCommand("configuration");

        JMenu aiMenu = new JMenu(translate("AI"));
        loadAiItem = new JMenuItem(translate("load_ai"));
        loadAiItem.setActionCommand("loadAI");
        loadAiItem.addActionListener(this);
//        saveAiItem.setActionCommand("saveAI");
//        saveAiItem.addActionListener(this);
        JMenuItem selectAIItem = new JMenuItem(translate("select_ai"));
        selectAIItem.setActionCommand("selectAI");
        selectAIItem.addActionListener(this);
        JMenuItem showAIToolItem = new JMenuItem(translate("show_ai_tool"));
        showAIToolItem.setActionCommand("showAITool");
        showAIToolItem.addActionListener(this);
        aiMenu.add(loadAiItem);
        aiMenu.add(selectAIItem);
//        aiMenu.add(saveAiItem);
        aiMenu.add(showAIToolItem);
        
		serverInfoMenu = new JMenu(translate("favorite"));
		officialServerInfoMenu = new JMenu(translate("server_list"));
		
		JMenu help = new JMenu(translate("help"));
		JMenuItem aboutItem = new JMenuItem(translate("help"));
		aboutItem.setActionCommand("version_info");
		
		menu.add(connectItem);
		menu.add(disconnectItem);
		menu.add(singleGameItem);
		menu.add(configItem);
        if (workDir != null)
            menu.add(replayItem);
		help.add(aboutItem);
		
		menubar.add(menu);
        if (workDir != null)
        {
            menubar.add(aiMenu);
        }
		menubar.add(officialServerInfoMenu);
        if (workDir!=null)
        {
            menubar.add(serverInfoMenu);
        }
		menubar.add(help);
		setJMenuBar(menubar);

		connectItem.addActionListener(this);
		disconnectItem.addActionListener(this);
		singleGameItem.addActionListener(this);
		configItem.addActionListener(this);
		replayItem.addActionListener(this);
		aboutItem.addActionListener(this);
		messageField.addKeyListener(this);
		
		FocusAdapter fa = new FocusAdapter()
		{
			public void focusGained(FocusEvent e)
			{
				messageField.requestFocusInWindow();
			}
		};
		
		mainPanel.addFocusListener(fa);
		messageArea.addFocusListener(fa);
		messageField.requestFocusInWindow();

		serverInfoPanel = new ServerInfoPanel(this)
        {
		    @Override
		    public void closeOperation(boolean approve)
		    {
                popPanel();
                if (approve)
                {
                    if (isEditableList())
                    {
                        saveResource(RESOURCE_SERVERINFOLIST, ClientXMLConverter.createDocument(serverInfoList.getServerInfos()).getDocumentElement());
                    }
                    updateServerInfo();
                    SmallServerInfo ssi = serverInfoPanel.getSelectedServerInfo();
                    if (ssi != null)
                    {
                        startSession(ssi);
                    }
                }
		    }
            @Override
            public void downloadServerList()
            {
                super.downloadServerList();
                try
                {
                    downloadOfficialServerInfo();
                }catch(Exception ex)
                {
                    JOptionPane.showMessageDialog(this, translate("download_failed"));
                    outputMessage(ex.getMessage());
                    ex.printStackTrace();
                }
            }
        };
		
		try
		{
			configurationInfo = ClientXMLConverter.createConfigurationInfo(loadResource(RESOURCE_CONFIGURATIONINFO));
		}catch(InvalidStructureException e)
		{
			configurationInfo = new ConfigurationInfo();
		}
        setConfigurationInfo(configurationInfo);
		
        officialServerInfoList = new ServerInfoTableModel();
        try
        {
            downloadOfficialServerInfo();
        }
        catch(IOException e)
        {
            try
            {
                try
                {
                    Element element = XMLConverter.readXML(getClass().getClassLoader().getResource("default." + RESOURCE_SERVERINFOLIST));
                    officialServerInfoList.setServerInfos(ClientXMLConverter.createServerInfos(element));
                }catch(SAXParseException exc)
                {
                    throw new RuntimeException(exc);
                }catch(IOException exc)
                {
                    throw new RuntimeException(exc);
                }
            }catch(InvalidStructureException exc)
            {
                exc.printStackTrace();
            }
            updateOfficialServerInfo();
        }
		
		serverInfoList = new ServerInfoTableModel();
		try
		{
			serverInfoList.setServerInfos(ClientXMLConverter.createServerInfos(loadResource(RESOURCE_SERVERINFOLIST)));
		}catch(InvalidStructureException e)
		{
			e.printStackTrace();
		}
		updateServerInfo();
        
        inputLoginInfoDialog = new InputLoginInfoDialog();
        inputLoginInfoDialog.setUsername(configurationInfo.getDefaultUserName());
        inputLoginInfoDialog.setNickname(configurationInfo.getDefaultNickName());
        
        aiSelectDialog = new AISelectDialog();
        try
        {
            Class aiClass = Class.forName(configurationInfo.getOtherAIClassName());
            AIInterface aiInterface = (AIInterface)(aiClass.newInstance());
            setAIInterface(aiInterface);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            AIInterface ar = new AIAlgorismicVer2();
            setAIInterface(ar);
        }
        serverInfoPanel.setServerInfoList(officialServerInfoList);
        serverInfoPanel.setEditableList(false);
        pushPanel(serverInfoPanel);
	}
    private void pushPanel(JPanel panel)
    {
        stagePanel.setDrawable(false);
        mainPanel.removeAll();
        mainPanel.add(panel);
        mainPanel.validate();
        mainPanel.repaint();
    }
    private void popPanel()
    {
        mainPanel.removeAll();
        mainPanel.add(stagePanel);
        mainPanel.validate();
        stagePanel.setDrawable(true);
        mainPanel.repaint();
        messageField.requestFocusInWindow();
    }

	public void actionPerformed(ActionEvent e)
	{
		if (e.getActionCommand().equals("connect"))
		{
			String arg = JOptionPane.showInputDialog(this, translate("input_server_address"), DataManager.DEFAULT_SERVER + ":" + DataManager.DEFAULT_PORT);
			if (arg== null)
				return;
			String[] tmp = arg.split("[:]",2);
			int port = DataManager.DEFAULT_PORT;
			if (tmp.length > 1)
			{
				try
				{
					port = Integer.parseInt(tmp[1]);
				}catch(NumberFormatException exp)
				{
				}
			}
            SmallServerInfo ssi = new SmallServerInfo();
            ssi.setHostName(tmp[0]);
            ssi.setPort(port);
            startSession(ssi);
            popPanel();
//            LoginInfo loginInfo = inputLoginInfoDialog.showDialog();
//            if (loginInfo == null)
//                return;
//            updateLoginInfo(loginInfo);
//			stagePanel.startSession(tmp[0],port,loginInfo);
        }
        else if (e.getActionCommand().equals("single_game"))
        {
            pushPanel(serverConfigurationInfoPanel);
        }
        else if (e.getActionCommand().equals("replay"))
        {
            JFileChooser jfc = new JFileChooser(workDir + "/replay");
            if (jfc.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
                return;
            File file = jfc.getSelectedFile();
            try
            {
                URL replay = file.toURL();
                stagePanel.startSession(replay);
            }
            catch(MalformedURLException exc)
            {
                exc.printStackTrace();
            }
            popPanel();
		}else if (e.getActionCommand().equals("disconnect"))
		{
			stagePanel.stopSession(true);
			
		}else if (e.getActionCommand().equals("configuration"))
		{
			configurationPanel.setConfigurationInfo(configurationInfo);
            pushPanel(configurationPanel);
//			if (configurationPanel.showDialog() == ConfigurationPanel.APPROVE_OPTION)
//			{
//				ConfigurationInfo ci = configurationPanel.getConfigurationInfo();
//                setConfigurationInfo(ci);
//				saveResource(RESOURCE_CONFIGURATIONINFO, ClientXMLConverter.createDocument(configurationInfo).getDocumentElement());
//			}
		}else if (e.getActionCommand().equals("show_server_info"))
		{
			serverInfoPanel.setServerInfoList(serverInfoList);
			serverInfoPanel.setEditableList(true);
            pushPanel(serverInfoPanel);
		}else if(e.getActionCommand().indexOf("jump_server_info_") == 0)
		{
			String tmp = e.getActionCommand();
			tmp = tmp.substring(17,tmp.length());
			SmallServerInfo sli;
			try
			{
				int i = Integer.parseInt(tmp);
				sli = serverInfoList.getServerInfo(i);
			}catch(Exception ex)
			{
				ex.printStackTrace();
				return;
			}
            startSession(sli);
            popPanel();
		}else if (e.getActionCommand().equals("show_official_server_info"))
		{
			serverInfoPanel.setServerInfoList(officialServerInfoList);
			serverInfoPanel.setEditableList(false);
            pushPanel(serverInfoPanel);
		}else if (e.getActionCommand().equals("update_official_server_info"))
		{
			try
			{
                downloadOfficialServerInfo();
			}catch(Exception ex)
			{
                JOptionPane.showMessageDialog(this, translate("download_failed"));
                outputMessage(ex.getMessage());
				ex.printStackTrace();
			}
		}else if(e.getActionCommand().indexOf("jump_official_server_info_") == 0)
		{
			String tmp = e.getActionCommand();
			tmp = tmp.substring(26,tmp.length());
			SmallServerInfo sli;
			try
			{
				int i = Integer.parseInt(tmp);
				sli = officialServerInfoList.getServerInfo(i);
			}catch(Exception ex)
			{
				ex.printStackTrace();
				return;
			}
            startSession(sli);
            popPanel();
		}else if (e.getActionCommand().equals("version_info"))
		{
			if (panelHelp != null)
			{
				panelHelp.setVisible(true);
			}
        }else if (e.getActionCommand().equals("loadAI"))
        {
            File file;
            if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
            {
                file = fileChooser.getSelectedFile();
                try
                {
                    loadAI(file);
                }
                catch(IOException exp)
                {
                    exp.printStackTrace();
                    JOptionPane.showMessageDialog(this,exp);
                }
            }
        }else if (e.getActionCommand().equals("selectAI"))
        {
            aiSelectDialog.setOtherAIClassName(configurationInfo.getOtherAIClassName());
            AIInterface aii = aiSelectDialog.showDialog();
            if (aii != null)
            {
                setAIInterface(aii);
                configurationInfo.setOtherAIClassName(aii.getClass().getName());
                saveResource(RESOURCE_CONFIGURATIONINFO, ClientXMLConverter.createDocument(configurationInfo).getDocumentElement());
            }
        }else if (e.getActionCommand().equals("showAITool"))
        {
            AiLearningTool gt = new AiLearningTool(workDir);
            JDialog dialog = new JDialog();
            dialog.setTitle("AILearningTool");
            dialog.getContentPane().add(gt);
            dialog.setSize(500,400);
            dialog.setModal(true);
            dialog.setVisible(true);
            gt.stopLearn();
        }
	}
    private void updateLoginInfo(LoginInfo loginInfo)
    {
        if (loginInfo.getMode() == LoginInfo.MODE_GUEST)
        {
            configurationInfo.setDefaultNickName(loginInfo.getNickname());
        }
        else if (loginInfo.getMode() == LoginInfo.MODE_LOGIN)
        {
            configurationInfo.setDefaultUserName(loginInfo.getUsername());
            
        }
        else if (loginInfo.getMode() == LoginInfo.MODE_NEWACOUNT)
        {
            configurationInfo.setDefaultUserName(loginInfo.getUsername());
            configurationInfo.setDefaultNickName(loginInfo.getNickname());
        }
        saveResource(RESOURCE_CONFIGURATIONINFO, ClientXMLConverter.createDocument(configurationInfo).getDocumentElement());
    }
    private Collection<ServerInfo> convToServerInfos(Collection<SmallServerInfo> ssis)
    {
        ArrayList<ServerInfo> sis = new ArrayList<ServerInfo>();
        Iterator<SmallServerInfo> ite = ssis.iterator();
        while(ite.hasNext())
        {
            SmallServerInfo ssi = ite.next();
            ServerInfo si = new ServerInfo();
            si.set(ssi);
            sis.add(si);
        }
        return sis;
    }
	private void updateOfficialServerInfo()
	{
		officialServerInfoMenu.removeAll();
		JMenuItem manageServerInfo = new JMenuItem(translate("show_server_list"));
		manageServerInfo.setActionCommand("show_official_server_info");
		manageServerInfo.addActionListener(this);
//		JMenuItem updateServerInfo = new JMenuItem(translate("update_server_list"));
//		updateServerInfo.setActionCommand("update_official_server_info");
//		updateServerInfo.addActionListener(this);
		
		officialServerInfoMenu.add(manageServerInfo);
//		officialServerInfoMenu.add(updateServerInfo);
//        if (workDir!=null)
//        {
//            officialServerInfoMenu.add(updateServerInfo);
//        }
		officialServerInfoMenu.addSeparator();
		for (int i=0;i<officialServerInfoList.size();i++)
		{
			JMenuItem entry = new JMenuItem(officialServerInfoList.getServerInfo(i).getLabel());
			entry.setActionCommand("jump_official_server_info_" + i);
			entry.addActionListener(this);
			officialServerInfoMenu.add(entry);
		}
	}
	private void updateServerInfo()
	{
		serverInfoMenu.removeAll();
		JMenuItem manageServerInfo = new JMenuItem(translate("show_server_list"));
		manageServerInfo.setActionCommand("show_server_info");
		manageServerInfo.addActionListener(this);
		serverInfoMenu.add(manageServerInfo);
		serverInfoMenu.addSeparator();
		for (int i=0;i<serverInfoList.size();i++)
		{
			JMenuItem entry = new JMenuItem(serverInfoList.getServerInfo(i).toString());
			entry.setActionCommand("jump_server_info_" + i);
			entry.addActionListener(this);
			serverInfoMenu.add(entry);
		}
	}

	public BStagePanel getStagePanel()
	{
		return stagePanel;
	}
	public File getWorkDir()
	{
	    return workDir;
	}
	public void keyTyped(KeyEvent e)
	{
		if (e.getKeyChar() == KeyEvent.VK_ENTER)
		{
			if (messageField.isEditable())
			{
				String tmp = messageField.getText();
				if (tmp.length() > 0)
				{
                    if (tmp.equals("/switchdraw"))
                    {
                        stagePanel.switchDraw();
                    }
                    else
                    {
                        stagePanel.setMessage(tmp);
                    }
				}
				messageField.setText("");
				messageField.setEditable(false);
			}else
			{
				messageField.setEditable(true);
			}
		}else if (e.getKeyChar() == KeyEvent.VK_ESCAPE)
		{
			if (messageField.isEditable())
			{
				messageField.setText("");
				messageField.setEditable(false);
			}
		}
	}

	public void keyPressed(KeyEvent e)
	{
		if (messageField.isEditable())
		{
			return;
		}
		else
		{
			stagePanel.keyPressed(e);
		}
	}

	public void keyReleased(KeyEvent e)
	{
		if (messageField.isEditable())
		{
			return;
		}
		else
		{
			stagePanel.keyReleased(e);
		}
	}
	public void outputMessage(String message)
	{
        int r = 1;
        int index = 0;
        while ((index = message.indexOf('\n', index)) != -1)
        {
            r++;
        }
        messageAreaRows += r;
		messageArea.append(message + "\n");
        index = 0;
        if (messageAreaRows > MESSAGE_ROWS)
        {
            for (int i=0;i<=messageAreaRows - MESSAGE_ROWS;i++)
            {
                index = messageArea.getText().indexOf('\n', index);
            }
            messageArea.replaceRange("", 0, index+1);
            messageAreaRows = 5;
        }
		messageArea.setCaretPosition(messageArea.getText().length());
	}
	public String translate(String key)
	{
		return languageTranslator.translate(key);
	}
	public String translate(String key, String[] opt)
	{
		return languageTranslator.translate(key,opt);
	}
	private Element loadResource(String name)
	{
		if (workDir != null)
		{
			File file = new File(workDir + "/" + name);
			try
			{
				Element element = XMLConverter.readXML(file);
				return element;
			}catch(SAXParseException e)
			{
			}catch(IOException e)
			{
			}
		}
		try
		{
			Element element = XMLConverter.readXML(getClass().getClassLoader().getResource("default." + name));
			if (saveResource(name,element))
				outputMessage(name + " is created.");
			return element;
		}catch(SAXParseException e)
		{
			throw new RuntimeException(e);
		}catch(IOException e)
		{
			throw new RuntimeException(e);
		}
	}
	private boolean saveResource(String name, Element element)
	{
		if (workDir != null)
		{
			File file = new File(workDir + "/" + name);
			try
			{
				XMLConverter.writeXML(file,element.getOwnerDocument());
				return true;
			}catch(IOException e)
			{
				return false;
			}
		}else
		{
			return false;
		}
	}
	private Element loadDynamicResource(String name)
	{
		if (workDir != null)
		{
			File file = new File(workDir + "/" + DataManager.getCurrentProtocolVersion() + "_" + name);
			if (!file.exists())
			{
                return null;
			}
			try
			{
				Element element = XMLConverter.readXML(file);
				return element;
			}catch(SAXParseException e)
			{
			}catch(IOException e)
			{
			}
		}
		return null;
	}
    private void downloadOfficialServerInfo() throws IOException
    {
        try
        {
            this.officialServerInfoList.setServerInfos(convToServerInfos(DataManager.downloadSmallServerinfos(configurationInfo.getServerInfoManagerAddress().getInetAddress(), DataManager.DEFAULT_TIMEOUT)));
            serverInfoPanel.setServerInfoList(this.officialServerInfoList);
            outputMessage(translate("server_list_is_downloaded"));
            updateOfficialServerInfo();
        }
        catch(AccessControlException e)
        {
        }
    }
//	private Element downloadDynamicResource(String name)
//	throws IOException
//	{
//		URL url = new URL(configurationInfo.getDynamicResourcePath() + "/" + DataManager.getCurrentProtocolVersion() + "_" + name);
//		try
//		{
//			Element element = XMLConverter.readXML(url);
//			saveDynamicResource(name, element);
//			JOptionPane.showMessageDialog(this, translate("download_success"));
//			return element;
//		}catch(SAXParseException e)
//		{
//			throw new IOException(e.getMessage());
//		}
//	}
	private boolean saveDynamicResource(String name, Element element)
	{
		if (workDir != null)
		{
			File file = new File(workDir + "/" + DataManager.getCurrentProtocolVersion() + "_" + name);
			try
			{
				XMLConverter.writeXML(file,element.getOwnerDocument());
				return true;
			}catch(IOException e)
			{
				return false;
			}
		}else
		{
			return false;
		}
	}

	public static LanguageTranslator createLanguageTranslator()
	{
		URL in = LanguageTranslator.class.getClassLoader().getResource("PhraseDic_"+ Locale.getDefault().getLanguage()+".xml");
		if (in == null)
   		in = LanguageTranslator.class.getClassLoader().getResource("PhraseDic.xml");
		if (in == null)
			return new LanguageTranslator();
		try
		{
			return ClientXMLConverter.createLanguageTranslator(XMLConverter.readXML(in));
		}catch(InvalidStructureException e)
		{
			e.printStackTrace();
			return new LanguageTranslator();
		}catch(SAXParseException e)
		{
			e.printStackTrace();
			return new LanguageTranslator();
		}catch(IOException e)
		{
			e.printStackTrace();
			return new LanguageTranslator();
		}
	}
    private void startSession(SmallServerInfo ssi)
    {
        LoginInfo loginInfo = inputLoginInfoDialog.showDialog();
        if (loginInfo == null)
            return;
        updateLoginInfo(loginInfo);
        stagePanel.startSession(ssi.getHostName(), ssi.getPort(), loginInfo);
    }
    
    @Override
    public void start()
    {
        super.start();
        stagePanel.setEnableSE(configurationInfo.isSoundEffect());
        stagePanel.setEnableBGM(configurationInfo.isBgm());
        
        String replayfilename = getParameter("replayfile");
        String onlyReplay = getParameter("onlyreplay");
        String restartStr = getParameter("restart");
        boolean restart = false;
        if (restartStr != null)
        {
            restart = restartStr.toLowerCase().equals("true");
            stagePanel.setForceReconnect(restart);
        }
        
        if (replayfilename != null)
        {
            try
            {
                URL url = new URL(getCodeBase(), replayfilename);
                stagePanel.startSession(url);
                popPanel();
            }
            catch(MalformedURLException e)
            {
                outputMessage(e.getMessage());
            }
        }
        if (onlyReplay != null && onlyReplay.toLowerCase().equals("true"))
        {
            setJMenuBar(null);
        }
    }
    public void stop()
    {
        super.stop();
        stagePanel.setEnableSE(false);
        stagePanel.setEnableBGM(false);
        stagePanel.stopSession(true);
    }
    public void setAIInterface(AIInterface aii)
    {
        if (aii == null)
            throw new NullPointerException();
        
//        if(workDir != null)
//        {
//            aii.setWorkDir(workDir);
//        }
        aii.setUserInterface(this);
        outputMessage(translate("ai_type",new String[]{aii.getAIName()}));
        stagePanel.setAIInterface(aii);
        loadAiItem.setEnabled(aii.isFileLoadable());
    }
    public void loadAI(File file) throws IOException
    {
        AIInterface ar = stagePanel.getAIInterface();
        try
        {
            ar.loadAI(file);
            outputMessage(translate("ai_file_is_loaded"));
            outputMessage(translate("ai_version", new String[]{"3(newest)"}));
        }
        catch(IOException e)
        {
            e.printStackTrace();
            outputMessage(translate("loading_ai_file_is_failed"));
            return;
        }
        catch(InvalidAIData e)
        {
            e.printStackTrace();
            outputMessage(translate("file_is_invalid"));
            return;
        }
    }
    
	public ConfigurationInfo getConfigurationInfo()
    {
        return configurationInfo;
    }
    public void setConfigurationInfo(ConfigurationInfo configurationInfo)
    {
        this.configurationInfo = configurationInfo;
        stagePanel.setVoumeSE(configurationInfo.getVolumeSE());
        stagePanel.setVoumeBGM(configurationInfo.getVolumeBGM());
        stagePanel.setEnableSE(configurationInfo.isSoundEffect());
        stagePanel.setEnableBGM(configurationInfo.isBgm());
        stagePanel.setForceReconnect(configurationInfo.isForceReconnect());
        stagePanel.setKeyConfig(configurationInfo.getKeyConfig());
        stagePanel.setFpsLimit(configurationInfo.getFpsLimit());
    }
    public BAudio getBAudio()
    {
        return baudio;
    }
    
    public static void main(String[] args)
	{
        JFrame dt = new JFrame();
		File workDir = new File(System.getProperty("user.home") + "/.bakutan");
		if (!workDir.exists())
		{
			workDir.mkdir();
		}
		BakutanClient bc = new BakutanClient(workDir);
		dt.getContentPane().add(bc);
		dt.setTitle("Bakutan build " + DataManager.getBuildNumber());
		dt.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		dt.setVisible(true);
		dt.setSize(dt.getPreferredSize());
		dt.validate();
		bc.messageField.requestFocusInWindow();
	}
}
