/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/04
 */
package bakutan.client;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;

import javax.swing.JDialog;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import bakutan.common.BUserInterface;
import bakutan.common.BodyState;
import bakutan.common.DataManager;
import bakutan.common.serverinfo.BStageState;
import bakutan.common.serverinfo.SmallServerInfo;
import bakutan.common.util.CompressedInputStream;
import bakutan.common.util.CompressedOutputStream;

/**
 * @author cattaka
 */
public class BakutanServerInfo extends JDialog
{
	private SmallServerInfo sli;
	private JTextArea textArea;
	private BUserInterface userinterfase;
	private BStageState bstageState;
	
	public BakutanServerInfo(BUserInterface userinterface, SmallServerInfo sli)
	{
		this.userinterfase = userinterface;
		this.sli = sli;
		
		setTitle(sli.getHostName() + ":" + sli.getPort());
		setSize(300,300);
		textArea = new JTextArea();
		textArea.setLineWrap(true);
		JScrollPane textPane = new JScrollPane(textArea,JScrollPane.VERTICAL_SCROLLBAR_NEVER,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		textArea.setEditable(false);
		getContentPane().add(textPane);
	}
	public BStageState getBstageState()
	{
		return bstageState;
	}
	public void startCheck()
	{
		new Thread()
		{
			public void run()
			{
				Socket socket = null;
				try
				{
					socket = new Socket(sli.getHostName(), sli.getPort());
					socket.setSoTimeout(5000);
					DataInputStream in = new DataInputStream(new CompressedInputStream(socket.getInputStream()));
					DataOutputStream out = new DataOutputStream(new CompressedOutputStream(socket.getOutputStream()));
					String protocol = in.readUTF();
					textArea.append(userinterfase.translate("protocol_version") + " : " + protocol + "\n");
					DataManager dataManager = DataManager.getAvailableDataManager(protocol);
                    if (dataManager != null)
					{
						out.writeByte(DataManager.REQUEST_BSTAGESTATE);
						out.flush();
						bstageState = DataManager.inputBStageState(in);
						BodyState[] bs = bstageState.getBodyStates();
						textArea.append(userinterfase.translate("login_message", new String[]{bstageState.getLoginMessage()}) + "\n");
						textArea.append(userinterfase.translate("note_message", new String[]{bstageState.getNoteMessage()}) + "\n");
						textArea.append(userinterfase.translate("clients_num", new String[]{bs.length + "/" + bstageState.getMaxClients()}) + "\n");
						textArea.append(userinterfase.translate("name") + " : " + userinterfase.translate("score") + "\n");
						for(int i=0;i<bs.length;i++)
						{
							textArea.append(bs[i].getName() + " : " + bs[i].getScore() + "\n");
						}
					}else
					{
						textArea.append(userinterfase.translate("different_protocol_version", new String[]{DataManager.getCurrentProtocolVersion(), protocol}) + "\n");
					}
					in.close();
					out.close();
					socket.close();
				}catch(Exception e)
				{
					textArea.append(userinterfase.translate("network_error") + "\n");
					e.printStackTrace();
					try
					{
						if (socket != null && socket.isConnected())
							socket.close();
					}catch(IOException e2)
					{
					}
				}
			}
		}.start();
	}
}
