/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/09
 */
package bakutan.client;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import bakutan.common.DataManager;
import bakutan.common.InvalidStructureException;
import bakutan.common.XMLConverter;
import bakutan.common.serverinfo.ServerInfo;
import bakutan.common.util.LanguageTranslator;

/**
 * @author cattaka
 */
public class ClientXMLConverter extends XMLConverter
{
	public static LanguageTranslator createLanguageTranslator(Element element)
	throws InvalidStructureException
	{
		LanguageTranslator lt = new LanguageTranslator();
		if (!(element.getTagName().equals("PhraseDic")))
			throw new InvalidStructureException("this is not PhraseDic");

		NodeList nl;
		String arg;
		nl = element.getElementsByTagName("Phrase");
		for (int i=0;i<nl.getLength();i++)
		{
			String key = ((Element)nl.item(i)).getAttribute("key");
			String value = ((Element)nl.item(i)).getAttribute("value");
			lt.setValue(key,value);
		}
		return lt;
	}

	public static ConfigurationInfo createConfigurationInfo(Element element)
	throws InvalidStructureException
	{
		ConfigurationInfo ci = new ConfigurationInfo();
		if (!(element.getTagName().equals("ConfigurationInfo")))
			throw new InvalidStructureException("this is not ConfigurationInfo");

		String tmp;
        
        //old format
        ci.setDefaultUserName(element.getAttribute("DefaultUserName"));
        ci.setDefaultNickName(element.getAttribute("DefaultNickName"));
        ci.setOtherAIClassName(element.getAttribute("OtherAIClassName"));
		tmp = element.getAttribute("Sound");
		if (tmp.equals("true"))
			ci.setSoundEffect(true);
		else if (tmp.equals("false"))
			ci.setSoundEffect(false);
		
		tmp = element.getAttribute("BGM");
		if (tmp.equals("true"))
			ci.setBgm(true);
		else if (tmp.equals("false"))
			ci.setBgm(false);

        tmp = element.getAttribute("FPSLimit");
        try
        {
            ci.setFpsLimit(Integer.parseInt(tmp));
        }
        catch (NumberFormatException e)
        {
        }
        //old format
        
        tmp = element.getAttribute("ForceReconnect");
        if (tmp.equals("true"))
            ci.setForceReconnect(true);
        else if (tmp.equals("false"))
            ci.setForceReconnect(false);

        NodeList nl = element.getElementsByTagName("KeyConfig");
        try
        {
            if (nl.getLength() > 0)
            {
                Element keyConfigElement = (Element)nl.item(0);
                KeyConfig keyConfig = createKeyConfig(keyConfigElement);
                ci.setKeyConfig(keyConfig);
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        
        nl = element.getElementsByTagName("Sound");
        try
        {
            if (nl.getLength() > 0)
            {
                Element sound = (Element)nl.item(0);
                ci.setSoundEffect(sound.getAttribute("SE").toLowerCase().equals("true"));
                ci.setBgm(sound.getAttribute("BGM").toLowerCase().equals("true"));
                int sv = Integer.parseInt(sound.getAttribute("SEVolume"));
                ci.setVolumeSE(sv);
                int bgmv = Integer.parseInt(sound.getAttribute("BGMVolume"));
                ci.setVolumeBGM(bgmv);
                
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        
        nl = element.getElementsByTagName("AI");
        try
        {
            if (nl.getLength() > 0)
            {
                Element ai = (Element)nl.item(0);
                ci.setOtherAIClassName(ai.getAttribute("OtherAIClassName"));
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }

        nl = element.getElementsByTagName("ServerInfoManagerAddress");
        try
        {
            if (nl.getLength() > 0)
            {
                Element sima = (Element)nl.item(0);
                ci.setServerInfoManagerAddress(createServerInfoManagerAddress(sima));
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }

        nl = element.getElementsByTagName("UserInfo");
        try
        {
            if (nl.getLength() > 0)
            {
                Element userinfo = (Element)nl.item(0);
                ci.setDefaultUserName(userinfo.getAttribute("DefaultUserName"));
                ci.setDefaultNickName(userinfo.getAttribute("DefaultNickName"));
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }

        nl = element.getElementsByTagName("Graphics");
        try
        {
            if (nl.getLength() > 0)
            {
                Element graphics = (Element)nl.item(0);
                ci.setFpsLimit(Integer.parseInt(graphics.getAttribute("FPSLimit")));
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        return ci;
	}

	public static Document createDocument(ConfigurationInfo ci)
	{
		Document document = createNewDocument("ConfigurationInfo");
		Element root = document.getDocumentElement();

//		root.setAttribute("DefaultUserName", ci.getDefaultUserName());
//        root.setAttribute("DefaultNickName", ci.getDefaultNickName());
//        root.setAttribute("OtherAIClassName", ci.getOtherAIClassName());
//        root.setAttribute("FPSLimit", String.valueOf(ci.getFpsLimit()));
        if (ci.isForceReconnect())
        {
            root.setAttribute("ForceReconnect", "true");
        }else
        {
            root.setAttribute("ForceReconnect", "false");
        }
        
        Element sound = document.createElement("Sound");
        if (ci.isSoundEffect())
        {
            sound.setAttribute("SE", "true");
        }else
        {
            sound.setAttribute("SE", "false");
        }
        if (ci.isBgm())
        {
            sound.setAttribute("BGM", "true");
        }else
        {
            sound.setAttribute("BGM", "false");
        }
        sound.setAttribute("SEVolume", String.valueOf(ci.getVolumeSE()));
        sound.setAttribute("BGMVolume", String.valueOf(ci.getVolumeBGM()));
        root.appendChild(sound);

        Element ai = document.createElement("AI");
        ai.setAttribute("OtherAIClassName", ci.getOtherAIClassName());
        root.appendChild(ai);
        
        Document simeDoc = createDocument(ci.getServerInfoManagerAddress());
        Node sima = document.importNode(simeDoc.getDocumentElement(), true);
        root.appendChild(sima);
        
        Element userinfo = document.createElement("UserInfo");
        userinfo.setAttribute("DefaultUserName", ci.getDefaultUserName());
        userinfo.setAttribute("DefaultNickName", ci.getDefaultNickName());
        root.appendChild(userinfo);

        Element graphics = document.createElement("Graphics");
        graphics.setAttribute("FPSLimit", String.valueOf(ci.getFpsLimit()));
        root.appendChild(graphics);

        Document keyConfigDoc = createDocument(ci.getKeyConfig());
        Node node = document.importNode(keyConfigDoc.getDocumentElement(), true);
        root.appendChild(node);
        
		return document;
	}

	public static Collection<ServerInfo> createServerInfos(Element element)
	throws InvalidStructureException
	{
		if (!(element.getTagName().equals("ServerInfoList")))
			throw new InvalidStructureException("this is not ServerInfoList");

        ArrayList<ServerInfo> bl = new ArrayList<ServerInfo>();
		NodeList nl;
		nl = element.getElementsByTagName("ServerInfo");
		for(int i=0;i<nl.getLength();i++)
		{
			try
			{
				ServerInfo be = createServerInfo((Element)(nl.item(i)));
				bl.add(be);
			}catch(InvalidStructureException e)
			{
			}
		}
		return bl;
	}
	
	public static ServerInfo createServerInfo(Element element)
	throws InvalidStructureException
	{
		ServerInfo be = new ServerInfo();
		if (!(element.getTagName().equals("ServerInfo")))
			throw new InvalidStructureException("this is not ServerInfoEntry");

		be.setLabel(element.getAttribute("Label"));
		be.setHostName(element.getAttribute("HostName"));
		int port = DataManager.DEFAULT_PORT;
        try
        {
            port = Integer.parseInt(element.getAttribute("Port"));
            be.setPort(port);
        }catch(NumberFormatException e)
        {
        }
		
		return be;
	}
	
	public static Document createDocument(Collection<ServerInfo> bl)
	{
		Document document = createNewDocument("ServerInfoList");
		Element root = document.getDocumentElement();
		
		Iterator<ServerInfo> ite = bl.iterator();
        while(ite.hasNext())
		{
			ServerInfo be = ite.next();
			Document bed = createDocument(be);
			Element element = (Element)(document.importNode(bed.getDocumentElement(),true));
			root.appendChild(element);
		}
		return document;
	}
    public static Document createDocument(KeyConfig keyConfig)
    {
        Document document = createNewDocument("KeyConfig");
        Element root = document.getDocumentElement();
        int[] events = KeyConfig.getActions();
        for (int i=0;i<events.length;i++)
        {
            int keyCode = keyConfig.getKeyCode(events[i]);
            if (keyCode == -1)
                continue;
            Element element = document.createElement("KeyMap");
            element.setAttribute("Key", String.valueOf(keyCode));
            element.setAttribute("Action", KeyConfig.getActionText(events[i]));
            root.appendChild(element);
        }
        for (int i=0;i<keyConfig.getMouseActionNum();i++)
        {
            Element element = document.createElement("MouseButtonMap");
            element.setAttribute("Button", String.valueOf(i));
            element.setAttribute("Action", KeyConfig.getActionText(keyConfig.getMouseAction(i)));
            root.appendChild(element);
        }
        return document;
    }
    
    public static KeyConfig createKeyConfig(Element element)
    throws InvalidStructureException
    {
        try
        {
            KeyConfig keyConfig = new KeyConfig();
            if (!(element.getTagName().equals("KeyConfig")))
                throw new InvalidStructureException("this is not KeyConfig.");

            NodeList nl = element.getElementsByTagName("KeyMap");
            for (int i=0;i<nl.getLength();i++)
            {
                Node node = nl.item(i);
                if (!(node instanceof Element))
                    continue;
                Element el = (Element)node;
                String keyCodeStr = el.getAttribute("Key");
                String actionText = el.getAttribute("Action");
                try
                {
                    int keyCode = Integer.parseInt(keyCodeStr);
                    int action = KeyConfig.getActionCode(actionText); 
                    keyConfig.putKeyToAction(keyCode,action);
                }
                catch(NumberFormatException e)
                {
                }
            }
            nl = element.getElementsByTagName("MouseButtonMap");
            for (int i=0;i<nl.getLength();i++)
            {
                Node node = nl.item(i);
                if (!(node instanceof Element))
                    continue;
                Element el = (Element)node;
                String keyCodeStr = el.getAttribute("Button");
                String actionText = el.getAttribute("Action");
                try
                {
                    int button = Integer.parseInt(keyCodeStr);
                    int action = KeyConfig.getActionCode(actionText); 
                    keyConfig.setMouseAction(button,action);
                }
                catch(NumberFormatException e)
                {
                }
            }

            return keyConfig;
        }catch (NullPointerException e)
        {
            throw new InvalidStructureException();
        }
    }
    
	public static Document createDocument(ServerInfo be)
	{
		Document document = createNewDocument("ServerInfo");
		Element root = document.getDocumentElement();
		root.setAttribute("Label",be.getLabel());
        root.setAttribute("HostName",be.getHostName());
        root.setAttribute("Port",String.valueOf(be.getPort()));
		return document;
	}
/*
	public static SmallServerInfo createSmallServerInfo(Element element)
	throws InvalidStructureException
	{
		try
		{
			SmallServerInfo sli = new SmallServerInfo();
			if (!(element.getTagName().equals("ServerInfo")))
				throw new InvalidStructureException("this is not ServerInfo.");

			NodeList nl;
			String arg;
			arg = element.getAttribute("HostName");
			sli.setHostName(arg);

			arg = element.getAttribute("Port");
			try
			{
				sli.setPort(Integer.valueOf(arg).intValue());
			}catch(NumberFormatException e)
			{
			}

			return sli;
		}catch (NullPointerException e)
		{
			throw new InvalidStructureException();
		}
	}

	public static Document createDocument(SmallServerInfo sli)
	{
		Document document = createNewDocument("ServerInfo");
		Element root = document.getDocumentElement();
		root.setAttribute("HostName",sli.getHostName());
		root.setAttribute("Port",String.valueOf(sli.getPort()));

		return document;
	}
*/
}
