/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

package bakutan.client;

import bakutan.common.DataManager;
import bakutan.common.serverinfo.ServerInfoManagerAddress;
import bakutan.common.serverinfo.SmallServerInfo;
import bakutan.game.deathmatch.AIAlgorismicVer2;

/*
 * 作成日: 2004/10/11
 */

/**
 * @author cattaka
 */
public class ConfigurationInfo
{
	private SmallServerInfo serverLinkInfo;
    private ServerInfoManagerAddress serverInfoManagerAddress;
	private String defaultUserName = "Bakutan";
    private String defaultNickName = "Bakutan";
    private String otherAIClassName = AIAlgorismicVer2.class.getName();
	private boolean soundEffect = true;
	private boolean bgm = true;
	private boolean forceReconnect = false;
    private KeyConfig keyConfig;
    private int fpsLimit = 60;
    private int volumeBGM = 96;
    private int volumeSE = 96;
    
	public ConfigurationInfo()
	{
		serverLinkInfo = new SmallServerInfo();
		serverLinkInfo.setHostName(DataManager.DEFAULT_SERVER);
		serverLinkInfo.setPort(DataManager.DEFAULT_PORT);
		serverInfoManagerAddress = new ServerInfoManagerAddress(DataManager.DEFAULT_SERVER, DataManager.DEFAULT_SIMA_PORT);
        keyConfig = new KeyConfig();
        BakutanKeyConfig.makeDefault(keyConfig);
	}
	
	public SmallServerInfo getSmallServerInfo()
	{
		SmallServerInfo sli = new SmallServerInfo();
		sli.set(this.serverLinkInfo);
		return sli;
	}
	public void setServerLinkInfo(SmallServerInfo sli)
	{
		this.serverLinkInfo.set(sli);
	}
	
	public String getOtherAIClassName()
    {
        return otherAIClassName;
    }

    public void setOtherAIClassName(String otherAIClass)
    {
        this.otherAIClassName = otherAIClass;
    }

    public void set(ConfigurationInfo ci)
	{
        setBgm(ci.isBgm());
        setSoundEffect(ci.isSoundEffect());
        setDefaultNickName(ci.getDefaultNickName());
        setDefaultUserName(ci.getDefaultUserName());
        setOtherAIClassName(ci.getOtherAIClassName());
		setServerLinkInfo(ci.getSmallServerInfo());
        setVolumeSE(ci.getVolumeSE());
        setVolumeBGM(ci.getVolumeBGM());
        setServerInfoManagerAddress(ci.getServerInfoManagerAddress());
	}
	public String getDefaultUserName()
	{
		return defaultUserName;
	}
	public void setDefaultUserName(String defaultUserName)
	{
		if (defaultUserName == null)
			throw new NullPointerException();
		this.defaultUserName = defaultUserName;
	}
    
	public String getDefaultNickName()
    {
        return defaultNickName;
    }

    public void setDefaultNickName(String defaultNickName)
    {
        this.defaultNickName = defaultNickName;
    }

    public boolean isSoundEffect() {
		return soundEffect;
	}
	public void setSoundEffect(boolean sound) {
		this.soundEffect = sound;
	}
	public boolean isBgm()
	{
		return bgm;
	}
	public void setBgm(boolean bgm)
	{
		this.bgm = bgm;
	}
	public String getDynamicResourcePath()
	{
		return DataManager.DYNAMIC_RESOURCE_PATH;
	}
    
    public boolean isForceReconnect()
    {
        return forceReconnect;
    }

    public void setForceReconnect(boolean forceReconnect)
    {
        this.forceReconnect = forceReconnect;
    }

    public KeyConfig getKeyConfig()
    {
        return keyConfig;
    }
    public void setKeyConfig(KeyConfig keyConfig)
    {
        this.keyConfig.set(keyConfig);
    }

    public int getFpsLimit()
    {
        return fpsLimit;
    }

    public void setFpsLimit(int fpsLimit)
    {
        this.fpsLimit = fpsLimit;
    }

    public int getVolumeSE()
    {
        return volumeSE;
    }

    public void setVolumeSE(int volumeSE)
    {
        this.volumeSE = volumeSE;
        if (this.volumeSE > 127)
            this.volumeSE = 127;
        else if (this.volumeSE < 0)
            this.volumeSE = 0;
    }

    public int getVolumeBGM()
    {
        return volumeBGM;
    }

    public void setVolumeBGM(int volumeBGM)
    {
        this.volumeBGM = volumeBGM;
        if (this.volumeBGM > 127)
            this.volumeBGM = 0;
        else if (this.volumeBGM < 0)
            this.volumeBGM = 0;
    }

    public ServerInfoManagerAddress getServerInfoManagerAddress()
    {
        return serverInfoManagerAddress;
    }

    public void setServerInfoManagerAddress(
            ServerInfoManagerAddress serverInfoManagerAddress)
    {
        this.serverInfoManagerAddress = serverInfoManagerAddress;
    }
    
}
