/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

package bakutan.client;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import bakutan.common.BUserInterface;
import bakutan.common.serverinfo.SmallServerInfo;
import bakutan.common.util.datainputpanel.DIPInfo;
import bakutan.common.util.datainputpanel.DIPInfoBoolean;
import bakutan.common.util.datainputpanel.DIPInfoInteger;
import bakutan.common.util.datainputpanel.DIPInfoSlider;
import bakutan.common.util.datainputpanel.DIPInfoString;
import bakutan.common.util.datainputpanel.DataInputPanel;

public class ConfigurationPanel extends JPanel implements ActionListener
{
	public static final int CANCEL_OPTION = 0;
	public static final int APPROVE_OPTION = 1;
	public static final int ERROR_OPTION = 2;

	private DataInputPanel dataInputPane;
    private KeyConfigPanel keyConfigPanel;

    private DIPInfoString dipDefaultUserName;
    private DIPInfoString dipDefaultNickName;
    private DIPInfoInteger dipFPSLimit;
    private DIPInfoBoolean dipSound;
    private DIPInfoBoolean dipBGM;
    private DIPInfoBoolean dipForceReconnect;
    private DIPInfoSlider dipVolumeBGM;
    private DIPInfoSlider dipVolumeSE;
    
	private JButton okB;
	private JButton cancelB;

	private SmallServerInfo serverLinkInfo;	
	
	private ConfigurationInfo config;
	private BUserInterface userinterface;
	
	public ConfigurationPanel(BUserInterface userinterface)
	{
		this.userinterface = userinterface;
		setSize(400,500);
		serverLinkInfo = new SmallServerInfo();
		
		initialize();
	}

	private void initialize()
	{
		GridBagLayout gbl;
		GridBagConstraints gbc;

		okB = new JButton(userinterface.translate("ok"));
		cancelB = new JButton(userinterface.translate("cancel"));
		
		/* generic panel */
        
        JTabbedPane mainTab = new JTabbedPane();
        // generic panel
        dipDefaultUserName = new DIPInfoString(userinterface.translate("default_user_name"),"bakutan");
        dipDefaultNickName = new DIPInfoString(userinterface.translate("default_nick_name"),"Bakutan");
        dipFPSLimit = new DIPInfoInteger(userinterface.translate("fps_limit"),"60");
        dipBGM = new DIPInfoBoolean(userinterface.translate("bgm"), true);
        dipSound = new DIPInfoBoolean(userinterface.translate("sound"), true);
        dipForceReconnect = new DIPInfoBoolean(userinterface.translate("force_reconnect"), false);
        dipVolumeBGM = new DIPInfoSlider(userinterface.translate("bgm_volume"), 0, 127, 64);
        dipVolumeSE = new DIPInfoSlider(userinterface.translate("se_volume"), 0, 127, 64);

        DIPInfo[] dipis = new DIPInfo[]{
                dipDefaultUserName,
                dipDefaultNickName,
                dipForceReconnect,
                dipFPSLimit,
                dipBGM,
                dipSound,
                dipVolumeBGM,
                dipVolumeSE,
        };
        dataInputPane = new DataInputPanel(dipis);
        dataInputPane.setShowButtons(false);

        JPanel genericPane = new JPanel();
        gbl = new GridBagLayout();
        gbc = new GridBagConstraints();
        gbc.gridx=0; gbc.gridy=0;
        gbc.fill = GridBagConstraints.NONE;
        gbl.setConstraints(dataInputPane, gbc);
        genericPane.setLayout(gbl);
        genericPane.add(dataInputPane);
        
		mainTab.add(userinterface.translate("generic"), genericPane);
		
		gbl = new GridBagLayout();
		gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.CENTER;
		
		gbc.gridx=0; gbc.gridy=0;
		gbc.gridwidth = 2;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx=1; gbc.weighty=1;
		gbl.setConstraints(mainTab,gbc);

        gbc.gridy++;
		gbc.gridwidth = 1;
		gbc.fill = GridBagConstraints.NONE;
		gbc.weighty=0;
		gbl.setConstraints(okB,gbc);
		gbc.gridx++;
		gbl.setConstraints(cancelB,gbc);

		setLayout(gbl);
		add(mainTab);
		add(okB);
		add(cancelB);
        
		okB.addActionListener(this);
		cancelB.addActionListener(this);

        //keyConfig
        keyConfigPanel = new KeyConfigPanel(userinterface);
        mainTab.add(userinterface.translate("key_config"), keyConfigPanel);
    }

	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == okB)
		{
            closeOperation(true);
		}
		else if (e.getSource() == cancelB)
		{
            closeOperation(false);
		}
	}

	public void setConfigurationInfo(ConfigurationInfo ci)
	{
        dipDefaultUserName.setValue(ci.getDefaultUserName());
        dipDefaultNickName.setValue(ci.getDefaultNickName());
        dipFPSLimit.setValue(ci.getFpsLimit());
        dipForceReconnect.setValue(ci.isForceReconnect());
        dipSound.setValue(ci.isSoundEffect());
		dipBGM.setValue(ci.isBgm());
        dipVolumeSE.setValue(ci.getVolumeSE());
        dipVolumeBGM.setValue(ci.getVolumeBGM());
        keyConfigPanel.setKeyConfig(ci.getKeyConfig());
	}
	
	public ConfigurationInfo getConfigurationInfo()
	{
		ConfigurationInfo ci = new ConfigurationInfo();
		ci.setDefaultUserName((String)dipDefaultUserName.getValue());
        ci.setDefaultNickName((String)dipDefaultNickName.getValue());
        try
        {
            int f = (Integer)dipFPSLimit.getValue();
            if (f < 0)
                f = 0;
            ci.setFpsLimit(f);
        }
        catch(NumberFormatException e)
        {
            ci.setFpsLimit(60);
        }
		ci.setSoundEffect((Boolean)dipSound.getValue());
		ci.setBgm((Boolean)dipBGM.getValue());
		ci.setForceReconnect((Boolean)dipForceReconnect.getValue());
        ci.setVolumeSE((Integer)dipVolumeSE.getValue());
        ci.setVolumeBGM((Integer)dipVolumeBGM.getValue());
        ci.setKeyConfig(keyConfigPanel.getKeyConfig());
		return ci;
	}
    public void closeOperation(boolean approve)
    {
    }
}
