/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */
package bakutan.client;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import bakutan.common.DataManager;
import bakutan.common.serverinfo.SmallServerInfo;


public class InputServerLinkInfoDialog extends JDialog implements ActionListener
{
	public static final int CANCEL_OPTION = 0;
	public static final int APPROVE_OPTION = 1;
	public static final int ERROR_OPTION = 2;

	private JTextField labelf = new JTextField();
	private JTextField hostf = new JTextField();
	private JTextField portf = new JTextField();
	private JButton bok = new JButton("OK");
	private JButton bcancel = new JButton("Cancel");
	private boolean bookmark = false;
	private int flag = CANCEL_OPTION;

	public InputServerLinkInfoDialog()
	{
		setTitle("Input HostName and Port");
		setSize(200,150);
		initialize();
	}

	public InputServerLinkInfoDialog(boolean bookmark)
	{
		setTitle("Input HostName and Port");
		setSize(200,150);
		this.bookmark = bookmark;
		initialize();
	}

	public void initialize()
	{
		JLabel labell = new JLabel("Label");
		JLabel hostl = new JLabel("Host");
		JLabel portl = new JLabel("Port");
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weighty=0;

		Dimension d;
		d = labelf.getPreferredSize(); d.width = 0;
		labelf.setPreferredSize(d);
		d = hostf.getPreferredSize(); d.width = 0;
		hostf.setPreferredSize(d);
		d = portf.getPreferredSize(); d.width = 0;
		portf.setPreferredSize(d);

		gbc.gridx=0; gbc.gridy=0;
		if (bookmark)
		{
			gbl.setConstraints(labell,gbc);
			gbc.gridy++;
		}
		gbl.setConstraints(hostl,gbc);
		gbc.gridy++;
		gbl.setConstraints(portl,gbc);
		gbc.gridy++;


		gbc.weightx=1;
		gbc.gridx=1; gbc.gridy=0;
		if (bookmark)
		{
			gbl.setConstraints(labelf,gbc);
			gbc.gridy++;
		}
		gbl.setConstraints(hostf,gbc);
		gbc.gridy++;
		gbl.setConstraints(portf,gbc);
		gbc.gridy++;

		gbc.weightx=0;
		gbc.fill = GridBagConstraints.NONE;
		gbc.gridx=0; gbc.gridy++;
		gbl.setConstraints(bok,gbc);
		gbc.gridx++;
		gbl.setConstraints(bcancel,gbc);

		getContentPane().setLayout(gbl);
		if (bookmark)
		{
			getContentPane().add(labell);
			getContentPane().add(labelf);
		}
		getContentPane().add(hostl);
		getContentPane().add(hostf);
		getContentPane().add(portl);
		getContentPane().add(portf);
		getContentPane().add(bok);
		getContentPane().add(bcancel);
		bok.addActionListener(this);
		bcancel.addActionListener(this);
	}

	public void setLabel(String arg)
	{
		labelf.setText(arg);
	}
	public void setHostName(String arg)
	{
		hostf.setText(arg);
	}
	public void setPort(int arg)
	{
		portf.setText(String.valueOf(arg));
	}

	public String getLabel()
	{
		return labelf.getText();
	}
	public String getHostName()
	{
		return hostf.getText();
	}
	public int getPort()
	{
		try
		{
			return Integer.valueOf(portf.getText()).intValue();
		}catch(Exception e)
		{
			return DataManager.DEFAULT_PORT;
		}
	}
	public int getFlag()
	{
		return flag;
	}

	public void setServerLinkInfo(SmallServerInfo sli)
	{
		setHostName(sli.getHostName());
		setPort(sli.getPort());
	}

	public SmallServerInfo getSmallServerInfo()
	{
		SmallServerInfo sli = new SmallServerInfo();
		sli.setHostName(getHostName());
		sli.setPort(getPort());
		return sli;
	}

	public int showInputDialog()
	{
		boolean b = isModal();
		setModal(true);
        setVisible(true);
		setModal(b);
		return flag;
	}

	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == bok)
		{
			flag = APPROVE_OPTION;
			setVisible(false);
		}else if (e.getSource() == bcancel)
		{
			flag = CANCEL_OPTION;
			setVisible(false);
		}

	}
}
