/*
 * 作成日: 2007/01/13
 *
 */
package bakutan.client;

import java.awt.event.KeyEvent;
import java.util.TreeMap;

public class KeyConfig
{
    public static final int ACTION_UP = 0;
    public static final int ACTION_DOWN = 1;
    public static final int ACTION_LEFT = 2;
    public static final int ACTION_RIGHT = 3;
    public static final int ACTION_WEAPON_NEXT = 10;
    public static final int ACTION_WEAPON_PREV = 11;
    public static final int ACTION_ATTACK = 50;
    public static final int ACTION_ACTION_1 = 101;
    public static final int ACTION_ACTION_2 = 102;
    public static final int ACTION_ACTION_3 = 103;
    public static final int ACTION_ACTION_4 = 104;
    public static final int ACTION_ACTION_5 = 105;
    public static final int ACTION_ACTION_6 = 106;
    public static final int ACTION_ACTION_7 = 107;
    public static final int ACTION_ACTION_8 = 108;
    public static final int ACTION_ACTION_9 = 109;
    public static final int ACTION_ACTION_10 = 110;
    public static final int ACTION_SELECT_ACTION_1 = 201;
    public static final int ACTION_SELECT_ACTION_2 = 202;
    public static final int ACTION_SELECT_ACTION_3 = 203;
    public static final int ACTION_SELECT_ACTION_4 = 204;
    public static final int ACTION_SELECT_ACTION_5 = 205;
    public static final int ACTION_SELECT_ACTION_6 = 206;
    public static final int ACTION_SELECT_ACTION_7 = 207;
    public static final int ACTION_SELECT_ACTION_8 = 208;
    public static final int ACTION_SELECT_ACTION_9 = 209;
    public static final int ACTION_SELECT_ACTION_10 = 210;
    public static final int ACTION_MOUSE_MOVE = 301;

    public static final int DEFAULT_MODE_BAKUTAN = 0;
    public static final int DEFAULT_MODE_BAKUSEN = 1;
    
    private TreeMap<Integer, Integer> keyToAction;
    private TreeMap<Integer, Integer> actionToKey;
    private int[] mouseAction = new int[]{-1, -1, -1, -1, -1};
    private int mode = DEFAULT_MODE_BAKUTAN;
    
    public KeyConfig()
    {
        super();
        keyToAction = new TreeMap<Integer, Integer>();
        actionToKey = new TreeMap<Integer, Integer>();
    }
    public KeyConfig(KeyConfig src)
    {
        set(src);
    }
    public void set(KeyConfig src)
    {
        keyToAction.clear();
        actionToKey.clear();
        keyToAction.putAll(src.keyToAction);
        actionToKey.putAll(src.actionToKey);
        this.mouseAction = new int[src.mouseAction.length];
        System.arraycopy(src.mouseAction, 0, this.mouseAction, 0, this.mouseAction.length);
    }
    public void putKeyToAction(int keyCode, int action)
    {
        if (keyCode == -1 || action == -1 || !isAvailableKeyCode(keyCode))
            return;
        
        if (actionToKey.values().contains(keyCode))
        {
            int e = keyToAction.get(keyCode);
            keyToAction.remove(keyCode);
            actionToKey.remove(e);
        }
        keyToAction.put(keyCode, action);
        actionToKey.put(action,keyCode);
    }
    public void removeAction(int action)
    {
        if (keyToAction.values().contains(action))
        {
            int key = actionToKey.get(action);
            actionToKey.remove(action);
            keyToAction.remove(key);
        }
    }
    public int getAction(int keyCode)
    {
        Integer action = keyToAction.get(keyCode);
        if (action == null)
            return -1;
        else
            return action.intValue();
    }
    public int getKeyCode(int action)
    {
        Integer keyCode = actionToKey.get(action);
        if (keyCode == null)
            return -1;
        else
            return keyCode.intValue();
    }
    public void clear()
    {
        keyToAction.clear();
        actionToKey.clear();
        for (int i=0;i<mouseAction.length;i++)
        {
            mouseAction[i] = -1;
        }
    }
    
    public static int[] getActions()
    {
        int[] events = new int[26];
        int i=0;
        events[i++] = ACTION_UP;
        events[i++] = ACTION_DOWN;
        events[i++] = ACTION_LEFT;
        events[i++] = ACTION_RIGHT;
        events[i++] = ACTION_WEAPON_NEXT;
        events[i++] = ACTION_WEAPON_PREV;
        events[i++] = ACTION_ATTACK;
        events[i++] = ACTION_SELECT_ACTION_1;
        events[i++] = ACTION_SELECT_ACTION_2;
        events[i++] = ACTION_SELECT_ACTION_3;
        events[i++] = ACTION_SELECT_ACTION_4;
        events[i++] = ACTION_SELECT_ACTION_5;
        events[i++] = ACTION_SELECT_ACTION_6;
        events[i++] = ACTION_SELECT_ACTION_7;
        events[i++] = ACTION_SELECT_ACTION_8;
        events[i++] = ACTION_SELECT_ACTION_9;
        events[i++] = ACTION_ACTION_1;
        events[i++] = ACTION_ACTION_2;
        events[i++] = ACTION_ACTION_3;
        events[i++] = ACTION_ACTION_4;
        events[i++] = ACTION_ACTION_5;
        events[i++] = ACTION_ACTION_6;
        events[i++] = ACTION_ACTION_7;
        events[i++] = ACTION_ACTION_8;
        events[i++] = ACTION_ACTION_9;
        events[i++] = ACTION_MOUSE_MOVE;
        return events;
    }
    public static String getActionText(int event)
    {
        if (event == ACTION_UP)
            return "up";
        else if (event == ACTION_DOWN)
            return "down";
        else if (event == ACTION_LEFT)
            return "left";
        else if (event == ACTION_RIGHT)
            return "right";
        else if (event == ACTION_WEAPON_NEXT)
            return "weapon_next";
        else if (event == ACTION_WEAPON_PREV)
            return "weapon_prev";
        else if (event == ACTION_ATTACK)
            return "attack";
        else if (event == ACTION_ACTION_1)
            return "action_1";
        else if (event == ACTION_ACTION_2)
            return "action_2";
        else if (event == ACTION_ACTION_3)
            return "action_3";
        else if (event == ACTION_ACTION_4)
            return "action_4";
        else if (event == ACTION_ACTION_5)
            return "action_5";
        else if (event == ACTION_ACTION_6)
            return "action_6";
        else if (event == ACTION_ACTION_7)
            return "action_7";
        else if (event == ACTION_ACTION_8)
            return "action_8";
        else if (event == ACTION_ACTION_9)
            return "action_9";
        else if (event == ACTION_ACTION_10)
            return "action_10";
        else if (event == ACTION_SELECT_ACTION_1)
            return "select_action_1";
        else if (event == ACTION_SELECT_ACTION_2)
            return "select_action_2";
        else if (event == ACTION_SELECT_ACTION_3)
            return "select_action_3";
        else if (event == ACTION_SELECT_ACTION_4)
            return "select_action_4";
        else if (event == ACTION_SELECT_ACTION_5)
            return "select_action_5";
        else if (event == ACTION_SELECT_ACTION_6)
            return "select_action_6";
        else if (event == ACTION_SELECT_ACTION_7)
            return "select_action_7";
        else if (event == ACTION_SELECT_ACTION_8)
            return "select_action_8";
        else if (event == ACTION_SELECT_ACTION_9)
            return "select_action_9";
        else if (event == ACTION_SELECT_ACTION_10)
            return "select_action_10";
        else if (event == ACTION_MOUSE_MOVE)
            return "move_by_mouse";
        else
            return null;
    }
    public static int getActionCode(String event)
    {
        if (event.equals("up"))
            return ACTION_UP;
        else if (event.equals("down"))
            return  ACTION_DOWN;
        else if (event.equals("left"))
            return  ACTION_LEFT;
        else if (event.equals("right"))
            return  ACTION_RIGHT;
        else if (event.equals("weapon_next"))
            return  ACTION_WEAPON_NEXT;
        else if (event.equals("weapon_prev"))
            return  ACTION_WEAPON_PREV;
        else if (event.equals("attack"))
            return  ACTION_ATTACK;
        else if (event.equals("action_1"))
            return  ACTION_ACTION_1;
        else if (event.equals("action_2"))
            return  ACTION_ACTION_2;
        else if (event.equals("action_3"))
            return  ACTION_ACTION_3;
        else if (event.equals("action_4"))
            return  ACTION_ACTION_4;
        else if (event.equals("action_5"))
            return  ACTION_ACTION_5;
        else if (event.equals("action_6"))
            return  ACTION_ACTION_6;
        else if (event.equals("action_7"))
            return  ACTION_ACTION_7;
        else if (event.equals("action_8"))
            return  ACTION_ACTION_8;
        else if (event.equals("action_9"))
            return  ACTION_ACTION_9;
        else if (event == "action_10")
            return ACTION_ACTION_10;
        else if (event.equals("select_action_1"))
            return  ACTION_SELECT_ACTION_1;
        else if (event.equals("select_action_2"))
            return  ACTION_SELECT_ACTION_2;
        else if (event.equals("select_action_3"))
            return  ACTION_SELECT_ACTION_3;
        else if (event.equals("select_action_4"))
            return  ACTION_SELECT_ACTION_4;
        else if (event.equals("select_action_5"))
            return  ACTION_SELECT_ACTION_5;
        else if (event.equals("select_action_6"))
            return  ACTION_SELECT_ACTION_6;
        else if (event.equals("select_action_7"))
            return  ACTION_SELECT_ACTION_7;
        else if (event.equals("select_action_8"))
            return  ACTION_SELECT_ACTION_8;
        else if (event.equals("select_action_9"))
            return  ACTION_SELECT_ACTION_9;
        else if (event.equals("select_action_10"))
            return ACTION_SELECT_ACTION_10;
        else if (event.equals("move_by_mouse"))
            return ACTION_MOUSE_MOVE;
        else
            return -1;
    }
    public static boolean isAvailableKeyCode(int keyCode)
    {
        switch (keyCode)
        {
            case KeyEvent.VK_F1 :
            case KeyEvent.VK_F2 :
            case KeyEvent.VK_F3 :
            case KeyEvent.VK_F4 :
            case KeyEvent.VK_F5 :
            case KeyEvent.VK_F6 :
            case KeyEvent.VK_F7 :
            case KeyEvent.VK_F8 :
            case KeyEvent.VK_F9 :
            case KeyEvent.VK_F10 :
            case KeyEvent.VK_F11 :
            case KeyEvent.VK_F12 :
            case KeyEvent.VK_ENTER :
                return false;
            default :
                return true;
        }
        
    }
    public int getMouseActionNum()
    {
        return mouseAction.length;
    }
    public int getMouseAction(int index)
    {
        return mouseAction[index];
    }
    public void setMouseAction(int index, int mouseAction)
    {
        if (index < 0 || this.mouseAction.length <= index)
            return;
        this.mouseAction[index] = mouseAction;
    }
    
}
