/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/19
 */
package bakutan.client;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Locale;

import javax.swing.JApplet;

import org.xml.sax.SAXParseException;

import bakutan.common.BUserInterface;
import bakutan.common.DataManager;
import bakutan.common.InvalidStructureException;
import bakutan.common.XMLConverter;
import bakutan.common.serverinfo.ServerInfo;
import bakutan.common.util.BAudio;
import bakutan.common.util.LanguageTranslator;
import bakutan.common.util.NullBAudio;

/**
 * @author cattaka
 */
public class ServerInfoApplet extends JApplet implements BUserInterface
{
	private ServerInfoPanel serverInfoPanel;
	private LanguageTranslator languageTranslator;
	private BAudio baudio = new NullBAudio();
    
	public ServerInfoApplet()
	{
		this.languageTranslator = getLanguageTranslator();
		serverInfoPanel = new ServerInfoPanel(this, true);
		getContentPane().add(serverInfoPanel);
	}
	public void init()
	{
		super.init();
		ServerInfoTableModel sil = new ServerInfoTableModel();
		String tmp = getParameter("servers");
		if (tmp == null)
		{
			serverInfoPanel.setServerInfoList(sil);
			return;
		}
		String[] svs = tmp.split("[,]");
		for (int i=0;i<svs.length;i++)
		{
			String[] t = svs[i].split(":",3);
			if (t.length < 2)
				continue;
			ServerInfo sie = new ServerInfo();
			sie.setLabel(t[0]);
			sie.setHostName(t[1]);
			int port = DataManager.DEFAULT_PORT;
			if (t.length >= 3)
			{
				try
				{
					port = Integer.parseInt(t[2]);
				}catch(NumberFormatException e)
				{
					continue;
				}
			}
			sie.setPort(port);
			sil.add(sie);
		}
		serverInfoPanel.setServerInfoList(sil);
		serverInfoPanel.getAllServerState();
		serverInfoPanel.select(0);
	}
	
	public void outputMessage(String message)
	{
		System.out.println(message);
	}
	public String translate(String key, String[] opt)
	{
		return languageTranslator.translate(key, opt);
	}
	public String translate(String key)
	{
		return languageTranslator.translate(key);
	}
	public static LanguageTranslator getLanguageTranslator()
	{
		URL in = LanguageTranslator.class.getClassLoader().getResource("PhraseDic_"+ Locale.getDefault().getLanguage()+".xml");
		if (in == null)
			in = LanguageTranslator.class.getClassLoader().getResource("PhraseDic.xml");
		if (in == null)
			return new LanguageTranslator();
		try
		{
			return ClientXMLConverter.createLanguageTranslator(XMLConverter.readXML(in));
		}catch(InvalidStructureException e)
		{
			e.printStackTrace();
			return new LanguageTranslator();
		}catch(SAXParseException e)
		{
			e.printStackTrace();
			return new LanguageTranslator();
		}catch(IOException e)
		{
			e.printStackTrace();
			return new LanguageTranslator();
		}
	}
    public BAudio getBAudio()
    {
        return baudio;
    }
    public File getWorkDir()
    {
        return null;
    }
}
