/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

package bakutan.client;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import bakutan.common.BUserInterface;
import bakutan.common.BodyState;
import bakutan.common.DataManager;
import bakutan.common.VersionBundle;
import bakutan.common.serverinfo.BStageState;
import bakutan.common.serverinfo.ServerInfo;

/*
 * 作成日: 2004/10/08
 */

/**
 * @author cattaka
 */
public class ServerInfoPanel extends JPanel implements ActionListener
{
	private JTable serverInfoTable;
	private JScrollPane serverInfoPane;
	private JTable playersTable;
	private JScrollPane playerPane;
	private JTextArea messageArea;
	private JScrollPane messagePane;

	private ServerInfoTableModel bookmarkList;
	private BUserInterface userinterface;
	private BodyStateModel bodyStateModel;
	
	private boolean appletMode;
    private JButton addB;
    private JButton delB;
    private JButton propertyB;
    private JButton connectB;
    private JButton closeB;
    private JButton upB;
    private JButton downB;
    private JButton updateStateB;
    private JButton downloadServerInfoB;
	private boolean editableList;
    
    private ServerInfo selectedServerInfo = null;
    
	public ServerInfoPanel(BUserInterface userinterface)
	{
        this(userinterface, false);
	}
	public ServerInfoPanel(BUserInterface userinterface, boolean appletMode)
	{
		this.userinterface = userinterface;
		setSize(400,400);
		this.appletMode = appletMode;
		makeLayout();
		if (appletMode)
		{
			setEditableList(!appletMode);
		}
	}

	private void makeLayout()
	{
		//bookmarkJList = new JList();
		serverInfoTable = new JTable()
		{
			public void valueChanged(ListSelectionEvent e)
			{
				super.valueChanged(e);
				if(ServerInfoPanel.this != null)
					updatePlayersTable(getSelectedRow());
			}
		};
		serverInfoPane = new JScrollPane(serverInfoTable);
		bodyStateModel = new BodyStateModel();
		playersTable = new JTable(bodyStateModel);
		playerPane = new JScrollPane(playersTable);
		messageArea = new JTextArea();
		messagePane = new JScrollPane(messageArea, JScrollPane.VERTICAL_SCROLLBAR_NEVER, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		messageArea.setLineWrap(true);
		messageArea.setEditable(false);
		
		addB = new JButton(userinterface.translate("add"));
		delB = new JButton(userinterface.translate("delete"));
		upB = new JButton(userinterface.translate("up"));
		downB = new JButton(userinterface.translate("down"));
		updateStateB = new JButton(userinterface.translate("update_server_state"));
        downloadServerInfoB = new JButton(userinterface.translate("download_server_list"));
		propertyB = new JButton(userinterface.translate("edit"));
		connectB = new JButton(userinterface.translate("connect"));
		closeB = new JButton(userinterface.translate("close"));
		addB.setActionCommand("add");
        delB.setActionCommand("del");
        upB.setActionCommand("up");
        downB.setActionCommand("down");
        updateStateB.setActionCommand("update_state");
        propertyB.setActionCommand("property");
        connectB.setActionCommand("connect");
        closeB.setActionCommand("close");
        downloadServerInfoB.setActionCommand("download_server_list");
        
		JLabel serverL = new JLabel(userinterface.translate("server_list"));
		JLabel infoL = new JLabel(userinterface.translate("player_list"));
		
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gc = new GridBagConstraints();

        gc.insets = new Insets(5,5,5,5);
        
		gc.anchor = GridBagConstraints.CENTER;
		gc.fill = GridBagConstraints.BOTH;

		gc.gridwidth = 3;
        gc.gridheight = 1;

		gc.gridx = 0;
        gc.gridy = 0;
		gc.weightx = 0;
        gc.weighty = 0;
		gbl.setConstraints(serverL, gc);
		
		gc.gridy++;
		gc.weightx = 1;
        gc.weighty = 1;
		serverInfoPane.setPreferredSize(new Dimension(0,0));
		gbl.setConstraints(serverInfoPane,gc);

		gc.gridy++;
		gc.weightx = 0;
        gc.weighty = 0;
		gbl.setConstraints(infoL, gc);

		gc.gridy++;
		gc.weightx = 1;
        gc.weighty = 0.5;
		playerPane.setPreferredSize(new Dimension(0,0));
		gbl.setConstraints(playerPane,gc);

        gc.gridy++;
		gc.weighty = 0.3;
		messagePane.setPreferredSize(new Dimension(0,0));
		gbl.setConstraints(messagePane,gc);

		int y = gc.gridy + 1;
		gc.fill = GridBagConstraints.NONE;
		gc.weightx = 1; 
        gc.weighty = 0;
		gc.gridwidth = 1; gc.gridheight = 1;
		gc.gridx = 0; gc.gridy = y;
		gbl.setConstraints(addB,gc);
		gc.gridy++;
		gbl.setConstraints(delB,gc);
        gc.gridy++;
        gbl.setConstraints(propertyB,gc);
        gc.gridy++;
        gbl.setConstraints(connectB,gc);
		
		gc.gridx = 1; gc.gridy = y;
		gbl.setConstraints(upB,gc);
		gc.gridy++;
		gbl.setConstraints(downB,gc);
		
		gc.gridx = 2; gc.gridy = y;
        gbl.setConstraints(downloadServerInfoB,gc);
        gc.gridy++;
        gbl.setConstraints(updateStateB,gc);
		gc.gridy++;
        gc.gridy++;
        gbl.setConstraints(closeB,gc);

		setLayout(gbl);
		add(serverL);
		add(infoL);
		add(serverInfoPane);
		add(playerPane);
		add(messagePane);
		add(addB);
		add(delB);
		add(updateStateB);
		add(upB);
		add(downB);
        add(downloadServerInfoB);
		if (!appletMode)
		{
			add(propertyB);
			add(closeB);
            add(connectB);
			propertyB.addActionListener(this);
			closeB.addActionListener(this);
            connectB.addActionListener(this);
		}
		addB.addActionListener(this);
		delB.addActionListener(this);
		updateStateB.addActionListener(this);
		upB.addActionListener(this);
		downB.addActionListener(this);
        downloadServerInfoB.addActionListener(this);
	}
	
	public void updateServerInfoTable()
	{
		serverInfoTable.updateUI();
		int i = serverInfoTable.getSelectedRow();
		updatePlayersTable(i);
	}
	public void updatePlayersTable(int i)
	{
		BStageState bss = bookmarkList.getBStageState(i);
		messageArea.setText("");
		if (bss != null)
		{
			bodyStateModel.setBodyStates(bss.getBodyStates());
			messageArea.append(userinterface.translate("login_message", new String[]{bss.getLoginMessage()}) + "\n");
			messageArea.append(userinterface.translate("note_message", new String[]{bss.getNoteMessage()}) + "\n");
		}else
		{
			bodyStateModel.setBodyStates(new BodyState[0]);
		}
		playersTable.updateUI();
	}
	public void getAllServerState()
	{
		int i = serverInfoTable.getSelectedRow();
		for (int j=0;j<bookmarkList.size();j++)
		{
			getServerState(j);
		}
	}
	public void select(int i)
	{
		if (i<0 || bookmarkList.size() <= i)
			return;
		serverInfoTable.changeSelection(i,0,false,false);
	}
	public void getServerState(int i)
	{
		ServerInfo sie = bookmarkList.getServerInfo(i);
		if (sie == null)
			return;
		BStageState bss = null;
		try
		{
			bss = DataManager.getBStageState(sie.getHostName(), sie.getPort(), 1000);
			sie.setBStageState(bss);
            //bookmarkList.setBStageState(sie, bss);
			bodyStateModel.setBodyStates(bss.getBodyStates());
		}catch(IOException e)
		{
			e.printStackTrace();
			sie.setBStageState(null);
            //bookmarkList.removeBStageState(sie);
		}
	}
	public void setServerInfoList(ServerInfoTableModel bl)
	{
		this.bookmarkList = bl;
		serverInfoTable.setModel(bookmarkList);
		for (int i = 0; i < 4; i++)
		{
		    TableColumn column = serverInfoTable.getColumnModel().getColumn(i);
		    switch(i)
			{
		    case 0:
		        column.setPreferredWidth(150);
		        break;
		    case 1:
		        column.setPreferredWidth(150);
		        break;
		    case 2:
		        column.setPreferredWidth(80);
		        break;
		    case 3:
		        column.setPreferredWidth(80);
		        break;
		    }
		}
		updateServerInfoTable();
	}
	/* for Interface Method */
	public void actionPerformed(ActionEvent ae)
	{
		if (ae.getActionCommand().equals("add"))
		{
			InputServerLinkInfoDialog ihpd = new InputServerLinkInfoDialog(true);
			if (ihpd.showInputDialog() == InputServerLinkInfoDialog.APPROVE_OPTION)
			{
				ServerInfo be = new ServerInfo();
				be.setLabel(ihpd.getLabel());
				be.setHostName(ihpd.getHostName());
				be.setPort(ihpd.getPort());
				bookmarkList.add(be);
				updateServerInfoTable();
			}
		}else if (ae.getActionCommand().equals("del"))
		{
			int i = serverInfoTable.getSelectedRow();
			ServerInfo sel = bookmarkList.getServerInfo(i);
			if (sel != null)
			{
				bookmarkList.remove(sel);
				updateServerInfoTable();
			}
		}else if (ae.getActionCommand().equals("up"))
		{
			int i = serverInfoTable.getSelectedRow();
			ServerInfo sel = bookmarkList.getServerInfo(i);
			if (sel != null)
			{
				//int i = bookmarkJList.getSelectedIndex();
				if (i>0)
				{
					bookmarkList.remove(sel);
					bookmarkList.add(i-1, sel);
					updateServerInfoTable();
					//bookmarkJList.setSelectedValue(sel,true);
					serverInfoTable.changeSelection(i-1,0,false,false);
				}
			}
		}else if (ae.getActionCommand().equals("down"))
		{
			int i = serverInfoTable.getSelectedRow();
			ServerInfo sel = bookmarkList.getServerInfo(i);
			if (sel != null)
			{
				//int i = bookmarkJList.getSelectedIndex();
				if (i<bookmarkList.size()-1)
				{
					bookmarkList.remove(sel);
					bookmarkList.add(i+1, sel);
					updateServerInfoTable();
					//bookmarkJList.setSelectedValue(sel,true);
					serverInfoTable.changeSelection(i+1,0,false,false);
				}
			}
		}else if (ae.getActionCommand().equals("property"))
		{
			int i = serverInfoTable.getSelectedRow();
			ServerInfo sel = bookmarkList.getServerInfo(i);
			if (sel != null)
			{
				ServerInfo be = (ServerInfo)sel;
				InputServerLinkInfoDialog ihpd = new InputServerLinkInfoDialog(true);
				ihpd.setLabel(be.getLabel());
				ihpd.setServerLinkInfo(be);
				if(ihpd.showInputDialog() == InputServerLinkInfoDialog.APPROVE_OPTION)
				{
					be.setLabel(ihpd.getLabel());
					be.setHostName(ihpd.getHostName());
					be.setPort(ihpd.getPort());
					updateServerInfoTable();
				}
			}
		}else if (ae.getActionCommand().equals("update_state"))
		{
			getAllServerState();
			updateServerInfoTable();
		}else if (ae.getActionCommand().equals("close"))
		{
            selectedServerInfo = null;
            closeOperation(false);
        }else if (ae.getActionCommand().equals("download_server_list"))
        {
            downloadServerList();
        }else if (ae.getActionCommand().equals("connect"))
        {
            int i = serverInfoTable.getSelectedRow();
            ServerInfo sel = bookmarkList.getServerInfo(i);
            if (sel != null)
            {
                selectedServerInfo = sel;
                closeOperation(true);
            }
        }
	}
    
	public ServerInfo getSelectedServerInfo()
    {
        return selectedServerInfo;
    }
    public static String getProtocolVersion()
	{
		return VersionBundle.getString("bakusen.protocolVersion");
	}
	public static String getBuildNumber()
	{
		return VersionBundle.getString("build.number");
	}

	public void setEditableList(boolean flag)
	{
        this.editableList = flag;
		addB.setVisible(flag);
		delB.setVisible(flag);
		upB.setVisible(flag);
		downB.setVisible(flag);
		propertyB.setVisible(flag);
        downloadServerInfoB.setVisible(!flag);
	}
    
    public boolean isEditableList()
    {
        return editableList;
    }
    public void closeOperation(boolean approve)
    {
    }
    public void downloadServerList()
    {
    }
}
class BodyStateModel implements TableModel
{
	private BodyState[] bodyStates = new BodyState[0];
	public BodyStateModel()
	{
		
	}
	public BodyState[] getBodyStates()
	{
		return bodyStates;
	}
	public void setBodyStates(BodyState[] bodyStates)
	{
		if (bodyStates == null)
			throw new NullPointerException();
		this.bodyStates = bodyStates;
	}
	
	public void addTableModelListener(TableModelListener l)
	{
	}
	public Class<?> getColumnClass(int columnIndex)
	{
		switch(columnIndex)
		{
		case 0:
			return String.class;
		case 1:
			return Integer.class;
		}
		return null;
	}
	public int getColumnCount()
	{
		return 2;
	}
	public String getColumnName(int columnIndex)
	{
		switch(columnIndex)
		{
		case 0:
			return "Name";
		case 1:
			return "Score";
		}
		return null;
	}
	public int getRowCount()
	{
		return bodyStates.length;
	}
	public Object getValueAt(int rowIndex, int columnIndex)
	{
		if (rowIndex < 0 || bodyStates.length <= rowIndex)
			return null;
		BodyState bs = bodyStates[rowIndex];
		switch(columnIndex)
		{
		case 0:
			return bs.getName();
		case 1:
			return new Integer(bs.getScore());
		}
		return null;
	}
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		return false;
	}
	public void removeTableModelListener(TableModelListener l)
	{

	}
	public void setValueAt(Object aValue, int rowIndex, int columnIndex)
	{
	}
}

