/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

package bakutan.client;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;

import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import bakutan.common.DataManager;
import bakutan.common.serverinfo.BStageState;
import bakutan.common.serverinfo.ServerInfo;
import bakutan.common.serverinfo.SmallBStageState;


/*
 * 作成日: 2004/10/07
 */

/**
 * @author cattaka
 */
public class ServerInfoTableModel implements TableModel
{
	private ArrayList<ServerInfo> list = new ArrayList<ServerInfo>();
	private HashSet<TableModelListener> tableModelListeners = new HashSet<TableModelListener>();
    
    public void setServerInfos(Collection<ServerInfo> sis)
    {
        list.clear();
        list.addAll(sis);
    }
    public Collection<ServerInfo> getServerInfos()
    {
        return new ArrayList<ServerInfo>(list);
    }
    
	public void add(ServerInfo sie)
	{
		list.add(sie);
	}
	public void add(int index, ServerInfo sie)
	{
		list.add(index, sie);
	}
	public void remove(ServerInfo sie)
	{
		list.remove(sie);
	}
	public ServerInfo getServerInfo(int i)
	{
		if (i<0 || list.size() <= i)
			return null;
		return (ServerInfo)list.get(i);
	}
	public ServerInfo[] toArray()
	{
		ServerInfo[] sies = new ServerInfo[list.size()];
		System.arraycopy(list.toArray(),0, sies,0, sies.length);
		return sies;
	}
	public int size()
	{
		return list.size();
	}
	public BStageState getBStageState(int i)
	{
		ServerInfo si = getServerInfo(i);
		if (si != null)
		{
			return si.getBStageState();
		}else
		{
			return null;
		}
	}
	public void addTableModelListener(TableModelListener l)
	{
		tableModelListeners.add(l);
	}
	public Class<String> getColumnClass(int columnIndex)
	{
		switch(columnIndex)
		{
		case 0:
			return String.class;
		case 1:
			return String.class;
		case 2:
			return String.class;
		case 3:
			return String.class;
        case 4:
            return String.class;
		}
		return null;
	}
	
	public int getColumnCount()
	{
		return 5;
	}
	public String getColumnName(int columnIndex)
	{
		switch(columnIndex)
		{
		case 0:
			return "Label";
		case 1:
			return "Server address";
		case 2:
			return "Protocol";
		case 3:
			return "Players";
        case 4:
            return "Rag";
		}
		return null;
	}
	public int getRowCount()
	{
		return list.size();
	}
	public Object getValueAt(int rowIndex, int columnIndex)
	{
		ServerInfo sie = getServerInfo(rowIndex);
		if (sie == null)
		{
			return null;
		}
		switch(columnIndex)
		{
    		case 0:
    			return sie.getLabel();
    		case 1:
    			return sie.getHostName() + ":" + sie.getPort();
    		case 2:
    		{
    			SmallBStageState bss = sie.getSmallBStageState();
    			if (bss == null)
    				return "---";
    			return bss.getProtocol();
    		}
    		case 3:
    		{
    			BStageState bss = sie.getBStageState();
    			
    			if (bss == null || !DataManager.getCurrentProtocolVersion().equals(bss.getProtocol()))
    				return "---";
    			return bss.getBodyStates().length + "/" + bss.getMaxClients();
    		}
            case 4:
            {
                BStageState bss = sie.getBStageState();
                if (bss == null || bss.getRag() == -1)
                    return "---";
                else
                    return String.valueOf(bss.getRag());
            }
		}
		
		return null;
	}
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		return false;
	}
	public void removeTableModelListener(TableModelListener l)
	{
		tableModelListeners.remove(l);
	}
	public void setValueAt(Object aValue, int rowIndex, int columnIndex)
	{
	}
}
