/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/01/26
 */
package bakutan.common;

import java.util.Comparator;
import java.util.Iterator;
import java.util.TreeSet;

/**
 * @author cattaka
 */
public class BPickResult
{
	private TreeSet<BPickIntersection> pickIntersections;
	private float offset = 0;
    public BPickResult()
	{
        pickIntersections = new TreeSet<BPickIntersection>(new BPickIntersectionComparator());
	}
	public void addBPickIntersection(BPickIntersection pi)
	{
		pickIntersections.add(pi);
	}
	public Iterator getBPickIntersections()
	{
		return pickIntersections.iterator();
	}
    public BPickIntersection getFirst()
    {
        BPickIntersection bpi = pickIntersections.first();
        return bpi;
    }
    public BPickIntersection removeFirst()
    {
        BPickIntersection bpi = pickIntersections.first();
        pickIntersections.remove(bpi);
        offset = bpi.getDistance();
        return bpi;
    }
	public int getSize()
	{
		return pickIntersections.size();
	}
    public void dispose()
    {
        pickIntersections.clear();
    }
    public float getOffset()
    {
        return offset;
    }
//    public void printAll()
//    {
//        Iterator<BPickIntersection> ite = pickIntersections.iterator();
//        while(ite.hasNext())
//        {
//            BPickIntersection pi = ite.next();
//            System.out.println(""+pi.getBody()+pi.getDistance());
//        }
//    }
}

class BPickIntersectionComparator implements Comparator<BPickIntersection>
{
    public int compare(BPickIntersection o1, BPickIntersection o2)
    {
        float d1 = o1.getDistance();
        float d2 = o2.getDistance();
        if (d1 < d2)
            return -1;
        if (d1 > d2)
            return 1;
        else
            return o1.hashCode() - o2.hashCode();
    }
}