/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/01/18
 *
 */
package bakutan.common;
 
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import bakutan.common.ai.AIInterface;
import bakutan.common.graphics.CharacterImageBundle;
import bakutan.common.graphics.EffectBundle;
import bakutan.common.math.BShape;
import bakutan.common.math.BMath;
import bakutan.common.math.BVector2f;
import bakutan.common.util.SEBundle;
import bakutan.common.util.BGMBundle;
 
/**
 * @author cattaka
 */
abstract public class BStage
{
    private HashMap<Integer,Body> bodys = new HashMap<Integer,Body>();
    private HashMap<Byte,HashSet<Body>> bodysByType = new HashMap<Byte,HashSet<Body>>();
//    private HashSet<Body> scoreBodys = new HashSet<Body>();
	private EventHandler eventHandler;
	private int nextID = 0;
	private BVector2f[] spawnPoint;
	private int drawOption = 0;
	private HashMap<Byte, String> stageMessages = new HashMap<Byte, String>();
	private BVector2f size = new BVector2f();
	private int maxClients = DataManager.DEFAULT_MAXCLIENTS;

    private BVector2f bufferCenter = new BVector2f();;
    private Dimension bufferOffset = new Dimension();
    private BShape drawMask = new BShape(DataManager.DEFAULT_RADIUS);

    private BVector2f[] freePoint;

	public BStage()
	{
        byte[] types = getBodyTypeArray();
        for(int i=0;i<types.length;i++)
        {
            bodysByType.put(types[i],new HashSet<Body>());
        }
	}
	
	abstract public void loadBStageFile(URL url) throws IOException;
	
	abstract public AIInterface createDefaultAIInterface();
    abstract public byte[] getBodyTypeArray();
	abstract public Body createBodyInstance(byte bodyType);
	abstract public ClientBody createClientBody();
    abstract public boolean isRemoteClientBody(byte type);
	abstract public byte getRemoteClientBodyType();
	
	abstract public CharacterImageBundle createCharacterImageBundle(GraphicsConfiguration gc);
	abstract public SEBundle createSEBundle();
	abstract public BGMBundle createBGMBundle();
	abstract public String getGameName();

	public int getClientActionHook(Body myBody, int action)
	{
	    return action;
	}

    public BVector2f getBufferCenter()
    {
        return bufferCenter;
    }
    public void setBufferCenter(BVector2f bufferCenter)
    {
        float mx = bufferCenter.x;
        float my = bufferCenter.y;
        float smx = DataManager.DEFAULT_WIDTH / 2;
        float smy = DataManager.DEFAULT_HEIGHT / 2;
        if (!isBufferCenterLock())
        {
            float mmx = getSize().x - smx;
            float mmy = getSize().y - smy;
            if (mx < smx)
                mx = smx;
            else if (mx > mmx)
                mx = mmx;
            if (my < smy)
                my = smy;
            else if (my > mmy)
                my = mmy;
        }
        if (DataManager.DEFAULT_WIDTH > getSize().x)
        {
            mx = getSize().x / 2;
        }
        if (DataManager.DEFAULT_HEIGHT > getSize().y)
        {
            my = getSize().y / 2;
        }
        this.bufferCenter.x = mx;
        this.bufferCenter.y = my;
        bufferOffset.width = (int)((mx-smx) * DataManager.BLOCKSIZE);
        bufferOffset.height = (int)((my-smy) * DataManager.BLOCKSIZE);
    }
    public Dimension getBufferOffset()
    {
        return new Dimension(bufferOffset);
    }
    public boolean isBufferCenterLock()
    {
        return true;
    }
    public BShape getDrawMask()
    {
        return drawMask;
    }

    //abstract public void drawBStage(Graphics bg, long currenttime, Dimension size, Dimension offset, Body myBody, Body myBodyGhost);
	public void drawBStage(Graphics bg, long currenttime, Dimension size, CharacterImageBundle cib, EffectBundle eb, Body myBody, Body myBodyGhost)
	{

        int sx = size.width;
		int sy = size.height;
		final HashSet<Body> hasString = new HashSet<Body>();
		final Font nameFont = new Font("Serif",Font.PLAIN,12);
        
		BStage bstage = this;
		
		hasString.clear();
		bg.setColor(Color.LIGHT_GRAY);
		bg.fillRect(0, 0, sx, sy);

        bg.translate(-bufferOffset.width, -bufferOffset.height);
        eb.drawPreEffect(bg, currenttime, size, this, myBody, myBodyGhost);

        Body[] bs = bstage.getSortedBodys();
		for(int i=0;i<bs.length;i++)
		{
            Body body = bs[i];
            if (myBody != null && body == myBody)
                body = myBodyGhost;
			if (!(body.isState(Body.STATE_INVISIBLE)))
			{
			    //if (body.isInSite(bufferOffset, size))
			    if (body.isInSite(bufferCenter, drawMask))
			    {
                    Dimension o = new Dimension(0,0);
                    BVector2f pos = body.getPosition();
                    int x = (int) (pos.x * DataManager.BLOCKSIZE);
                    int y = (int) (pos.y * DataManager.BLOCKSIZE);
                    bg.translate(x,y);
                    body.drawCharacter(bg,currenttime,cib, eb);
                    bg.translate(-x,-y);
                }
				if (body.hasName() || body.getMessage().length() > 0)
				hasString.add(body);
			}
		}
        eb.drawPostEffect(bg, currenttime, size, this, myBody, myBodyGhost);
        bg.translate(bufferOffset.width, bufferOffset.height);

		bg.setFont(nameFont);
		bg.setColor(Color.WHITE);
		Iterator<Body> ite = hasString.iterator();
		while(ite.hasNext())
		{
			Body b = ite.next();
			
			int offsetx = b.getCharacterImage().getWidth() / 2;
			int offsety = b.getCharacterImage().getHeight() / 2;
			int x = -bufferOffset.width + (int) (b.getPosition().x * DataManager.BLOCKSIZE - offsetx);
			int y = -bufferOffset.height + (int) (b.getPosition().y * DataManager.BLOCKSIZE - offsety);
			if (b.getName().length() > 0)
			{
				int width = DataManager.BLOCKSIZE / 2 - (int)(bg.getFontMetrics().getStringBounds(b.getName(),bg).getWidth() / 2) ;
				bg.drawString(b.getName(), x+width, y);
			}
			if ((currenttime - b.getLastMessageChange() < DataManager.MESSAGESHOWTIME) && b.getMessage().length() > 0)
			{
				int width = DataManager.BLOCKSIZE / 2 - (int)(bg.getFontMetrics().getStringBounds(b.getMessage(),bg).getWidth() / 2) ;
				bg.drawString(b.getMessage(), x+width, y + DataManager.BLOCKSIZE + bg.getFont().getSize());
			}
		}
	}
	abstract public void drawScore(Graphics bg, long currenttime, Dimension size, Body myBody);
	abstract public void drawStatus(Graphics bg, long currenttime, Dimension size, ClientBody myBody, BUserInterface ui);
	abstract public void drawRadar(Graphics bg, long currenttime, Dimension size, Body myBody, BVector2f center);

	public boolean isDrawOption(int type)
	{
		return ((drawOption & (1<<type)) == (1<<type));
	}
	public void setDrawOption(boolean flag, int type)
	{
		if (flag)
		{
			drawOption = (drawOption | (1<<type));
		}else
		{
			drawOption = (drawOption & ~(1<<type));
		}
	}
	public void addBody(Body body)
	{
		if (body == null)
			throw new NullPointerException();
		body.setBstage(this);
		if (body.getID() == -1)
		{
			throw new RuntimeException("ID is not set.");
		}
//		if (body.hasScore())
//		{
//			this.scoreBodys.add(body);
//		}
		this.bodys.put(body.getID(),body);
        this.bodysByType.get(body.getType()).add(body);
	}
	public boolean removeBody(Body body)
	{
		if (body == null)
			throw new NullPointerException();
		body.setBstage(null);
//		if (body.hasScore())
//		{
//			this.scoreBodys.remove(body);
//		}
        this.bodysByType.get(body.getType()).remove(body);
		return bodys.remove(new Integer(body.getID())) != null;
	}
	public Body getBody(int id)
	{
		return bodys.get(new Integer(id));
	}
    public int getBodysNum()
    {
        return bodys.size();
    }
	public Body[] getBodys()
	{
	    return bodys.values().toArray(new Body[bodys.size()]);
	}
    public Body[] getBodysByType(byte type)
    {
        HashSet<Body> hs = bodysByType.get(type);
        return hs.toArray(new Body[hs.size()]);
    }
    public Iterator<Body> getBodysIterator()
    {
        return bodys.values().iterator();
    }
    public Iterator getBodysIteratorByType(byte type)
    {
        return bodysByType.get(type).iterator();
    }
    abstract public Body[] getBodysHasScore();
	abstract public Body[] getSortedBodys();
	
	public void updateAll(long currenttime)
	{
		Iterator<Body> ite = bodys.values().iterator();
		while (ite.hasNext())
		{
            Body body = ite.next();
            body.update(currenttime);
		}
	}
	public void setStageMessage(byte type, String message)
	{
		stageMessages.put(new Byte(type), message);
	}
	public byte[] getStageMessageTypes()
	{
		Object[] objs = stageMessages.keySet().toArray();
		byte[] types = new byte[objs.length];
		for (int i=0;i<types.length;i++)
		{
			types[i] = ((Byte)objs[i]).byteValue();
		}
		return types;
	}
	public String getStageMessage(byte type)
	{
		return (String)stageMessages.get(new Byte(type));
	}
	public void removeStageMessage(byte type)
	{
		stageMessages.remove(new Byte(type));
	}
	
	public BPickIntersection pickClosest(BVector2f start, BVector2f end, int state)
	{
		Iterator<Body> ite = bodys.values().iterator();
		float d = Float.MAX_VALUE;
		Body col = null;
		while(ite.hasNext())
		{
			Body target = ite.next();
			if (!(target.isState(state)))
			{
				continue;
			}
			float t = BMath.getClosestIntersection(start,end,target.getPosition(),target.getShape());
			if (!(Float.isNaN(t)) && t<d)
			{
				d = t;
				col = target;
			}
		}
		if (col != null)
		{
			return new BPickIntersection(d,col);
		}else
		{
			return null;
		}
	}
	public boolean isIntersectOther(Body body, int state)
	{
		Iterator<Body> ite = bodys.values().iterator();
		while(ite.hasNext())
		{
			Body target = ite.next();
			if (body == target || !(target.isState(state)))
			{
				continue;
			}
			if (BMath.isIntersect(body.getPosition(),body.getShape(),target.getPosition(),target.getShape()))
			{
				return true;
			}
		}
		return false;
	}
	public boolean isIntersectOther(BVector2f position, BShape pg, int state)
	{
		Iterator<Body> ite = bodys.values().iterator();
		while(ite.hasNext())
		{
			Body target = ite.next();
			if (!(target.isState(state)))
			{
				continue;
			}
			if (BMath.isIntersect(position,pg,target.getPosition(),target.getShape()))
			{
				return true;
			}
		}
		return false;
	}
	public void pickAll(Body body, Set<Body> destination)
	{
		Iterator<Body> ite = bodys.values().iterator();
		BPickResult pr = new BPickResult();
		while(ite.hasNext())
		{
			Body target = ite.next();
			if (body == target || !(body.isCheckIntersectWith(target) || body.isCheckCollidWith(target)))
			{
				continue;
			}
			if (BMath.isIntersect(body.getPosition(),body.getShape(),target.getPosition(),target.getShape()))
			{
                destination.add(target);
			}
		}
	}
	public void pickAllByType(BVector2f pos, BShape pg, byte type, Set<Body> destination)
	{
		Iterator<Body> ite = bodysByType.get(type).iterator();
		BPickResult pr = new BPickResult();
		while(ite.hasNext())
		{
			Body target = ite.next();
			if (BMath.isIntersect(pos, pg, target.getPosition(),target.getShape()))
			{
                destination.add(target);
			}
		}
	}
	public BPickResult estimateCollision(Body body)
	{
		if (body == null)
			throw new NullPointerException();
        BPickResult bpr = new BPickResult();
		Iterator<Body> ite;
		byte[] types = body.getCheckCollidType();
        if (types == null)
            types = getBodyTypeArray();
        for (int i=0;i<types.length;i++)
        {
            ite = bodysByType.get(types[i]).iterator();
            while(ite.hasNext())
            {
                Body target = ite.next();
                if (body == target)
                {
                    continue;
                }
                if (BMath.isIntersect(body.getPosition(),body.getShape(),target.getPosition(),target.getShape()))
                {
                    BPickIntersection pi = new BPickIntersection(0,target);
                    bpr.addBPickIntersection(pi);
                    //continue;
                }
                float t = BMath.nextCollid(body.getPosition(),body.getShape(),body.getVelocity(),target.getPosition(),target.getShape(),target.getVelocity());
                if (!Float.isNaN(t))
                {
                    BPickIntersection pi = new BPickIntersection(t,target);
                    bpr.addBPickIntersection(pi);
                }
            }
        }
        return bpr;
	}
	public BPickResult estimateIntersection(Body body)
	{
		if (body == null)
			throw new NullPointerException();
        BPickResult bpr = new BPickResult();
		Iterator<Body> ite;
        byte[] types = body.getCheckIntersectType();
        if (types == null)
            types = getBodyTypeArray();
        for (int i=0;i<types.length;i++)
        {
            ite = bodysByType.get(types[i]).iterator();
            
    		while(ite.hasNext())
    		{
    			Body target = ite.next();
    			if (body == target)
    			{
    				continue;
    			}
    			if (BMath.isIntersect(body.getPosition(),body.getShape(),target.getPosition(),target.getShape()))
    			{
                    BPickIntersection pi = new BPickIntersection(0f,target);
                    bpr.addBPickIntersection(pi);
                    continue;
    			}
    			float t = BMath.nextCollid(body.getPosition(),body.getShape(),body.getVelocity(),target.getPosition(),target.getShape(),target.getVelocity());
    			if (!Float.isNaN(t))
    			{
                    BPickIntersection pi = new BPickIntersection(t,target);
                    bpr.addBPickIntersection(pi);
    			}
    		}
        }
        return bpr;
	}
	public BPickResult estimateIntersection(Body body, Set<Body> ignore)
	{
		if (body == null)
			throw new NullPointerException();
        BPickResult bpr = new BPickResult();
		Iterator<Body> ite;
        byte[] types = body.getCheckIntersectType();
        if (types == null)
            types = getBodyTypeArray();
        for (int i=0;i<types.length;i++)
        {
            ite = bodysByType.get(types[i]).iterator();
            while(ite.hasNext())
    		{
    			Body target = ite.next();
    			if (body == target
    				|| ignore.contains(target)
                )
    			{
    				continue;
    			}
                if (BMath.isIntersect(body.getPosition(),body.getShape(),target.getPosition(),target.getShape()))
                {
                    BPickIntersection pi = new BPickIntersection(0f,target);
                    bpr.addBPickIntersection(pi);
                    continue;
                }
    			float t = BMath.nextCollid(body.getPosition(),body.getShape(),body.getVelocity(),target.getPosition(),target.getShape(),target.getVelocity());
    			if (!Float.isNaN(t))
    			{
                    BPickIntersection pi = new BPickIntersection(t,target);
                    bpr.addBPickIntersection(pi);
    			}
    		}
        }
        return bpr;
	}
    public void pickIntersection(Body body, Set<Body> destination)
    {
        if (body == null || destination == null)
            throw new NullPointerException();
        BPickResult bpr = new BPickResult();
        Iterator<Body> ite;
        byte[] types = body.getCheckIntersectType();
        if (types == null)
            types = getBodyTypeArray();
        for (int i=0;i<types.length;i++)
        {
            ite = bodysByType.get(types[i]).iterator();
            while(ite.hasNext())
            {
                Body target = ite.next();
                if (body == target || (!(body.isCheckIntersectWith(target) || body.isCheckCollidWith(target))))
                {
                    continue;
                }
                if (BMath.isIntersect(body.getPosition(),body.getShape(),target.getPosition(),target.getShape()))
                {
                    destination.add(target);
                }
            }
        }
    }
	public EventThread getEventThread()
	{
		return eventHandler.getEventThread();
	}
	public long getCurrentTime()
	{
		return eventHandler.getEventThread().getCurrentTime();
	}
	public int getNextID()
	{
		do
        {
            nextID++;
		    if (nextID == Short.MAX_VALUE)
			{
				nextID = 0;
            }
		}while (getBody(nextID) != null);
		//System.out.println(nextID);
		return nextID;
	}
	public BVector2f getRandomeSpawnPoint()
	{
		return getSpawnPoint()[(int)(Math.random() * getSpawnPoint().length)];
	}
	public BVector2f[] getSpawnPoint()
	{
		return spawnPoint;
	}
	public void setSpawnPoint(BVector2f[] spawnPoint)
	{
		this.spawnPoint = spawnPoint;
	}
	public EventHandler getEventHandler()
	{
		return eventHandler;
	}
	public void setEventHandler(EventHandler eventHandler)
	{
		this.eventHandler = eventHandler;
	}
	public BVector2f getSize()
	{
		return size;
	}
	public void setSize(BVector2f size)
	{
		this.size.set(size);
	}
	public int getMaxClients()
	{
		return maxClients;
	}
	public void setMaxClients(int maxClients)
	{
		this.maxClients = maxClients;
	}
	public BVector2f[] getFreePoint()
	{
		return freePoint;
	}
	public void setFreePoint(BVector2f[] freePoint)
	{
		this.freePoint = freePoint;
	}
	public BVector2f getRandomeFreePoint()
	{
		return getFreePoint()[(int)(Math.random() * getFreePoint().length)];
	}
	public void eventAction(int action)
	{
	}
	public void eventClientPreAdd(long currenttime, Body body)
	{
	}
	public void eventClientPostAdd(long currenttime, Body body)
	{
	}
//	public void eventClientDead(long currenttime, Body body)
//	{
//	}
//    abstract public void changeActionMode(int change);
//    abstract public byte getActionMode();
//    abstract public byte getActionNum();
//    abstract public void setActionMode(byte mode);
}