/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/01
 */
package bakutan.common;

import java.io.File;

import bakutan.common.util.BAudio;

/**
 * 爆裂タンク用のユーザーインターフェース。
 * メッセージの出力、変換、作業用ディレクトリを得ることができる。
 * @author cattaka
 */
public interface BUserInterface
{
    /**
     * メッセージを出力する。
     * @param message
     */
	public void outputMessage(String message);
    /**
     * ローカライズ用の辞書(PhraseDic_??.xml)に従って入力されたキーを変換する。
     * @param key 辞書に対するキー
     * @return ローカライズ化されたメッセージ
     */
	public String translate(String key);
    /**
     * ローカライズ用の辞書(PhraseDic_??.xml)に従って入力されたキーを変換する。
     * @param key 辞書に対するキー
     * @param opt メッセージに埋め込む文字列
     * @return ローカライズ化されたメッセージ
     */
	public String translate(String key, String[] opt);
    /**
     * 爆裂タンクの作業用ディレクトリを取得する。
     * Javaアプリケーション、WebStart版の場合は作業用ディレクトリを得ることができるが、
     * Javaアプレット時、制限付きWebStart版の場合はnullを返す。
     * @return 作業用ディレクトリ、設定されてない場合はfalse。
     */
    public File getWorkDir();
    
    public BAudio getBAudio();
}
