/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/01/18
 */
package bakutan.common;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import bakutan.common.event.BodyActionLockEnergyEvent;
import bakutan.common.event.BodyActionEvent;
import bakutan.common.event.BodyAddEvent;
import bakutan.common.event.BodyClientIDEvent;
import bakutan.common.event.BodyCommandEvent;
import bakutan.common.event.BodyDeleteEvent;
import bakutan.common.event.BodyEvent;
import bakutan.common.event.BodyArmorEvent;
import bakutan.common.event.BodyKnock;
import bakutan.common.event.BodyMessageEvent;
import bakutan.common.event.BodyMoveEvent;
import bakutan.common.event.BodyMultiAddEvent;
import bakutan.common.event.BodyOptionEvent;
import bakutan.common.event.BodyPushEvent;
import bakutan.common.event.BodyScoreEvent;
import bakutan.common.event.BodySetPotisionEvent;
import bakutan.common.event.BodySmallActionEvent;
import bakutan.common.event.BodySoundEvent;
import bakutan.common.event.BodyStageMessageEvent;
import bakutan.common.event.BodyStateEvent;
import bakutan.common.event.GameRestartEvent;
import bakutan.common.event.GameSetEvent;
import bakutan.common.event.StageEvent;
import bakutan.common.math.BShape;
import bakutan.common.math.BVector2f;
import bakutan.common.serverinfo.BStageState;
import bakutan.common.serverinfo.ServerInfo;
import bakutan.common.serverinfo.SmallBStageState;
import bakutan.common.serverinfo.SmallServerInfo;
import bakutan.common.util.CompressedInputStream;
import bakutan.common.util.CompressedOutputStream;


/**
 * @author cattaka
 */
public class DataManager
{
    public static final String DYNAMIC_RESOURCE_PATH = "http://www.cattaka.net/files/bakutan/dynamicresource/";

	public static final String DEFAULT_SERVER = "cattaka.myhome.cx";
    public static final int DEFAULT_SIMA_PORT = 9400;
	public static final int DEFAULT_PORT = 9410;
	public static final int DEFAULT_MAXCLIENTS = 16;
	public static final int MAX_MESSAGE_LENGTH = 100;
	public static final int DEFAULT_TIMEOUT = 600000;
	public static final int DEFAULT_TIMELIMIT = 900000;
	//public static final long MAX_INTERVAL = 3600000;
	public static final long MAX_EVENT_REPORT_INTERVAL = 60000;
    public static final long MIM_MOVE_REPORT_INTERVAL = 100;
    public static final long MAX_MOVE_REPORT_INTERVAL = 1000;
    public static final long KNOCK_INTERVAL = 5000;
	
	public static int BLOCKSIZE = 32;
	public static float DEFAULT_WIDTH = 24;
	public static float DEFAULT_HEIGHT = 18;
	public static float DEFAULT_RADIUS = (float)(Math.sqrt(DEFAULT_WIDTH*DEFAULT_WIDTH + DEFAULT_HEIGHT*DEFAULT_HEIGHT)/2d);
    
	public static final byte IDPASS = 1;
	public static final byte USERID = 2;
	public static final byte EVENT_ADD = 3;
	public static final byte EVENT_DELETE = 4;
	public static final byte EVENT_MOVE = 5;
	public static final byte EVENT_STATE = 7;
	public static final byte EVENT_SETPOTISION = 9;
	public static final byte EVENT_CHAT = 10;
	public static final byte EVENT_SCORE = 11;
	public static final byte EVENT_ACTION = 12;
	public static final byte EVENT_OPTION = 13;
	public static final byte EVENT_MESSAGE = 14;
	public static final byte EVENT_SOUND = 15;
	public static final byte EVENT_STAGEMESSAGE = 16;
	public static final byte EVENT_PUSH = 17;
	public static final byte EVENT_GAMESET = 18;
	public static final byte EVENT_ARMOR = 19;
    public static final byte EVENT_KNOCK = 20;
    public static final byte EVENT_COLLISION = 21;
    public static final byte EVENT_CLIENTMOVE = 22;
    public static final byte EVENT_CLIENTACTION = 23;
    public static final byte EVENT_CLIENTADD = 24;
    public static final byte EVENT_SCORECHANGE = 25;
    public static final byte EVENT_STAGEACTION = 26;
    public static final byte EVENT_ACTIONCANCEL = 27;
    public static final byte EVENT_SMALLACTION = 28;
    public static final byte EVENT_MULTIADD = 29;
    public static final byte EVENT_CLIENTID = 30;
    public static final byte EVENT_GAMERESTART = 31;
    public static final byte EVENT_RECHECKINTERSECT = 32;
    public static final byte EVENT_CLIENTREMOVE = 33;
    
    public static final byte SERVERMANAGE_ADD = 0;
    public static final byte SERVERMANAGE_GET = 1;

	public static final byte MESSAGE_SYSTEM = 0;
	public static final byte MESSAGE_LOGIN = 1;
	public static final byte MESSAGE_NOTE = 2;
	public static final byte MESSAGE_CHAT = 3;
	
	public static final byte REQUEST_LOGIN = 0;
	public static final byte REQUEST_BSTAGESTATE = 1;
    public static final byte REQUEST_SMALLBSTAGESTATE = 2;

    public static final byte AUTH_SUCCEED = 0;
    public static final byte AUTH_FAILED = 1;
    public static final byte AUTH_FAILED_INVALID = 2;
    public static final byte AUTH_FAILED_EXIST = 3;
    public static final byte AUTH_TOO_MANY_CLIENTS = 4;
    
    public static final byte LOGINMODE_GUEST = 0;
    public static final byte LOGINMODE_USER = 1;
    public static final byte LOGINMODE_NEWUSER = 2;
    
    public static final int USERNAME_LENGTH = 10;
    public static final int PASSWORD_LENGTH = 20;
    public static final int NICKNAME_LENGTH = 10;
    
	public static final float RAG_DELAY = 0.5f;
	public static final float RAG_ATTENUATION = 0.25f;
	public static int ANIMINTERVAL = 200;
	public static long MESSAGESHOWTIME = 5000;
    
	protected DataManager()
    {
    }
    
	public static String getCurrentProtocolVersion()
	{
		return VersionBundle.getString("bakusen.protocolVersion");
	}
	public static String getBuildNumber()
	{
		return VersionBundle.getString("build.number");
	}
    public static DataManager getAvailableDataManager()
    {
        return new DataManager();
    }
    
	public static DataManager getAvailableDataManager(String protocol)
    {
        if (protocol.equals(VersionBundle.getString("bakusen.protocolVersion")))
            return new DataManager();
        else if (protocol.equals("0.34"))
            return new DataManager();
        else if (protocol.equals("0.35"))
            return new DataManager();
        else
            return null;
    }
    
	public void outputData(DataOutputStream out, BStage bstage)
	throws IOException
	{
		out.writeUTF(bstage.getClass().getName());
		out.writeFloat(bstage.getSize().x);
		out.writeFloat(bstage.getSize().y);
	}
	public BStage inputBStage(DataInputStream in)
	throws IOException
	{
		Class bstageClass;
		try
		{
			bstageClass = Class.forName(in.readUTF());
		}catch(ClassNotFoundException e)
		{
			throw new IOException("unknown BStage class:"+e.getMessage());
		}
		
		if (!(BStage.class.isAssignableFrom(bstageClass)))
		{
			throw new IOException("invalid BStage class");
		}
		BStage bstage;
		try
		{
			bstage= (BStage)bstageClass.newInstance();
		}catch(IllegalAccessException e)
		{
			throw new IOException("Could not create BStage");
		}catch(InstantiationException e)
		{
			throw new IOException("Could not create BStage");
		}
		float w = in.readFloat();
		float h = in.readFloat();
		if (Float.isNaN(w) || Float.isNaN(h))
		{
			throw new IOException("Could not create BStage");
		}
		bstage.setSize(new BVector2f(w,h));
		return bstage;
	}
	public void outputData(DataOutputStream out, Body[] bs)
	throws IOException
	{
		out.writeInt(bs.length);
		for (int i=0;i<bs.length;i++)
		{
			outputData(out,bs[i]);
		}
	}
	public Body[] inputBodys(DataInputStream in, BStage bstage)
	throws IOException
	{
		int l = in.readInt();
		Body[] bs = new Body[l];
		for (int i=0;i<l;i++)
		{
			bs[i] = inputBody(in, bstage);
		}
		return bs;
	}
	
	public void outputData(DataOutputStream out, Body body)
	throws IOException
	{
		out.writeByte(body.getType());
        body.outputData(this, out);
        return;
        /*
		if (body.hasName())
			out.writeUTF(body.getName());
		out.writeShort(body.getID());
		if (body.hasState())
		    out.writeShort(body.getState());
        if (body.hasArmor())
            out.writeShort(body.getArmor());
        if (body.hasOption())
        {
            int[] bs = body.getOptions();
            out.writeByte(bs.length);
            for (int i=0;i<bs.length;i++)
            {
                out.writeShort((short)bs[i]);
            }
        }
        outputData(out, body.getPosition());
        outputData(out, body.getVelocity());
        //outputData(out, body.getDirection());
        out.writeByte(convBVectorToByte(body.getDirection()));

		if (body.hasScore())
			out.writeInt(body.getScore());
		if (body.hasVariableBCircle())
			outputData(out,body.getCircle());
		*/
	}
	public Body inputBody(DataInputStream in, BStage bstage)
	throws IOException
	{
		Body body;
		body = bstage.createBodyInstance(in.readByte());
        body.inputData(this, in);
        return body;
        /*
		if (body.hasName())
			body.setName(in.readUTF());
		body.setID(in.readShort());
        if (body.hasState())
            body.setState(in.readShort());
        if (body.hasArmor())
            body.setArmor(in.readShort());
        if (body.hasOption())
        {
            byte l = in.readByte();
            int[] bs = new int[l];
            for (int i=0;i<bs.length;i++)
            {
                bs[i] = in.readShort();
            }
            body.setOptions(bs);
        }
        BVector2f temp;
        temp = inputBVector2d(in);
        if (Float.isNaN(temp.x) || Float.isNaN(temp.y))
        {
            throw new IOException("Broken data");
        }
        body.getPosition().set(temp);
        temp = inputBVector2d(in);
        if (Float.isNaN(temp.x) || Float.isNaN(temp.y))
        {
            throw new IOException("Broken data");
        }
        body.getVelocity().set(temp);
        //temp = inputBVector2d(in);
        temp = convByteToBVector(in.readByte());
        if (Float.isNaN(temp.x) || Float.isNaN(temp.y))
        {
            throw new IOException("Broken data");
        }
        body.getDirection().set(temp);

        if (body.hasScore())
			body.setScore(in.readInt());
		if (body.hasVariableBCircle())
			body.setCircle(inputBPolygon(in));
		return body;
        */
	}
	public void outputData(DataOutputStream out, BShape bp)
	throws IOException
	{
        out.writeByte(bp.getType());
        if (bp.getType() == BShape.CIRCLE)
        {
            out.writeFloat(bp.getRadius());
        }
        else if (bp.getType() == BShape.LINE)
        {
            out.writeFloat(bp.getPoint1().x);
            out.writeFloat(bp.getPoint1().y);
            out.writeFloat(bp.getPoint2().x);
            out.writeFloat(bp.getPoint2().y);
        }
        else if (bp.getType() == BShape.RECTANGLE)
        {
            out.writeFloat(bp.getRectangle().a);
            out.writeFloat(bp.getRectangle().b);
            out.writeFloat(bp.getRectangle().c);
            out.writeFloat(bp.getRectangle().d);
        }
        else if (bp.getType() == BShape.COMPOSITE)
        {
            BShape[] sps = bp.getComposite();
            out.writeByte(sps.length);
            for (int i=0;i<sps.length;i++)
            {
                outputData(out, sps[i]);
            }
        }
	}
	public BShape inputBShape(DataInputStream in) throws IOException
	{
        byte type = in.readByte();
        if (type == BShape.CIRCLE)
        {
            float radius = in.readFloat();
            return new BShape(radius);
        }
        else if (type == BShape.LINE)
        {
            float x1,y1,x2,y2;
            x1 = in.readFloat();
            y1 = in.readFloat();
            x2 = in.readFloat();
            y2 = in.readFloat();
            if (Float.isNaN(x1) || Float.isNaN(y1) || Float.isNaN(x2) || Float.isNaN(y2))
                throw new IOException();
            BShape s = new BShape();
            s.setType(type);
            s.getPoint1().set(x1,y1);
            s.getPoint2().set(x2,y2);
            return s;
        }
        else if (type == BShape.RECTANGLE)
        {
            float x1,y1,x2,y2;
            x1 = in.readFloat();
            x2 = in.readFloat();
            y1 = in.readFloat();
            y2 = in.readFloat();
            if (Float.isNaN(x1) || Float.isNaN(y1) || Float.isNaN(x2) || Float.isNaN(y2))
                throw new IOException();
            BShape s = new BShape();
            s.setType(type);
            s.getRectangle().set(x1,x2,y1,y2);
            return s;
        }
        else if (type == BShape.COMPOSITE)
        {
            int num = in.readByte();
            BShape[] sps = new BShape[num];
            for (int i=0;i<num;i++)
            {
                sps[i] = inputBShape(in);
            }
            return new BShape(sps);
        }
        else
        {
            throw new IOException("Unknown BShape type");
        }
	}
	public void outputData(DataOutputStream out, BVector2f[] vs)
	throws IOException
	{
		out.writeInt(vs.length);
		for (int i=0;i<vs.length;i++)
		{
            outputData(out, vs[i]);
		}
	}
    public void outputData(DataOutputStream out, BVector2f vec)
    throws IOException
    {
        out.writeFloat(vec.x);
        out.writeFloat(vec.y);
    }
	public BVector2f[] inputBVector2ds(DataInputStream in)
	throws IOException
	{
		int l = in.readInt();
		BVector2f[] vs = new BVector2f[l];
		for (int i=0;i<vs.length;i++)
		{
			vs[i] = inputBVector2d(in);
			if (Float.isNaN(vs[i].x) || Float.isNaN(vs[i].y))
			{
				throw new IOException("Broken data");
			}
		}
		return vs;
	}
    public BVector2f inputBVector2d(DataInputStream in)
    throws IOException
    {
        BVector2f vec = new BVector2f();
        vec.x = in.readFloat();
        vec.y = in.readFloat();
        return vec;
    }
	public void outputData(DataOutputStream out, StageEvent se)
	throws IOException
	{
		out.writeByte(se.getTargets().length);
		for (int i=0;i<se.getTargets().length;i++)
		{
			out.writeShort(se.getTargets()[i].getID());
		}
		outputData(out,se.getBodyEvent());
	}
    public void outputData(DataOutputStream out, ClientStageEvent cse)
    throws IOException
    {
        int[] ids = cse.getBodysID();
        out.writeByte(ids.length);
        for (int i=0;i<ids.length;i++)
        {
            out.writeShort(ids[i]);
        }
        outputData(out,cse.getBodyEvent());
    }
	
	public void outputData(DataOutputStream out, BodyEvent be)
	throws IOException
	{
		if (be.getEventID() == DataManager.EVENT_ADD)
		{
			out.writeByte(EVENT_ADD);
			outputData(out,((BodyAddEvent)be).getBody());
		}
        else if (be.getEventID() == DataManager.EVENT_MULTIADD)
        {
            out.writeByte(EVENT_MULTIADD);
            Body[] bodys = ((BodyMultiAddEvent)be).getBodys();
            out.writeShort((short)bodys.length);
            for (int i=0;i<bodys.length;i++)
            {
                outputData(out,bodys[i]);
            }
        }
		else if (be.getEventID() == DataManager.EVENT_DELETE)
		{
			BodyDeleteEvent bde = (BodyDeleteEvent)be;
			out.writeByte(EVENT_DELETE);
		}
		else if (be.getEventID() == DataManager.EVENT_STATE)
		{
			BodyStateEvent bse = (BodyStateEvent)be;
			out.writeByte(EVENT_STATE);
			out.writeByte(bse.getType());
			out.writeInt(bse.getState());
		}
		else if (be.getEventID() == DataManager.EVENT_OPTION)
		{
			BodyOptionEvent boe = (BodyOptionEvent)be;
			out.writeByte(EVENT_OPTION);
			out.writeByte(boe.getType());
			out.writeInt(boe.getValue());
		}
		else if (be.getEventID() == DataManager.EVENT_ACTION)
		{
			BodyActionEvent bae = (BodyActionEvent)be;
			out.writeByte(EVENT_ACTION);
			out.writeFloat(bae.getPosition().x);
			out.writeFloat(bae.getPosition().y);
            out.writeFloat(bae.getTarget().x);
            out.writeFloat(bae.getTarget().y);
			out.writeByte(bae.getAction());
		}
        else if (be.getEventID() == DataManager.EVENT_SMALLACTION)
        {
            BodySmallActionEvent bsae = (BodySmallActionEvent)be;
            out.writeByte(EVENT_SMALLACTION);
            out.writeByte(bsae.getAction());
        }
		else if (be.getEventID() == DataManager.EVENT_MOVE)
		{
			BodyMoveEvent bme = (BodyMoveEvent)be;
			out.writeByte(EVENT_MOVE);
            outputData(out,bme.getPosition());
            outputData(out,bme.getVelocity());
            out.writeByte(convBVectorToByte(bme.getDirection()));
		}
		else if (be.getEventID() == DataManager.EVENT_SETPOTISION)
		{
			BodySetPotisionEvent bse = (BodySetPotisionEvent)be;
			out.writeByte(EVENT_SETPOTISION);
			out.writeFloat(bse.getPosition().x);
			out.writeFloat(bse.getPosition().y);
		}
		else if (be.getEventID() == DataManager.EVENT_CHAT)
		{
			BodyCommandEvent bce = (BodyCommandEvent)be;
			out.writeByte(EVENT_CHAT);
			out.writeUTF(bce.getCommand());
		}
		else if (be.getEventID() == DataManager.EVENT_MESSAGE)
		{
			BodyMessageEvent bme = (BodyMessageEvent)be;
			out.writeByte(EVENT_MESSAGE);
			out.writeByte(bme.getType());
			out.writeUTF(bme.getMessage());
		}
		else if (be.getEventID() == DataManager.EVENT_STAGEMESSAGE)
		{
			BodyStageMessageEvent bme = (BodyStageMessageEvent)be;
			out.writeByte(EVENT_STAGEMESSAGE);
			out.writeByte(bme.getType());
			out.writeUTF(bme.getMessage());
		}
		else if (be.getEventID() == DataManager.EVENT_SOUND)
		{
			BodySoundEvent bse = (BodySoundEvent)be;
			out.writeByte(EVENT_SOUND);
            if (bse.getMode() == BodySoundEvent.MODE_POSITION)
            {
                out.writeByte(bse.getSound());
                out.writeByte(bse.getMode());
                outputData(out,bse.getPosition());
            }
            else if (bse.getMode() == BodySoundEvent.MODE_TARGET)
            {
                out.writeByte(bse.getSound());
                out.writeByte(bse.getMode());
            }
            else
            {
                throw new IOException("Not supported mode.");
            }
		}
		else if (be.getEventID() == DataManager.EVENT_SCORE)
		{
			BodyScoreEvent bse = (BodyScoreEvent)be;
			out.writeByte(EVENT_SCORE);
			out.writeShort(bse.getScores().length);
			for(int i=0;i<bse.getScores().length;i++)
			{
				out.writeShort(bse.getScores()[i]);
			}
		}
		else if (be.getEventID() == DataManager.EVENT_PUSH)
		{
			BodyPushEvent bpe = (BodyPushEvent)be;
			out.writeByte(EVENT_PUSH);
			out.writeInt(bpe.getTargetID());
		}
		else if (be.getEventID() == DataManager.EVENT_GAMESET)
		{
			GameSetEvent gse = (GameSetEvent)be;
			out.writeByte(EVENT_GAMESET);
			out.writeLong(gse.getTimeLimit());
		}
        else if (be.getEventID() == DataManager.EVENT_GAMERESTART)
        {
            out.writeByte(EVENT_GAMERESTART);
        }
		else if (be.getEventID() == DataManager.EVENT_ARMOR)
		{
			BodyArmorEvent bhe = (BodyArmorEvent)be;
			out.writeByte(EVENT_ARMOR);
			out.writeByte(bhe.getType());
			out.writeInt(bhe.getArmor());
		}
        else if (be.getEventID() == DataManager.EVENT_KNOCK)
        {
            out.writeByte(EVENT_KNOCK);
        }
        else if (be.getEventID() == DataManager.EVENT_ACTIONCANCEL)
        {
            BodyActionLockEnergyEvent bace = (BodyActionLockEnergyEvent)be;
            out.writeByte(EVENT_ACTIONCANCEL);
            out.writeByte(bace.getAction());
            out.writeFloat(bace.getEnergy());
            out.writeShort(bace.getActionLock());
        }
        else if (be.getEventID() == DataManager.EVENT_CLIENTID)
        {
            BodyClientIDEvent bcie = (BodyClientIDEvent)be;
            out.writeByte(EVENT_CLIENTID);
            out.writeShort(bcie.getId());
        }
		else
		{
			throw new IOException(be.toString());
		}
	}
	public BodyEvent inputBodyEvent(DataInputStream in, BStage bstage)
	throws IOException
	{
		byte b = in.readByte();
		if (b == EVENT_ADD)
		{
			Body body = inputBody(in, bstage);
			BodyAddEvent bae = new BodyAddEvent(body);
			return bae;
        }else if (b == EVENT_MULTIADD)
        {
            int num = in.readShort();
            Body[] bodys = new Body[num];
            for (int i=0;i<num;i++)
            {
                bodys[i] = inputBody(in, bstage);
            }
            return new BodyMultiAddEvent(bodys);
		}else if (b == EVENT_DELETE)
		{
			BodyDeleteEvent bde = new BodyDeleteEvent();
			return bde;
		}else if (b == EVENT_MOVE)
		{
            BVector2f temp;
            BVector2f pos = new BVector2f();
            BVector2f vel = new BVector2f();
            BVector2f dir = new BVector2f();
            temp = inputBVector2d(in);
            if (Float.isNaN(temp.x) || Float.isNaN(temp.y))
            {
                throw new IOException("Broken data");
            }
            pos.set(temp);
            temp = inputBVector2d(in);
            if (Float.isNaN(temp.x) || Float.isNaN(temp.y))
            {
                throw new IOException("Broken data");
            }
            vel.set(temp);
            //temp = inputBVector2d(in);
            temp = convByteToBVector(in.readByte());
            if (Float.isNaN(temp.x) || Float.isNaN(temp.y))
            {
                throw new IOException("Broken data");
            }
            dir.set(temp);

            BodyMoveEvent bme = new BodyMoveEvent(pos,vel,dir);
			return bme;
		/*
		}else if (b == EVENT_PUTBOM)
		{
			float x = in.readFloat();
			float y = in.readFloat();
			if (Float.isNaN(x) || Float.isNaN(y))
				throw new IOException("Broken data");
			BodyPutBomEvent bpe = new BodyPutBomEvent(x,y);
			return bpe;
		*/
		}else if (b == EVENT_ACTION)
		{
			float x = in.readFloat();
			float y = in.readFloat();
            float tx = in.readFloat();
            float ty = in.readFloat();
			if (Float.isNaN(x) || Float.isNaN(y))
				throw new IOException("Broken data");
			byte action = in.readByte();
			BodyActionEvent bae = new BodyActionEvent(x,y,tx,ty,action);
			return bae;
        }else if (b == EVENT_SMALLACTION)
        {
            byte action = in.readByte();
            BodySmallActionEvent bsae = new BodySmallActionEvent(action);
            return bsae;
		}else if (b==EVENT_STATE)
		{
			byte t = in.readByte();
			int s = in.readInt();
			BodyStateEvent bse = new BodyStateEvent(t,s);
			return bse;
		}else if (b==EVENT_OPTION)
		{
			byte t = in.readByte();
			int o = in.readInt();
			BodyOptionEvent boe = new BodyOptionEvent(t,o);
			return boe;
		}else if (b == EVENT_SETPOTISION)
		{
			float x = in.readFloat();
			float y = in.readFloat();
			if (Float.isNaN(x) || Float.isNaN(y))
				throw new IOException("Broken data");
			BodySetPotisionEvent bse = new BodySetPotisionEvent(new BVector2f(x,y));
			return bse;
		}else if (b == EVENT_CHAT)
		{
			String message = in.readUTF();
			BodyCommandEvent bce = new BodyCommandEvent(message);
			return bce;
		}else if (b == EVENT_MESSAGE)
		{
			byte type = in.readByte();
			String message = in.readUTF();
			BodyMessageEvent bme = new BodyMessageEvent(type, message);
			return bme;
		}else if (b == EVENT_STAGEMESSAGE)
		{
			byte type = in.readByte();
			String message = in.readUTF();
			BodyStageMessageEvent bme = new BodyStageMessageEvent(type, message);
			return bme;
		}else if (b == EVENT_SOUND)
		{
			byte type = in.readByte();
            byte mode = in.readByte();
            if (mode == BodySoundEvent.MODE_POSITION)
            {
                return new BodySoundEvent(type, inputBVector2d(in));
            }
            else if (mode == BodySoundEvent.MODE_TARGET)
            {
                return new BodySoundEvent(type);
            }
            else
            {
                throw new IOException("Not supported mode.");
            }
		}else if (b == EVENT_SCORE)
		{
			int[] scores = new int[in.readShort()];
			for (int i=0;i<scores.length;i++)
			{
				scores[i] = in.readShort();
			}
			BodyScoreEvent bse = new BodyScoreEvent(scores);
			return bse;
		}else if (b == EVENT_PUSH)
		{
			int targetID = in.readInt();
			BodyPushEvent bpe = new BodyPushEvent(targetID);
			return bpe;
		}else if(b == EVENT_GAMESET)
		{
			long timeLimit = in.readLong();
			GameSetEvent gse = new GameSetEvent(timeLimit);
			return gse;
        }else if(b == EVENT_GAMERESTART)
        {
            GameRestartEvent gre = new GameRestartEvent();
            return gre;
		}else if(b == EVENT_ARMOR)
		{
			byte type = in.readByte();
			int armor = in.readInt();
			return new BodyArmorEvent(type, armor);
        }else if(b == EVENT_KNOCK)
        {
            return new BodyKnock();
        }else if(b == EVENT_ACTIONCANCEL)
        {
            byte action = in.readByte();
            float energy = in.readFloat();
            short actionLock = in.readShort();
            return new BodyActionLockEnergyEvent(action, energy, actionLock);
        }else if (b == EVENT_CLIENTID)
        {
            int id = in.readShort();
            BodyClientIDEvent bcie = new BodyClientIDEvent(id);
            return bcie;
		}else
		{
			throw new IOException("unknown event:"+b);
		}
	}

	/*
	public static void outputData(DataOutputStream out, StageEvent be)
	throws IOException
	{
		if (be instanceof BodyAddEvent)
		{
			out.writeByte(EVENT_BODYADD);
			outputData(out,((BodyAddEvent)be).getBody());
		}
		else if (be instanceof BodyDeleteEvent)
		{
			BodyDeleteEvent bde = (BodyDeleteEvent)be;
			out.writeByte(EVENT_BODYDELETE);
			out.writeInt(bde.getBody().getID());
		}
		else if (be instanceof BodyStateEvent)
		{
			BodyStateEvent bse = (BodyStateEvent)be;
			out.writeByte(EVENT_BODYSTATE);
			out.writeInt(bse.getBody().getID());
			out.writeInt(bse.getState());
		}
		else if (be instanceof BodyMoveEvent)
		{
			BodyMoveEvent bme = (BodyMoveEvent)be;
			out.writeByte(EVENT_BODYADD);
			out.writeInt(bme.getBody().getID());
			out.writeFloat(bme.getPosition().x);
			out.writeFloat(bme.getPosition().y);
			out.writeFloat(bme.getVelocity().x);
			out.writeFloat(bme.getVelocity().y);
		}
		else
		{
			throw new IOException();
		}
	}
	public static BodyEvent inputBodyEvent(DataInputStream in)
	throws IOException
	{
		byte b = in.readByte();
		if (b == EVENT_BODYADD)
		{
			Body body = inputBody(in);
			BodyAddEvent bae = new BodyAddEvent(body);
			return bae;
		}else
		{
			throw new IOException();
		}
	}
	*/
	public static void outputData(DataOutputStream out, BodyState bs)
	throws IOException
	{
		out.writeUTF(bs.getName());
		out.writeInt(bs.getScore());
		out.writeUTF(bs.getMisc());
	}
	public static BodyState inputBodyState(DataInputStream in)
	throws IOException
	{
		String name = in.readUTF();
		int score = in.readInt();
		String misc = in.readUTF();
		BodyState bs = new BodyState(name,score,misc);
		return bs;
	}
	public static void outputData(DataOutputStream out, BStageState bstageState)
	throws IOException
	{
        out.writeUTF(bstageState.getProtocol());
        out.writeShort(bstageState.getMaxClients());
        out.writeInt(bstageState.getOption());
        
        out.writeUTF(bstageState.getLoginMessage());
        out.writeUTF(bstageState.getNoteMessage());
		BodyState[] bss = bstageState.getBodyStates();
		out.writeShort(bss.length);
		for (int i=0;i<bss.length;i++)
		{
			outputData(out,bss[i]);
		}
	}
	public static BStageState inputBStageState(DataInputStream in)
	throws IOException
	{
        long starttime = System.currentTimeMillis();
        
		BStageState bss = new BStageState();
        bss.setProtocol(in.readUTF());
        bss.setMaxClients(in.readShort());
        bss.setOption(in.readInt());
        
		bss.setLoginMessage(in.readUTF());
		bss.setNoteMessage(in.readUTF());
		int l = in.readShort();
		for (int i=0;i<l;i++)
		{
			bss.addBodyState(inputBodyState(in));
		}
        long endtime = System.currentTimeMillis();
        bss.setRag((int)(endtime-starttime));
		return bss;
	}
    public static void outputData(DataOutputStream out, SmallBStageState bstageState)
    throws IOException
    {
        out.writeUTF(bstageState.getProtocol());
        out.writeInt(bstageState.getOption());
    }
    public static SmallBStageState inputSmallBStageState(DataInputStream in)
    throws IOException
    {
        long starttime = System.currentTimeMillis();
        SmallBStageState bss = new SmallBStageState();
        bss.setProtocol(in.readUTF());
        bss.setOption(in.readInt());
        long endtime = System.currentTimeMillis();
        bss.setRag((int)(endtime-starttime));
        return bss;
    }
    public static void outputData(DataOutputStream out, Collection<SmallServerInfo> sies) throws IOException
    {
        out.writeShort(sies.size());
        Iterator<SmallServerInfo> ite =  sies.iterator();
        while(ite.hasNext())
        {
            outputData(out, ite.next());
        }
    }
    public static Collection<SmallServerInfo> inputSmallServerInfos(DataInputStream in) throws IOException
    {
        ArrayList<SmallServerInfo> sies = new ArrayList<SmallServerInfo>();
        int num = in.readShort();
        for(int i=0;i<num;i++)
        {
            sies.add(inputSmallServerInfo(in));
        }
        return sies;
    }
    public static void outputData(DataOutputStream out, ServerInfo sie) throws IOException
    {
        out.writeUTF(sie.getLabel());
        out.writeUTF(sie.getHostName());
        out.writeInt(sie.getPort());
        BStageState bss = sie.getBStageState();
        if ( bss == null )
        {
            out.writeBoolean(false);
        }
        else
        {
            out.writeBoolean(true);
            outputData(out,bss);
        }
    }
    public static ServerInfo inputServerInfo(DataInputStream in) throws IOException
    {
        ServerInfo sie = new ServerInfo();
        sie.setLabel(in.readUTF());
        sie.setHostName(in.readUTF());
        sie.setPort(in.readInt());
        
        if (in.readBoolean())
        {
            sie.setBStageState(inputBStageState(in));
        }
        return sie;
    }
    public static void outputData(DataOutputStream out, SmallServerInfo sie) throws IOException
    {
        out.writeUTF(sie.getLabel());
        out.writeUTF(sie.getHostName());
        out.writeInt(sie.getPort());
        SmallBStageState bss = sie.getSmallBStageState();
        if ( bss == null )
        {
            out.writeBoolean(false);
        }
        else
        {
            out.writeBoolean(true);
            outputData(out,bss);
        }
    }
    public static SmallServerInfo inputSmallServerInfo(DataInputStream in) throws IOException
    {
        SmallServerInfo sie = new ServerInfo();
        sie.setLabel(in.readUTF());
        sie.setHostName(in.readUTF());
        sie.setPort(in.readInt());
        
        if (in.readBoolean())
        {
            sie.setSmallBStageState(inputSmallBStageState(in));
        }
        return sie;
    }
    
    public static BStageState getBStageState(String host, int port, int timeout)
    throws IOException
    {
        return getBStageState(new InetSocketAddress(host, port), timeout);
    }

    public static BStageState getBStageState(InetSocketAddress address, int timeout)
	throws IOException
	{
		Socket socket = null;
		socket = new Socket();
        try
        {
    		socket.setSoTimeout(timeout);
    		socket.connect(address, timeout);
    		DataInputStream in = new DataInputStream(new CompressedInputStream(socket.getInputStream()));
    		DataOutputStream out = new DataOutputStream(new CompressedOutputStream(socket.getOutputStream()));
    		String protocol = in.readUTF();
    		BStageState bss;
			out.writeByte(DataManager.REQUEST_BSTAGESTATE);
			out.flush();
			bss = DataManager.inputBStageState(in);
            
    		bss.setProtocol(protocol);
    		in.close();
    		out.close();
    		socket.close();
    //		System.out.println(min.getReceive() + "byte received. " + mout.getSend() + "byte send.");
    		return bss;
        }
        catch(IOException e)
        {
            socket.close();
            throw new IOException(e.getMessage());
        }
	}
    public static void uploadSmallServerInfo(InetSocketAddress address, SmallServerInfo ssi, int timeout)
    throws IOException
    {
        Socket socket = null;
        socket = new Socket();
        try
        {
            socket.setSoTimeout(timeout);
            socket.connect(address, timeout);
            DataInputStream in = new DataInputStream(new CompressedInputStream(socket.getInputStream()));
            DataOutputStream out = new DataOutputStream(new CompressedOutputStream(socket.getOutputStream()));
            out.writeByte(SERVERMANAGE_ADD);
            DataManager.outputData(out, ssi);
            out.flush();
            in.close();
            out.close();
            socket.close();
        }
        catch(IOException e)
        {
            socket.close();
            throw new IOException(e.getMessage());
        }
    }
    public static Collection<SmallServerInfo> downloadSmallServerinfos(InetSocketAddress address,  int timeout)
    throws IOException
    {
        Socket socket = null;
        socket = new Socket();
        try
        {
            socket.setSoTimeout(timeout);
            socket.connect(address, timeout);
            DataInputStream in = new DataInputStream(new CompressedInputStream(socket.getInputStream()));
            DataOutputStream out = new DataOutputStream(new CompressedOutputStream(socket.getOutputStream()));
            //DataManager dataManager = DataManager.getAvailableDataManager();
            out.writeByte(SERVERMANAGE_GET);
            out.flush();
            Collection<SmallServerInfo> ssis = inputSmallServerInfos(in);
            in.close();
            out.close();
            socket.close();
    //      System.out.println(min.getReceive() + "byte received. " + mout.getSend() + "byte send.");
            return ssis;
        }
        catch(IOException e)
        {
            socket.close();
            throw new IOException(e.getMessage());
        }
    }
    
    public static SmallBStageState getSmallBStageState(String host, int port, int timeout)
    throws IOException
    {
        Socket socket = null;
        socket = new Socket();
        try
        {
            InetSocketAddress address = new InetSocketAddress(host, port);
            socket.connect(address, timeout);
            DataInputStream in = new DataInputStream(new CompressedInputStream(socket.getInputStream()));
            DataOutputStream out = new DataOutputStream(new CompressedOutputStream(socket.getOutputStream()));
            String protocol = in.readUTF();
            SmallBStageState bss;
            out.writeByte(DataManager.REQUEST_SMALLBSTAGESTATE);
            out.flush();
            bss = DataManager.inputSmallBStageState(in);
            //bss.setProtocol(protocol);
            in.close();
            out.close();
            socket.close();
            return bss;
        }
        catch(IOException e)
        {
            socket.close();
            throw new IOException(e.getMessage());
        }
    }
    public static byte convBVectorToByte(BVector2f vec)
    {
        if (vec.getLength()==0)
            return 0;
        
        double rad = vec.x / vec.getLength();
        rad = Math.acos(rad);
        if (vec.y < 0)
        rad = -rad;
        double angle = (rad / Math.PI * 180)/2;
        return (byte)angle;
    }
    public static BVector2f convByteToBVector(byte src)
    {
        double ang = (double)src * 2;
        double rad = ang * Math.PI / 180;
        BVector2f vec = new BVector2f();
        vec.x = (float)Math.cos(rad);
        vec.y = (float)Math.sin(rad);
        return vec;
    }
}
