/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/01/11
 *
 */
package bakutan.common;

import bakutan.common.event.StageEvent;
import bakutan.common.util.BakutanClock;


/**
 * @author cattaka
 *
 */
public class EventThread
{
	private StageEvent tail;
	private StageEvent current;
	private Thread waitingThread;
	private boolean passNext = false;
	private int size = 0;
	
	public EventThread()
	{
		long l = BakutanClock.getCurrenttime();
		current = new StageEvent(l,l);
	}
	public StageEvent waitForNextEvent()
	{
		synchronized(this)
		{
			waitingThread = Thread.currentThread();
			while(true)
			{
				if (current.getNext() == null)
				{
					try
					{
						this.wait();
					}catch(InterruptedException e)
					{
						continue;
					}
				}else
				{
					long ct = current.getNext().getGenerationTime() - BakutanClock.getCurrenttime();//System.currentTimeMillis();
					try
					{
						if (ct > 0)
						{
							this.wait(ct);
						}
					}catch(InterruptedException e)
					{
						continue;
					}
                    StageEvent prev = current;
					current = current.getNext();
					//prev.setNext(null);
					prev.dispose();
                    size--;
					if (passNext)
					{
						passNext = false;
						continue;
					}
					waitingThread = null;
					return current;
				}
			}
		}
	}
	public long getCurrentTime()
	{
		return current.getGenerationTime();
	}
	public void putStageEvent(StageEvent be)
	{
        //debug code
//        if(be.getBodyEvent().getEventID() == 7 && be.getGenerationTime()-getCurrentTime() > 150000)
//            throw new RuntimeException();
	    //safety code
//        if (be.getGenerationTime() - getCurrentTime() > DataManager.MAX_EVENT_REPORT_INTERVAL)
//        {
//            return;
//        }
		synchronized(this)
		{
			current.addNear(be);
            size++;
			if (waitingThread != null)
			{
				waitingThread.interrupt();
			}
		}
	}
	public int getSize()
	{
		synchronized(this)
		{
            return size;
		}
	}
    public void deleteEventAbout(Body target)
    {
        synchronized (this)
        {
            StageEvent se = current.getNext();
            if (se == null)
                return;
            while(se.getNext() != null)
            {
                StageEvent se2 = se.getNext();
                Body[] bs = se2.getTargets();
//                for (int i=0;i<bs.length;i++)
//                {
//                    if (bs[i] == target)
//                    {
//                        se2.remove();
//                        continue outer;
//                    }
//                }
//                se = se.getNext();
                if (bs.length > 0 && bs[0] == target)
                {
                    size--;
                    se.removeNext();
                }
                else
                {
                    se = se.getNext();
                }
            }
        }
    }
    public int[] calcEventDistribution()
    {
        synchronized (this)
        {
            int[] dist = new int[40];
            StageEvent se = current;
            do
            {
                dist[se.getBodyEvent().getEventID()]++;
//                if (se.getBodyEvent().getEventID() == 7)
//                {
//                    System.out.print(""+(se.getGenerationTime()-getCurrentTime())+" : ");
//                }
                se = se.getNext();
            }while(se.getNext() != null);
//            System.out.println();
            return dist;
        }
    }
}
