/*
 * 作成日: 2007/01/17
 */
package bakutan.common;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.URL;
import java.util.zip.GZIPInputStream;

import bakutan.common.event.BodyAddEvent;
import bakutan.common.event.BodyEvent;
import bakutan.common.event.DisconnectEvent;
import bakutan.common.event.GameSetEvent;
import bakutan.common.util.BakutanClock;
import bakutan.common.util.CompressedInputStream;
import bakutan.common.util.CompressedOutputStream;
import bakutan.common.util.DataFlusher;
import bakutan.common.util.ImplementDataFlusher;
import bakutan.common.util.MesureInputStream;
import bakutan.common.util.MesureOutputStream;
import bakutan.common.util.NullDataFlusher;
import bakutan.common.util.TimerInputStream;

public class NetworkThread extends Thread
{
    public static final int MODE_CLIENT = 0;
    public static final int MODE_REPLAY = 1;
    public static final int MODE_DIRECT = 2;
    
    protected BUserInterface userinterface;
    protected DataManager dataManager;
    private GameOption gameOption;
    private String host;
    private int port;
    private LoginInfo loginInfo;
    private Socket socket;
    private MesureInputStream min;
    private MesureOutputStream mout;
    private DataInputStream in;
    private DataOutputStream out;
    private DataFlusher dataFlusher;
    private URL replayURL;
    private boolean goflag = false;
    private GameThread gameThread;
    private int mode = -1;
    
    public NetworkThread(BUserInterface userinterface, GameOption gameOption)
    {
        super("Network Thread");
        this.userinterface = userinterface;
        this.gameOption = gameOption;
    }
    public void run()
    {
        if (mode == -1)
            throw new RuntimeException("need to call from startSession()");
        
        boolean wait = false;
        outer: do
        {
            //connect to server
            if (wait)
            {
                //foolish busy wait
                for (int i=0;i<50;i++)
                {
                    try
                    {
                        Thread.sleep(100);
                    }catch(InterruptedException e)
                    {
                        e.printStackTrace();
                    }
                    if (!goflag)
                    {
                        break outer;
                    }
                }
            }
            wait = true;
            if (mode == MODE_CLIENT || mode == MODE_DIRECT)
            {
                if (mode == MODE_CLIENT)
                {
                    closeAll();
                    try
                    {
                        userinterface.outputMessage(userinterface.translate("connecting_to", new String[]{host,String.valueOf(port)}));
                        socket = new Socket();
                        InetSocketAddress isa = new InetSocketAddress(host, port);
                        socket.connect(isa, 3000);
                        min = new MesureInputStream(socket.getInputStream());
                        mout = new MesureOutputStream(socket.getOutputStream());
                        in = new DataInputStream(new CompressedInputStream(min,32768));
                        out = new DataOutputStream(new CompressedOutputStream(mout,32768));
                        userinterface.outputMessage(userinterface.translate("connected"));
                    }catch(IOException e)
                    {
                        userinterface.outputMessage(userinterface.translate("connection_refused"));
                        e.printStackTrace();
                        continue;
                    }
                }
                try
                {
                    String protocol = in.readUTF();
                    dataManager = DataManager.getAvailableDataManager(protocol);
                    if (dataManager == null)
                    {
                        userinterface.outputMessage(userinterface.translate("different_protocol_version", new String[]{DataManager.getCurrentProtocolVersion(), protocol}));
                        socket.close();
                        break outer;
                    }
                    out.writeByte(DataManager.REQUEST_LOGIN);
                    if (loginInfo.getMode() == LoginInfo.MODE_LOGIN)
                    {
                        out.writeByte(DataManager.LOGINMODE_USER);
                        out.writeUTF(loginInfo.getUsername());
                        out.writeUTF(loginInfo.getPassword());
                    }
                    else if (loginInfo.getMode() == LoginInfo.MODE_NEWACOUNT)
                    {
                        out.writeByte(DataManager.LOGINMODE_NEWUSER);
                        out.writeUTF(loginInfo.getUsername());
                        out.writeUTF(loginInfo.getPassword());
                        out.writeUTF(loginInfo.getNickname());
                        loginInfo.setMode(LoginInfo.MODE_LOGIN);
                    }
                    else
                    {
                        out.writeByte(DataManager.LOGINMODE_GUEST);
                        out.writeUTF(loginInfo.getNickname());
                    }
                    out.flush();
                    if (mode == MODE_CLIENT)
                        dataFlusher = new ImplementDataFlusher("DataFlusher Thread", out, Thread.MAX_PRIORITY);
                    else
                        dataFlusher = new NullDataFlusher(out);
                    dataFlusher.start();
                }catch(IOException e)
                {
                    e.printStackTrace();
                    continue;
                }
            }
            else if (mode == MODE_REPLAY)
            {
                closeAll();
                mout = null;
                out = null;
                try
                {
                    InputStream replayStream = replayURL.openStream();
                    min = new MesureInputStream(new TimerInputStream(new GZIPInputStream(replayStream,2<<12)));
                    in = new DataInputStream(min);
                    String protocol = in.readUTF();
                    dataManager = DataManager.getAvailableDataManager(protocol);
                    if (dataManager == null)
                    {
                        userinterface.outputMessage(userinterface.translate("different_version_replay", new String[]{DataManager.getCurrentProtocolVersion(), protocol}));
                        break outer;
                    }
                }
                catch(IOException e)
                {
                    e.printStackTrace();
                    break outer;
                }
                userinterface.outputMessage(userinterface.translate("replayfile_is_loaded"));
            }
            else
            {
                throw new RuntimeException("Unknown mode");
            }

            try
            {
                while(goflag)
                {
                    BStage bstage;
                    
                    //when replay mode ignore this block
                    if (mode == MODE_CLIENT || mode == MODE_DIRECT)
                    {
                        int status = in.readByte();
                        if (status != DataManager.AUTH_SUCCEED)
                        {
                            if (status == DataManager.AUTH_FAILED_INVALID)
                                userinterface.outputMessage(userinterface.translate("authorization_failed_invalid"));
                            else if (status == DataManager.AUTH_FAILED_EXIST)
                                userinterface.outputMessage(userinterface.translate("authorization_failed_exist"));
                            else if (status == DataManager.AUTH_TOO_MANY_CLIENTS)
                                userinterface.outputMessage(userinterface.translate("too_many_clients"));
                            else
                                userinterface.outputMessage(userinterface.translate("authorization_failed"));
                            break outer;
                        }
                    }
                    
                    bstage = dataManager.inputBStage(in);
                    if (gameThread != null)
                    {
                        gameThread.stopDrawThread();
                        gameThread = null;
                        System.gc();
                    }
                    gameThread = createGameThread();
                    //drawThread = new DrawableGameThread(dataManager);
                    Body[] bodys = dataManager.inputBodys(in, bstage);
                    for (int i=0;i<bodys.length;i++)
                    {
                        BodyAddEvent bae = new BodyAddEvent(bodys[i]);
                        ClientStageEvent cse = new ClientStageEvent(new int[0], bae);
                        gameThread.putStageEvent(cse);
                    }
                    if (dataFlusher != null)
                        gameThread.startDrawThread(bstage, dataFlusher);
                    else
                        gameThread.startDrawThread(bstage);
                    int l;
                    //while(socket.isConnected())
                    //long lastReceive = BakutanClock.getCurrenttime();
                    while((l = in.readByte()) != -1 && goflag)
                    {
                        int[] bodysID = new int[l];
                        for (int i=0;i<l;i++)
                        {
                            bodysID[i] = in.readShort();
                        }
                        BodyEvent be = dataManager.inputBodyEvent(in,bstage);
                        long currenttime = BakutanClock.getCurrenttime();
                        //System.out.println(currenttime - lastReceive);
                        //lastReceive = currenttime;
                        ClientStageEvent cse = new ClientStageEvent(bodysID, be);
                        cse.setGenerationTime(currenttime);
                        gameThread.putStageEvent(cse);
                        if (be instanceof GameSetEvent)
                        {
                            GameSetEvent gse = (GameSetEvent)be;
                            if (gse.getTimeLimit() == 0)
                            {
                                break;
                            }
                        }
                    }
                }
            }catch(IOException e)
            {
                //e.printStackTrace();
                ClientStageEvent disconnectEvent = new ClientStageEvent(new int[0], new DisconnectEvent());
                if (gameThread != null)
                    gameThread.putStageEvent(disconnectEvent);
                if (replayURL == null)
                    userinterface.outputMessage(userinterface.translate("connection_disconnected"));
                else
                    userinterface.outputMessage(userinterface.translate("end_of_replay_file"));
            }
        }
        while(gameOption.isForceReconnect());
    }
    public synchronized void startSession(String host, int port, LoginInfo loginInfo)
    {
        if (this.isAlive())
        {
            stopNetworkThread();
        }
        this.host = host;
        this.port = port;
        this.loginInfo = loginInfo;
        this.goflag = true;
        this.mode = MODE_CLIENT;
        this.start();
    }
    public synchronized void startSession(URL replay)
    {
        if (this.isAlive())
        {
            stopNetworkThread();
        }
        this.replayURL = replay;
        this.goflag = true;
        this.mode = MODE_REPLAY;
        this.start();
    }
    public synchronized void startSession(InputStream in, OutputStream out, LoginInfo loginInfo)
    {
        this.loginInfo = loginInfo;
        this.min = new MesureInputStream(in);
        this.mout = new MesureOutputStream(out);
        this.in = new DataInputStream(min);
        this.out = new DataOutputStream(mout);

        if (this.isAlive())
        {
            stopNetworkThread();
        }
        this.goflag = true;
        this.mode = MODE_DIRECT;
        this.start();
    }
    private void closeAll()
    {
        if (in != null)
        {
            try
            {
                in.close();
            }catch(IOException e)
            {
            }
        }
        if (dataFlusher != null)
        {
            dataFlusher.dispose();
        }
        if (socket != null && socket.isConnected())
        {
            try
            {
                socket.close();
            }catch(IOException e)
            {
            }
        }
    }
    public void stopNetworkThread()
    {
        this.goflag = false;
        closeAll();
        try
        {
            join();
        }catch(InterruptedException e)
        {
        }
        if (gameThread != null)
        {
            gameThread.stopDrawThread();
        }
    }
    public int getSend()
    {
        if (mout == null)
            return 0;
        
        return mout.getSend();
    }
    public int getReceive()
    {
        if (min == null)
            return 0;

        return min.getReceive();
    }
    public void resetSend()
    {
        if (mout != null)
            mout.resetSend();
    }
    public void resetReceive()
    {
        if (min != null)
            min.resetReceive();
    }
    public void setDrawOptionEx(boolean value, int type)
    {
        gameThread.setDrawOptionEx(value, type);
    }
    public void setMessage(String message)
    {
        gameThread.setMessage(message);
    }
    public boolean checkDrawOption(int type)
    {
        return gameThread.checkDrawOption(type);
    }
    protected GameThread createGameThread()
    {
        return new GameThread(dataManager, userinterface, gameOption);
    }
}
