/*
 * 作成日: 2007/01/17
 */
package bakutan.common;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.xml.sax.SAXParseException;

import bakutan.client.ClientXMLConverter;
import bakutan.common.util.BAudio;
import bakutan.common.util.LanguageTranslator;
import bakutan.common.util.NullBAudio;

public class SimpleUserInterface implements BUserInterface
{
    private LanguageTranslator languageTranslator;
    private File workDir = null;
    private DateFormat dateFormat = new SimpleDateFormat();
    private BAudio baudio = new NullBAudio();
    private boolean outputDate = false;
    
    public SimpleUserInterface(File workDir)
    {
        super();
        this.languageTranslator = createLanguageTranslator();
        this.workDir = workDir;
    }
    public SimpleUserInterface()
    {
        super();
        this.languageTranslator = createLanguageTranslator();
    }
    public static LanguageTranslator createLanguageTranslator()
    {
        URL in = LanguageTranslator.class.getClassLoader().getResource("PhraseDic_"+ Locale.getDefault().getLanguage()+".xml");
        if (in == null)
        in = LanguageTranslator.class.getClassLoader().getResource("PhraseDic.xml");
        if (in == null)
            return new LanguageTranslator();
        try
        {
            return ClientXMLConverter.createLanguageTranslator(XMLConverter.readXML(in));
        }catch(InvalidStructureException e)
        {
            e.printStackTrace();
            return new LanguageTranslator();
        }catch(SAXParseException e)
        {
            e.printStackTrace();
            return new LanguageTranslator();
        }catch(IOException e)
        {
            e.printStackTrace();
            return new LanguageTranslator();
        }
    }

    public void outputMessage(String message)
    {
        if (outputDate)
            System.out.println(dateFormat.format(new Date()) + " : " + message);
        else
            System.out.println(dateFormat.format(message));
    }

    public String translate(String key)
    {
        return languageTranslator.translate(key);
    }

    public String translate(String key, String[] opt)
    {
        return languageTranslator.translate(key, opt);
    }

    public File getWorkDir()
    {
        return workDir;
    }

    public BAudio getBAudio()
    {
        return baudio;
    }
    public boolean isOutputDate()
    {
        return outputDate;
    }
    public void setOutputDate(boolean outputDate)
    {
        this.outputDate = outputDate;
    }
    
}
