/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

package bakutan.common;

import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.DOMImplementation;
import org.xml.sax.SAXParseException;

import bakutan.common.serverinfo.ServerInfoManagerAddress;
import bakutan.common.util.LanguageTranslator;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.IOException;
import java.io.File;
import java.io.FileOutputStream;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;

public class XMLConverter
{
    public static Document createDocument(ServerInfoManagerAddress sima)
    {
        Document document = createNewDocument("ServerInfoManagerAddress");
        Element root = document.getDocumentElement();
        root.setAttribute("HostName",sima.getHostName());
        root.setAttribute("Port",String.valueOf(sima.getPort()));
        return document;
    }
    public static ServerInfoManagerAddress createServerInfoManagerAddress(Element element)
    throws InvalidStructureException
    {
        try
        {
            if (!(element.getTagName().equals("ServerInfoManagerAddress")))
                throw new InvalidStructureException("this is not ServerInfoManagerAddress.");

            String hostname = element.getAttribute("HostName");

            int port = DataManager.DEFAULT_SIMA_PORT;
            String arg = element.getAttribute("Port");
            try
            {
                port = Integer.valueOf(arg).intValue();
            }catch(NumberFormatException e)
            {
            }
            
            return new ServerInfoManagerAddress(hostname, port);
        }catch (NullPointerException e)
        {
            throw new InvalidStructureException();
        }
    }
    
	public static Document createDocument(LanguageTranslator lt)
	{
		Document document = createNewDocument("PhraseDic");
		Element root = document.getDocumentElement();
		Element element;

		String[] keys = lt.getKeys();
		for (int i=0;i<keys.length;i++)
		{
			element = document.createElement("Phrase");
			element.setAttribute("key",keys[i]);
			element.setAttribute("value",lt.getValue(keys[i]));
			root.appendChild(element);
		}
		return document;
	}

    public static Document createNewDocument(String qualifiedName)
    {
        try
        {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();         
            DOMImplementation domImpl=builder.getDOMImplementation();
            return domImpl.createDocument("",qualifiedName,null);
        }catch(Exception e)
        {
            throw new RuntimeException("createNewDocument:" + e.toString());
        }
    }

    public static Element readXML(URL url)
    throws IOException,SAXParseException
    {
    	URLConnection uc = url.openConnection();
    	InputStream in = uc.getInputStream();
    	Element el = readXML(in);
    	in.close();
    	return el;
    }
 
    public static Element readXML(File file)
    throws IOException,SAXParseException
    {
        try
        {
            Document document= DocumentBuilderFactory.newInstance()
                .newDocumentBuilder().parse(file);
            return document.getDocumentElement();
        }catch(Exception e)
        {
            throw new IOException("XMLConverter:Invalid file:"+ file);
        }
    }
    
    public static Element readXML(InputStream in)
    throws IOException,SAXParseException
    {
        try
        {
            Document document= DocumentBuilderFactory.newInstance()
                .newDocumentBuilder().parse(in);
            return document.getDocumentElement();
        }catch(Exception e)
        {
        	//e.printStackTrace();
            throw new IOException("XMLConverter:InputStream error");
        }
    }

    public static void writeXML(File file,Document document)
        throws IOException
    {
        try
        {
            TransformerFactory transFactory = TransformerFactory.newInstance();
            Transformer transformer = transFactory.newTransformer();
            
            DOMSource source = new DOMSource(document);
            FileOutputStream out = new FileOutputStream(file); 
            StreamResult result = new StreamResult(out); 
            transformer.transform(source, result);
            
            out.close();
        }catch(Exception e)
        {
            throw new IOException(e.toString());
        }
    }

    public static void main(String[] args)
    {
        if (args.length < 1)
        {
            System.out.println("arg1:filename");
            return;
        }
        try
        {
            BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
            File file = new File(args[0]);

            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance(); 
            DocumentBuilder builder = factory.newDocumentBuilder(); 
        
            DOMImplementation domImpl=builder.getDOMImplementation();
            System.out.print("qualifiedName:");
            String qualifiedName = in.readLine();
            Document document = domImpl.createDocument("",qualifiedName,null);

            Element element = document.getDocumentElement();
            while(true)
            {
                System.out.print("nodeName:");
                String nodeName = in.readLine();
                if (nodeName.length()==0)
                    break;
                System.out.print("nodeValue:");
                String nodeValue = in.readLine();
                Element node = document.createElement(nodeName);
                //Text text = document.createTextNode(nodeValue);
                //node.appendChild(text);
                node.setAttribute("value",nodeValue);
                element.appendChild(node);
            }


            TransformerFactory transFactory = TransformerFactory.newInstance();
            Transformer transformer = transFactory.newTransformer();
        
            DOMSource source = new DOMSource(document);
            FileOutputStream out = new FileOutputStream(file); 
            StreamResult result = new StreamResult(out); 
            transformer.transform(source, result);

            out.close();
            in.close();
        }catch(Exception e)
        {
            e.printStackTrace();
        }
    }
    public static BotInfo creataBotInfo(Element element)
    throws InvalidStructureException
    {
        try
        {
            if (!(element.getTagName().equals("BotInfo")))
                throw new InvalidStructureException("this is not BotInfo.");

            String fpsLimitStr = element.getAttribute("FPSLimit");
            String aiClassName = element.getAttribute("AIClassName");
            String nickname = element.getAttribute("NickName");
            String username = element.getAttribute("UserName");
            String password = element.getAttribute("Password");

            BotInfo botInfo = new BotInfo();
            try
            {
                botInfo.setFpsLimit(Integer.parseInt(fpsLimitStr));
            }
            catch(NumberFormatException e)
            {
            }
            botInfo.setAiClassName(aiClassName);
            if (nickname.length() > 0)
            {
                LoginInfo loginInfo = new LoginInfo();
                loginInfo.setNickname(nickname);
                loginInfo.setMode(LoginInfo.MODE_GUEST);
                botInfo.setLoginInfo(loginInfo);
            }
            else if (username.length() > 0 && password.length() > 0)
            {
                LoginInfo loginInfo = new LoginInfo();
                loginInfo.setUsername(username);
                loginInfo.setPassword(password);
                loginInfo.setMode(LoginInfo.MODE_LOGIN);
                botInfo.setLoginInfo(loginInfo);
            }
            else
            {
                throw new InvalidStructureException("BotInfo needs NickName or UserName and Password.");
            }
            
            return botInfo;
        }catch (NullPointerException e)
        {
            throw new InvalidStructureException();
        }
    }
}
