/*
 * 作成日: 2006/11/24
 *
 * 
 * ウィンドウ - 設定 - Java - コード・スタイル - コード・テンプレート
 */
package bakutan.common.ai;

import java.io.File;
import java.io.IOException;

import bakutan.common.BUserInterface;

/**
 * クライアントプログラムで使用されるAIのインターフェースとなるクラス
 */
public abstract class AIInterface
{
    private BUserInterface userinterface = null;
    
    /**
     * このAIの名前を返す。
     * @return このAIの名前 
     */
    abstract public String getAIName();
    
    /**
     * ステージの状態を入力する。
     * getAIOutput()が呼び出される前に必ずこのメソッドによりステージの情報を与えられる。
     * @param mybody 自キャラの情報
     * @param obstage ステージの情報。
     * @param currenttime 現在の時刻を表すlong値。
     * @param actionlock アクションがロックされる時間で単位はミリ秒。
     * この値が０以下の時は即座にアクションを行えるが、それ以上の時はアクションが行えない。
     * @param energy 現在の残りエネルギー
     * @see #getAIOutput() getAIOutput()
     */
    abstract public void input(BodyInfoForAI mybody, BStageInfoForAI obstage, long currenttime, int actionlock, float energy);
    
    /**
     * 自キャラの操作を表すAIOutputを返す。
     * このメソッドは呼び出される前に必ずinputメソッドによって情報が与えられる。
     * @return 自キャラの操作を表すAIOutput。
     * @see #input(BodyInfoForAI, BStageInfoForAI, long, int, float) input
     */
    abstract public AIOutput getAIOutput();

    /**
     * isRecordAI()がtrueを返すときに学習用にユーザーの入力が与えられる。
     * @param currenttime 現在の時刻
     * @param out ユーザーの入力を表すAIOutput
     */
    public void inputDesired(long currenttime,AIOutput out)
    {
    }
    
    /**
     * AIの動作についてファイルを読み込むことが可能ならばtrueを返す。
     * @return ファイルを読み込むことが可能ならばtrue、そうでなければfalseを返す。
     * @see #loadAI(File) loadAI(File)
     */
    public boolean isFileLoadable()
    {
        return false;
    }
    
    /**
     * 爆裂タンクの作業用ディレクトリを得る。
     * getUserInterface().getWorkDIr()と同値である。
     * @return 作業用ディレクトリ
     */
    public File getWorkDir()
    {
        if (getUserInterface() == null)
            return null;
        return getUserInterface().getWorkDir();
    }
    
    /**
     * 爆裂タンクのユーザーインターフェースを取得する。
     * このインターフェースから作業用ディレクトリの取得や
     * メッセージの出力、変換が行える。
     * @return 爆裂タンクのユーザーインターフェース
     */
    public BUserInterface getUserInterface()
    {
        return userinterface;
    }
    /**
     * 爆裂タンクのユーザーインターフェースを設定する。
     * これはクライアントプログラムにより与えられる。
     * @param userinterface ユーザーインターフェース
     */
    public void setUserInterface(BUserInterface userinterface)
    {
        this.userinterface = userinterface;
    }

    /**
     * 学習用にユーザーの入力をレコード中であるかどうかを返す。
     * @return レコード中ならばtrueを返す。そうでなければfalseを返す。
     */
    public boolean isRecordAI()
    {
        return false;
    }

    /**
     * クライアントがレコード操作を行ったときに呼び出される。
     * @param recordAI レコードを開始するならばtrue、終了するならfalseを返す。
     */
    public void setRecordAI(boolean recordAI)
    {
    }
    
    /**
     * isFileLoadable()メソッドがtrueを返すとき、クライアントのAI読み込みの操作により
     * このメソッドにファイルが与えられる。
     * @param file 与えられたファイル
     * @throws IOException 入出力エラーが起こったときにスローされる。
     * @throws InvalidAIData ファイルが不正な場合にスローされる
     * @see #isFileLoadable() isFileLoadable()
     */
    public void loadAI(File file) throws IOException, InvalidAIData
    {
        return;
    }
}
