/*
 * 作成日: 2006/12/27
 */
package bakutan.common.ai;

import bakutan.common.BStage;
import bakutan.common.Body;
import bakutan.common.math.BVector2f;

/**
 * @author cattaka
 *
 * AIInterfaceに与えられるステージの状態を得るためのクラス。
 * 各Body(キャラクタ)の情報はgetBodyInfoForAIs()メソッド、getBodyInfoForAI()メソッドによって
 * 得ることができる。
 */
public class BStageInfoForAI
{
    private BStage bstage;
    public BStageInfoForAI(BStage bstage)
    {
        super();
        this.bstage = bstage;
    }
    /**
     * ステージに存在するすべてのBodyの情報を返す。
     * @return すべてのBodyのBodyInfoForAIの配列
     */
    public BodyInfoForAI[] getBodyInfoForAIs()
    {
        Body[] bodys = bstage.getBodys();
        BodyInfoForAI[] bifas = new BodyInfoForAI[bodys.length]; 
        for (int i=0;i<bodys.length;i++)
        {
            bifas[i] = new BodyInfoForAI(bodys[i]);
        }
        return bifas;
    }
    
    /**
     * ステージに存在する指定されたタイプのBodyの情報を返す。
     * @param type 指定するタイプ
     * @return 指定されたタイプのBodyInfoForAIの配列
     */
    public BodyInfoForAI[] getBodyInfoForAIsByType(byte type)
    {
        Body[] bodys = bstage.getBodysByType(type);
        BodyInfoForAI[] bifas = new BodyInfoForAI[bodys.length]; 
        for (int i=0;i<bodys.length;i++)
        {
            bifas[i] = new BodyInfoForAI(bodys[i]);
        }
        return bifas;
    }
    /**
     * 指定したIDを持つBodyの情報を返す。
     * @param id 指定するID
     * @return 指定したIDを持つBodyのBodyInfoForAi、指定したIDが存在しない場合はnullを返す。
     */
    public BodyInfoForAI getBodyInfoForAI(int id)
    {
        Body body = bstage.getBody(id);
        if (body == null)
            return null;
        return new BodyInfoForAI(body);
    }
    /**
     * ステージ上に存在しているBodyの数を返す。
     * @return ステージ上に存在しているBodyの数
     */
    public int getBodysNum()
    {
        return bstage.getBodysNum();
    }
    /**
     * ゲームでサポートするすべてのBodyのタイプの配列を返す。
     * @return Bodyのタイプの配列
     */
    public byte[] getBodyTypeArray()
    {
        return bstage.getBodyTypeArray();
    }
    /**
     * ステージの範囲を表すベクトルを返す。
     * 範囲は(0,0)からこのベクトルが表す座標までの長方形である。
     * @return ステージの範囲を表すベクトル
     */
    public BVector2f getSize()
    {
        return new BVector2f(bstage.getSize());
    }
}
