/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/01/18
 */
package bakutan.common.graphics;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.Image;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;

import javax.imageio.ImageIO;

import bakutan.common.Body;
import bakutan.common.math.BVector2f;

/**
 * Eight Direction ver
 * @author cattaka
 */
public class CharacterImageED implements CharacterImage
{
	private Image[][] imgs;
	private int width = 32;
	private int height = 32;
    
	public CharacterImageED(GraphicsConfiguration gc, InputStream in)
	throws IOException
	{
		BufferedImage img = ImageIO.read(in);
//		BufferedImage tmp = new BufferedImage(img.getWidth(), img.getHeight(), BufferedImage.TYPE_INT_ARGB_PRE);
//		Graphics g = tmp.getGraphics();
//		g.drawImage(img,0,0,null);
//        g.dispose();
        
		createImage(gc, img);
	}
	private void createImage(GraphicsConfiguration gc, BufferedImage img)
	{
		Color bgcolor;
		bgcolor = new Color(img.getRGB(0,0));
		for (int i=0;i<17;i++)
		{
			int y = i * height;
			Image[] tmp1 = new Image[2];
            Image[] tmp2 = new Image[2];
            for (int j=0;j<2;j++)
            {
                int x = j * width;
                if (gc != null)
                    tmp1[j] = gc.createCompatibleImage(width,height,Transparency.BITMASK);
                else
                    tmp1[j] = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB_PRE);
                Graphics g = tmp1[j].getGraphics();
                g.drawImage(img, 0, 0, width, height, x, y, x+width, y+height,null);
                g.dispose();
            }
            for (int j=0;j<2;j++)
            {
                int x = (j+2) * width;
                if (gc != null)
                    tmp2[j] = gc.createCompatibleImage(width,height,Transparency.BITMASK);
                else
                    tmp2[j] = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB_PRE);
                Graphics g = tmp1[j].getGraphics();
                g.drawImage(img, 0, 0, width, height, x, y, x+width, y+height,null);
                g.dispose();
            }
            
//			tmp1 = new BufferedImage[]{img.getSubimage(0,y,width,height),img.getSubimage(width,y,width,height)};
//			tmp2 = new BufferedImage[]{img.getSubimage(width * 2,y,width,height),img.getSubimage(width*3,y,width,height)};
			switch(i)
			{
			case 0:
				imgs[WIN] = tmp1;
				imgs[WINSTOP] = tmp2;
				break;
			case 1:
				imgs[DEADRIGHT] = tmp1;
				imgs[DEADRIGHTSTOP] = tmp2;
				break;
			case 2:
				imgs[DEADUP] = tmp1;
				imgs[DEADUPSTOP] = tmp2;
				break;
			case 3:
				imgs[DEADLEFT] = tmp1;
				imgs[DEADLEFTSTOP] = tmp2;
				break;
			case 4:
				imgs[DEADDOWN] = tmp1;
				imgs[DEADDOWNSTOP] = tmp2;
				imgs[DEAD] = tmp1;
				imgs[DEADSTOP] = tmp2;
				break;
			case 5:
				imgs[DEADUPRIGHT] = tmp1;
				imgs[DEADUPRIGHTSTOP] = tmp2;
				break;
			case 6:
				imgs[DEADUPLEFT] = tmp1;
				imgs[DEADUPLEFTSTOP] = tmp2;
				break;
			case 7:
				imgs[DEADDOWNLEFT] = tmp1;
				imgs[DEADDOWNLEFTSTOP] = tmp2;
				break;
			case 8:
				imgs[DEADDOWNRIGHT] = tmp1;
				imgs[DEADDOWNRIGHTSTOP] = tmp2;
				break;
			case 9:
				imgs[RIGHT] = tmp1;
				imgs[MOVERIGHT] = tmp2;
				break;
			case 10:
				imgs[UP] = tmp1;
				imgs[MOVEUP] = tmp2;
				break;
			case 11:
				imgs[LEFT] = tmp1;
				imgs[MOVELEFT] = tmp2;
				break;
			case 12:
				imgs[DOWN] = tmp1;
				imgs[MOVEDOWN] = tmp2;
				break;
			case 13:
				imgs[UPRIGHT] = tmp1;
				imgs[MOVEUPRIGHT] = tmp2;
				break;
			case 14:
				imgs[UPLEFT] = tmp1;
				imgs[MOVEUPLEFT] = tmp2;
				break;
			case 15:
				imgs[DOWNLEFT] = tmp1;
				imgs[MOVEDOWNLEFT] = tmp2;
				break;
			case 16:
				imgs[DOWNRIGHT] = tmp1;
				imgs[MOVEDOWNRIGHT] = tmp2;
				break;
			}
		}
		return;
	}
	public Image[] getImages(int state)
	{
		return imgs[state];
	}
	public Image getImages(Body body, int count)
	{
		if (body.isState(Body.STATE_WIN))
		{
			if (count < imgs[WIN].length)
			{
				return imgs[WIN][count];
			}else
			{
				return imgs[WINSTOP][count % imgs[WINSTOP].length];
			}
		}

		int dir = getDirNum(body.getDirection());
		if (body.isState(Body.STATE_DEAD))
		{
			Image[] tmp;
			switch(dir)
			{
			case 0:
				tmp = imgs[DEADRIGHT];
				break;
			case 1:
				tmp = imgs[DEADUPRIGHT];
				break;
			case 2:
				tmp = imgs[DEADUP];
				break;
			case 3:
				tmp = imgs[DEADUPLEFT];
				break;
			case 4:
				tmp = imgs[DEADLEFT];
				break;
			case 5:
				tmp = imgs[DEADDOWNLEFT];
				break;
			case 6:
				tmp = imgs[DEADDOWN];
				break;
			case 7:
				tmp = imgs[DEADDOWNRIGHT];
				break;
			default:
				tmp = imgs[DEAD];
			}
			
			if (count < tmp.length)
			{
				return tmp[count];
			}else
			{
				switch(dir)
				{
				case 0:
					tmp = imgs[DEADRIGHTSTOP];
					break;
				case 1:
					tmp = imgs[DEADUPRIGHTSTOP];
					break;
				case 2:
					tmp = imgs[DEADUPSTOP];
					break;
				case 3:
					tmp = imgs[DEADUPLEFTSTOP];
					break;
				case 4:
					tmp = imgs[DEADLEFTSTOP];
					break;
				case 5:
					tmp = imgs[DEADDOWNLEFTSTOP];
					break;
				case 6:
					tmp = imgs[DEADDOWNSTOP];
					break;
				case 7:
					tmp = imgs[DEADDOWNRIGHTSTOP];
					break;
				default:
					tmp = imgs[DEADSTOP];
				}
				return tmp[count % tmp.length];
			}
		}else
		{
			int imageMode;
			if (!body.getVelocity().isZeroVector())
			{
				switch(dir)
				{
				case 0:
					imageMode = MOVERIGHT;
					break;
				case 1:
					imageMode = MOVEUPRIGHT;
					break;
				case 2:
					imageMode = MOVEUP;
					break;
				case 3:
					imageMode = MOVEUPLEFT;
					break;
				case 4:
					imageMode = MOVELEFT;
					break;
				case 5:
					imageMode = MOVEDOWNLEFT;
					break;
				case 6:
					imageMode = MOVEDOWN;
					break;
				case 7:
					imageMode = MOVEDOWNRIGHT;
					break;
				default:
					imageMode = MOVEDOWN;
				}
			}
			else
			{
				switch(dir)
				{
				case 0:
					imageMode = RIGHT;
					break;
				case 1:
					imageMode = UPRIGHT;
					break;
				case 2:
					imageMode = UP;
					break;
				case 3:
					imageMode = UPLEFT;
					break;
				case 4:
					imageMode = LEFT;
					break;
				case 5:
					imageMode = DOWNLEFT;
					break;
				case 6:
					imageMode = DOWN;
					break;
				case 7:
					imageMode = DOWNRIGHT;
					break;
				default:
					imageMode = DOWN;
				}
			}
			return imgs[imageMode][count % imgs[imageMode].length];
		}
	}
	private int getDirNum(BVector2f direction)
	{
	    BVector2f v = new BVector2f(direction);
        if ( v.isZeroVector())
            return 0;
        v.normalize();
        float l = v.getLength();
        if (l == 0)
            return 6;
        double rad = Math.acos(v.x/l);
        if (v.y > 0)
            rad = 2 * Math.PI - rad;
        rad += (2 * Math.PI) / 16;
        int n = (int)(rad / (2 * Math.PI) * 8);
        if (n > 7)
            n = 0;
        return n;
//		BVector2f v = new BVector2f(direction);
//		v.normalize();
//		float a = (float)(Math.sqrt(3) / 2);
//		float b = 1 / 2;
//		if (v.x >= a)
//			return 0;
//		else if (v.x <= -a)
//			return 4;
//		else if (v.y < 0)
//		{
//			if (v.x > b)
//				return 1;
//			else if (v.x < -b)
//				return 3;
//			else
//				return 2;
//		}else
//		{
//			if (v.x > b)
//				return 7;
//			else if (v.x < -b)
//				return 5;
//			else
//				return 6;
//		}
	}
	public int getHeight()
	{
		return height;
	}
	public int getWidth()
	{
		return width;
	}
    public void dispose()
    {        
        for (int i=0;i<imgs.length;i++)
        {
            for (int j=0;j<imgs[i].length;j++)
            {
                imgs[i][j].flush();
            }
        }
    }
}
