/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/01/18
 */
package bakutan.common.graphics;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.Image;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;

import javax.imageio.ImageIO;

import bakutan.common.Body;
import bakutan.common.math.BVector2f;

/**
 * Four Direction ver
 * @author cattaka
 */
public class CharacterImageND implements CharacterImage
{
	private Image[][] imgs;
	private int width = 32;
	private int height = 32;
	
	public CharacterImageND(GraphicsConfiguration gc, InputStream in)
	throws IOException
	{
		BufferedImage img = ImageIO.read(in);
		//BufferedImage tmp = new BufferedImage(img.getWidth(), img.getHeight(), BufferedImage.TYPE_INT_ARGB_PRE);
//		Graphics g = tmp.getGraphics();
//		g.drawImage(img,0,0,null);
//        g.dispose();
		createImage(gc, img);
	}
	private void createImage(GraphicsConfiguration gc, BufferedImage img)
	{
		Color bgcolor;
		bgcolor = new Color(img.getRGB(0,0));
		imgs = new Image[6][2];
		for (int i=0;i<imgs.length;i++)
		{
			int y = i * height;
            for (int j=0;j<2;j++)
            {
                int x = width * j;
                if (gc != null)
                    imgs[i][j] = gc.createCompatibleImage(width,height,Transparency.BITMASK);
                else
                    imgs[i][j] = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB_PRE);
                //System.out.println(""+imgs[i][j]);
                //System.out.println(imgs[i][j]);
                //imgs[i][j] = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB_PRE);
                Graphics g = imgs[i][j].getGraphics();
                g.drawImage(img, 0, 0, width, height, x, y,x+width, y+height,null);
                g.dispose();
            }
		}
	}
	public Image[] getImages(int state)
	{
	    if (state == MOVEDOWN
                || state == MOVEDOWNLEFT
                || state == MOVELEFT
                || state == MOVEUPLEFT
                || state == MOVEUP
                || state == MOVEUPRIGHT
                || state == MOVERIGHT
                || state == MOVEDOWNRIGHT
                )
            return imgs[5];
        else if (state == DOWN
                || state == DOWNLEFT
                || state == LEFT
                || state == UPLEFT
                || state == UP
                || state == UPRIGHT
                || state == RIGHT
                || state == DOWNRIGHT
                )
            return imgs[4];
        else
            return imgs[state];
	}
	public Image getImages(Body body, int count)
	{
		if (body.isState(Body.STATE_WIN))
		{
		    return imgs[WINSTOP][count % imgs[WINSTOP].length];
		}
		else if (body.isState(Body.STATE_DEAD))
		{
		    return imgs[DEADSTOP][count % imgs[DEADSTOP].length];
		}else
		{
			BVector2f vec = body.getDirection();
			int imageMode = DOWN;
			if (vec.isZeroVector())
			{
                return imgs[4][count % imgs[4].length];
			}
			else
			{
                return imgs[5][count % imgs[5].length];
			}
		}
	}
	public int getHeight()
	{
		return height;
	}
	public int getWidth()
	{
		return width;
	}
//    private int getDirNum(BVector2f direction)
//    {
//        BVector2f v = new BVector2f(direction);
//        if (v.isZeroVector())
//            return 0;
//        v.normalize();
//        float l = v.getLength();
//        if (l == 0)
//            return 3;
//        double rad = Math.acos(v.x/l);
//        if (v.y > 0)
//            rad = 2 * Math.PI - rad;
//        rad += (2 * Math.PI) / 8;
//        int n = (int)(rad / (2 * Math.PI) * 4);
//        if (n > 3)
//            n = 0;
//        return n;
//    }
    public void dispose()
    {
        for (int i=0;i<imgs.length;i++)
        {
            for (int j=0;j<imgs[i].length;j++)
            {
                imgs[i][j].flush();
            }
        }
    }
}
