/*
 * 作成日: 2007/01/04
 */
package bakutan.common.graphics;

import java.awt.Dimension;
import java.awt.Graphics;
import java.util.TreeSet;

import bakutan.common.BStage;
import bakutan.common.Body;
import bakutan.common.DataManager;
import bakutan.common.math.BVector2f;

public class EffectBundle
{
    private TreeSet<EffectDrawer> preEffectSet = new TreeSet<EffectDrawer>(new EffectComparator());
    private TreeSet<EffectDrawer> postEffectSet = new TreeSet<EffectDrawer>(new EffectComparator());
    private int effectLevel = 0;
    
    public EffectBundle()
    {
        super();
    }
    public void putPreEffect(EffectDrawer effect)
    {
        preEffectSet.add(effect);
    }
    public void putPostEffect(EffectDrawer effect)
    {
        postEffectSet.add(effect);
    }
    public void drawPreEffect(Graphics bg, long currenttime, Dimension size, BStage bstage, Body myBody, Body myBodyGhost)
    {
        EffectDrawer[] eds = new EffectDrawer[preEffectSet.size()]; 
        preEffectSet.toArray(eds); 
        for (int i=0;i<eds.length;i++)
        {
            EffectDrawer ed = eds[i];
            Body target = ed.getTarget();
            if (target != null && target == myBody)
                ed.setTarget(myBodyGhost);
            ed.update(currenttime);
            if (ed.isFinish(currenttime))
            {
                preEffectSet.remove(ed);
            }
            else if (ed.isInSite(bstage.getBufferCenter(), bstage.getDrawMask()))
            {
                BVector2f pos = ed.getPosition();
                int x = (int) (pos.x * DataManager.BLOCKSIZE);
                int y = (int) (pos.y * DataManager.BLOCKSIZE);
                bg.translate(x,y);
                ed.drawCharacter(bg,currenttime,effectLevel);
                bg.translate(-x,-y);
                if (target != null)
                    ed.setTarget(target);

            }
        }
        //System.out.println("pre :" + preEffectSet.size());
    }
    public void drawPostEffect(Graphics bg, long currenttime, Dimension size, BStage bstage, Body myBody, Body myBodyGhost)
    {
        EffectDrawer[] eds = new EffectDrawer[postEffectSet.size()]; 
        postEffectSet.toArray(eds); 
        for (int i=0;i<eds.length;i++)
        {
            EffectDrawer ed = eds[i];
            Body target = ed.getTarget();
            if (target != null && target == myBody)
                ed.setTarget(myBodyGhost);
            ed.update(currenttime);
            if (ed.isFinish(currenttime))
            {
                postEffectSet.remove(ed);
            }
            else if (ed.isInSite(bstage.getBufferCenter(), bstage.getDrawMask()))
            {
                BVector2f pos = ed.getPosition();
                int x = (int) (pos.x * DataManager.BLOCKSIZE);
                int y = (int) (pos.y * DataManager.BLOCKSIZE);
                bg.translate(x,y);
                ed.drawCharacter(bg,currenttime,effectLevel);
                bg.translate(-x,-y);
                if (target != null)
                    ed.setTarget(target);
            }
        }
        //System.out.println("post:" + postEffectSet.size());
    }
    public int getEffectLevel()
    {
        return effectLevel;
    }
    public void setEffectLevel(int effectLevel)
    {
        this.effectLevel = effectLevel;
        if (this.effectLevel < 0)
            this.effectLevel = 0;
        if (this.effectLevel > 5)
            this.effectLevel = 5;
    }
    
//    private void updateAll(long currenttime)
//    {
//        Iterator<EffectDrawer> ite = effectSet.iterator();
//        while (ite.hasNext())
//        {
//            EffectDrawer ed = ite.next();
//            ed.update(currenttime);
//        }
//    }
}
