/*
 * 作成日: 2007/01/04
 */
package bakutan.common.graphics;

import java.awt.Graphics;

import bakutan.common.Body;
import bakutan.common.math.BMath;
import bakutan.common.math.BShape;
import bakutan.common.math.BVector2f;

public abstract class EffectDrawer
{
    private BVector2f position = new BVector2f();
    private BVector2f velocity = new BVector2f();
    private float radius = 1f; 
    private long lastUpdate = 0l;
    private long startTime = 0l;
    private Body target;
    
    public EffectDrawer(long startTime)
    {
        super();
        this.startTime = startTime;
    }

    public EffectDrawer(long startTime, Body target)
    {
        super();
        this.startTime = startTime;
        this.target = target;;
    }

    public long getLastUpdate()
    {
        return lastUpdate;
    }


    public void setLastUpdate(long lastUpdate)
    {
        this.lastUpdate = lastUpdate;
    }


    public float getRadius()
    {
        return radius;
    }


    public void setRadius(float radius)
    {
        this.radius = radius;
    }


    public BVector2f getPosition()
    {
        return position;
    }

    public void setPosition(BVector2f position)
    {
        this.position.set(position);
    }

    public BVector2f getVelocity()
    {
        return velocity;
    }

    public void setVelocity(BVector2f velocity)
    {
        this.velocity.set(velocity);
    }
    public void update(long currenttime)
    {
        if (target != null)
        {
            setPosition(target.getPosition());
            setVelocity(target.getVelocity());
        }
        else
        {
            long dl = currenttime - lastUpdate;
            float d = (float)(currenttime - lastUpdate) * 0.001f;
            this.lastUpdate = currenttime;
            position.x += velocity.x * d;
            position.y += velocity.y * d;
        }
    }
    
    public boolean isInSite(BVector2f pos, BShape shape)
    {
        if (target != null)
        {
            return BMath.isIntersectCirclexCirle(pos, shape.getRadius(), target.getPosition(), getRadius());
        }
        else
        {
            return BMath.isIntersectCirclexCirle(pos, shape.getRadius(), getPosition(), getRadius());
        }
    }
    
    public long getStartTime()
    {
        return startTime;
    }

    public void setStartTime(long startTime)
    {
        this.startTime = startTime;
    }


    public Body getTarget()
    {
        return target;
    }

    public void setTarget(Body target)
    {
        this.target = target;
    }

    abstract public void drawCharacter(Graphics g , long currenttime, int effectLevel);
    abstract public boolean isFinish(long currenttime);
}
