/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/01/14
 */
package bakutan.common.math;


/**
 * @author cattaka
 * ベクトルの各種演算を行うためのクラス。
 */
public class BMath
{
//    public static final int CROSS = 0;
//    public static final int PARALLEL = 1;
//    public static final int ONPLAIN = 2;
//    public static final int ININNER = 3;
//    public static final int INOUTER = 4;
//    public static final int ONLINE = 5;
    
//    public static final int LEFT = 6;
//    public static final int RIGHT = 7;
//    public static final int FRONT = 8;
//    public static final int REAR = 9;
//    public static final int START = 10;
//    public static final int INNER = 11;
//    public static final int END = 12;
    
//    private static final float max_distance = (float)Long.MAX_VALUE / 1000f;
    
    /**
     * a1とa2の線分の内部にbが存在するか調べる。
     * a1==bまたはa2==bならばfalseを返す。
     * @return a1とa2の線分の内部にbが存在するならばtrue、そうでなければfalse。
     */
    public static boolean isOnLineInner(BVector2f a1, BVector2f a2, BVector2f b)
    {
        return (a1.x - b.x)*(a2.x - b.x) + (a1.y - b.y)*(a2.y - b.y) < 0; 
    }
    /**
     * a1とa2の線分上にbが存在するか調べる。
     * a1==bまたはa2==bならばtrueを返す。
     * @return a1とa2の線分上にbが存在するならばtrue、そうでなければfalse。
     */
    public static boolean isOnLine(BVector2f a1, BVector2f a2, BVector2f b)
    {
        return (a1.x - b.x)*(a2.x - b.x) + (a1.y - b.y)*(a2.y - b.y) <= 0; 
    }
    
    /**
     * a1とa2の線分の内部にbが存在するか調べる。
     * a1==bまたはa2==bならばfalseを返す。
     * @return a1とa2の線分の内部にbが存在するならばtrue、そうでなければfalse。
     */
    public static boolean isOnLineInner(BVector3f a1, BVector3f a2, BVector3f b)
    {
        return (a1.x - b.x)*(a2.x - b.x) + (a1.y - b.y)*(a2.y - b.y) + (a1.z - b.z)*(a2.z - b.z) < 0; 
    }
    /**
     * a1とa2の線分上にbが存在するか調べる。
     * a1==bまたはa2==bならばtrueを返す。
     * @return a1とa2の線分上にbが存在するならばtrue、そうでなければfalse。
     */
    public static boolean isOnLine(BVector3f a1, BVector3f a2, BVector3f b)
    {
        return (a1.x - b.x)*(a2.x - b.x) + (a1.y - b.y)*(a2.y - b.y) + (a1.z - b.z)*(a2.z - b.z) <= 0; 
    }
//    public static void createPlane(BVector4f dest, BVector2f p1, BVector2f p2, BVector2f v)
//    {
//        BVector3f pp = new BVector3f(p2.x - p1.x, p2.y-p1.y, 0);
//        BVector3f v3 = new BVector3f(v.x, v.y, 1);
//        BVector3f q = new BVector3f();
//        BVector3f.cross(q, v3, pp);
//        BVector4f plane = new BVector4f();
//        plane.a = q.x;
//        plane.b = q.y;
//        plane.c = q.z;
//        plane.d = -(q.x * p1.x + q.y * p1.y);
//    }
//    public static int getCrossPoint(BVector3f dest, BVector4f plane, BVector2f point, BVector2f velocity)
//    {
//        float d1 = plane.a * velocity.x + plane.b * velocity.y + plane.c;
//        float d2 = plane.a * point.x + plane.b * point.y + plane.d;
//        if (d1 == 0)
//        {
//            if (d2 == 0)
//                return ONPLAIN;
//            else
//                return PARALLEL;
//        }
//        dest.z = d2 / d1;
//        dest.x = velocity.x * dest.z + point.x;
//        dest.y = velocity.y * dest.z + point.y;
//        return CROSS;
//    }
//    public static int getCrossPoint(BVector3f dest, BVector2f p1, BVector2f v1, BVector2f p2, BVector2f v2)
//    {
//        if (v1.x != v2.x)
//        {
//            dest.z = -(p1.x - p2.x) / (v1.x - v2.x);
//        }
//        else if (v1.y != v2.y)
//        {
//            dest.z = -(p1.y - p2.y) / (v1.y - v2.y);
//        }
//        else
//        {
//            return PARALLEL;
//        }
//        dest.x = v1.x * dest.z + p1.x;
//        dest.y = v1.y * dest.z + p1.y;
//        return CROSS;
//    }
//    public static int checkPosition(BVector4f plane, BVector3f point)
//    {
//        double d = plane.a * point.x + plane.b * point.y + plane.c * point.z + plane.d;
//        if (d > 0)
//            return ININNER;
//        else if (d < 0)
//            return INOUTER;
//        else
//            return ONPLAIN;
//    }

    /**
     * 線分p11-p12の内部と線分p21-p22の内部（両端を含まない）の共有点を求める。
     * @param dest 交点の出力先
     * @param p11
     * @param p12
     * @param p21
     * @param p22
     * @return 共有点が存在するならばtrue。そうでなければfalse。
     */
    public static boolean isIntersectInner(BVector2f dest, BVector2f p11, BVector2f p12, BVector2f p21, BVector2f p22)
    {
        float a1,a2,b1,b2,f;
        f = p11.x*p12.y - p12.x*p11.y;
        if (f == 0)
            return false;
        a1 = -(p12.y - p11.y) / f;
        b1 =  (p12.x - p11.x) / f;

        f = p21.x*p22.y - p22.x*p21.y;
        if (f == 0)
            return false;
        a2 = -(p22.y - p21.y) / f;
        b2 =  (p22.x - p21.x) / f;

        f = a1*b2 - a2*b1;
        if (f == 0)
            return false;
        dest.x = -(b2-b1) / f;
        dest.y =  (a2-a1) / f;
        
        return (isOnLineInner(p11,p12,dest) && isOnLineInner(p21,p22,dest));
    }
    /**
     * 線分p11-p12と線分p21-p22（両端を含む）の共有点を求める。
     * @param dest 交点の出力先
     * @param p11
     * @param p12
     * @param p21
     * @param p22
     * @return 共有点が存在するならばtrue。そうでなければfalse。
     */
     public static boolean isIntersect(BVector2f dest, BVector2f p11, BVector2f p12, BVector2f p21, BVector2f p22)
    {
        float a1,a2,b1,b2,f;
        f = p11.x*p12.y - p12.x*p11.y;
        if (f == 0)
            return false;
        a1 = -(p12.y - p11.y) / f;
        b1 =  (p12.x - p11.x) / f;

        f = p21.x*p22.y - p22.x*p21.y;
        if (f == 0)
            return false;
        a2 = -(p22.y - p21.y) / f;
        b2 =  (p22.x - p21.x) / f;

        f = a1*b2 - a2*b1;
        if (f == 0)
            return false;
        dest.x = -(b2-b1) / f;
        dest.y =  (a2-a1) / f;
        
        return (isOnLine(p11,p12,dest) && isOnLine(p21,p22,dest));
    }
//    public static int checkStraightenPosition(BVector2f a1, BVector2f a2, BVector2f b)
//    {
//        if (isOnLineInner(a1,b,a2))
//        {
//            return FRONT;
//        }
//        else if (isOnLineInner(a2,b,a1))
//        {
//            return REAR;
//        }
//        else if (a1.equals(b))
//        {
//            return START;
//        }
//        else if (a2.equals(b))
//        {
//            return END;
//        }
//        else
//        {
//            return INNER;
//        }
//    }
//    public static boolean isLeft(BVector2f a1, BVector2f a2, BVector2f b)
//    {
//        return ((a2.x - a1.x)*(b.y - a1.y) - (a2.y - a1.y)*(b.x - a1.x)) > 0;
//    }
    
    
//    public static int contains(BVector2f offset, BVector2f[] vs, BVector2f point)
//    {
//        if (vs.length <= 2)
//        {
//            return INOUTER;
//        }
//        int hits = 0;
//
//        BVector2f last = new BVector2f(vs[vs.length - 1]);
//        BVector2f cur = new BVector2f();
//        last.add(offset);
//
//        for (int i = 0; i < vs.length; last.set(cur), i++)
//        //for (int i = vs.length - 1; i >= 0; last.set(cur), i--)
//        {
//            cur.set(vs[i]);
//            cur.add(offset);
//
//            if (cur.y == last.y)
//            {
//                if (cur.y == point.y)
//                {
//                    if ((cur.x <= point.x && point.x <= last.x) || (cur.x >= point.x && point.x >= last.x))
//                        return ONLINE;
//                }
//                continue;
//            }
//
//            float leftx;
//            if (cur.x < last.x)
//            {
//                if (point.x > last.x)
//                {
//                    continue;
//                }
//                leftx = cur.x;
//            }
//            else
//            {
//                if (point.x > cur.x)
//                {
//                    continue;
//                }
//                leftx = last.x;
//            }
//
//            float test1, test2;
//            if (cur.y < last.y)
//            {
//                if (point.y < cur.y || point.y > last.y)
//                {
//                    continue;
//                }
//                if (point.x < leftx)
//                {
//                    hits++;
//                    continue;
//                }
//                test1 = point.x - cur.x;
//                test2 = point.y - cur.y;
//            }
//            else
//            {
//                if (point.y < last.y || point.y > cur.y)
//                {
//                    continue;
//                }
//                if (point.x < leftx)
//                {
//                    hits++;
//                    continue;
//                }
//                test1 = point.x - last.x;
//                test2 = point.y - last.y;
//            }
//
//            test2 = (test2 / (last.y - cur.y) * (last.x - cur.x));
//            if (test1 < test2)
//            {
//                hits++;
//            }
//            else if (test1 == test2)
//            {
//                return ONLINE;
//            }
//                
//        }
//
//        if ((hits & 1) != 0)
//            return ININNER;
//        else
//            return INOUTER;
//    }
     /**
      * 0 = args.x * x^2 + args.y * y + args.zの二次関数の解2つを引数ansに出力する。
      * @param ans 長さ2の配列で解の出力先
      * @param args 二次関数を表すベクトル
      * @return 実数解を持つならばtrue、そうでないならfalse。
      */
     public static boolean solveQuadEquation(float[] ans, BVector3f args)
    {
        float f = args.y * args.y - 4 * args.x * args.z;
        if (f < 0)
            return false;
        f = (float)Math.sqrt(f);
        ans[0] = (- args.y + f) / (2 * args.x);
        ans[1] = (- args.y - f) / (2 * args.x);
        return true;
    }
//    public static float getClosestIntersection(BVector2f start, BVector2f end, BVector2f pos, BVector2f[] vs)
//    {
//        BVector2f pos1 = new BVector2f();
//        BVector2f pos2 = new BVector2f();
//        BVector2f v = new BVector2f();
//        pos1.x = pos.x + vs[vs.length-1].x;
//        pos1.y = pos.y + vs[vs.length-1].y;
//        float distance = Float.MAX_VALUE;
//        for (int i=0;i<vs.length;i++)
//        {
//            pos2.x = pos.x + vs[i].x;
//            pos2.y = pos.y + vs[i].y;
//            if(getIntersection(v,start,end,pos1,pos2))
//            {
//                float d = (v.x-start.x)*(v.x-start.x)+(v.y-start.y)*(v.y-start.y);
//                if (d<distance)
//                {
//                    distance = d;
//                }
//            }
//            pos1.set(pos2);
//        }
//        if (distance > max_distance)
//        {
//            return Float.NaN;
//        }
//        else
//        {
//            return (float)Math.sqrt(distance);
//        }
//    }
     /**
      * 線分p11-p12と線分p21-p22（両端を含む）の共有点を求める。
      * @param dest 交点の出力先
      * @param p11
      * @param p12
      * @param p21
      * @param p22
      * @return 共有点が存在するならばtrue。そうでなければfalse。
      */
    public static boolean getIntersection(BVector2f dest, BVector2f p11, BVector2f p12, BVector2f p21, BVector2f p22)
    {
        float a1 = -(p12.y - p11.y);
        float a2 = -(p22.y - p21.y);
        float b1 = p12.x - p11.x;
        float b2 = p22.x - p21.x;
        float c1 = p11.x*p12.y - p12.x*p11.y;
        float c2 = p21.x*p22.y - p22.x*p21.y;
        float c = a1*b2 - a2*b1;
        if (c == 0)
        {
            return false;
        }else
        {
            BVector2f v = new BVector2f();
            v.x = (b1*c2 - b2*c1)/c;
            v.y = (c1*a2 - c2*a1)/c;
            //if (isInInner(v,p11,p12) && isInInner(v,p21,p22))
            if (isOnLine(p11,p12,v) && isOnLine(p21,p22,v))
            {
                dest.set(v);
                return true;
            }else
            {
                return false;
            }
        }
    }
//    public static float getIntersection(BVector4f p, BVector2f fx, BVector2f fy)
//    {
//        float a = p.a * fx.x + p.b * fy.x + p.c;
//        float b = p.a * fx.y + p.b * fy.y + p.d;
//        if (a == 0)
//            return Float.NaN;
//        return -b/a;
//    }
//    private static boolean isInInner(BVector2f a, BVector2f b1, BVector2f b2)
//    {
//
//        if (((a.x-b1.x)*(b2.x-b1.x) + (a.y-b1.y)*(b2.y-b1.y) >= 0)
//                && ((a.x-b2.x)*(b1.x-b2.x) + (a.y-b2.y)*(b1.y-b2.y) >= 0))
//
//        {
//            return true;
//        }else
//        {
//            return false;
//        }
//    }
    /**
     * pos1とpos2を通る一次関数を得る。
     * dest.x * x + dest.y * y + dest.z = 0
     * @param dest 一次関数の出力先
     * @param pos1
     * @param pos2
     */
    public static void getLineFunction(BVector3f dest, BVector2f pos1, BVector2f pos2)
    {
        dest.x = pos2.y - pos1.y;
        dest.y = -(pos2.x - pos1.x);
        dest.z = pos2.x*pos1.y - pos1.x * pos2.y;
    }
    
//    /*
//     * 位置p1、半径circle1、速度v1で動く円1と位置p2、半径circle2、速度v2で動く円2の
//     * 衝突するまでの時間とその法線を得る。
//     * @param normaldest1 円1上の衝突時の法線
//     * @param normaldest2 円2上の衝突時の法線
//     * @param p1 円1の位置
//     * @param circle1 円1の半径
//     * @param v1 円1の速度
//     * @param p2 円2の位置
//     * @param circle2 円2の半径
//     * @param v2 円2の速度
//     * @return 衝突するまでの時間。衝突しない場合はFloat.NaN。
//     */
//    public static float nextCollid(BVector2f normaldest1,BVector2f normaldest2, BVector2f p1, BShape circle1, BVector2f v1, BVector2f p2, BShape circle2, BVector2f v2)
//    {
//        return nextCollidCirclexCircle(normaldest1, normaldest2, p1, circle1.getRadius(), v1, p2, circle2.getRadius(), v2);
//    }
//    /*
//     * 位置p1、半径circle1、速度v1で動く円1と位置p2、半径circle2、速度v2で動く円2の
//     * 衝突するまでの時間とその法線を得る。
//     * @param normaldest1 円1上の衝突時の法線
//     * @param normaldest2 円2上の衝突時の法線
//     * @param p1 円1の位置
//     * @param radius1 円1の半径
//     * @param v1 円1の速度
//     * @param p2 円2の位置
//     * @param radius2 円2の半径
//     * @param v2 円2の速度
//     * @return 衝突するまでの時間。衝突しない場合はFloat.NaN。
//     */
//    public static float nextCollidCirclexCircle(BVector2f normaldest1, BVector2f normaldest2, BVector2f p1, float radius1, BVector2f v1, BVector2f p2, float radius2, BVector2f v2)
//    {
//        if (isOff(p1,v1,p2,v2))
//            return Float.NaN;
//        
//        float a,b,c;
//        float t1,t2,t3,t4;
//        t1 = v1.x - v2.x;
//        t2 = v1.y - v2.y;
//        t3 = p1.x - p2.x;
//        t4 = p1.y - p2.y;
//        a = t1*t1 + t2*t2;
//        b = 2*( t1 * t3 + t2 * t4 );
//        c = t3*t3 + t4*t4 - (radius1 + radius2)*(radius1 + radius2);
//        
//        float pm = b*b - 4 * a * c;
//        if (pm < 0)
//            return Float.NaN;
//        pm = (float)Math.sqrt(pm);
//        float time1 = (-b - pm)/(2*a);
//        float time2 = (-b + pm)/(2*a);
//        if (time1 <= 0)
//        {
//            if (time2 <= 0)
//            {
//                return Float.NaN;
//            }
//            time1 = time2;
//        }
//        BVector2f pos1 = new BVector2f();
//        BVector2f pos2 = new BVector2f();
//        pos1.set(p1);
//        pos1.scaleAdd(time1, v1);
//        pos2.set(p2);
//        pos2.scaleAdd(time1, v2);
//        normaldest1.set(pos2);
//        normaldest1.sub(pos1);
//        if (normaldest1.getLength()==0)
//            return Float.NaN;
//        normaldest1.normalize();
//        normaldest2.set(normaldest1);
//        normaldest2.scale(-1f);
//
//        return time1;
//    }
    /**
     * 物体1と物体2の衝突するまでの時間を得る。現在はCIRCLEとRECTANGLEのみ正常に作動する。
     * @param p1 物体1の位置
     * @param shape1 物体1の形状
     * @param v1 物体1の速度
     * @param p2 物体2の位置
     * @param shape2 物体2の形状
     * @param v2 物体2の速度
     * @return 衝突するまでの時間。衝突しない場合はFloat.NaN。
     */
    public static float nextCollid(BVector2f p1, BShape shape1, BVector2f v1, BVector2f p2, BShape shape2, BVector2f v2)
    {
        if (shape1.getType() == BShape.COMPOSITE)
        {
            BShape[] sps1 = shape1.getComposite();
            boolean flag = false;
            float time = Float.MAX_VALUE;
            for (int i=0;i<sps1.length;i++)
            {
                BShape sp1 = sps1[i];
                float t = Float.NaN;
                t = nextCollid(p1, sp1, v1, p2, shape2, v2);
                if (!Float.isNaN(t) && t>0 && t < time)
                {
                    flag = true;
                    time = t;
                }
            }
            if (flag)
                return time;
            else
                return Float.NaN;
        }
        else if (shape2.getType() == BShape.COMPOSITE)
        {
            BShape[] sps2 = shape2.getComposite();
            boolean flag = false;
            float time = Float.MAX_VALUE;
            for (int i=0;i<sps2.length;i++)
            {
                BShape sp2 = sps2[i];
                float t = Float.NaN;
                t = nextCollid(p1, shape1, v1, p2, sp2, v2);
                if (!Float.isNaN(t) && t>0 && t < time)
                {
                    flag = true;
                    time = t;
                }
            }
            if (flag)
                return time;
            else
                return Float.NaN;
        }
        if (shape1.getType() == BShape.CIRCLE)
        {
            if (shape2.getType() == BShape.CIRCLE)
                return nextCollidCirclexCircle(p1,shape1.getRadius(), v1, p2, shape2.getRadius(), v2);
            else if (shape2.getType() == BShape.RECTANGLE)
                return nextCollidCirclexRectangle(p1, shape1.getRadius(), v1, p2, shape2.getRectangle(), v2); 
            else if (shape2.getType() == BShape.LINE)
                return nextCollidCirclexLine(p1, shape1.getRadius(), v1, p2, shape2.getPoint1(), shape2.getPoint2(), v2); 
            else
                return Float.NaN;
        }
        else if (shape1.getType() == BShape.LINE)
        {
            if (shape2.getType() == BShape.CIRCLE)
                return nextCollidCirclexLine(p2, shape2.getRadius(), v2, p1, shape1.getPoint1(), shape1.getPoint2(), v1); 
            else
                return Float.NaN;
        }
        else if (shape1.getType() == BShape.RECTANGLE)
        {
            if (shape2.getType() == BShape.CIRCLE)
                return nextCollidCirclexRectangle(p2, shape2.getRadius(), v2, p1, shape1.getRectangle(), v1); 
            else
                return Float.NaN;
        }
        else
        {
            return Float.NaN;
        }
    }
    /**
     * 円1と円2の衝突するまでの時間とその法線を得る。
     * @param p1 点1の位置
     * @param v1 点1の速度
     * @param p2 円2の位置
     * @param radius2 円2の半径
     * @param v2 円2の速度
     * @return 衝突するまでの時間。衝突しない場合はFloat.NaN。
     */
    public static float nextCollidPointxCircle(BVector2f p1, BVector2f v1, BVector2f p2, float radius2, BVector2f v2)
    {
        if (isOff(p1,v1,p2,v2))
            return Float.NaN;
        
        float a,b,c;
        float t1,t2,t3,t4;
        t1 = v1.x - v2.x;
        t2 = v1.y - v2.y;
        t3 = p1.x - p2.x;
        t4 = p1.y - p2.y;
        a = t1*t1 + t2*t2;
        b = 2*( t1 * t3 + t2 * t4 );
        c = t3*t3 + t4*t4 - (radius2)*(radius2);
        
        float pm = b*b - 4 * a * c;
        if (pm < 0)
            return Float.NaN;
        pm = (float)Math.sqrt(pm);
        float time1 = (-b - pm)/(2*a);
        float time2 = (-b + pm)/(2*a);
        if (time1 <= 0)
        {
            if (time2 <= 0)
            {
                return Float.NaN;
            }
            time1 = time2;
        }
        return time1;
    }
    /**
     * 円1と円2の衝突するまでの時間を得る。
     * @param p1 円1の位置
     * @param radius1 円1の半径
     * @param v1 円1の速度
     * @param p2 円2の位置
     * @param radius2 円2の半径
     * @param v2 円2の速度
     * @return 衝突するまでの時間。衝突しない場合はFloat.NaN。
     */
    public static float nextCollidCirclexCircle(BVector2f p1, float radius1, BVector2f v1, BVector2f p2, float radius2, BVector2f v2)
    {
        return nextCollidPointxCircle(p1, v1, p2, radius1+radius2,v2);
//        if (isOff(p1,v1,p2,v2))
//            return Float.NaN;
//        
//        float a,b,c;
//        float t1,t2,t3,t4;
//        t1 = v1.x - v2.x;
//        t2 = v1.y - v2.y;
//        t3 = p1.x - p2.x;
//        t4 = p1.y - p2.y;
//        a = t1*t1 + t2*t2;
//        b = 2*( t1 * t3 + t2 * t4 );
//        c = t3*t3 + t4*t4 - (radius1 + radius2)*(radius1 + radius2);
//        
//        float pm = b*b - 4 * a * c;
//        if (pm < 0)
//            return Float.NaN;
//        pm = (float)Math.sqrt(pm);
//        float time1 = (-b - pm)/(2*a);
//        float time2 = (-b + pm)/(2*a);
//        if (time1 <= 0)
//        {
//            if (time2 <= 0)
//            {
//                return Float.NaN;
//            }
//            time1 = time2;
//        }
//        return time1;
    }
    /**
     * 点と線の衝突するまでの時間とその法線を得る。
     * @param p1 点の位置
     * @param v1 点の速度
     * @param p2 線のオフセット
     * @param p21 線の視点
     * @param p22 線の位置
     * @param v2 線の速度
     * @return 衝突するまでの時間。衝突しない場合はFloat.NaN。
     */
    public static float nextCollidPointxLine(BVector2f p1, BVector2f v1, BVector2f p2, BVector2f p21, BVector2f p22, BVector2f v2)
    {
        BVector2f p21d = new BVector2f(p21);
        BVector2f p22d = new BVector2f(p22);
        BVector2f pc = new BVector2f(0f,0f);
        p21d.add(p2);
        p22d.add(p2);
        pc.scaleAdd(0.5f, p21d);
        pc.scaleAdd(0.5f, p22d);
        BVector3f line = new BVector3f();
        getLineFunction(line, p21d, p22d);
        BVector2f vd = new BVector2f(v1);
        vd.sub(v2);

        BVector2f n = new BVector2f(line.x , line.y);
        n.normalize();
        double dist = - line.z / Math.sqrt(line.x*line.x + line.y*line.y) - BVector2f.getDot(n, p1);
        double vel = BVector2f.getDot(n, vd);
        if (dist * vel <= 0)
            return Float.NaN;
        float t = (float)(dist/vel);
        
        
        //reuse pc
        pc.set(p1);
        pc.scaleAdd(t, v1);
        if (isOnLine(p21d, p22d, pc))
            return t;
        return Float.NaN;
    }
    /**
     * 円と位置pos、線の衝突するまでの時間とその法線を得る。
     * @param p1 円の位置
     * @param radius1 円の半径
     * @param v1 円の速度
     * @param p2 線のオフセット
     * @param p21 線の視点
     * @param p22 線の位置
     * @param v2 線の速度
     * @return 衝突するまでの時間。衝突しない場合はFloat.NaN。
     */
    public static float nextCollidCirclexLine(BVector2f p1, float radius1, BVector2f v1, BVector2f p2, BVector2f p21, BVector2f p22, BVector2f v2)
    {
        BVector2f p21d = new BVector2f(p21);
        BVector2f p22d = new BVector2f(p22);
        BVector2f pc = new BVector2f(0f,0f);
        p21d.add(p2);
        p22d.add(p2);
        pc.scaleAdd(0.5f, p21d);
        pc.scaleAdd(0.5f, p22d);
        BVector3f line = new BVector3f();
        getLineFunction(line, p21d, p22d);
        BVector2f vd = new BVector2f(v1);
        vd.sub(v2);

        BVector2f n = new BVector2f(line.x , line.y);
        n.normalize();
        double dist = - line.z / Math.sqrt(line.x*line.x + line.y*line.y) - BVector2f.getDot(n, p1);
        double vel = BVector2f.getDot(n, vd);
        if (dist * vel <= 0)
            return Float.NaN;
        float r = radius1;
        float t1 = (float)((dist + r)/vel);
        float t2 = (float)((dist - r)/vel);
        float t;
        
        if (t1 < 0)
        {
            if (t2 < 0)
                return Float.NaN;
            t = t2;
        }
        else
        {
            if (t1 < t2)
                t = t1;
            else
                t = t2;
        }
        //reuse pc
        pc.set(p1);
        pc.scaleAdd(t, v1);
        if (vel < 0)
            pc.scaleAdd(-r, n);
        else
            pc.scaleAdd(r, n);
        if (isOnLine(p21d, p22d, pc))
            return t;

        t1 = nextCollidPointxCircle(p1,v1,p21d,r,v2);
        t2 = nextCollidPointxCircle(p1,v1,p22d,r,v2);
        if (Float.isNaN(t1))
        {
            if (Float.isNaN(t2))
                return Float.NaN;
            else
                t = t2;
        }
        else
        {
            if (Float.isNaN(t2))
                t = t1;
            else
            {
                if (t1<t2)
                    t = t1;
                else
                    t = t2;
            }
        }
        return t;
    }
    /**
     * 点と長方形の衝突するまでの時間を得る。
     * @param p1 点の位置
     * @param v1 点の速度
     * @param p2 長方形の位置
     * @param sq2 長方形の形状
     * @param v2 長方形の速度
     * @return 衝突するまでの時間
     */
    public static float nextCollidPointxRectangle(BVector2f p1, BVector2f v1, BVector2f p2, BVector4f sq2, BVector2f v2)
    {
        //123
        //456
        //789
        int pos1;
        //int pos2;
        float x = p1.x - p2.x;
        float y = p1.y - p2.y;
        float x1 = sq2.a;
        float x2 = sq2.b;
        float y1 = sq2.c;
        float y2 = sq2.d;
        BVector2f vel = new BVector2f(v1);
        vel.sub(v2);
        if (x <= x1)
        {
            if (y <= y1)
                pos1 = 7;
            else if (y < y2)
                pos1 = 4;
            else
                pos1 = 1;
        }
        else if (x < x2)
        {
            if (y <= y1)
                pos1 = 8;
            else if (y < y2)
                pos1 = 5;
            else
                pos1 = 2;
        }
        else
        {
            if (y <= y1)
                pos1 = 9;
            else if (y < y2)
                pos1 = 6;
            else
                pos1 = 3;
        }
        if (vel.x >= 0 && (pos1 == 3 || pos1 == 6 || pos1 == 9))
            return Float.NaN;
        else if (vel.x <= 0 && (pos1 == 1 || pos1 == 4 || pos1 == 7))
            return Float.NaN;
        if (vel.y >= 0 && (pos1 == 1 || pos1 == 2 || pos1 == 3))
            return Float.NaN;
        else if (vel.y <= 0 && (pos1 == 7 || pos1 == 8 || pos1 == 9))
            return Float.NaN;
        
        BVector3f line;
        //do not connect else
        if (pos1 == 1 || pos1 == 2 || pos1 == 3)
        {
            float t = (y2 - y) / vel.y;
            float xc = x + vel.x * t;
            if (x1 <= xc && xc <= x2)
                return t;
        }
        if (pos1 == 7 || pos1 == 8 || pos1 == 9)
        {
            float t = (y1 - y) / vel.y;
            float xc = x + vel.x * t;
            if (x1 <= xc && xc <= x2)
                return t;
        }
        if (pos1 == 3 || pos1 == 6 || pos1 == 9)
        {
            float t = (x2 - x) / vel.x;
            float yc = y + vel.y * t;
            if (y1 <= yc && yc <= y2)
                return t;
        }
        if (pos1 == 1 || pos1 == 4 || pos1 == 7)
        {
            float t = (x1 - x) / vel.x;
            float yc = y + vel.y * t;
            if (y1 <= yc && yc <= y2)
                return t;
        }
        return Float.NaN;
    }
    /**
     * 円と長方形の衝突するまでの時間を得る。
     * @param p1 円の位置
     * @param radius1 円の半径
     * @param v1 円の速度
     * @param p2 長方形の位置
     * @param sq2 長方形の大きさ
     * @param v2 長方形の速度
     * @return 衝突するまでの時間。衝突しない場合はFloat.NaN。
     */
    public static float nextCollidCirclexRectangle(BVector2f p1, float radius1, BVector2f v1, BVector2f p2, BVector4f sq2, BVector2f v2)
    {
        //123
        //456
        //789
        //10 = 5 of upleft
        //11 = 5 of upright
        //12 = 5 of downleft
        //13 = 5 of downright
        int pos1;
        //int pos2;
        float x = p1.x - p2.x;
        float y = p1.y - p2.y;
        float x0 = sq2.a - radius1;
        float x1 = sq2.a;
        float x2 = sq2.b;
        float x3 = sq2.b + radius1;
        float y0 = sq2.c - radius1;
        float y1 = sq2.c;
        float y2 = sq2.d;
        float y3 = sq2.d + radius1;
        BVector2f pos = new BVector2f(x,y);
        BVector2f vel = new BVector2f(v1);
        BVector2f zeroVel = new BVector2f(0,0);
        vel.sub(v2);
        if (x <= x0)
        {
            if (y <= y0)
                pos1 = 7;
            else if (y < y3)
                pos1 = 4;
            else
                pos1 = 1;
        }
        else if (x < x3)
        {
            if (y <= y0)
                pos1 = 8;
            else if (y < y3)
            {
                pos1 = 5;
                if (x < x1)
                {
                   if (y < y1)
                       pos1 = 12;
                   else if (y2 < y)
                       pos1 = 10;
                }
                else if (x2 < x)
                {
                    if (y < y1)
                        pos1 = 13;
                    else if (y2 < y)
                        pos1 = 11;
                }
            }
            else
                pos1 = 2;
        }
        else
        {
            if (y <= y0)
                pos1 = 9;
            else if (y < y3)
                pos1 = 6;
            else
                pos1 = 3;
        }
        if (vel.x >= 0 && (pos1 == 3 || pos1 == 6 || pos1 == 9))
            return Float.NaN;
        else if (vel.x <= 0 && (pos1 == 1 || pos1 == 4 || pos1 == 7))
            return Float.NaN;
        if (vel.y >= 0 && (pos1 == 1 || pos1 == 2 || pos1 == 3))
            return Float.NaN;
        else if (vel.y <= 0 && (pos1 == 7 || pos1 == 8 || pos1 == 9))
            return Float.NaN;
        
        BVector3f line;
        //do not connect else
        //System.out.print(""+ pos1 + ":");
        if (pos1 == 1 || pos1 == 2 || pos1 == 3 || pos1 == 4 || pos1 == 7 || pos1 == 10)
        {
            //System.out.print("10 ");
            BVector2f cp = new BVector2f(x1,y2);
            float t = nextCollidPointxCircle(pos, vel, cp, radius1, zeroVel);
            if (!Float.isNaN(t))
                return t;
        }
        if (pos1 == 1 || pos1 == 2 || pos1 == 3 || pos1 == 6 || pos1 == 9 || pos1 == 11)
        {
            //System.out.print("11 ");
            BVector2f cp = new BVector2f(x2,y2);
            float t = nextCollidPointxCircle(pos, vel, cp, radius1, zeroVel);
            if (!Float.isNaN(t))
                return t;
        }
        if (pos1 == 1 || pos1 == 4 || pos1 == 7 || pos1 == 8 || pos1 == 9 || pos1 == 12)
        {
            //System.out.print("12 ");
            BVector2f cp = new BVector2f(x1,y1);
            float t = nextCollidPointxCircle(pos, vel, cp, radius1, zeroVel);
            if (!Float.isNaN(t))
                return t;
        }
        if (pos1 == 3 || pos1 == 6 || pos1 == 9 || pos1 == 7 || pos1 == 8 || pos1 == 13)
        {
            //System.out.print("13 ");
            BVector2f cp = new BVector2f(x2,y1);
            float t = nextCollidPointxCircle(pos, vel, cp, radius1, zeroVel);
            if (!Float.isNaN(t))
                return t;
        }
        if (pos1 == 1 || pos1 == 2 || pos1 == 3)
        {
            float t = (y3 - y) / vel.y;
            float xc = x + vel.x * t;
            if (x1 <= xc && xc <= x2)
                return t;
        }
        if (pos1 == 7 || pos1 == 8 || pos1 == 9)
        {
            float t = (y0 - y) / vel.y;
            float xc = x + vel.x * t;
            if (x1 <= xc && xc <= x2)
                return t;
        }
        if (pos1 == 3 || pos1 == 6 || pos1 == 9)
        {
            float t = (x3 - x) / vel.x;
            float yc = y + vel.y * t;
            if (y1 <= yc && yc <= y2)
                return t;
        }
        if (pos1 == 1 || pos1 == 4 || pos1 == 7)
        {
            float t = (x0 - x) / vel.x;
            float yc = y + vel.y * t;
            if (y1 <= yc && yc <= y2)
                return t;
        }
        return Float.NaN;
    }
    /**
     * 未実装。必ずRuntimeExceptionをスローする。
     * @param p1
     * @param v1
     * @param p2
     * @param sq1
     * @param v2
     * @return 衝突するまでの時間
     */
    public static float nextCollidRectanglexRectangle(BVector2f p1, BVector2f v1, BVector2f p2, BVector4f sq1, BVector2f v2)
    {
        //TODO 実装
        throw new RuntimeException("not implement yet");
    }
    
    /**
     * 点p1から速度v1で動く点と点p2から速度v2で動く点が遠ざかっているか調べる。
     * @param p1
     * @param v1
     * @param p2
     * @param v2
     * @return 二点が離れる一方ならばtrue、そうでなければfalse。
     */
    public static boolean isOff(BVector2f p1, BVector2f v1, BVector2f p2, BVector2f v2)
    {
        return (p2.x - p1.x)*(v2.x - v1.x) + (p2.y - p1.y)*(v2.y - v1.y) > 0;
    }
    
    /**
     * 2つの物体が交差しているか判定する。
     * @param p1 物体1の位置
     * @param shape1 物体1の形状
     * @param p2 物体2の位置
     * @param shape2 物体2の形状
     * @return 2つの物体が交差するならtrue、そうでなければfalse。
     */
    public static boolean isIntersect(BVector2f p1, BShape shape1, BVector2f p2, BShape shape2)
    {
        if (shape1.getType() == BShape.COMPOSITE)
        {
            BShape[] sps1 = shape1.getComposite();
            for (int i=0;i<sps1.length;i++)
            {
                BShape sp1 = sps1[i];
                if (isIntersect(p1, sp1, p2, shape2))
                    return true;
            }
            return false;
        }
        else if (shape2.getType() == BShape.COMPOSITE)
        {
            BShape[] sps2 = shape2.getComposite();
            for (int i=0;i<sps2.length;i++)
            {
                BShape sp2 = sps2[i];
                if (isIntersect(p1, shape1, p2, sp2))
                    return true;
            }
            return false;
        }
        if (shape1.getType() == BShape.CIRCLE)
        {
            if (shape2.getType() == BShape.CIRCLE)
                return isIntersectCirclexCirle(p1,shape1.getRadius(), p2, shape2.getRadius());
            else if (shape2.getType() == BShape.RECTANGLE)
                return isIntersectCirclexRectangle(p1, shape1.getRadius(), p2, shape2.getRectangle()); 
            else
                return false;
        }
        else if (shape1.getType() == BShape.RECTANGLE)
        {
            if (shape2.getType() == BShape.CIRCLE)
                return isIntersectCirclexRectangle(p2, shape2.getRadius(), p1, shape1.getRectangle()); 
            else if (shape2.getType() == BShape.RECTANGLE)
                return isIntersectRectanglexRectangle(p1,shape1.getRectangle(),p2,shape2.getRectangle());
            else
                return false;
        }
        else
        {
            return false;
        }
    }
    /*
     * 位置p1の点と位置pos、線p21-p22、太さradius2の線が交差するか調べる。
     * @param p1 点の位置
     * @param p2 線のオフセット
     * @param p21 線の視点
     * @param p22 線の位置
     * @param radius2 線の太さ
     * @return 2つが交差するならtrue、そうでなければfalse。
     */
//    public static boolean isIntersectPointxLine(BVector2f p1, BVector2f p2, BVector2f p21, BVector2f p22)
//    {
//        //fix this
//        BVector2f p21d = new BVector2f(p21);
//        BVector2f p22d = new BVector2f(p22);
//        p21d.add(p2);
//        p22d.add(p2);
//
//        BVector3f line = new BVector3f();
//        getLineFunction(line, p21d, p22d);
//
//        BVector2f n = new BVector2f(line.x , line.y);
//        n.normalize();
//        float dist = (float)(-line.z / Math.sqrt(line.x*line.x + line.y*line.y) - BVector2f.getDot(n, p1));
//        float r = radius2;
//        if (Math.abs(dist) > r)
//            return false;
//        
//        BVector2f pc = new BVector2f();
//        pc.set(p1);
//        pc.scaleAdd(dist, n);
//        
//        if (isOnLine(p21d, p22d, pc))
//            return true;
//        else if (isIntersectPointxCirle(p1, p21d,r))
//            return true;
//        else if (isIntersectPointxCirle(p1, p22d,r))
//            return true;
//        else
//            return false;
//    }
    /**
     * 位置p1、半径radius1の円1と位置pos、線p21-p22の線が交差するか調べる。
     * @param p1 円の位置
     * @param radius1 円の半径
     * @param p2 線のオフセット
     * @param p21 線の視点
     * @param p22 線の位置
     * @return 2つが交差するならtrue、そうでなければfalse。
     */
    public static boolean isIntersectCirclexLine(BVector2f p1, float radius1, BVector2f p2, BVector2f p21, BVector2f p22)
    {
        BVector2f p21d = new BVector2f(p21);
        BVector2f p22d = new BVector2f(p22);
        p21d.add(p2);
        p22d.add(p2);

        BVector3f line = new BVector3f();
        getLineFunction(line, p21d, p22d);

        BVector2f n = new BVector2f(line.x , line.y);
        n.normalize();
        float dist = (float)(-line.z / Math.sqrt(line.x*line.x + line.y*line.y) - BVector2f.getDot(n, p1));
        float r = radius1;
        if (Math.abs(dist) > r)
            return false;
        
        BVector2f pc = new BVector2f();
        pc.set(p1);
        pc.scaleAdd(dist, n);
        
        if (isOnLine(p21d, p22d, pc))
            return true;
        else if (isIntersectPointxCirle(p1, p21d,r))
            return true;
        else if (isIntersectPointxCirle(p1, p22d,r))
            return true;
        else
            return false;
    }
    /**
     * 位置p1の円と位置p2、半径radius2の円が共有部分を持つか調べる。
     * @param p1
     * @param p2
     * @param radius2
     * @return 2つが交差するならtrue、そうでなければfalse。
     */
    public static boolean isIntersectPointxCirle(BVector2f p1, BVector2f p2, float radius2)
    {
        return BVector2f.getDistance(p1,p2) < radius2;
    }
    /**
     * 位置p1、半径radius1の円と位置p2、半径radius2の円が共有部分を持つか調べる。
     * @param p1
     * @param radius1
     * @param p2
     * @param radius2
     * @return 2つの円が交差するならtrue、そうでなければfalse。
     */
    public static boolean isIntersectCirclexCirle(BVector2f p1, float radius1, BVector2f p2, float radius2)
    {
        return BVector2f.getDistance(p1,p2) <= radius1 + radius2;
    }
    
    public static boolean isIntersectPointxRectangle(BVector2f p1, BVector2f p2, BVector4f sq2)
    {
        float x = p1.x - p2.x;
        float y = p1.y - p2.y;
        return ((sq2.a < x && x < sq2.b) && (sq2.c < y && y < sq2.d));
    }
    public static boolean isIntersectCirclexRectangle(BVector2f p1, float radius1, BVector2f p2, BVector4f sq2)
    {
        BVector2f pos1 = new BVector2f(p1);
        pos1.sub(p2);
        float x = pos1.x;
        float y = pos1.y;
        float x0 = sq2.a - radius1;
        float x1 = sq2.a;
        float x2 = sq2.b;
        float x3 = sq2.b + radius1;
        float y0 = sq2.c - radius1;
        float y1 = sq2.c;
        float y2 = sq2.d;
        float y3 = sq2.d + radius1;
        if (x < x0 || x3 < x || y < y0 || y3 < y)
        {
            return false;
        }
        if (x1 <= x && x <= x2)
        {
            if (y0 <= y && y <= y3)
                return true;
            else
                return false;
        }
        else if (y1 <= y && y <= y2)
        {
            if (x0 <= x && x <= x3)
                return true;
            else
                return false;
        }
        else
        {
            if (x < x1)
            {
                if (y < y1)
                {
                    BVector2f pos2 = new BVector2f(x1,y1);
                    if (BVector2f.getDistance(pos1, pos2) <= radius1)
                        return true;
                }
                else if (y2 < y)
                {
                    BVector2f pos2 = new BVector2f(x1,y2);
                    if (BVector2f.getDistance(pos1, pos2) <= radius1)
                        return true;
                }
            }
            else if (x2 < x)
            {
                if (y <= y1)
                {
                    BVector2f pos2 = new BVector2f(x2,y1);
                    if (BVector2f.getDistance(pos1, pos2) <= radius1)
                        return true;
                }
                else if (y2 < y)
                {
                    BVector2f pos2 = new BVector2f(x2,y2);
                    if (BVector2f.getDistance(pos1, pos2) <= radius1)
                        return true;
                }
            }
            return false;
        }
    }
    public static boolean isIntersectRectanglexRectangle(BVector2f p1, BVector4f sq1, BVector2f p2, BVector4f sq2)
    {
        //123
        //456
        //789
        int pos1;
        //int pos2;
        float ox = p1.x - p2.x;
        float oy = p1.y - p2.y;
        float x1 = ox + sq1.a;
        float y1 = oy + sq1.c;
        float x2 = ox + sq1.b;
        float y2 = oy + sq1.d;
        if (x1 < sq2.a)
        {
            if (y1 < sq2.c)
            {
                pos1 = 7;
            }
            else if (y1 <= sq2.d)
            {
                pos1 = 4;
            }
            else
            {
                //pos1 = 1;
                return false;
            }
        }
        else if (x1 <= sq2.b)
        {
            if (y1 < sq2.c)
            {
                pos1 = 8;
            }
            else if (y1 <= sq2.d)
            {
                pos1 = 5;
                return true;
            }
            else
            {
                //pos1 = 2;
                return false;
            }
        }
        else
        {
//            pos1 = 9;
//            pos1 = 6;
//            pos1 = 3;
            return false;
        }
        if (x2 < sq2.a)
        {
            return false;
        }
        else if (x2 <= sq2.b)
        {
            if (y2 < sq2.c)
            {
                //pos2 = 8;
                return false;
            }
            else if (y2 <= sq2.d)
            {
                //pos2 = 5;
                return true;
            }
            else
            {
                //pos2 = 2;
                return (pos1 == 4 || pos1 == 5 || pos1 == 7 || pos1 == 8);
            }
        }
        else
        {
            if (y2 < sq2.c)
            {
                //pos2 = 9;
                return false;
            }
            else if (y2 <= sq2.d)
            {
                //pos2 = 6;
                return (pos1 == 4 || pos1 == 5 || pos1 == 7 || pos1 == 8);
            }
            else
            {
                //pos2 = 3;
                return (pos1 == 4 || pos1 == 5 || pos1 == 7 || pos1 == 8);
            }
        }
    }

    /**
     * 線分start-end（両端含む）と位置posの半径円の最近の交点を得る。
     * @param start 線分の始点
     * @param end 線分の終点
     * @param pos 円の位置
     * @param radius 円の半径
     * @return 始点から交点までの最近距離、交差しない場合はFloat.NaNを返す。
     */
    public static float getClosestIntersection(BVector2f start, BVector2f end, BVector2f pos, float radius)
    {
        BVector2f startp = new BVector2f(start);
        BVector2f endp = new BVector2f(end);
        startp.sub(pos);
        endp.sub(pos);
        
        BVector3f linef = new BVector3f();
        getLineFunction(linef, startp, endp);
        float cp = linef.z;
        //float cp = linef.z + linef.x*pos.x + linef.y*pos.y;
        float pm = linef.x*linef.x*radius*radius + linef.y*linef.y*radius*radius - cp*cp;
        if (pm <0)
            return Float.NaN;
        pm = (float)Math.sqrt(pm);
        float t = linef.x*linef.x + linef.y*linef.y;
        BVector2f p1 = new BVector2f();
        BVector2f p2 = new BVector2f();
        p1.x = (-linef.x*cp+linef.y*pm) / t;
        p1.y = (-linef.y*cp-linef.x*pm) / t;
        p2.x = (-linef.x*cp-linef.y*pm) / t;
        p2.y = (-linef.y*cp+linef.x*pm) / t;
        if (isOnLine(startp,endp,p1))
        {
            if (isOnLine(startp,endp,p2))
            {
                float d1 = BVector2f.getDistance(startp,p1);
                float d2 = BVector2f.getDistance(startp,p2);
                return (d1 < d2) ? d1 : d2;
            }
            else
            {
                return BVector2f.getDistance(startp,p1);
            }
        }
        else if (isOnLine(startp,endp,p2))
        {
            return BVector2f.getDistance(startp,p2);
        }
        return Float.NaN;
    }
    /**
     * 線分start-end（両端含む）と位置posの半径円の最近の交点を得る。
     * @param start 線分の始点
     * @param end 線分の終点
     * @param pos 円の位置
     * @param circle 円の半径
     * @return 始点から交点までの最近距離、交差しない場合はFloat.NaNを返す。
     */
    public static float getClosestIntersection(BVector2f start, BVector2f end, BVector2f pos, BShape circle)
    {
        return getClosestIntersection(start, end, pos, circle.getRadius());
    }

    /**
     * 点p1から方向vに伸びる線上の点p2にもっとも近い点を得る。
     * @param dest 出力先
     * @param p1
     * @param v
     * @param p2
     */
    public static void getClosestPoint(BVector2f dest, BVector2f p1, BVector2f v, BVector2f p2)
    {
        float d = v.getLength();
        if (d == 0)
        {
            dest.x = Float.NaN;
            dest.y = Float.NaN;
            return;
        }
        float t = ((p1.x - p2.x)*(v.x) + (p1.y-p2.y)*(v.y)) / d;
        dest.x = p1.x + v.x * t;
        dest.y = p1.y + v.y * t;
    }
    /**
     * 2つの物体が接触しているときの法線を得る。現在はCIRCLEとRECTANGLEのみ正常に作動する。
     * @param dest1 物体1の法線
     * @param dest2 物体2の法線
     * @param p1 物体1の位置
     * @param shape1 物体1の形状
     * @param p2 物体2の位置
     * @param shape2 物体2の形状
     * @return 正常に法線が得られたときにtrue、そうでないときはfalse。
     */
    public static boolean calcNormal(BVector2f dest1, BVector2f dest2, BVector2f p1, BShape shape1, BVector2f p2, BShape shape2)
    {
        if (shape1.getType() == BShape.CIRCLE)
        {
            if (shape2.getType() == BShape.CIRCLE)
            {
                return calcNormalCirclexCircle(dest1, dest2, p1, p2);
            }
            else if (shape2.getType() == BShape.RECTANGLE)
            {
                return calcNormalCirclexRectangle(dest1, dest2, p1, shape1.getRadius(), p2, shape2.getRectangle());
            }
            else
                throw new RuntimeException("Not implement yet.");

        }
        else if (shape1.getType() == BShape.RECTANGLE)
        {
            if (shape2.getType() == BShape.CIRCLE)
            {
                return calcNormalCirclexRectangle(dest2, dest1, p2, shape2.getRadius(), p1, shape1.getRectangle());
            }
            else
                throw new RuntimeException("Not implement yet.");
        }
        else
        {
            throw new RuntimeException("Not implement yet.");
        }
    }
    public static boolean calcNormalCirclexCircle(BVector2f dest1, BVector2f dest2, BVector2f p1, BVector2f p2)
    {
        if (BVector2f.getDistance(p1, p2) == 0)
            return false;
        dest1.set(p2);
        dest1.sub(p1);
        dest1.normalize();
        dest2.set(-dest1.x, -dest1.y);
        return true;
    }
    public static boolean calcNormalCirclexLine(BVector2f dest1, BVector2f dest2, BVector2f p1, float radius1, BVector2f p2, BVector2f p21, BVector2f p22)
    {
        BVector2f p21d = new BVector2f(p21);
        BVector2f p22d = new BVector2f(p22);
        p21d.add(p2);
        p22d.add(p2);
        BVector2f pos = new BVector2f(p1);
        
        BVector3f line = new BVector3f();
        getLineFunction(line, p21d, p22d);

        BVector2f n = new BVector2f(line.x , line.y);
        n.normalize();
        float dist = (float)(- line.z / Math.sqrt(line.x*line.x + line.y*line.y) - BVector2f.getDot(n, p1));
        pos.scaleAdd(dist,n);

        if (isOnLine(p21d, p22d, pos))
        {
            pos.sub(p1);
            if (!pos.isZeroVector())
            {
                pos.normalize();
                dest1.set(pos);
                dest2.set(-pos.x, -pos.y);
                return true;
            }
            else
            {
                return false;
            }
        }
        float d1 = BVector2f.getDistance(p1, p21d);
        float d2 = BVector2f.getDistance(p1, p22d);
        if (d1 < d2)
        {
            return calcNormalCirclexCircle(dest1, dest2, p1, p21d);
        }
        else
        {
            return calcNormalCirclexCircle(dest1, dest2, p1, p22d);
        }
    }

    public static boolean calcNormalCirclexRectangle(BVector2f dest1, BVector2f dest2, BVector2f p1, float radius1, BVector2f p2, BVector4f sq2)
    {
        BVector2f pos1 = new BVector2f(p1);
        pos1.sub(p2);
        float x = pos1.x;
        float y = pos1.y;
        float x1 = sq2.a;
        float x2 = sq2.b;
        float y1 = sq2.c;
        float y2 = sq2.d;
        if (x1 < x && x < x2)
        {
            if (y1 < y && y < y2)
            {
                if (pos1.isZeroVector())
                    return false;
                pos1.normalize();
                dest2.set(pos1);
                dest1.set(-pos1.x, -pos1.y);
                return true;
            }
            else if (y1 <= y)
            {
                dest1.set(0,-1f);
                dest2.set(0, 1f);
                return true;
            }
            else
            {
                dest1.set(0, 1f);
                dest2.set(0,-1f);
                return true;
            }
        }
        else if (y1 < y && y < y2)
        {
            if (x1 <= x)
            {
                dest1.set(-1f,0f);
                dest2.set( 1f,0f);
                return true;
            }
            else
            {
                dest1.set( 1f,0f);
                dest2.set(-1f,0f);
                return true;
            }            
        }
        else
        {
            if (x1 < x)
            {
                if (y1 < y)
                {
                    pos1.x -= x1;
                    pos1.y -= y1;
                    if (pos1.isZeroVector())
                    {
                        pos1.set(-1,-1);
                    }
                    pos1.normalize();
                    dest2.set(pos1);
                    dest1.set(-pos1.x, -pos1.y);
                    return true;
                }
                else
                {
                    pos1.x -= x1;
                    pos1.y -= y2;
                    if (pos1.isZeroVector())
                    {
                        pos1.set(-1, 1);
                    }
                    pos1.normalize();
                    dest2.set(pos1);
                    dest1.set(-pos1.x, -pos1.y);
                    return true;
                }
            }
            else
            {
                if (y1 < y)
                {
                    pos1.x -= x2;
                    pos1.y -= y1;
                    if (pos1.isZeroVector())
                    {
                        pos1.set( 1,-1);
                    }
                    pos1.normalize();
                    dest2.set(pos1);
                    dest1.set(-pos1.x, -pos1.y);
                    return true;
                }
                else
                {
                    pos1.x -= x2;
                    pos1.y -= y2;
                    if (pos1.isZeroVector())
                    {
                        pos1.set( 1, 1);
                    }
                    pos1.normalize();
                    dest2.set(pos1);
                    dest1.set(-pos1.x, -pos1.y);
                    return true;
                }
            }
        }
    }
    
    public static void main(String[] args)
    {
        test4();
    }
    private static void test1()
    {
        BVector2f start = new BVector2f(-10,0);
        BVector2f end = new BVector2f(10,0);
        BVector2f pos = new BVector2f(0,0);
        float radius = 10;
        
        float dest = getClosestIntersection(start, end, pos, radius);
        System.out.println(dest);
    }
    private static void test2()
    {
        BVector2f normaldest1 = new BVector2f();
        BVector2f normaldest2 = new BVector2f();
        BVector2f p1 = new BVector2f(-5,0);
        float radius1 = 1;
        BVector2f v1 = new BVector2f(1,0);
        BVector2f p2 = new BVector2f(5,0);
        float radius2 = 2;
        BVector2f v2 = new BVector2f(-1,0);
        float time = nextCollidCirclexCircle(p1,radius1,v1,p2,radius2,v2);
        System.out.println(time);
        System.out.println(normaldest1);
        System.out.println(normaldest2);
        
    }
    private static void test3()
    {
        BVector2f start = new BVector2f(5,5);
        BVector2f end = new BVector2f(-5,-5);
        BVector2f pos = new BVector2f(1,1);
        float p = BMath.getClosestIntersection(start,end,pos,1);
        System.out.println(p);
    }
    private static void test4()
    {
        BVector2f p1 = new BVector2f(0,-4);
        float radius1 = 0.5f;
        BVector2f v1 = new BVector2f(1,2);
        BVector2f p2 = new BVector2f(0,0);
        BVector2f p21 = new BVector2f( 10, 0);
        BVector2f p22 = new BVector2f(-10,0);
        float radius2 = 0.5f;
        BVector2f v2 = new BVector2f(0,0);
        float t = BMath.nextCollidCirclexLine(p1, radius1, v1, p2, p21, p22, v2);
        System.out.println(t);

        BVector2f pos = new BVector2f(-4.2f, 0);
        boolean b = BMath.isIntersectCirclexLine(pos, radius1, p2, p21,p22);
        
        System.out.println(b);
    }
    private static void test5()
    {
        BVector4f sq1 = new BVector4f(-1f, 1f, -1f, 1f);
        BVector4f sq2 = new BVector4f(-1f, 1f, -1f, 1f);
        BVector2f p1 = new BVector2f( 5.0f, 5.0f);
        BVector2f p2 = new BVector2f( 5.0f, 5.0f);
        //boolean flag = isIntersectRectanglexRectangle(p1, sq1, p2, sq2);
        boolean flag = isIntersectCirclexRectangle(p1, 1f, p2, sq2);
        System.out.println(flag);
    }
    private static void test6()
    {
        BVector4f sq1 = new BVector4f(-1f, 1f, -1f, 1f);
        BVector4f sq2 = new BVector4f(-1f, 1f, -1f, 1f);
        BVector2f p1 = new BVector2f( 0.0f, 0.0f);
        BVector2f p2 = new BVector2f( 0.0f, 0.0f);
        BVector2f v1 = new BVector2f( 0.0f, 0.0f);
        BVector2f v2 = new BVector2f( 1.0f, 1.0f);
        //boolean flag = isIntersectRectanglexRectangle(p1, sq1, p2, sq2);
        int n = 16;
        for (int i = 0; i<n;i++)
        {
            double rad = Math.PI * 2 * ((double)i/(double)n);
            p1.set(2.5f,0);
            p1.rotate(rad);
            v1.set(p1);
            v1.scale(-1f);
            v1.normalize();
            p1.add(p2);
            v1.add(v2);
            //float t = nextCollidPointxRectangle(p1, v1, p2, sq2, v2);
            float t = nextCollidCirclexRectangle(p1, 1f, v1,p2, sq2, v2);
            System.out.println(t);
        }
    }
}
