/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */
/*
 * 作成日: 2005/07/22
 */
package bakutan.common.math;

/**
 * @author cattaka
 * 
 * 形状を表すクラス
 */
public class BShape
{
    public static final int CIRCLE = 0;
    public static final int LINE = 1;
    public static final int RECTANGLE = 2;
    public static final int COMPOSITE = 3;
    private float radius;
    private int type;
    private BVector2f p1;
    private BVector2f p2;
    private BVector4f rectangle;
    private BShape[] composite;
    public BShape()
    {
        super();
        setType(CIRCLE);
    }
    public BShape(float radius)
    {
        super();
        this.radius = radius;
        setType(CIRCLE);
    }
    public BShape(BVector2f p1, BVector2f p2)
    {
        super();
        setType(LINE);
        setPoint1(p1);
        setPoint2(p2);
    }
    public BShape(BVector4f square)
    {
        super();
        this.radius = 0;
        setType(RECTANGLE);
        setRectangle(square);
    }
    public BShape(BShape[] shapes)
    {
        super();
        setType(COMPOSITE);
        setComposite(shapes);
    }
    /**
     * 半径を得る
     * @return 半径
     */
    public float getRadius()
    {
        return radius;
    }
    /**
     * 半径を設定する
     * @param radius 設定する半径
     */
    public void setRadius(float radius)
    {
        this.radius = radius;
    }
    public void setType(int type)
    {
        this.type = type;
        if (type == CIRCLE)
        {
            p1 = null;
            p2 = null;
        }
        else if (type == LINE)
        {
            p1 = new BVector2f();
            p2 = new BVector2f();
        }
        else if (type == RECTANGLE)
        {
            rectangle = new BVector4f();
        }
    }
    public int getType()
    {
        return type;
    }
    public BVector2f getPoint1()
    {
        return p1;
    }
    public void setPoint1(BVector2f p1)
    {
        this.p1.set(p1);
    }
    public BVector2f getPoint2()
    {
        return p2;
    }
    public void setPoint2(BVector2f p2)
    {
        this.p2.set(p2);
    }
    public BVector4f getRectangle()
    {
        return rectangle;
    }
    public void setRectangle(BVector4f sq)
    {
        rectangle = sq;
        if (rectangle.a > rectangle.b)
        {
            rectangle.a = sq.b;
            rectangle.b = sq.a;
        }
        if (rectangle.c > rectangle.d)
        {
            rectangle.c = sq.d;
            rectangle.d = sq.c;
        }
    }
    public BShape[] getComposite()
    {
        return composite;
    }
    public void setComposite(BShape[] composite)
    {
        this.composite = new BShape[composite.length];
        System.arraycopy(composite, 0, this.composite, 0, this.composite.length);
    }
    
}
