/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/03/16
 */
package bakutan.common.util;

import javax.sound.midi.ControllerEventListener;
import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MetaEventListener;
import javax.sound.midi.MetaMessage;
import javax.sound.midi.MidiChannel;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Sequence;
import javax.sound.midi.Sequencer;
import javax.sound.midi.ShortMessage;
import javax.sound.midi.Synthesizer;

public class BBGM implements MetaEventListener, ControllerEventListener
{
	private Sequencer sequencer;
    private Synthesizer synthesizer;
    private MidiChannel[] channels;
    private int[] orgVollume;
    private BGMBundle bgmBundle;
	private int volume = 64;
    private float volumeF = 0.5f;
    //private boolean enable = true;
    
	public BBGM()
	{
	}
	public void setEnable(boolean enable)
	{
		if (enable)
		{
            if (!(sequencer != null && sequencer.isOpen()))
            {
                try
                {
                    if (synthesizer == null)
                    {
                        synthesizer = MidiSystem.getSynthesizer();
                        synthesizer.open();
                        channels = synthesizer.getChannels();
                        orgVollume = new int[channels.length];
                    }
                    sequencer = MidiSystem.getSequencer(false);
                    sequencer.getTransmitter().setReceiver(synthesizer.getReceiver());

                    sequencer.addControllerEventListener(this, new int[]{7});
                    sequencer.addMetaEventListener(this);
                    sequencer.open();
                }catch(MidiUnavailableException e)
                {
                    e.printStackTrace();
                    this.sequencer = null;
                }
            }
		}
        else
        {
            stop();
            if (sequencer != null && sequencer.isOpen())
            {
                sequencer.close();
                sequencer = null;
            }
        }
	}
	
	public void play()
	{
        if (sequencer == null || bgmBundle == null)
            return;
        stop();
        
		try
		{
			Sequence seq = bgmBundle.getRandom();
			if (seq != null)
			{
				sequencer.setSequence(seq);
				sequencer.start();
			}
        }
        catch(InvalidMidiDataException e)
		{
			e.printStackTrace();
		}
	}
	public void stop()
	{
		if (sequencer == null)
			return;
        if (sequencer.isRunning())
            sequencer.stop();
	}
	public boolean isEnable()
	{
		return (sequencer != null);
	}
	public BGMBundle getBgmBundle()
	{
		return bgmBundle;
	}
	public void setBgmBundle(BGMBundle bgmBundle)
	{
		this.bgmBundle = bgmBundle;
	}
    private void createSequencer()
    {
        if (!(sequencer != null && sequencer.isOpen()))
        {
            try
            {
                if (synthesizer == null)
                {
                    synthesizer = MidiSystem.getSynthesizer();
                    synthesizer.open();
                    channels = synthesizer.getChannels();
                    orgVollume = new int[channels.length];
                }
                sequencer = MidiSystem.getSequencer(false);
                sequencer.getTransmitter().setReceiver(synthesizer.getReceiver());

                sequencer.addControllerEventListener(this, new int[]{7});
                sequencer.addMetaEventListener(this);
                sequencer.open();
            }catch(MidiUnavailableException e)
            {
                e.printStackTrace();
                this.sequencer = null;
                setEnable(false);
            }
        }
    }
    public void meta(MetaMessage meta)
    {
        if (meta.getType() == 47)
        {
//            for (int i=0;i<channels.length;i++)
//            {
//                //channels[i].allNotesOff();
//                channels[i].allSoundOff();
//            }
//            System.out.println(meta);
            new Thread()
            {
                public void run() {}
                {
                    try
                    {
                        Thread.sleep(1000);
                    }
                    catch(InterruptedException e)
                    {
                    }
                    play();
                }
            }.start();
        }
    }
    public void controlChange(ShortMessage message)
    {
        int channel = message.getChannel();
        MidiChannel ch = synthesizer.getChannels()[channel];
        orgVollume[channel] = ch.getController(7);
        //System.out.print(ch.getController(7) +":");
        ch.controlChange(7,(int)(volumeF * (float)orgVollume[channel]));
        //System.out.println(ch.getController(7));
    }
    
    public int getVolume()
    {
        return volume;
    }
    public void setVolume(int volume)
    {
        this.volume = volume;
        if (this.volume > 127)
            this.volume = 127;
        if (this.volume < 0)
            this.volume = 0;
        volumeF = (float)volume/127f;
        if (channels != null)
        {
            for (int i=0, n=channels.length; i<n; i++)
            {
                //System.out.print(channels[i].getController(7) +":");
                channels[i].controlChange(7,(int)(volumeF * (float)orgVollume[i]));
                //System.out.println(channels[i].getController(7));
            }
            //System.out.println("ok");
        }
    }
 
}
