/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/04
 */
package bakutan.common.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.LinkedList;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;

import bakutan.common.math.BVector2f;



/**
 * @author cattaka
 */
public class BSound implements Runnable
{
    public static final float DISTANCE = 5;
    public static final float WIDTH = 0f;
	private SEBundle audioBundle;
    private SourceDataLine[] lines;
    //private FloatControl[] controls;
    private PlayInfo[] playInfos;
    private int next = 0;
    private int lineNum = 0;
    private LinkedList<PlayInfo> playList = new LinkedList<PlayInfo>();
	private Thread thread;
    private boolean goflag = false;
    private int volume = 64;
    private float volumeF = 0.5f;
    private long[] endTime;
    
	public BSound(int lineNum)
	{
        setLineNum(lineNum);
	}

    public void setLineNum(int lineNum)
    {
        if (this.lineNum != lineNum)
        {
            this.lineNum = lineNum;
            setEnable(true);
        }
        else
        {
            this.lineNum = lineNum;
        }
    }
    
    public synchronized void setEnable(boolean enable)
    {
        if (enable)
        {
            if ((lines != null && lines.length != lineNum)
                    || (thread != null && thread.isAlive()))
            {
                setEnable(false);
            }
            try
            {
//                URL url = getClass().getClassLoader().getResource("sound/fire.wav");
//                AudioFileFormat fileFormat = AudioSystem.getAudioFileFormat(url);
//                AudioFormat format = fileFormat.getFormat();
                //PCM_SIGNED 22050.0 Hz, 16 bit, mono, 2 bytes/frame, little-endian
                AudioFormat format = new AudioFormat(22050.0f, 16, 1, true, false);
                //DataLine.Info dataInfo = new DataLine.Info(SourceDataLine.class, format);
                lines = new SourceDataLine[lineNum];
                //controls = new FloatControl[lineNum];
                playInfos = new PlayInfo[lineNum];
                endTime = new long[lineNum];
//                soundData = new byte[lineNum][];
//                wroteData = new int[lineNum];
                for (int i=0;i<lineNum;i++)
                {
                    //lines[i] = (SourceDataLine)AudioSystem.getLine(dataInfo);
                    lines[i] = AudioSystem.getSourceDataLine(format);
                    lines[i].open(format);
//                    lines[i].start();
//                    controls[i] = (FloatControl)lines[i].getControl(FloatControl.Type.VOLUME);
                }
                next = 0;
                this.thread = new Thread(this,"Bakutan Sound thread");
                this.thread.start();
            }catch(Exception e)
            {
                lines = null;
                e.printStackTrace();
                setEnable(false);
                return;
            }
        }
        else
        {
            if (thread != null && thread.isAlive())
            {
                goflag = false;
                synchronized (playList)
                {
                    playList.notify();
                }
                try
                {
                    thread.join();
                }
                catch(InterruptedException e)
                {
                }
            }
                
            if (lines != null)
            {
                for (int i=0;i<lines.length;i++)
                {
                    lines[i].stop();
                    lines[i].close();
                }
                lines = null;
            }
        }
    }
    public boolean isRunning()
    {
        return (thread != null && thread.isAlive());
    }
    public void run()
    {
        goflag = true;
        byte[] d = new byte[4];
        while(goflag)
        {
            synchronized (playList)
            {
                try
                {
                    playList.wait(50);
                }
                catch(InterruptedException e)
                {
                }
            }
            long currenttime = System.currentTimeMillis();
            for (int i=0;i<lines.length;i++)
            {
                PlayInfo pi = playInfos[i];
                if (pi != null)
                {
                    byte[] sd = pi.soundData;
                    int wd = pi.wroteData;
                    if (sd.length == wd)
                    {
                        if (!lines[i].isActive() || currenttime - endTime[i] > 0)
                        {
                            lines[i].stop();
                            playInfos[i] = null;
                        }
                    }
                    else
                    {
                        int w = lines[i].available();
                        if (wd + w > sd.length)
                        {
                            w = sd.length - wd;
                        }
                        byte[] td = new byte[w];
                        for (int j=0;j<w;j++)
                        {
                            td[j] = (byte)(sd[wd+j] * pi.vr);
                        }
                        lines[i].write(td,0,td.length);
                        wd += w;
                        pi.wroteData = wd;
                    }
                }
            }
            synchronized (playList)
            {
                boolean wait = true;
                while(playList.size() > 0)
                {
                    wait = false;
                    PlayInfo pi = playList.removeFirst();
                    for (int i=0;i<lines.length;i++)
                    {
                        if (playInfos[i] == null)
                        {
                            playInfos[i] = pi;
                            endTime[i] = currenttime + (pi.soundData.length / 22);
                            lines[i].flush();
                            lines[i].start();
//                            controls[i].setValue(pi.vr);
                            break;
                        }
                    }
                }
                if (wait)
                {
                    try
                    {
                        playList.wait(100);
                    }catch(InterruptedException e)
                    {
                    }
                }
            }
        }
    }
	public static byte[] loadAudioFile(URL url)
	{
		try
		{
			ByteArrayOutputStream bao = new ByteArrayOutputStream(1024);
			AudioInputStream ais;
			ais = AudioSystem.getAudioInputStream(url);
			byte[] bs = new byte[(int)ais.getFrameLength()];
			while(true)
			{
				int s = ais.read(bs);
				if (s == -1)
					break;
				bao.write(bs,0,s);
			}
			bs = bao.toByteArray();
			ais.close();
			bao.close();
			return bs;
		}catch(IOException e)
		{
			e.printStackTrace();
			return null;
		}catch(UnsupportedAudioFileException e)
		{
			e.printStackTrace();
			return null;
		}
	}
	
	public synchronized void play(byte type, int rag)
	{
		if (!(isRunning()) || audioBundle == null)
			return;
        byte[] sd = audioBundle.getAudioData(type);
        if (sd == null)
            return;
		PlayInfo pi = new PlayInfo(type, rag, volumeF, sd);
        synchronized (playList)
        {
            playList.add(pi);
            playList.notify();
        }
	}
    public synchronized void play(byte type, int rag, BVector2f center, BVector2f pos)
    {
        if (!(isRunning()) || audioBundle == null)
            return;
        byte[] sd = audioBundle.getAudioData(type);
        if (sd == null)
            return;
        PlayInfo pi = new PlayInfo(type, rag, volumeF, sd, center, pos);
        synchronized (playList)
        {
            playList.add(pi);
            playList.notify();
        }
    }

    public SEBundle getSEBundle()
	{
		return audioBundle;
	}
	public void setSEBundle(SEBundle audioBundle)
	{
		this.audioBundle = audioBundle;
	}
    public int getVolume()
    {
        return volume;
    }
    public void setVolume(int volume)
    {
        this.volume = volume;
        if (this.volume > 127)
            this.volume = 127;
        if (this.volume < 0)
            this.volume = 0;
        volumeF = (float)volume/127f;
    }
}
class PlayInfo
{
    private byte type;
    private int rag;
    int wroteData;
    byte[] soundData;
//    float vl = 1f;
    float vr = 1f;
    
    public PlayInfo(byte type, int rag, float volume, byte[] soundData)
    {
        this.type = type;
        this.rag = rag;
        this.wroteData = 0;
        this.soundData = soundData;
        this.vr = volume;
    }
    public PlayInfo(byte type, int rag, float volume, byte[] soundData, BVector2f center, BVector2f pos)
    {
        this(type,rag, volume, soundData);
        float d = BSound.DISTANCE;
        float f;
        d = d*d;
        
        BVector2f p = new BVector2f();
        p.set(center);
//        p.x += BAudio.WIDTH / 2;
        p.sub(pos);
        f = p.getLengthSquare();
        vr = (float)Math.sqrt(d/(f+d)) * volume;
        
//        p.set(center);
//        p.x -= BAudio.WIDTH / 2;
//        p.sub(pos);
//        f = p.getLengthSquare();
//        vl = d/(f+d); //(float)Math.sqrt(d/(f+d));
    }
    public int getRag()
    {
        return rag;
    }
    public byte getType()
    {
        return type;
    }
    
}
