/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/03/02
 */

package bakutan.common.util;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;

public class ImplementDataFlusher implements DataFlusher,Runnable
{
	private OutputStream out;
	private ArrayList<byte[]> buf = new ArrayList<byte[]>();
	private boolean goflag = false;
	private Thread thread;
    private String name;
    private int priority;
    
	public ImplementDataFlusher(String name, OutputStream out, int priority)
	{
        this.name = name;
		this.priority = priority;
		this.out = out;
	}
	public ImplementDataFlusher(String name, OutputStream out)
	{
        this.name = name;
        this.priority = Thread.NORM_PRIORITY;
		this.out = out;
	}
	public ImplementDataFlusher(OutputStream out)
	{
        this.priority = Thread.NORM_PRIORITY;
		this.out = out;
	}
	
    public void start()
    {
        if (thread != null)
            throw new RuntimeException("Alread started");
        thread = new Thread(this);
        if (this.name != null)
            thread.setName(this.name);
        thread.setPriority(priority);
        thread.start();
    }
    
    public boolean isAlive()
    {
        return (thread != null && thread.isAlive());
    }
    
	public void run()
	{
		try
		{
			goflag = true;
			while(true)
			{
				byte[] bs;
				int n;
				synchronized(buf)
				{
					n = buf.size();
					if (n == 0)
					{
						if (!goflag)
						{
							break;
						}
						try
						{
							buf.wait();
						}catch(InterruptedException e)
						{
							e.printStackTrace();
						}
						continue;
					}else
					{
						n--;
						bs = buf.remove(0);
					}
				}
				out.write(bs);
				if (n == 0)
				{
					out.flush();
				}
			}
		}catch(IOException e)
		{
			//e.printStackTrace();
		}
        try
        {
            out.close();
        }
        catch(IOException e)
        {
        }
		afterStop();
	}
	public void sendPacket(byte[] packet)
	{
		if (packet == null)
		{
			throw new NullPointerException();
		}
		synchronized(buf)
		{
			if (goflag)
			{
				buf.add(packet);
				buf.notify();
			}
		}
	}
    public void sendPacket(byte[] packet, boolean flush)
    {
        if (packet == null)
        {
            throw new NullPointerException();
        }
        synchronized(buf)
        {
            if (goflag)
            {
                buf.add(packet);
                if (flush)
                    buf.notify();
            }
        }
    }
	public void dispose()
	{
		synchronized(buf)
		{
			goflag = false;
			buf.notify();
		}
	}
	public void afterStop()
	{
	}
	public void waitForStop()
	{
		try
		{
            if (thread != null)
                thread.join();
		}catch(InterruptedException e)
		{
		}
	}
}
