/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

package bakutan.common.util;
import java.util.TreeMap;

public class LanguageTranslator
{
    private TreeMap<String,String> phrases;
    public LanguageTranslator()
    {
        phrases = new TreeMap<String,String>();
    }
    public String[] getKeys()
    {
        Object[] objs = phrases.keySet().toArray();
        String[] keys = new String[objs.length];
        for(int i=0;i<objs.length;i++)
            keys[i] = (String)objs[i];
        return keys;
    }
    public String getValue(String key)
    {
        return (String)(phrases.get(key));
    }
    public void setValue(String key,String value)
    {
        phrases.put(key,value);
    }

    public String translate(String key)
    {
    	return translate(key,null);
    }
    
    public String translate(String key, String[] opt)
    {
        String [] args = opt;
        if (args == null)
            args = new String[0];
        String phrase = (String)(phrases.get(key));
        if (phrase != null)
        {
            for (int i=0;i<args.length;i++)
            {
                phrase = phrase.replaceAll("<arg"+i+">",args[i]);
            }
            return phrase;
        }else{
            StringBuffer buf = new StringBuffer(key);
			if (args.length > 0)
			{
				buf.append(":");
				for (int i=0;i<args.length;i++)
				{
					buf.append(args[i]);
					if (i+1<args.length)
						buf.append(",");
				}
			}
            return buf.toString();
        }
    }
}
