/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/01/19
 */
package bakutan.common.util;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import bakutan.common.BStage;
import bakutan.common.Body;
import bakutan.common.ClientStageEvent;
import bakutan.common.DataManager;
import bakutan.common.event.BodyEvent;
import bakutan.common.event.StageEvent;
import bakutan.common.serverinfo.BStageState;
import bakutan.common.serverinfo.SmallBStageState;

/**
 * @author cattaka
 */
public class PacketManager
{
    private ByteArrayOutputStream bout;
	private DataOutputStream out;
    private DataManager dataManager;
	public PacketManager(DataManager dataManager)
	{
        this.dataManager = dataManager;
		bout = new ByteArrayOutputStream(1<<20); // 1 mega byte
		out = new DataOutputStream(bout);
	}
	
	public byte[] makePacket(StageEvent se)
	{
		try
		{
			bout.reset();
			dataManager.outputData(out,se);
			return bout.toByteArray();
		}catch(IOException e)
		{
			throw new RuntimeException(e);
		}
	}
    public byte[] makePacket(ClientStageEvent cse)
    {
        try
        {
            bout.reset();
            dataManager.outputData(out,cse);
            return bout.toByteArray();
        }catch(IOException e)
        {
            throw new RuntimeException(e);
        }
    }
	public byte[] makePacket(BodyEvent be)
	{
		try
		{
			bout.reset();
			dataManager.outputData(out,be);
			return bout.toByteArray();
		}catch(IOException e)
		{
			throw new RuntimeException(e);
		}
	}
	/*
	public byte[] makePacket(BodyEvent be)
	{
		try
		{
			bout.reset();
			dataManager.outputData(out,be);
			return bout.toByteArray();
		}catch(IOException e)
		{
			throw new RuntimeException(e);
		}
	}
	*/
	public byte[] makePacket(BStage bstage)
	{
		try
		{
			bout.reset();
			dataManager.outputData(out,bstage);
			return bout.toByteArray();
		}catch(IOException e)
		{
			throw new RuntimeException(e);
		}
	}
	public byte[] makePacket(Body[] bs)
	{
		try
		{
			bout.reset();
			dataManager.outputData(out,bs);
			return bout.toByteArray();
		}catch(IOException e)
		{
			throw new RuntimeException(e);
		}
	}
	public byte[] makePacket(int i)
	{
		try
		{
			bout.reset();
			out.writeInt(i);
			return bout.toByteArray();
		}catch(IOException e)
		{
			throw new RuntimeException(e);
		}
	}
    public byte[] makePacket(String str)
    {
        try
        {
            bout.reset();
            out.writeUTF(str);
            return bout.toByteArray();
        }catch(IOException e)
        {
            throw new RuntimeException(e);
        }
    }
	public byte[] makePacket(BStageState bss)
	{
		try
		{
			bout.reset();
			DataManager.outputData(out,bss);
			return bout.toByteArray();
		}catch(IOException e)
		{
			throw new RuntimeException(e);
		}
	}
    public byte[] makePacket(SmallBStageState bss)
    {
        try
        {
            bout.reset();
            DataManager.outputData(out,bss);
            return bout.toByteArray();
        }catch(IOException e)
        {
            throw new RuntimeException(e);
        }
    }
}
