/*
 * 作成日: 2007/01/30
 */
package bakutan.common.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

public class PipedStream
{
    private byte[] buf;
    private int bufSize;
    private int pointIn = 0;
    private int pointOut = 0;
    private InputStream in;
    private OutputStream out;
    private boolean goflag = false;
    
    public PipedStream()
    {
        this(1024);
    }
    public PipedStream(int bufSize)
    {
        super();
        this.bufSize = bufSize;
        buf = new byte[bufSize];
        in = new PipeInput();
        out = new PipeOutput();
        goflag = true;
    }
    
    class PipeInput extends InputStream
    {
        @Override
        public int read() throws IOException
        {
            synchronized (buf)
            {
                if (!goflag)
                    return -1;
                while (pointIn == pointOut)
                {
                    try
                    {
                        buf.wait();
                    }
                    catch(InterruptedException e)
                    {
                    }
                }
                if (pointOut == pointIn-1)
                {
                    buf.notify();
                }
                int b = buf[pointIn++] & (0xFF);
//                System.out.println("<"+(pointIn-1)+":"+b);
                pointIn = pointIn % bufSize;
                return b;
            }
        }
        @Override
        public void close() throws IOException
        {
            super.close();
            goflag = false;
        }
    }
    class PipeOutput extends OutputStream
    {
        @Override
        public void write(int b) throws IOException
        {
            synchronized (buf)
            {
                if (!goflag)
                    throw new IOException();
                while (pointOut == pointIn-1)
                {
                    try
                    {
                        buf.wait();
                    }catch(InterruptedException e)
                    {
                    }
                }
                if (pointIn == pointOut)
                {
                    buf.notifyAll();
                }
//                System.out.print(">"+pointOut+":"+(b & 0xFF));
                buf[pointOut++] = (byte)(b & 0xFF);
                pointOut = pointOut % bufSize;
            }
        }
        @Override
        public void close() throws IOException
        {
            super.close();
            goflag = false;
        }
    }
    public InputStream getIn()
    {
        return in;
    }
    public OutputStream getOut()
    {
        return out;
    }
}
