/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/01/11
 *
 */
package bakutan.debug;

import java.io.IOException;
import java.io.InputStream;

/**
 * @author cattaka
 *
 */
public class RaggedInputStream extends InputStream implements Runnable
{
	private Thread waitingThread;
	private int current = 0;
	private int last = 0;
	private int[] buffer = new int[10000];
	private long[] stime = new long[10000];
	private InputStream in;
	
	private long basicRag = 500;
	private long ragWave = 100;
	
	public RaggedInputStream(InputStream in, long basicRag, long ragWave)
	{
	    this.basicRag = basicRag;
	    this.ragWave = ragWave;
		this.in = in;
		new Thread(this).start();
	}
	public void run()
	{
	    
		while(true)
		{
		    int b;
		    try
		    {		    
		        b = in.read();
		        if (b == -1)
		        {
		            throw new IOException();
		        }
		    }catch(IOException e)
		    {
		        try
		        {
			        in.close();
		        }catch(IOException ex)
		        {
		        }
		        this.notify();
			    buffer[last] = -1;
		        return;
		    }
	        synchronized(this)
	        {
			    if (last >= buffer.length)
		    		last = 0;
			    
			    buffer[last] = b;
			    
		    	stime[last] = System.currentTimeMillis();
	            last++;
		    	this.notify();
	        }
		}
	}
    public void close() throws IOException
    {
        in.close();
    }
    public int read() throws IOException
    {
        int b;
        long rag;
        synchronized(this)
		{
	    	//System.out.println("" + current + " : " +last );
			if (current == last)
			{
				try
				{
				    this.wait();
				}catch(InterruptedException e)
				{
				}
			}
			b = buffer[current];
			rag = stime[current] - System.currentTimeMillis() + basicRag + (long)((ragWave * 2) * Math.random()) - ragWave;
		}
	    current++;
	    if (current >= buffer.length)
	    	current = 0;
		try
		{
			if (rag > 0)
			{
			    Thread.sleep(rag);
			}
		}catch(InterruptedException e)
		{
		}
		return b;
    }
}
