/*
 * 作成日: 2007/02/06
 */
package bakutan.game.capturetheflag;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.geom.Ellipse2D;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;

import bakutan.common.Body;
import bakutan.common.ClientBody;
import bakutan.common.DataManager;
import bakutan.common.graphics.CharacterImage;
import bakutan.common.graphics.CharacterImageBundle;
import bakutan.common.graphics.CharacterImageND;
import bakutan.common.math.BMath;
import bakutan.common.math.BShape;
import bakutan.common.math.BVector2f;
import bakutan.common.math.BVector4f;
import bakutan.common.util.BSound;
import bakutan.common.util.SEBundle;
import bakutan.game.deathmatch.BStageDM;
import bakutan.game.deathmatch.BodyHoming;
import bakutan.game.deathmatch.BodyItem;

public class BStageCTF extends BStageDM
{
    /**
     * BodyのタイプID、フラグを表す。
     * @see Body#getType() getType()
     * @see BodyHoming BodyHoming
     */
    public static final byte TYPE_FLAG = 21;

    public static final byte SOUND_FLAG_RESET = 20;
    public static final byte SOUND_FLAG_PILLAGED = 21;
    public static final byte SOUND_FLAG_CAPTURED = 22;

    private HashMap<Integer, BVector2f[]> teamSpawnPoints = new HashMap<Integer, BVector2f[]>();

    public BStageCTF()
    {
        super();
    }
    @Override
    public String getGameName()
    {
        return "CTF";
    }
    @Override
    public byte[] getBodyTypeArray()
    {
        return new byte[]{
                TYPE_WALL1,
                TYPE_PLAYER1,
                TYPE_MISSILE1,
                TYPE_FIRE1,
                TYPE_ITEM,
                TYPE_SOFTWALL,
                TYPE_BOMB1,
                TYPE_SLUG1,
                TYPE_HOMING1,
                TYPE_CLUSTERBOMB,
                TYPE_FLAG
                };
    }
    @Override
    public Body[] getBodysHasScore()
    {
        Body[] bs = super.getBodysHasScore();
        Body[] flags = getBodysByType(TYPE_FLAG);
        Body[] ret = new Body[bs.length + flags.length];
        System.arraycopy(bs,0,ret,0,bs.length);
        System.arraycopy(flags,0,ret,bs.length,flags.length);
        return ret;
    }
    @Override
    public ClientBody createClientBody()
    {
        return new BodyCTFPlayer();
    }
    @Override
    public Body createBodyInstance(byte bodyType)
    {
        if (bodyType == TYPE_FLAG)
            return new BodyFlag();
        else if (bodyType == TYPE_PLAYER1)
            return new BodyCTFPlayer();
        return super.createBodyInstance(bodyType);
    }
    @Override
    public SEBundle createSEBundle()
    {
        SEBundle ad = super.createSEBundle();
        ArrayList<URL> al = new ArrayList<URL>();
        al.add(getClass().getClassLoader().getResource("sound/flagreset.wav"));
        al.add(getClass().getClassLoader().getResource("sound/flaggot.wav"));
        al.add(getClass().getClassLoader().getResource("sound/flagcaptured.wav"));
        
        for (byte i = 0;i< al.size();i++)
        {
            URL url;
            url = al.get(i);
            if (url != null)
                ad.putAudioData((byte)(i+20),BSound.loadAudioFile(url));
        }
        return ad;
    }
    @Override
    public CharacterImageBundle createCharacterImageBundle(GraphicsConfiguration gc)
    {
        try
        {
            CharacterImageBundle cib = super.createCharacterImageBundle(gc);
            CharacterImage img_flag = new CharacterImageND(gc, getClass().getClassLoader().getResourceAsStream("image/flag.png"));
            cib.putCharacterImage(TYPE_FLAG, img_flag);
            return cib;
        }catch(IOException e)
        {
            e.printStackTrace();
            throw new RuntimeException("Can't load CharacterImages");
        }
    }
    private static char[][] getChars(URL mapfile)
    throws IOException
    {
        BufferedReader in = new BufferedReader(new InputStreamReader(mapfile.openStream()));
        String tmp;
        ArrayList<char[]> al = new ArrayList<char[]>();
        while((tmp = in.readLine()) != null)
        {
            char[] cs = tmp.toCharArray();
            al.add(cs);
        }
        in.close();
        char[][] css = new char[al.size()][];
        for (int i=0;i<css.length;i++)
        {
            char[] cs = al.get(i);
            css[i] = cs;
        }
        return css;
    }
    @Override
    public void loadBStageFile(URL url) throws IOException
    {
        char[][] data;

        data = getChars(url);
            
        float h = data.length;
        float w = 0;
        for (int i=0;i<h;i++)
        {
            if (w < data[i].length)
                w = data[i].length;
        }
        
        setSize(new BVector2f(w, h));
        
        BStageCTF bstage = this;
        //bstage.setEventHandler(eventHandler);
        
        BodyFlag redFlag = null;
        BodyFlag blueFlag = null;
        ArrayList<BVector2f> defSpawnPoints = new ArrayList<BVector2f>();
        ArrayList<BVector2f> redSpawnPoints = new ArrayList<BVector2f>();
        ArrayList<BVector2f> blueSpawnPoints = new ArrayList<BVector2f>();
        ArrayList<BVector2f> freePoints = new ArrayList<BVector2f>();
        for (int i=1;i<data.length-1;i++)
        {
            for (int j=1;j<data[i].length-1;j++)
            {
                if (data[i][j] == ' ')
                {
                    BVector2f pos = new BVector2f();
                    pos.x = ((float)j) + 0.5f;
                    pos.y = ((float)i) + 0.5f;
                    freePoints.add(pos);
                }
                else if (data[i][j] == 'W')
                {
                    Body body = createBodyInstance(TYPE_WALL1);
                    body.setID(bstage.getNextID());
                    body.getPosition().x = ((float)j) + 0.5f;
                    body.getPosition().y = ((float)i) + 0.5f;
                    bstage.addBody(body);
                }
                else if (data[i][j] == 'S')
                {
                    Body body = createBodyInstance(TYPE_SOFTWALL);
                    body.setID(bstage.getNextID());
                    body.getPosition().x = ((float)j) + 0.5f;
                    body.getPosition().y = ((float)i) + 0.5f;
                    bstage.addBody(body);

                    BVector2f pos = new BVector2f();
                    pos.x = ((float)j) + 0.5f;
                    pos.y = ((float)i) + 0.5f;
                    freePoints.add(pos);
                }
                else if (data[i][j] == 's')
                {
                    BVector2f pos = new BVector2f();
                    pos.x = ((float)j) + 0.5f;
                    pos.y = ((float)i) + 0.5f;
                    defSpawnPoints.add(pos);
                }
                else if (data[i][j] == 'R')
                {
                    if (redFlag != null)
                        throw new IOException("Red flag already exists.");
                    redFlag = (BodyFlag)createBodyInstance(TYPE_FLAG);
                    redFlag.setName("Red");
                    redFlag.setID(bstage.getNextID());
                    redFlag.getPosition().x = ((float)j) + 0.5f;
                    redFlag.getPosition().y = ((float)i) + 0.5f;
                    redFlag.setDefaultPosition(redFlag.getPosition());
                    bstage.addBody(redFlag);
                    redFlag.setOption(BodyFlag.OPTION_TEAM, 0);

                    BVector2f pos = new BVector2f();
                    pos.x = ((float)j) + 0.5f;
                    pos.y = ((float)i) + 0.5f;
                    freePoints.add(pos);
                }
                else if (data[i][j] == 'B')
                {
                    if (blueFlag != null)
                        throw new IOException("Blue flag already exists.");
                    blueFlag = (BodyFlag)createBodyInstance(TYPE_FLAG);
                    blueFlag.setName("Blue");
                    blueFlag.setID(bstage.getNextID());
                    blueFlag.getPosition().x = ((float)j) + 0.5f;
                    blueFlag.getPosition().y = ((float)i) + 0.5f;
                    blueFlag.setDefaultPosition(blueFlag.getPosition());
                    bstage.addBody(blueFlag);
                    blueFlag.setOption(BodyFlag.OPTION_TEAM, 1);

                    BVector2f pos = new BVector2f();
                    pos.x = ((float)j) + 0.5f;
                    pos.y = ((float)i) + 0.5f;
                    freePoints.add(pos);
                }
                else if (data[i][j] == 'r')
                {
                    BVector2f pos = new BVector2f();
                    pos.x = ((float)j) + 0.5f;
                    pos.y = ((float)i) + 0.5f;
                    redSpawnPoints.add(pos);
                }
                else if (data[i][j] == 'b')
                {
                    BVector2f pos = new BVector2f();
                    pos.x = ((float)j) + 0.5f;
                    pos.y = ((float)i) + 0.5f;
                    blueSpawnPoints.add(pos);
                }
                else if (data[i][j] == 'I')
                {
                    BodyItem body = (BodyItem)createBodyInstance(TYPE_ITEM);
                    body.setID(bstage.getNextID());
                    body.selectRundom();
                    body.getPosition().x = ((float)j) + 0.5f;
                    body.getPosition().y = ((float)i) + 0.5f;
                    body.setRespawnable(true);
                    bstage.addBody(body);
                }
            }
        }
        
        if (redFlag == null)
            throw new IOException("Red flag is not found.");
        if (blueFlag == null)
            throw new IOException("Blue flag is not found.");
        
        BVector4f rect = new BVector4f();
        Body wallN = createBodyInstance(TYPE_WALL1);
        wallN.setID(bstage.getNextID());
        wallN.getPosition().set( w/2, 0.5f);
        wallN.getShape().setType(BShape.RECTANGLE);
        wallN.getShape().getRectangle().set(-w/2f, w/2f, -0.5f, 0.5f);
        bstage.addBody(wallN);
        Body wallS = createBodyInstance(TYPE_WALL1);
        wallS.setID(bstage.getNextID());
        wallS.getPosition().set( w/2, h-0.5f);
        wallS.getShape().setType(BShape.RECTANGLE);
        wallS.getShape().getRectangle().set(-w/2f, w/2f, -0.5f, 0.5f);
        bstage.addBody(wallS);
        Body wallW = createBodyInstance(TYPE_WALL1);
        wallW.setID(bstage.getNextID());
        wallW.getPosition().set( 0.5f, h/2);
        wallW.getShape().setType(BShape.RECTANGLE);
        wallW.getShape().getRectangle().set(-0.5f, 0.5f, -h/2f, h/2f);
        bstage.addBody(wallW);
        Body wallE = createBodyInstance(TYPE_WALL1);
        wallE.setID(bstage.getNextID());
        wallE.getPosition().set( w-0.5f, h/2);
        wallE.getShape().setType(BShape.RECTANGLE);
        wallE.getShape().getRectangle().set(-0.5f, 0.5f, -h/2f, h/2f);
        bstage.addBody(wallE);
        

        BVector2f[] sps = defSpawnPoints.toArray(new BVector2f[defSpawnPoints.size()]);
        BVector2f[] rsps = redSpawnPoints.toArray(new BVector2f[redSpawnPoints.size()]);
        BVector2f[] bsps = blueSpawnPoints.toArray(new BVector2f[blueSpawnPoints.size()]);
        BVector2f[] fps = freePoints.toArray(new BVector2f[freePoints.size()]);
        bstage.setSpawnPoint(sps);
        teamSpawnPoints.put(0, rsps);
        teamSpawnPoints.put(1, bsps);
        bstage.setFreePoint(fps);
    }
    public BVector2f getRandomeSpawnPoint(int team)
    {
        BVector2f[] sps = teamSpawnPoints.get(team);
        return sps[(int)(Math.random() * sps.length)];
    }
    public BVector2f[] getSpawnPoint(int team)
    {
        BVector2f[] vs = teamSpawnPoints.get(team);
        if (vs != null)
            return vs;
        return getSpawnPoint();
    }
    @Override
    public void drawRadar(Graphics bg, long currenttime, Dimension size, Body myBody, BVector2f center)
    {
        Body[] bs = getBodys();
        BVector2f radarCenter;
        if (myBody != null)
            radarCenter = myBody.getPosition();
        else
            radarCenter = center;
        final float radius = DataManager.DEFAULT_HEIGHT * 2;
        final float scale = 3;
        final BShape radarShape = new BShape(radius);
//      final Color backColor = new Color(127, 127,  127, 32);
//      final Color myBodyColor = new Color(0, 127, 0,191);
//      final Color playerColor = new Color(0, 0, 255,191);
//      final Color bomColor = new Color(255,0,0,191);
//      final Color softWallColor = new Color(255,255,255,127);
//      final Color wallColor = new Color(255,255,255,191);
//      final Color itemColor = new Color(255,255,0,191);

        final Color backColor = new Color(127, 127,  127);
        final Color myBodyColor = new Color(0, 127, 0);
        final Color playerColor = new Color(0, 0, 255);
        final Color bomColor = new Color(255,0,0);
        final Color softWallColor = new Color(255,255,255);
        final Color wallColor = new Color(255,255,255);
        final Color itemColor = new Color(255,255,0);
        final Color flagColor = new Color(255, 0, 255);
        
        int r = (int)(radius * scale);
        int cx = size.width - r - DataManager.BLOCKSIZE;
        int cy = size.height - r - DataManager.BLOCKSIZE*2;
        bg.translate(cx,cy);
        final Ellipse2D clipShape = new Ellipse2D.Float(-r, -r, r*2, r*2);
        bg.setColor(backColor);
        bg.drawOval(-r, -r, r*2, r*2);
        bg.drawOval(-r/2, -r/2, r, r);
//        bg.drawLine(-r, 0, r, 0);
//        bg.drawLine( 0,-r, 0, r);
        bg.setClip(clipShape);
        BVector2f vec = new BVector2f();
        for(int i=0;i<bs.length;i++)
        {
            int ms = 8;
            Body body = bs[i];
            switch(body.getType())
            {
            case TYPE_PLAYER1:
                if (myBody != null && body == myBody)
                    bg.setColor(myBodyColor);
                else
                    bg.setColor(playerColor);
                break;
            case TYPE_FLAG:
                bg.setColor(flagColor);
                break;
            case TYPE_ITEM:
//              continue;
                ms = 4;
                bg.setColor(itemColor);
                break;
            case TYPE_SOFTWALL:
//              continue;
                ms = 4;
                bg.setColor(softWallColor);
                break;
            case TYPE_WALL1:
//                if (body.getShape().getType() != BShape.LINE)
//                    continue;
                bg.setColor(wallColor);
                break;
            default:
                ms = 6;
                bg.setColor(bomColor);
            }
            if (!BMath.isIntersect(radarCenter, radarShape, body.getPosition(), body.getShape()))
                continue;
            vec.set(body.getPosition());
            vec.sub(radarCenter);

            BShape shape = body.getShape();
            int x = (int)((vec.x)*scale); 
            int y = (int)((vec.y)*scale);
            if (shape.getType() == BShape.CIRCLE)
            {
                bg.fillOval(x-ms/2,y-ms/2,ms, ms);
            }
            else if (shape.getType() == BShape.LINE)
            {
                int px1 = x + (int)((shape.getPoint1().x) * scale); 
                int py1 = y + (int)((shape.getPoint1().y) * scale);
                int px2 = x + (int)((shape.getPoint2().x) * scale); 
                int py2 = y + (int)((shape.getPoint2().y) * scale);
                bg.drawLine(px1, py1, px2, py2);
            }
            else if (shape.getType() == BShape.RECTANGLE)
            {
                BVector4f rec = shape.getRectangle();
                int x1 = x + (int)((rec.a) * scale); 
                int y1 = y + (int)((rec.c) * scale);
                int w = (int)((rec.b-rec.a) * scale); 
                int h = (int)((rec.d-rec.c) * scale);
                bg.fillRect(x1, y1, w, h);
            }
        }
        bg.setClip(null);
        bg.translate(-cx,-cy);
    }
    @Override
    public void drawScore(Graphics bg, long currenttime, Dimension size, Body myBody)
    {
        final Font teamFont = new Font("Serif",Font.BOLD,30);
        final Font scoreFont = new Font("Serif",Font.BOLD,20);
        Body[] bodys = getBodysByType(TYPE_PLAYER1);
        Body[] flags = getBodysByType(TYPE_FLAG);
        int x1 = DataManager.BLOCKSIZE * 4;
        int y1 = DataManager.BLOCKSIZE * 3;
        int x2 = DataManager.BLOCKSIZE * 12;
        int y2 = DataManager.BLOCKSIZE * 3;
        int s = bg.getFont().getSize();
        
        bg.setFont(teamFont);
        for (int i=0;i<flags.length;i++)
        {
            Body flag = flags[i];
            if (myBody != null && myBody.getOption(BodyCTFPlayer.OPTION_TEAM) == flag.getOption(BodyFlag.OPTION_TEAM))
                bg.setColor(Color.YELLOW);
            else
                bg.setColor(Color.WHITE);
                
            if (flag.getOption(BodyFlag.OPTION_TEAM) == 0)
            {
                bg.drawString(flag.getName(), x1, y1);
                bg.drawString(String.valueOf(flag.getScore()), x1 + DataManager.BLOCKSIZE * 7, y1);
                y1 += bg.getFont().getSize();
            }
            else if (flag.getOption(BodyFlag.OPTION_TEAM) == 1)
            {
                bg.drawString(flag.getName(), x2, y2);
                bg.drawString(String.valueOf(flag.getScore()), x2 + DataManager.BLOCKSIZE * 7, y2);
                y2 += bg.getFont().getSize();
            }
        }
            
        bg.setFont(scoreFont);
        for (int i=0;i<bodys.length;i++)
        {
            Body body = bodys[i];
            if (bodys[i] == myBody)
                bg.setColor(Color.YELLOW);
            else
                bg.setColor(Color.WHITE);
            if (body.getOption(BodyCTFPlayer.OPTION_TEAM) == 0)
            {
                bg.drawString(bodys[i].getName(), x1, y1);
                bg.drawString(String.valueOf(body.getScore()), x1 + DataManager.BLOCKSIZE * 7, y1);
                y1 += bg.getFont().getSize();
            }
            else if (body.getOption(BodyCTFPlayer.OPTION_TEAM) == 1)
            {
                bg.drawString(bodys[i].getName(), x2, y2);
                bg.drawString(String.valueOf(body.getScore()), x2 + DataManager.BLOCKSIZE * 7, y2);
                y2 += bg.getFont().getSize();
            }
        }
    }
}
