/*
 * 作成日: 2007/02/06
 */
package bakutan.game.capturetheflag;

import java.awt.Color;
import java.awt.Graphics;
import java.text.SimpleDateFormat;
import java.util.Date;

import bakutan.common.BStage;
import bakutan.common.Body;
import bakutan.common.DataManager;
import bakutan.common.EventHandler;
import bakutan.common.event.BodyArmorEvent;
import bakutan.common.graphics.CharacterImageBundle;
import bakutan.common.graphics.EffectBundle;
import bakutan.common.math.BVector2f;
import bakutan.game.deathmatch.BStageDM;
import bakutan.game.deathmatch.BodyItem;
import bakutan.game.deathmatch.BodyPlayer;

public class BodyCTFPlayer extends BodyPlayer
{
    /**
     * チームのオプションを得るための定数。
     * getOption(int)に使用する。
     */
    public static final byte OPTION_TEAM = 5;

    public BodyCTFPlayer()
    {
        super(BStageCTF.TYPE_PLAYER1, 6);
        setOption(OPTION_TEAM, -1);
    }
    @Override
    public void eventCommand(EventHandler eventHandler, BStage bstage, String command)
    {
        long currenttime = bstage.getCurrentTime();
        if (command.length() != 0)
        {
            if (command.charAt(0) != '/')
            {
                eventHandler.message(currenttime, currenttime, this, DataManager.MESSAGE_CHAT,command);
            }else if (command.length() > 6 && command.substring(0,6).equals("/note "))
            {
                SimpleDateFormat sdf = new SimpleDateFormat();
                String note = "["+this.getName()+"]{"+ sdf.format(new Date()) +")"+command.substring(6, command.length());
                eventHandler.message(currenttime, currenttime, this,DataManager.MESSAGE_NOTE, note);
            }
            else
            {
                String[] strs = command.split(" ", 2);
                if (strs.length < 2)
                    return;
                if (strs[0].toLowerCase().equals("/team"))
                {
                    BStageCTF stage = (BStageCTF)bstage;
                    if (strs[1].toLowerCase().equals("red"))
                    {
                        if (getOption(OPTION_TEAM) != 0)
                        {
                            eventHandler.option(currenttime, currenttime, this, OPTION_TEAM, 0);
                            eventHandler.setPosition(currenttime, currenttime, this, stage.getRandomeSpawnPoint(0));
                        }
                    }
                    else if (strs[1].toLowerCase().equals("blue"))
                    {
                        if (getOption(OPTION_TEAM) != 1)
                        {
                            eventHandler.option(currenttime, currenttime, this, OPTION_TEAM, 1);
                            eventHandler.setPosition(currenttime, currenttime, this, stage.getRandomeSpawnPoint(1));
                        }
                    }
                }
            }
        }
    }
    @Override
    public void eventStateChange(EventHandler eventHandler, BStage obstage, byte type, int state)
    {
        BStageCTF bstage = (BStageCTF)obstage;
        if ((type == Body.SET
                || type == Body.ENABLE)
                && (state & Body.STATE_DEAD) == Body.STATE_DEAD)
        {
            long currenttime = bstage.getCurrentTime();
            eventHandler.setPosition(currenttime, currenttime, this, getPosition());
            eventHandler.setPosition(currenttime, currenttime + 3000, this, bstage.getRandomeSpawnPoint(getOption(OPTION_TEAM)));

            for (int i=0;i<getOption(OPTION_FIRE);i++)
            {
                BVector2f pos = ((BStageDM)bstage).getRandomeFreePoint();
                BodyItem body = new BodyItem();
                body.setOption(BodyItem.OPTION_TYPE, BodyItem.ITEM_FIRE);
                body.setPosition(pos);
                if (!(bstage.isIntersectOther(body,STATE_COLLIDABLE)))
                {
                    eventHandler.add(currenttime, currenttime, body);
                    eventHandler.state(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL, body, ENABLE, STATE_DEAD);
                    eventHandler.delete(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL + 500, body);
                }
            }
            for (int i=0;i<getOption(OPTION_SPEED);i++)
            {
                BodyItem body = new BodyItem();
                body.setOption(BodyItem.OPTION_TYPE, BodyItem.ITEM_SPEED);
                body.setPosition(((BStageDM)bstage).getRandomeFreePoint());
                if (!(bstage.isIntersectOther(body,STATE_COLLIDABLE)))
                {
                    eventHandler.add(currenttime, currenttime, body);
                    eventHandler.state(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL, body, ENABLE, STATE_DEAD);
                    eventHandler.delete(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL + 500, body);
                }
            }
            for (int i=0;i<getOption(OPTION_CHARGE);i++)
            {
                BodyItem body = new BodyItem();
                body.setOption(BodyItem.OPTION_TYPE, BodyItem.ITEM_CHARGE);
                body.setPosition(((BStageDM)bstage).getRandomeFreePoint());
                if (!(bstage.isIntersectOther(body,STATE_COLLIDABLE)))
                {
                    eventHandler.add(currenttime, currenttime, body);
                    eventHandler.state(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL, body, ENABLE, STATE_DEAD);
                    eventHandler.delete(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL + 500, body);
                }
            }
            if ((getOption(OPTION_WEAPON) & WEAPON_KICK) == WEAPON_KICK)
            {
                BodyItem body = new BodyItem();
                body.setOption(BodyItem.OPTION_TYPE, BodyItem.ITEM_WEAPON);
                body.setWeapon(WEAPON_KICK);
                body.setPosition(((BStageDM)bstage).getRandomeFreePoint());
                if (!(bstage.isIntersectOther(body,STATE_COLLIDABLE)))
                {
                    eventHandler.add(currenttime, currenttime, body);
                    eventHandler.state(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL, body, ENABLE, STATE_DEAD);
                    eventHandler.delete(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL + 500, body);
                }
            }
            
            //Respawn
            eventHandler.state(currenttime, currenttime + 3000, this, Body.DISABLE, Body.STATE_LIFE | Body.STATE_DEAD);
            eventHandler.armor(currenttime,currenttime + 3000, this, this, BodyArmorEvent.SET, BodyPlayer.DEFAULT_ARMOR);
            eventHandler.state(currenttime, currenttime + 3000, this, Body.ENABLE, Body.STATE_MARKER);
            eventHandler.state(currenttime, currenttime + 6000, this, Body.DISABLE, Body.STATE_MARKER);
            eventHandler.state(currenttime, currenttime + 6000, this, Body.ENABLE, Body.STATE_LIFE);
            eventHandler.option(currenttime, currenttime + 3000, this, BodyPlayer.OPTION_FIRE, 0);
            eventHandler.option(currenttime, currenttime + 3000, this, BodyPlayer.OPTION_SPEED, 0);
            eventHandler.option(currenttime, currenttime + 3000, this, BodyPlayer.OPTION_CHARGE, 0);
            eventHandler.option(currenttime, currenttime + 3000, this, BodyPlayer.OPTION_WEAPON, 0);
            
            //Score
            if (lastAttacked.getType() == BStageCTF.TYPE_PLAYER1)
            {
                if (this.getOption(OPTION_TEAM) != lastAttacked.getOption(OPTION_TEAM))
                {
                    eventHandler.scoreChange(currenttime,currenttime, this, new Body[]{lastAttacked}, new int[]{1});
                }
                else
                {
                    eventHandler.scoreChange(currenttime,currenttime, this, new Body[]{lastAttacked}, new int[]{-1});
                }
            }
        }
    }
    @Override
    public void preAdd(BStage bstage)
    {
        int red = 0;
        int blue = 0;
        Body[] bs = bstage.getBodysByType(BStageCTF.TYPE_PLAYER1);
        for (int i=0;i<bs.length;i++)
        {
            if (bs[i].getOption(OPTION_TEAM) == 0)
            {
                red++;
            }
            else if (bs[i].getOption(OPTION_TEAM) == 1)
            {
                blue++;
            }
        }
        BStageCTF stage = (BStageCTF)bstage;
        if (red < blue)
        {
            setOption(OPTION_TEAM, 0);
            setPosition(stage.getRandomeSpawnPoint(0));
        }
        else if (red > blue)
        {
            setOption(OPTION_TEAM, 1);
            setPosition(stage.getRandomeSpawnPoint(1));
        }
        else
        {
            if (Math.random() < 0.5)
            {
                setOption(OPTION_TEAM, 0);
                setPosition(stage.getRandomeSpawnPoint(0));
            }
            else
            {
                setOption(OPTION_TEAM, 1);
                setPosition(stage.getRandomeSpawnPoint(1));
            }
        }
        //System.out.println(""+red+":"+blue);
    }
    @Override
    public void drawCharacter(Graphics g, long currenttime, CharacterImageBundle cib, EffectBundle eb)
    {
        super.drawCharacter(g, currenttime, cib, eb);
        if (getOption(OPTION_TEAM) == 0)
        {
            g.setColor(Color.RED);
        }
        else if (getOption(OPTION_TEAM) == 1)
        {
            g.setColor(Color.BLUE);
        }
        int rx = getCharacterImage().getWidth() / 2;
        int ry = getCharacterImage().getHeight() / 2;
        g.drawOval(-rx, -ry, rx*2, ry*2);

    }
}
