/*
 * 作成日: 2007/02/06
 */
package bakutan.game.capturetheflag;

import java.awt.Graphics;
import java.awt.Image;

import bakutan.common.BStage;
import bakutan.common.BUserInterface;
import bakutan.common.Body;
import bakutan.common.ClientBody;
import bakutan.common.DataManager;
import bakutan.common.EventHandler;
import bakutan.common.event.BodyActionEvent;
import bakutan.common.event.BodyEvent;
import bakutan.common.graphics.CharacterImage;
import bakutan.common.graphics.CharacterImageBundle;
import bakutan.common.graphics.EffectBundle;
import bakutan.common.math.BShape;
import bakutan.common.math.BVector2f;

public class BodyFlag extends Body
{
    public static final byte ACTION_FOLLOW = 0;
    public static final byte ACTION_RETURN = 1;
    public static final byte ACTION_PILLAGED = 2;
    public static final byte ACTION_CAPTURED = 3;
    
    public static final byte OPTION_TEAM = 0;
    public static final byte OPTION_STATE = 1;
    
    public static final int OPTION_STATE_BASE = 0;
    public static final int OPTION_STATE_CAPTURE = 1;
    public static final int OPTION_STATE_DROP = 2;

    public static final long ACTION_FOLLOW_INTERVAL = 200;
    
    private Body follow = null;
    private BVector2f defaultPosition = new BVector2f();
    
    public BodyFlag()
    {
        super(BStageCTF.TYPE_FLAG, 2);
        setShape( new BShape(BStageCTF.WALL_WIDTH));
    }

    public BVector2f getDefaultPosition()
    {
        return defaultPosition;
    }

    public void setDefaultPosition(BVector2f defaultPosition)
    {
        this.defaultPosition.set(defaultPosition);
    }

    @Override
    public boolean hasScore()
    {
        return true;
    }
    
    @Override
    public void actionForClient(BodyEvent bodyEvent, ClientBody mybody, CharacterImageBundle cib, EffectBundle eb, BUserInterface ui, long currenttime, BVector2f bufferCenter)
    {
        if (bodyEvent instanceof BodyActionEvent)
        {
            BodyActionEvent bae = (BodyActionEvent)bodyEvent;
            byte action = bae.getAction();
            if (action == ACTION_RETURN)
            {
                ui.getBAudio().playSE(BStageCTF.SOUND_FLAG_RESET, 0);
                ui.outputMessage(ui.translate("flag_is_returned", new String[]{getName()}));
            }
            else if (action == ACTION_PILLAGED)
            {
                ui.getBAudio().playSE(BStageCTF.SOUND_FLAG_PILLAGED, 0);
                ui.outputMessage(ui.translate("flag_is_pillaged", new String[]{getName()}));
            }
            else if (action == ACTION_CAPTURED)
            {
                ui.getBAudio().playSE(BStageCTF.SOUND_FLAG_CAPTURED, 0);
                ui.outputMessage(ui.translate("flag_is_captured", new String[]{getName()}));
            }
        }
    }
    @Override
    public void drawCharacter(Graphics g, long currenttime, CharacterImageBundle cib, EffectBundle eb)
    {
        CharacterImage characterImage = getCharacterImage();
        if (characterImage == null)
            return;
        
        int offsetx = characterImage.getWidth() / 2;
        int offsety = characterImage.getHeight() / 2;
        int x = -offsetx;
        int y = -offsety;
        int n = ((int)(currenttime - getLastStateChange()))/DataManager.ANIMINTERVAL;
        
        Image[] img = characterImage.getImages(getOption(OPTION_TEAM));
        g.drawImage(img[n%img.length], x, y, null);
    }
    @Override
    public boolean eventAction(EventHandler eventHandler, BStage bstage, BVector2f position, BodyActionEvent event)
    {
        long currenttime = eventHandler.getEventThread().getCurrentTime();
        if (event.getAction() == ACTION_FOLLOW)
        {
            if (getOption(OPTION_STATE) != OPTION_STATE_CAPTURE)
                return false;
            
            if (follow == null || follow.isState(STATE_DEAD) || follow.getOption(BodyCTFPlayer.OPTION_TEAM) == getOption(OPTION_TEAM))
            {
                follow = null;
                setOption(OPTION_STATE, OPTION_STATE_DROP);
                eventHandler.setPosition(currenttime, currenttime, this, getPosition());
                return true;
            }
            else
            {
                eventHandler.move(currenttime, currenttime,this, follow.getPosition(), follow.getVelocity(), follow.getDirection());
                eventHandler.smallAction(currenttime, currenttime+ACTION_FOLLOW_INTERVAL, this, ACTION_FOLLOW);
                return true;
            }
        }
        else if (event.getAction() == ACTION_RETURN)
        {
            eventHandler.setPosition(currenttime, currenttime, this, defaultPosition);
            eventHandler.option(currenttime, currenttime, this, OPTION_STATE, OPTION_STATE_BASE);
            return true;
        }
        else if (event.getAction() == ACTION_PILLAGED)
        {
            if (getOption(OPTION_STATE) == OPTION_STATE_CAPTURE || follow == null)
                return false;
            eventHandler.option(currenttime, currenttime, this, OPTION_STATE, OPTION_STATE_CAPTURE);
            eventHandler.smallAction(currenttime, currenttime, this, ACTION_FOLLOW);
            return true;
        }
        else if (event.getAction() == ACTION_CAPTURED)
        {
            eventHandler.setPosition(currenttime, currenttime, this, defaultPosition);
            eventHandler.option(currenttime, currenttime, this, OPTION_STATE, OPTION_STATE_BASE);
            return true;
        }
        return false;
    }
    @Override
    public void eventCollisition(EventHandler eventHandler, BStage bstage, Body target)
    {
        long currenttime = eventHandler.getEventThread().getCurrentTime();
        if (target.getType() == BStageCTF.TYPE_PLAYER1)
        {
            if (getOption(OPTION_TEAM) == target.getOption(BodyCTFPlayer.OPTION_TEAM))
            {
                if (getOption(OPTION_STATE) == OPTION_STATE_DROP)
                {
                    eventHandler.smallAction(currenttime, currenttime, this, ACTION_RETURN);
                }
            }
            else
            {
                if (getOption(OPTION_STATE) == OPTION_STATE_BASE || getOption(OPTION_STATE) == OPTION_STATE_DROP)
                {
                    if (getOption(OPTION_TEAM) != target.getOption(BodyCTFPlayer.OPTION_TEAM))
                    {
                        follow = target;
                        eventHandler.smallAction(currenttime, currenttime, this, ACTION_PILLAGED);
                    }
                }
            }
        }
        if (target.getType() == BStageCTF.TYPE_FLAG)
        {
            if (target.getOption(OPTION_STATE) == OPTION_STATE_BASE)
            {
                eventHandler.scoreChange(currenttime, currenttime, this, new Body[]{target}, new int[]{1});
                eventHandler.smallAction(currenttime, currenttime, this, ACTION_CAPTURED);
            }
        }
        super.eventCollisition(eventHandler, bstage, target);
    }
    @Override
    public byte[] getCheckCollidType()
    {
        return new byte[]{BStageCTF.TYPE_PLAYER1, BStageCTF.TYPE_FLAG};
    }

    @Override
    public byte[] getCheckIntersectType()
    {
        return new byte[]{BStageCTF.TYPE_PLAYER1, BStageCTF.TYPE_FLAG};
    }

}
