/*
 * 作成日: 2006/12/25
 */
package bakutan.game.deathmatch;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.TreeSet;

import bakutan.common.Body;
import bakutan.common.ai.AIInterface;
import bakutan.common.ai.AIOutput;
import bakutan.common.ai.BStageInfoForAI;
import bakutan.common.ai.BodyInfoForAI;
import bakutan.common.ai.BodyInfoForAIComparator;
import bakutan.common.ai.InvalidAIData;
import bakutan.common.math.BMath;
import bakutan.common.math.BVector2f;

public class AIAlgorismicVer1 extends AIInterface
{
    //並べ替え用のTreeSet群
    private TreeSet<BodyInfoForAI> wallSorter;
    private TreeSet<BodyInfoForAI> softwallSorter;
    private TreeSet<BodyInfoForAI> itemSorter;
    private TreeSet<BodyInfoForAI> playerSorter;
    private TreeSet<BodyInfoForAI> attackSorter;
    private TreeSet<BodyInfoForAI> fireSorter;
    private BodyInfoForAIComparator comparator;

    private AIOutput aout;
    private BVector2f randomWalkVec = new BVector2f();
    private long randomWalkTimer = 0;
    
    public AIAlgorismicVer1()
    {
        comparator = new BodyInfoForAIComparator();
        wallSorter = new TreeSet<BodyInfoForAI>(comparator);
        softwallSorter = new TreeSet<BodyInfoForAI>(comparator);
        itemSorter = new TreeSet<BodyInfoForAI>(comparator);
        playerSorter = new TreeSet<BodyInfoForAI>(comparator);
        attackSorter = new TreeSet<BodyInfoForAI>(comparator);
        fireSorter = new TreeSet<BodyInfoForAI>(comparator);
    }

    public String getAIName()
    {
        return "AIAlgorismicVer1";
    }

    public AIOutput getAIOutput()
    {
        return aout;
    }

    public void input(BodyInfoForAI mybody, BStageInfoForAI obstage, long currenttime, int actionlock, float energy)
    {
        aout = new AIOutput();

        BodyInfoForAI[] bodys = obstage.getBodyInfoForAIs();
        //並べ替え用の集合の初期化
        wallSorter.clear();
        softwallSorter.clear();
        itemSorter.clear();
        playerSorter.clear();
        attackSorter.clear();
        fireSorter.clear();
        comparator.setTarget(mybody.getPosition());
        
        boolean move = false;
        
        //bodysの要素を種別ごとに分ける
        for (int i=0;i<bodys.length;i++)
        {
            if (bodys[i].getType() == BStageDM.TYPE_WALL1)
                wallSorter.add(bodys[i]);
            else if (bodys[i].getType() == BStageDM.TYPE_SOFTWALL)
                softwallSorter.add(bodys[i]);
            else if (bodys[i].getType() == BStageDM.TYPE_ITEM)
                itemSorter.add(bodys[i]);
            else if (bodys[i].getType() == BStageDM.TYPE_PLAYER1)
            {
                if (bodys[i].getID() != mybody.getID())
                    playerSorter.add(bodys[i]);
            }
            else if (bodys[i].getType() == BStageDM.TYPE_MISSILE1)
            {
                attackSorter.add(bodys[i]);
            }
            else if (bodys[i].getType() == BStageDM.TYPE_SLUG1)
            {
                attackSorter.add(bodys[i]);
                fireSorter.add(bodys[i]);
            }
            else if (bodys[i].getType() == BStageDM.TYPE_HOMING1)
            {
                attackSorter.add(bodys[i]);
            }
            else if (bodys[i].getType() == BStageDM.TYPE_BOMB1)
            {
                attackSorter.add(bodys[i]);
            }
            else if (bodys[i].getType() == BStageDM.TYPE_FIRE1)
            {
                attackSorter.add(bodys[i]);
                fireSorter.add(bodys[i]);
            }
        }
        BVector2f pos = new BVector2f(mybody.getPosition());
        BVector2f velocity = new BVector2f(mybody.getVelocity());
        velocity.set(0,0);
        Iterator<BodyInfoForAI> ite;
        
        //近くにあるアイテムを探し取りに行く動作をする。
        {
            ite = itemSorter.iterator();
            while(ite.hasNext())
            {
                BodyInfoForAI body = ite.next();
                if (body.getOption(BodyItem.OPTION_TYPE) == BodyItem.ITEM_FIRE)
                {
                    if (mybody.getOption(BodyPlayer.OPTION_FIRE) >= BodyPlayer.MAX_FIRE)
                        continue;
                }
                else if (body.getOption(BodyItem.OPTION_TYPE) == BodyItem.ITEM_CHARGE)
                {
                    if (mybody.getOption(BodyPlayer.OPTION_CHARGE) >= BodyPlayer.MAX_CHARGE)
                        continue;
                }
                else if (body.getOption(BodyItem.OPTION_TYPE) == BodyItem.ITEM_SPEED)
                {
                    if (mybody.getOption(BodyPlayer.OPTION_SPEED) >= BodyPlayer.MAX_SPEED)
                        continue;
                }
                else if (body.getOption(BodyItem.OPTION_TYPE) == BodyItem.ITEM_ARMOR)
                {
                    if (mybody.getArmor() >= BodyPlayer.DEFAULT_ARMOR)
                        continue;
                }
                else
                {
                    continue;
                }
                
                if (BVector2f.getDistance(mybody.getPosition(), body.getPosition()) > 20f)
                {
                    continue;
                }
                
                if (getClosistIntersect(pos, body.getPosition(), wallSorter) != null
                    || getClosistIntersect(pos, body.getPosition(), softwallSorter) != null)
                {
                    continue;
                }
                    
                BVector2f vec = new BVector2f();
                vec.set(body.getPosition());
                vec.sub(pos);
                if (!vec.isZeroVector())
                {
                    move = true;
                    vec.normalize();
                    velocity.set(vec);
                    break;
                }
            }
        }

        
        //アイテムを取りに行かないときにランダムウォークを行う
        if (!move)
        {
            if (randomWalkTimer < currenttime)
            {
                randomWalkTimer = currenttime + (long)(Math.random() * 2000d);
                randomWalkVec.x = (float)(Math.random() * 2d - 1d);
                randomWalkVec.y = (float)(Math.random() * 2d - 1d);
                move = true;
            }
            velocity.set(randomWalkVec);
        }
        
        //当たりそうな攻撃を探して、あれば回避行動をとる。
        {
            ite = attackSorter.iterator();
            while(ite.hasNext())
            {
                BodyInfoForAI body = ite.next();
                float time = BMath.nextCollid(pos, mybody.getShape(), velocity, body.getPosition(), body.getShape(), body.getVelocity());
                if (Float.isNaN(time) || time < 1f)
                    continue;
                BVector2f vec = new BVector2f();
                vec.set(pos);
                vec.sub(body.getPosition());
                if (body.getVelocity().isZeroVector())
                {
                    if (!vec.isZeroVector())
                    {
                        vec.normalize();
                        velocity.set(vec);
                        move = true;
                        break;
                    }
                    else
                    {
                        continue;
                    }
                }
                
                float f = BVector2f.getCross(body.getVelocity(),vec);
                
                if (f > 0)
                {
                    vec.set(body.getVelocity());
                    vec.rotate(Math.PI / 2d);
                    vec.normalize();
                    velocity.set(vec);
                    move = true;
                }
                else if (f < 0)
                {
                    vec.set(body.getVelocity());
                    vec.rotate(-Math.PI / 2d);
                    vec.normalize();
                    velocity.set(vec);
                    move = true;
                }
                else
                {
                    if (!vec.isZeroVector())
                    {
                        vec.normalize();
                        velocity.set(vec);
                        move = true;
                        break;
                    }
                }
            }
        }
        
        //移動を設定する。
        aout.setVelocity(velocity);

        //アクションを初期化
        aout.setAction(-1);

        //アクションが可能ならば攻撃を行う。
        //最も近くて視線が通った敵に攻撃を行う。
        if (actionlock <= 0 && energy > 0)
        {
            ite = playerSorter.iterator();
            while(ite.hasNext())
            {
                BodyInfoForAI body = ite.next();
                if (body.isState(Body.STATE_DEAD))
                {
                    continue;
                }
                //視線のチェック、視線が通っていなければ次の対象に移る。
                if (getClosistIntersect(pos, body.getPosition(), wallSorter) != null
                        || getClosistIntersect(pos, body.getPosition(), softwallSorter) != null)
                {
                    continue;
                }
                //爆風のチェック。一直線上の近くに爆風があれば攻撃せず、次の対象に移る。
                BodyInfoForAI closistFire = getClosistIntersect(pos, body.getPosition(), fireSorter);
                if (closistFire != null && BVector2f.getDistance(pos, closistFire.getPosition()) < 100)
                {
                    continue;
                }
                
                //射撃の照準を設定する
                aout.setActiontarget(body.getPosition());
                
                //距離により攻撃する武器を選ぶ
                float distance = BVector2f.getDistance(pos, body.getPosition());
                if (distance < 10f)
                {
                    //ミサイルで攻撃
                    aout.setAction(BodyPlayer.ACTION_MISSILE);
                }
                else if (distance < 12f)
                {
                    //爆弾で攻撃
                    aout.setAction(BodyPlayer.ACTION_BOMB);
                }
                else
                {
                    if (Math.random() < 0.5f)
                    {
                        //スラグで攻撃
                        aout.setAction(BodyPlayer.ACTION_SLUG);
                    }
                    else
                    {
                        //ホーミングで攻撃
                        aout.setAction(BodyPlayer.ACTION_HOMING);
                   }
                }
            }
        }
    }
    /**
     * 線分start-endに与えられた集合bodysの要素で交差する最も近い要素を返す。
     * @param start 線分の始点
     * @param end 線分の終点
     * @param bodys 対象となる要素の集合
     * @return もっともstartに近い線分start-endに交差する要素
     */
    private BodyInfoForAI getClosistIntersect(BVector2f start, BVector2f end, TreeSet<BodyInfoForAI> bodys)
    {
        BodyInfoForAI lastBody = null;
        float lastDistance = Float.MAX_VALUE;
        Iterator<BodyInfoForAI> ite = bodys.iterator();
        while(ite.hasNext())
        {
            BodyInfoForAI body = ite.next();
            float distance = BMath.getClosestIntersection(start, end, body.getPosition(), body.getShape());
            if (!Float.isNaN(distance))
            {
                if (distance < lastDistance)
                {
                    lastDistance = distance;
                    lastBody = body;
                }
            }
        }
        return lastBody;
    }

    public void inputDesired(long currenttime, AIOutput out)
    {
        return;
    }

    public boolean isRecordAI()
    {
        return false;
    }

    public void loadAI(File file) throws IOException, InvalidAIData
    {
        return;
    }

    public void setRecordAI(boolean recordAI)
    {
    }

    public boolean isFileLoadable()
    {
        return false;
    }
    
}
