/*
 * 作成日: 2006/11/24
 */
package bakutan.game.deathmatch;

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.zip.GZIPOutputStream;

import bakutan.common.ai.AIInterface;
import bakutan.common.ai.AIOutput;
import bakutan.common.ai.BStageInfoForAI;
import bakutan.common.ai.BodyInfoForAI;
import bakutan.common.ai.BodyInfoForAIComparator;
import bakutan.common.ai.InvalidAIData;
import neuralnetwork.NeuralNetwork;
/*
 * -In 11
 * OnOff
 * type(man,missile,bom,softwall,wall)
 * position
 * velocity
 * armor
 * 
 * -Out 8
 * velocity
 * actiontarget
 * action1
 * action2
 * action3
 * action4
 */

public class AINeuralnetworkV3 extends AIInterface
{
    public static final int TIMER_NUM = 4; 
    public static final int MYBODY_PN = 7;
    public static final int SITE_WALL_NUM = 20;
    public static final int SITE_WALL_PN = 3;
    public static final int SITE_SOFTWALL_NUM = 10;
    public static final int SITE_SOFTWALL_PN = 4;
    public static final int SITE_ITEM_NUM = 3;
    public static final int SITE_ITEM_PN = 3;
    public static final int SITE_BODY_NUM = 5;
    public static final int SITE_BODY_PN = 9;
    public static final int SITE_MISSLE_NUM = 10;
    public static final int SITE_MISSLE_PN = 6;
    public static final int SITE_BOM_NUM = 10;
    public static final int SITE_BOM_PN = 6;
    public static final int SITE_SLUG_NUM = 3;
    public static final int SITE_SLUG_PN = 6;
    public static final int SITE_HOMING_NUM = 3;
    public static final int SITE_HOMING_PN = 6;
    public static final int SITE_FIRE_NUM = 10;
    public static final int SITE_FIRE_PN = 6;
    //public static final int INTPUT_NEURON_NUM = NEURON_PAR_BODY * SITE_NUM + 3;
    public static final int INTPUT_NEURON_NUM = TIMER_NUM + MYBODY_PN
    + SITE_WALL_NUM * SITE_WALL_PN
    + SITE_SOFTWALL_NUM * SITE_SOFTWALL_PN
    + SITE_ITEM_NUM * SITE_ITEM_PN
    + SITE_BODY_NUM * SITE_BODY_PN
    + SITE_MISSLE_NUM * SITE_MISSLE_PN
    + SITE_BOM_NUM * SITE_BOM_PN
    + SITE_SLUG_NUM * SITE_SLUG_PN
    + SITE_HOMING_NUM * SITE_HOMING_PN
    + SITE_FIRE_NUM * SITE_FIRE_PN;
    public static final int OUTPUT_NEURON_NUM = 8;
    
    private NeuralNetwork network;
    private TreeSet<BodyInfoForAI> wallSorter;
    private TreeSet<BodyInfoForAI> softwallSorter;
    private TreeSet<BodyInfoForAI> itemSorter;
    private TreeSet<BodyInfoForAI> manSorter;
    private TreeSet<BodyInfoForAI> missileSorter;
    private TreeSet<BodyInfoForAI> bomSorter;
    private TreeSet<BodyInfoForAI> slugSorter;
    private TreeSet<BodyInfoForAI> homingSorter;
    private TreeSet<BodyInfoForAI> fireSorter;
    private BodyInfoForAIComparator comparator;
    private double[] inputValues;
    private double[] desiredValues;
    //private File workDir;
    private DataOutputStream recordStream;
    private boolean recordAI = false;
    private long lastRecordTime = 0;
    private float myPosX = 0f;
    private float myPosY = 0f;
    private AIOutput out;

    public AINeuralnetworkV3()
    {
        super();
        inputValues = new double[INTPUT_NEURON_NUM];
        desiredValues = new double[OUTPUT_NEURON_NUM];
        comparator = new BodyInfoForAIComparator();
        wallSorter = new TreeSet<BodyInfoForAI>(comparator);
        softwallSorter = new TreeSet<BodyInfoForAI>(comparator);
        itemSorter = new TreeSet<BodyInfoForAI>(comparator);
        manSorter = new TreeSet<BodyInfoForAI>(comparator);
        missileSorter = new TreeSet<BodyInfoForAI>(comparator);
        bomSorter = new TreeSet<BodyInfoForAI>(comparator);
        slugSorter = new TreeSet<BodyInfoForAI>(comparator);
        homingSorter = new TreeSet<BodyInfoForAI>(comparator);
        fireSorter = new TreeSet<BodyInfoForAI>(comparator);
        NeuralNetwork nn = new NeuralNetwork();
        nn.initialize(new int[]{INTPUT_NEURON_NUM, 10, 10, OUTPUT_NEURON_NUM});
        nn.setLearningRate(0.01);
        nn.setLinearOutput(false);
        try
        {
            setNetwork(nn);
        }
        catch(InvalidAIData e)
        {
            throw new RuntimeException(e);
        }
    }
    
    public String getAIName()
    {
        return "AINeuralnetworkVer3";
    }
//    
//    public NeuralNetwork getNetwork()
//    {
//        return network;
//    }

    private void setNetwork(NeuralNetwork network) throws InvalidAIData
    {
        if (network.getInputLayerNodeNum() != inputValues.length
            || network.getOutputLayerNodeNum() != desiredValues.length)
            throw new InvalidAIData();
        this.network = network;
    }

//    private void input(double[] input)
//    {
//        if (input.length != INTPUT_NEURON_NUM)
//            throw new ArrayIndexOutOfBoundsException();
//        for (int i=0;i<input.length;i++)
//        {
//            network.setInput(i,input[i]);
//        }
//        network.feedForward();
//    }
    public void input(BodyInfoForAI mybody, BStageInfoForAI bstage, long currenttime, int actionlock, float energy)
    {
        out = new AIOutput();
        //BStageThrowBom bstage = (BStageThrowBom)obstage;
        
        BodyInfoForAI[] bs = bstage.getBodyInfoForAIs();
        wallSorter.clear();
        softwallSorter.clear();
        itemSorter.clear();
        manSorter.clear();
        missileSorter.clear();
        bomSorter.clear();
        slugSorter.clear();
        homingSorter.clear();
        fireSorter.clear();
        
        comparator.setTarget(mybody.getPosition());
        for (int i=0;i<bs.length;i++)
        {
            if (bs[i].getType() == BStageDM.TYPE_WALL1)
                wallSorter.add(bs[i]);
            else if (bs[i].getType() == BStageDM.TYPE_SOFTWALL)
                softwallSorter.add(bs[i]);
            else if (bs[i].getType() == BStageDM.TYPE_ITEM)
                itemSorter.add(bs[i]);
            else if (bs[i].getType() == BStageDM.TYPE_PLAYER1)
            {
                if (bs[i].getID() != mybody.getID())
                    manSorter.add(bs[i]);
            }
            else if (bs[i].getType() == BStageDM.TYPE_MISSILE1)
                missileSorter.add(bs[i]);
            else if (bs[i].getType() == BStageDM.TYPE_SLUG1)
                slugSorter.add(bs[i]);
            else if (bs[i].getType() == BStageDM.TYPE_HOMING1)
                homingSorter.add(bs[i]);
            else if (bs[i].getType() == BStageDM.TYPE_BOMB1)
                bomSorter.add(bs[i]);
            else if (bs[i].getType() == BStageDM.TYPE_FIRE1)
                fireSorter.add(bs[i]);
        }

        inputValues[0] = (double)(actionlock) / 1000d;
        if (inputValues[0] < 0)
            inputValues[0] = 0;
        inputValues[1] = (double)(currenttime %   1000l) /   1000d;
        inputValues[2] = (double)(currenttime %  10000l) /  10000d;
        inputValues[3] = (double)(currenttime % 100000l) / 100000d;
        int offset = TIMER_NUM;

        //mybody
        myPosX = mybody.getPosition().x;
        myPosY = mybody.getPosition().y;
        BodyInfoForAI body = mybody;
        inputValues[offset+0] = 1d;
//        inputValues[offset+1] = body.getPosition().x;
//        inputValues[offset+2] = body.getPosition().y;
        inputValues[offset+1] = body.getVelocity().x;
        inputValues[offset+2] = body.getVelocity().y;
        inputValues[offset+3] = (double)body.getArmor();
        inputValues[offset+4] = body.getOption(BodyPlayer.OPTION_FIRE);
        inputValues[offset+5] = body.getOption(BodyPlayer.OPTION_CHARGE);
        inputValues[offset+6] = body.getOption(BodyPlayer.OPTION_SPEED);
        offset += MYBODY_PN;
        
        float mx = mybody.getPosition().x;
        float my = mybody.getPosition().y;
        float mvx = mybody.getVelocity().x;
        float mvy = mybody.getVelocity().y;
        
        //wall
        Iterator<BodyInfoForAI> ite;
        ite = wallSorter.iterator();
        for (int i=0;i<SITE_WALL_NUM;i++)
        {
            int s = offset + i*SITE_WALL_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
//                inputValues[s+1] = body.getPosition().x;
//                inputValues[s+2] = body.getPosition().y;
                inputValues[s+1] = body.getPosition().x - mx;
                inputValues[s+2] = body.getPosition().y - my;
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = 0d;
                inputValues[s+2] = 0d;
//                inputValues[s+1] = mybody.getPosition().x;
//                inputValues[s+2] = mybody.getPosition().y;
            }
        }
        offset += SITE_WALL_NUM * SITE_WALL_PN;
        
        //softwall
        ite = softwallSorter.iterator();
        for (int i=0;i<SITE_SOFTWALL_NUM;i++)
        {
            int s = offset + i*SITE_SOFTWALL_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
//              inputValues[s+1] = body.getPosition().x;
//              inputValues[s+2] = body.getPosition().y;
                inputValues[s+1] = body.getPosition().x - mx;
                inputValues[s+2] = body.getPosition().y - my;
                inputValues[s+3] = (double)body.getArmor();
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = 0d;
                inputValues[s+2] = 0d;
//                inputValues[s+1] = mybody.getPosition().x;
//                inputValues[s+2] = mybody.getPosition().y;
                inputValues[s+3] = 0d;
            }
        }
        offset += SITE_SOFTWALL_NUM * SITE_SOFTWALL_PN;

        //item
        ite = itemSorter.iterator();
        for (int i=0;i<SITE_ITEM_NUM;i++)
        {
            int s = offset + i*SITE_ITEM_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
//              inputValues[s+1] = body.getPosition().x;
//              inputValues[s+2] = body.getPosition().y;
                inputValues[s+1] = body.getPosition().x - mx;
                inputValues[s+2] = body.getPosition().y - my;
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = 0d;
                inputValues[s+2] = 0d;
//                inputValues[s+1] = mybody.getPosition().x;
//                inputValues[s+2] = mybody.getPosition().y;
            }
        }
        offset += SITE_ITEM_NUM * SITE_ITEM_PN;
        
        //man
        ite = manSorter.iterator();
        for (int i=0;i<SITE_BODY_NUM;i++)
        {
            int s = offset + i*SITE_BODY_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
//              inputValues[s+1] = body.getPosition().x;
//              inputValues[s+2] = body.getPosition().y;
                inputValues[s+1] = body.getPosition().x - mx;
                inputValues[s+2] = body.getPosition().y - my;
//                inputValues[s+3] = body.getVelocity().x;
//                inputValues[s+4] = body.getVelocity().y;
                inputValues[s+3] = body.getVelocity().x - mvx;
                inputValues[s+4] = body.getVelocity().y - mvy;
                inputValues[s+5] = (double)body.getArmor();
                inputValues[s+6] = body.getOption(BodyPlayer.OPTION_FIRE);
                inputValues[s+7] = body.getOption(BodyPlayer.OPTION_CHARGE);
                inputValues[s+8] = body.getOption(BodyPlayer.OPTION_SPEED);
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = 0d;
                inputValues[s+2] = 0d;
//                inputValues[s+1] = mybody.getPosition().x;
//                inputValues[s+2] = mybody.getPosition().y;
                inputValues[s+3] = 0;
                inputValues[s+4] = 0;
//                inputValues[s+3] = mybody.getVelocity().x;
//                inputValues[s+4] = mybody.getVelocity().y;
                inputValues[s+5] = 0d;
                inputValues[s+6] = 0d;
                inputValues[s+7] = 0d;
                inputValues[s+8] = 0d;
            }
        }
        offset += SITE_BODY_NUM * SITE_BODY_PN;

        //missile
        ite = missileSorter.iterator();
        for (int i=0;i<SITE_MISSLE_NUM;i++)
        {
            int s = offset + i*SITE_MISSLE_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
//              inputValues[s+1] = body.getPosition().x;
//              inputValues[s+2] = body.getPosition().y;
                inputValues[s+1] = body.getPosition().x - mx;
                inputValues[s+2] = body.getPosition().y - my;
//                inputValues[s+3] = body.getVelocity().x;
//                inputValues[s+4] = body.getVelocity().y;
                inputValues[s+3] = body.getVelocity().x - mvx;
                inputValues[s+4] = body.getVelocity().y - mvy;
                inputValues[s+5] = (double)body.getArmor();
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = 0d;
                inputValues[s+2] = 0d;
//                inputValues[s+1] = mybody.getPosition().x;
//                inputValues[s+2] = mybody.getPosition().y;
                inputValues[s+3] = 0;
                inputValues[s+4] = 0;
//                inputValues[s+3] = mybody.getVelocity().x;
//                inputValues[s+4] = mybody.getVelocity().y;
                inputValues[s+5] = 0d;
            }
        }
        offset += SITE_MISSLE_NUM * SITE_MISSLE_PN;

        //bom
        ite = bomSorter.iterator();
        for (int i=0;i<SITE_BOM_NUM;i++)
        {
            int s = offset + i*SITE_BOM_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
//              inputValues[s+1] = body.getPosition().x;
//              inputValues[s+2] = body.getPosition().y;
                inputValues[s+1] = body.getPosition().x - mx;
                inputValues[s+2] = body.getPosition().y - my;
//                inputValues[s+3] = body.getVelocity().x;
//                inputValues[s+4] = body.getVelocity().y;
                inputValues[s+3] = body.getVelocity().x - mvx;
                inputValues[s+4] = body.getVelocity().y - mvy;
                inputValues[s+5] = (double)body.getArmor();
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = 0d;
                inputValues[s+2] = 0d;
//                inputValues[s+1] = mybody.getPosition().x;
//                inputValues[s+2] = mybody.getPosition().y;
                inputValues[s+3] = 0;
                inputValues[s+4] = 0;
//                inputValues[s+3] = mybody.getVelocity().x;
//                inputValues[s+4] = mybody.getVelocity().y;
                inputValues[s+5] = 0d;
            }
        }
        offset += SITE_BOM_NUM * SITE_BOM_PN;

        //slug
        ite = slugSorter.iterator();
        for (int i=0;i<SITE_SLUG_NUM;i++)
        {
            int s = offset + i*SITE_SLUG_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
//              inputValues[s+1] = body.getPosition().x;
//              inputValues[s+2] = body.getPosition().y;
                inputValues[s+1] = body.getPosition().x - mx;
                inputValues[s+2] = body.getPosition().y - my;
//                inputValues[s+3] = body.getVelocity().x;
//                inputValues[s+4] = body.getVelocity().y;
                inputValues[s+3] = body.getVelocity().x - mvx;
                inputValues[s+4] = body.getVelocity().y - mvy;
                inputValues[s+5] = (double)body.getArmor();
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = 0d;
                inputValues[s+2] = 0d;
//                inputValues[s+1] = mybody.getPosition().x;
//                inputValues[s+2] = mybody.getPosition().y;
                inputValues[s+3] = 0;
                inputValues[s+4] = 0;
//                inputValues[s+3] = mybody.getVelocity().x;
//                inputValues[s+4] = mybody.getVelocity().y;
                inputValues[s+5] = 0d;
            }
        }
        offset += SITE_SLUG_NUM * SITE_SLUG_PN;

        //horming
        ite = slugSorter.iterator();
        for (int i=0;i<SITE_HOMING_NUM;i++)
        {
            int s = offset + i*SITE_HOMING_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
//              inputValues[s+1] = body.getPosition().x;
//              inputValues[s+2] = body.getPosition().y;
                inputValues[s+1] = body.getPosition().x - mx;
                inputValues[s+2] = body.getPosition().y - my;
//                inputValues[s+3] = body.getVelocity().x;
//                inputValues[s+4] = body.getVelocity().y;
                inputValues[s+3] = body.getVelocity().x - mvx;
                inputValues[s+4] = body.getVelocity().y - mvy;
                inputValues[s+5] = (double)body.getArmor();
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = 0d;
                inputValues[s+2] = 0d;
//                inputValues[s+1] = mybody.getPosition().x;
//                inputValues[s+2] = mybody.getPosition().y;
                inputValues[s+3] = 0;
                inputValues[s+4] = 0;
//                inputValues[s+3] = mybody.getVelocity().x;
//                inputValues[s+4] = mybody.getVelocity().y;
                inputValues[s+5] = 0d;
            }
        }
        offset += SITE_HOMING_NUM * SITE_HOMING_PN;

        //fire
        ite = fireSorter.iterator();
        for (int i=0;i<SITE_FIRE_NUM;i++)
        {
            int s = offset + i*SITE_FIRE_PN;
            body = (ite.hasNext()) ? ite.next() : null;
            
            if (body != null)
            {
                inputValues[s+0] = 1d;
//              inputValues[s+1] = body.getPosition().x;
//              inputValues[s+2] = body.getPosition().y;
                inputValues[s+1] = body.getPosition().x - mx;
                inputValues[s+2] = body.getPosition().y - my;
//                inputValues[s+3] = body.getVelocity().x;
//                inputValues[s+4] = body.getVelocity().y;
                inputValues[s+3] = body.getVelocity().x - mvx;
                inputValues[s+4] = body.getVelocity().y - mvy;
                inputValues[s+5] = body.getShape().getRadius();
            }
            else
            {
                inputValues[s+0] = 0d;
                inputValues[s+1] = 0d;
                inputValues[s+2] = 0d;
//                inputValues[s+1] = mybody.getPosition().x;
//                inputValues[s+2] = mybody.getPosition().y;
                inputValues[s+3] = 0d;
                inputValues[s+4] = 0d;
//                inputValues[s+3] = mybody.getVelocity().x;
//                inputValues[s+4] = mybody.getVelocity().y;
                inputValues[s+5] = 0d;
            }
        }
        offset += SITE_FIRE_NUM * SITE_FIRE_PN;
        if (offset != INTPUT_NEURON_NUM)
            throw new RuntimeException("error");
//        input(inputValues);
        network.setInput(inputValues);
    }

    public AIOutput getAIOutput()
    {
        network.feedForward();
        out.getVelocity().x = (float)(logit(network.getOutput(0)));
        out.getVelocity().y = (float)(logit(network.getOutput(1)));
//        out.getActiontarget().x = (float)(logit(network.getOutput(2)));
//        out.getActiontarget().y = (float)(logit(network.getOutput(3)));
        out.getActiontarget().x = (float)(logit(network.getOutput(2)) + myPosX);
        out.getActiontarget().y = (float)(logit(network.getOutput(3)) + myPosY);

        if (out.getVelocity().getLength() < 0.5)
            out.getVelocity().set(0f,0f);
        else
        {
            if (out.getVelocity().getLength() > 1f)
                out.getVelocity().normalize();
        }
        double value = 0;
        int action = -1;
        for (int i=0;i<4;i++)
        {
            if (value < network.getOutput(i+4))
            {
                value = network.getOutput(i+4);
                action = i;
            }
        }
        if (value > 0.5d)
        {
            out.setAction(action);
        }
        else
        {
            out.setAction(-1);
        }
        return out;
    }
    public void inputDesired(long currenttime, AIOutput out)
    {
        desiredValues[0] = logistic(out.getVelocity().x);
        desiredValues[1] = logistic(out.getVelocity().y);
//        desiredValues[2] = logistic(out.getActiontarget().x);
//        desiredValues[3] = logistic(out.getActiontarget().y);
        desiredValues[2] = logistic(out.getActiontarget().x - myPosX);
        desiredValues[3] = logistic(out.getActiontarget().y - myPosY);
        boolean[] actions = new boolean[4];
        int action = out.getAction();
        desiredValues[4] = (action == 0) ? 0.7d:0.3d;
        desiredValues[5] = (action == 1) ? 0.7d:0.3d;
        desiredValues[6] = (action == 2) ? 0.7d:0.3d;
        desiredValues[7] = (action == 3) ? 0.7d:0.3d;
        
        //inputDesired(desiredValues);
        network.setDesiredOutput(desiredValues);
        writeRecord(currenttime);
    }
//    private void inputDesired(double[] input)
//    {
//        if (input.length != OUTPUT_NEURON_NUM)
//            throw new ArrayIndexOutOfBoundsException();
//        
//        for (int i=0;i<input.length;i++)
//        {
//            network.setDesiredOutput(i,input[i]);
//        }
//    }
//    public void backPropagate()
//    {
//        network.backPropagate();
//    }
//    public double[] getDesiredValues()
//    {
//        return desiredValues;
//    }
    private static double logistic(double x)
    {
        double d = x/100;
        return 1.0/(1+Math.exp(-d));
    }
    private static double logit(double x)
    {
        return Math.log(x/(1-x)) * 100;
    }

//    public File getWorkDir()
//    {
//        return workDir;
//    }
//
//    public void setWorkDir(File recordFile)
//    {
//        this.workDir = recordFile;
//    }

    public boolean isRecordAI()
    {
        return recordAI;
    }

    public void setRecordAI(boolean recordAI)
    {
        this.recordAI = recordAI;
        if (recordAI)
        {
            if (getWorkDir() == null)
            {
                recordAI = false;
                return;
            }
            if (recordStream != null)
                return;
            SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
            File recordFile = new File( getWorkDir() + "/ver3_" + format.format(new Date()) + ".rec");
            
            if (!recordFile.exists())
            {
                try
                {
                    if (!recordFile.createNewFile())
                    {
                        recordAI = false;
                        return;
                    }
                }
                catch(IOException e)
                {
                    e.printStackTrace();
                    System.out.println(recordFile);
                    recordAI = false;
                    return;
                }
            }
            
            try
            {
                recordStream = new DataOutputStream(new GZIPOutputStream(new FileOutputStream(recordFile),1024*1024));
                recordStream.writeInt(INTPUT_NEURON_NUM);
                recordStream.writeInt(OUTPUT_NEURON_NUM);
            }
            catch(IOException e)
            {
                recordAI = false;
                return;
            }
        }
        else
        {
            if (recordStream != null)
            {
                try
                {
                    recordStream.close();
                }
                catch(IOException e)
                {
                    e.printStackTrace();
                }
                recordStream = null;
            }
        }
    }
    private void writeRecord(long currenttime)
    {
        if (!recordAI || recordStream == null)
            return;
        if (currenttime - lastRecordTime < 100)
        {
            return;
        }
        lastRecordTime = currenttime;

        try
        {
            double[] ds;
            ds = inputValues;
            recordStream.writeInt(ds.length);
            for (int i=0;i<ds.length;i++)
            {
                recordStream.writeDouble(ds[i]);
            }
            ds = desiredValues;
            recordStream.writeInt(ds.length);
            for (int i=0;i<ds.length;i++)
            {
                recordStream.writeDouble(ds[i]);
            }
        }
        catch(IOException e)
        {
            e.printStackTrace();
            setRecordAI(false);
        }

        
    }
//    public int getInputNeuronNum()
//    {
//        return INTPUT_NEURON_NUM;
//    }
//    public int getOutputNeuronNum()
//    {
//        return OUTPUT_NEURON_NUM;
//    }

    public void loadAI(File file) throws IOException, InvalidAIData
    {
        FileInputStream in = new FileInputStream(file);
        NeuralNetwork nn = NeuralNetwork.loadFromStream(in);
        in.close();
        setNetwork(nn);
    }

    public boolean isFileLoadable()
    {
        return true;
    }
//    }
}

