/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/07
 */
package bakutan.game.deathmatch;

import java.awt.Graphics;
import java.awt.Image;
import java.util.HashSet;

import bakutan.common.BStage;
import bakutan.common.BUserInterface;
import bakutan.common.Body;
import bakutan.common.ClientBody;
import bakutan.common.DataManager;
import bakutan.common.EventHandler;
import bakutan.common.event.BodyAddEvent;
import bakutan.common.event.BodyEvent;
import bakutan.common.event.BodyArmorEvent;
import bakutan.common.graphics.CharacterImage;
import bakutan.common.graphics.CharacterImageBundle;
import bakutan.common.graphics.EffectBundle;
import bakutan.common.math.BShape;
import bakutan.common.math.BVector2f;

/**
 * @author cattaka
 */
public class BodyFire extends Body
{
    /**
     * 火のサイズのオプションを得るための定数。
     * getOption(int)に使用する。
     */
	public static final byte OPTION_SIZE = 0;
	private int damage = 20;
    private HashSet<Body> hit = new HashSet<Body>();
    private Body owner;
    
	public BodyFire()
	{
        super(BStageDM.TYPE_FIRE1, 1);
//        setType(BStageThrowBom.TYPE_FIRE1);
//		setOptions(new int[1]);
	}
    @Override
	public boolean hasName()
	{
		return false;
	}
    @Override
	public boolean hasVariableBCircle()
	{
		return true;
	} 
    @Override
    public boolean hasState()
    {
        return false;
    }
    @Override
    public boolean hasArmor()
    {
        return false;
    }
    @Override
    public boolean hasArmorReport()
    {
        return false;
    }
    @Override
    public void setOption(int type, int value)
    {
        if (type == OPTION_SIZE)
        {
            setShape(new BShape(BodyFire.calcRadius(value)));
        }
        super.setOption(type, value);
    }
	public boolean hasScore()
	{
		return false;
	}
	public void drawCharacter(Graphics g, long currenttime, CharacterImageBundle cib, EffectBundle eb)
	{
	    //float scale = (BStageThrowBom.FIRE_WIDTH + BStageThrowBom.FIRE_WIDTH_INC * getOption(OPTION_SIZE)) / BStageThrowBom.FIRE_WIDTH;
        float scale = (BStageDM.FIRE_WIDTH + BStageDM.FIRE_WIDTH_INC * getOption(OPTION_SIZE)) / BStageDM.FIRE_WIDTH;
		int width = (int)(scale * getCharacterImage().getWidth());
		int height = (int)(scale * getCharacterImage().getHeight());
		int n = ((int)(currenttime - getLastStateChange()))/DataManager.ANIMINTERVAL;
		Image[] img;
		
		img = getCharacterImage().getImages(CharacterImage.WIN);
//		int x = -offset.width + (int) (getPosition().x * DataManager.BLOCKSIZE - width / 2);
//		int y = -offset.height + (int) (getPosition().y * DataManager.BLOCKSIZE - height / 2);
        int x =  - width / 2;
        int y =  - height / 2;
		g.drawImage(img[n%img.length], x, y, width, height, null);
		
		/*
		img = getCharacterImage().getImages(CharacterImage.MOVERIGHT);
		for (int j=1;j<=getOption(OPTION_RIGHT) && j<getOption(OPTION_LENGTH);j++)
		{
			int x = (int) (getPosition().x * DataManager.CHARACTERSIZE - offsetx) + DataManager.CHARACTERSIZE * j;
			int y = (int) (getPosition().y * DataManager.CHARACTERSIZE - offsety);
			g.drawImage(img[n%img.length], x, y, null);
		}
		img = getCharacterImage().getImages(CharacterImage.MOVEUP);
		for (int j=1;j<=getOption(OPTION_UP) && j<getOption(OPTION_LENGTH);j++)
		{
			int x = (int) (getPosition().x * DataManager.CHARACTERSIZE - offsetx);
			int y = (int) (getPosition().y * DataManager.CHARACTERSIZE - offsety) - DataManager.CHARACTERSIZE * j;
			g.drawImage(img[n%img.length], x, y, null);
		}
		img = getCharacterImage().getImages(CharacterImage.MOVELEFT);
		for (int j=1;j<=getOption(OPTION_LEFT) && j<getOption(OPTION_LENGTH);j++)
		{
			int x = (int) (getPosition().x * DataManager.CHARACTERSIZE - offsetx) - DataManager.CHARACTERSIZE * j;
			int y = (int) (getPosition().y * DataManager.CHARACTERSIZE - offsety);
			g.drawImage(img[n%img.length], x, y, null);
		}
		img = getCharacterImage().getImages(CharacterImage.MOVEDOWN);
		for (int j=1;j<=getOption(OPTION_DOWN) && j<getOption(OPTION_LENGTH);j++)
		{
			int x = (int) (getPosition().x * DataManager.CHARACTERSIZE - offsetx);
			int y = (int) (getPosition().y * DataManager.CHARACTERSIZE - offsety) + DataManager.CHARACTERSIZE * j;
			g.drawImage(img[n%img.length], x, y, null);
		}
		if (getOption(OPTION_RIGHT) == getOption(OPTION_LENGTH))
		{
			img = getCharacterImage().getImages(CharacterImage.RIGHT);
			int x = (int) (getPosition().x * DataManager.CHARACTERSIZE - offsetx) + DataManager.CHARACTERSIZE * getOption(OPTION_LENGTH);
			int y = (int) (getPosition().y * DataManager.CHARACTERSIZE - offsety);
			g.drawImage(img[n%img.length], x, y, null);
		}
		if (getOption(OPTION_UP) == getOption(OPTION_LENGTH))
		{
			img = getCharacterImage().getImages(CharacterImage.UP);
			int x = (int) (getPosition().x * DataManager.CHARACTERSIZE - offsetx);
			int y = (int) (getPosition().y * DataManager.CHARACTERSIZE - offsety) - DataManager.CHARACTERSIZE * getOptions()[OPTION_LENGTH];
			g.drawImage(img[n%img.length], x, y, null);
		}
		if (getOptions()[OPTION_LEFT] == getOption(OPTION_LENGTH))
		{
			img = getCharacterImage().getImages(CharacterImage.LEFT);
			int x = (int) (getPosition().x * DataManager.CHARACTERSIZE - offsetx) - DataManager.CHARACTERSIZE * getOption(OPTION_LENGTH);
			int y = (int) (getPosition().y * DataManager.CHARACTERSIZE - offsety);
			g.drawImage(img[n%img.length], x, y, null);
		}
		if (getOption(OPTION_DOWN) == getOption(OPTION_LENGTH))
		{
			img = getCharacterImage().getImages(CharacterImage.DOWN);
			int x = (int) (getPosition().x * DataManager.CHARACTERSIZE - offsetx);
			int y = (int) (getPosition().y * DataManager.CHARACTERSIZE - offsety) + DataManager.CHARACTERSIZE * getOption(OPTION_LENGTH);
			g.drawImage(img[n%img.length], x, y, null);
		}
		img = getCharacterImage().getImages(CharacterImage.WIN);
		int x = (int) (getPosition().x * DataManager.CHARACTERSIZE - offsetx);
		int y = (int) (getPosition().y * DataManager.CHARACTERSIZE - offsety);
		g.drawImage(img[n%img.length], x, y, null);
		*/
		
		
	}
	public void actionForClient(BodyEvent bodyEvent, ClientBody mybody, CharacterImageBundle cib, EffectBundle eb, BUserInterface ui, long currenttime, BVector2f bufferCenter)
	{
		if (bodyEvent instanceof BodyAddEvent)
		{
		    //baudio.play(BStageThrowBom.SOUND_FIRE, 0, bufferCenter, getPosition());
            EffectBlast effect = new EffectBlast(getPosition(), getVelocity(), 30, getShape().getRadius()*5, currenttime);
            eb.putPostEffect(effect);
		}
	}
	public void eventCollisition(EventHandler eventHandler, BStage bstage, Body target)
	{
		if (target.isState(Body.STATE_LIFE))
		{
            if (hit.contains(target))
                return;
            hit.add(target);
			long currenttime = bstage.getCurrentTime();
			eventHandler.armor(currenttime, currenttime, target, this.getOwner(), BodyArmorEvent.SUB, damage);
		}
	}
    @Override
    public byte[] getCheckCollidType()
    {
        return new byte[]{
                };
    }
    @Override
    public byte[] getCheckIntersectType()
    {
        return new byte[]{
                BStageDM.TYPE_PLAYER1,
                BStageDM.TYPE_MISSILE1,
                BStageDM.TYPE_FIRE1,
                BStageDM.TYPE_ITEM,
                BStageDM.TYPE_SOFTWALL,
                BStageDM.TYPE_BOMB1,
                BStageDM.TYPE_SLUG1,
                BStageDM.TYPE_HOMING1,
                BStageDM.TYPE_CLUSTERBOMB
                };
    }
	public boolean isCheckCollidWith(Body target)
	{
		return false;
	}
	
	public boolean isCheckIntersectWith(Body target)
	{
        if (target.getType() == BStageDM.TYPE_WALL1)
            return false;
        else
            return true;
	}
    public int getDamage()
    {
        return damage;
    }
    public void setDamage(int damage)
    {
        this.damage = damage;
    }
    public static float calcRadius(int size)
    {
        float radius = BStageDM.FIRE_WIDTH + BStageDM.FIRE_WIDTH_INC * size;
        return radius;
    }
    public Body getOwner()
    {
        return owner;
    }
    public void setOwner(Body owner)
    {
        this.owner = owner;
    }
    public void putHit(Body target)
    {
        hit.add(target);
    }
}
