/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/08
 */
package bakutan.game.deathmatch; 

import java.awt.Graphics;
import java.awt.Image;

import bakutan.common.BStage;
import bakutan.common.BUserInterface;
import bakutan.common.Body;
import bakutan.common.ClientBody;
import bakutan.common.DataManager;
import bakutan.common.EventHandler;
import bakutan.common.event.BodyEvent;
import bakutan.common.event.BodyArmorEvent;
import bakutan.common.graphics.CharacterImage;
import bakutan.common.graphics.CharacterImageBundle;
import bakutan.common.graphics.EffectBundle;
import bakutan.common.math.BShape;
import bakutan.common.math.BVector2f;

/**
 * @author cattaka
 */
public class BodyItem extends Body
{
    /**
     * 火力アップのアイテムを表す定数
     */
    public static final byte ITEM_FIRE = 0;
    /**
     * 爆弾数アップのアイテムを表す定数
     */
    public static final byte ITEM_CHARGE = 1;
    /**
     * スピードアップのアイテムを表す定数
     */
    public static final byte ITEM_SPEED = 2;
    /**
     * 未使用
     */
    public static final byte ITEM_WEAPON = 3;
    /**
     * 装甲回復のアイテムを表す定数
     */
    public static final byte ITEM_ARMOR = 4;

    /**
     * アイテムのオプションを得るための定数。
     * getOption(int)に使用する。
     * @see BodyItem#ITEM_FIRE ITEM_FIRE
     * @see BodyItem#ITEM_CHARGE ITEM_CHARGE
     * @see BodyItem#ITEM_SPEED ITEM_SPEED
     * @see BodyItem#ITEM_WEAPON ITEM_WEAPON
     * @see BodyItem#ITEM_ARMOR ITEM_ARMOR
     */
	public static final byte OPTION_TYPE = 0;
	//private Body parent;
	private boolean respawnable = false;
	private int weapon;
	
	public BodyItem()
	{
        super(BStageDM.TYPE_ITEM, 2);
//        setType(BStageThrowBom.TYPE_ITEM);
//		setOptions(new int[2]);
		enableState(STATE_FIREWALL);
		enableState(STATE_LIFE);
		setShape(new BShape(BStageDM.ITEM_WIDTH));
		setArmor(1);
	}
    @Override
	public boolean hasName()
	{
		return false;
	}
    @Override
	public boolean hasVariableBCircle()
	{
		return false;
	} 
    @Override
	public boolean hasScore()
	{
		return false;
	}
    @Override
    public boolean hasArmor()
    {
        return false;
    }
    @Override
    public boolean hasArmorReport()
    {
        return false;
    }
	public void actionForClient(BodyEvent bodyEvent, ClientBody mybody, CharacterImageBundle cib, EffectBundle eb, BUserInterface ui, long currenttime, BVector2f bufferCenter)
	{
//		if (bodyEvent instanceof BodyDeleteEvent)
//		{
//			if (!isState(STATE_DEAD))
//			{
//				baudio.play(BStageThrowBom.SOUND_ITEMGET, 0);
//			}
//		}
	}
	public void eventCollisition(EventHandler eventHandler, BStage bstage, Body target)
	{
		if (target.getType() == BStageDM.TYPE_PLAYER1 && !(isState(Body.STATE_DEAD)))
		{
			long currenttime = bstage.getCurrentTime();
			byte item_id = (byte)getOption(OPTION_TYPE);
			if ((item_id == ITEM_FIRE && target.getOption(BodyPlayer.OPTION_FIRE) < BodyPlayer.MAX_FIRE)
				|| (item_id == ITEM_SPEED && target.getOption(BodyPlayer.OPTION_SPEED) < BodyPlayer.MAX_SPEED)
				|| (item_id == ITEM_CHARGE) && target.getOption(BodyPlayer.OPTION_CHARGE) < BodyPlayer.MAX_CHARGE)
			{
				eventHandler.option(currenttime, currenttime, target, item_id, (byte)(target.getOption(item_id)+1));
				//eventHandler.sound(currenttime,currenttime,BStageThrowBom.SOUND_ITEMGET);
				eventHandler.delete(currenttime,currenttime,this);
				respawn(eventHandler, currenttime,bstage);
			}else if(item_id == ITEM_WEAPON)
			{
				if ((target.getOption(BodyPlayer.OPTION_WEAPON) & getWeapon()) == 0)
				{
					if (getWeapon() == BodyPlayer.WEAPON_KICK)
					{
						int w = (target.getOption(BodyPlayer.OPTION_WEAPON) | getWeapon());
						eventHandler.option(currenttime, currenttime, target, BodyPlayer.OPTION_WEAPON,w);
						//eventHandler.sound(currenttime,currenttime,BStageThrowBom.SOUND_ITEMGET);
						eventHandler.delete(currenttime,currenttime,this);
						respawn(eventHandler, currenttime,bstage);
					}
				}
            }else if(item_id == ITEM_ARMOR)
            {
                if (target.getArmor() > 0 && target.getArmor() < 100)
                {
                    int h = 20;
                    if (target.getArmor() > 80)
                    {
                        h = 100 - target.getArmor();
                    }
                    eventHandler.armor(currenttime, currenttime, target,this, BodyArmorEvent.ADD,h);
                    eventHandler.delete(currenttime,currenttime,this);
                    respawn(eventHandler, currenttime,bstage);
                }
            }
		}
	}
	public void eventStateChange(EventHandler eventHandler, BStage bstage, byte type, int state)
	{
		if ((type == Body.SET || type == Body.ENABLE)
				&& (state & Body.STATE_DEAD) == Body.STATE_DEAD)
		{
			long currenttime = bstage.getCurrentTime();
			eventHandler.delete(currenttime,currenttime + 500,this);
			
			respawn(eventHandler,currenttime, bstage);;
		}
	}
	
	private void respawn(EventHandler eventHandler, long currenttime, BStage bstage)
	{
		if (respawnable)
		{
			respawnable = false;
			BodyItem body = new BodyItem();
			//body.setID(bstage.getNextID());
			
			body.selectRundom();
			body.setRespawnable(true);
			body.setPosition(getPosition());
			body.enableState(STATE_DEAD);
			body.disableState(STATE_FIREWALL);
			eventHandler.add(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL, body);
			eventHandler.state(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL + 1000, body, DISABLE, STATE_DEAD);
			eventHandler.state(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL + 1000, body, ENABLE, STATE_FIREWALL);
            eventHandler.recheckIntersect(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL + 1000, body);
		}
	}

	public void drawCharacter(Graphics g ,long currenttime, CharacterImageBundle cib, EffectBundle eb)
	{
		CharacterImage characterImage = getCharacterImage();
		if (characterImage == null)
			return;
		
		int offsetx = characterImage.getWidth() / 2;
		int offsety = characterImage.getHeight() / 2;
        int x = -offsetx;
        int y = -offsety;
		int n = ((int)(currenttime - getLastStateChange()))/DataManager.ANIMINTERVAL;
		
		if (isState(Body.STATE_DEAD))
		{
			Image[] img = characterImage.getImages(CharacterImage.DEAD);
			if (n < img.length)
			{
				g.drawImage(img[n], x, y, null);
			}else
			{
				img = characterImage.getImages(CharacterImage.DEADSTOP);
				g.drawImage(img[n%img.length], x, y, null);
			}
		}else
		{
			Image[] img = characterImage.getImages(CharacterImage.WIN);
			if (n < img.length)
			{
				g.drawImage(img[n], x, y, null);
			}else
			{
				if (getOption(OPTION_TYPE) == ITEM_FIRE)
				{
					img = characterImage.getImages(CharacterImage.RIGHT);
				}else if (getOption(OPTION_TYPE) == ITEM_SPEED)
				{
					img = characterImage.getImages(CharacterImage.UP);
				}else if (getOption(OPTION_TYPE) == ITEM_CHARGE)
				{
					img = characterImage.getImages(CharacterImage.LEFT);
				}else if (getOption(OPTION_TYPE) == ITEM_WEAPON)
				{
				    img = characterImage.getImages(CharacterImage.DOWN);
                }else if (getOption(OPTION_TYPE) == ITEM_ARMOR)
                {
                    img = characterImage.getImages(CharacterImage.MOVELEFT);
                }
				g.drawImage(img[n%img.length], x, y, null);
			}
		}
	}
	/*
	public Body getParent()
	{
		return parent;
	}
	public void setParent(Body parent)
	{
		this.parent = parent;
	}
	*/
	public void selectRundom()
	{
		float r = (float)Math.random();
		if (r < 1d/4d)
			setOption(BodyItem.OPTION_TYPE, ITEM_FIRE);
		else if (r < 2d/4d)
			setOption(BodyItem.OPTION_TYPE, ITEM_SPEED);
		else if (r < 3d/4d)
			setOption(BodyItem.OPTION_TYPE, ITEM_CHARGE);
        else
            setOption(BodyItem.OPTION_TYPE, ITEM_ARMOR);
	}
	public int getWeapon()
	{
		return weapon;
	}
	public void setWeapon(int weapon)
	{
		this.weapon = weapon;
	}
	public boolean isRespawnable()
	{
		return respawnable;
	}
	public void setRespawnable(boolean respawnable)
	{
		this.respawnable = respawnable;
	}
    @Override
    public byte[] getCheckCollidType()
    {
        return null;
    }
    @Override
    public byte[] getCheckIntersectType()
    {
        return new byte[]{
                BStageDM.TYPE_PLAYER1,
                BStageDM.TYPE_FIRE1,
                };
    }
	public boolean isCheckCollidWith(Body target) {
		return false;
	}
	
	public boolean isCheckIntersectWith(Body target)
	{
		if (target.getType() == BStageDM.TYPE_PLAYER1
			|| target.getType() == BStageDM.TYPE_FIRE1
			)
			return true;
		else
			return false;
	}
}
