/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/07
 */
package bakutan.game.deathmatch;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.AffineTransform;

import bakutan.common.BStage;
import bakutan.common.BUserInterface;
import bakutan.common.Body;
import bakutan.common.ClientBody;
import bakutan.common.DataManager;
import bakutan.common.EventHandler;
import bakutan.common.event.BodyActionEvent;
import bakutan.common.event.BodyEvent;
import bakutan.common.event.BodyArmorEvent;
import bakutan.common.graphics.CharacterImage;
import bakutan.common.graphics.CharacterImageBundle;
import bakutan.common.graphics.EffectBundle;
import bakutan.common.math.BShape;
import bakutan.common.math.BVector2f;

/**
 * @author cattaka
 */
public class BodyMissile extends Body
{
    /**
     * 火力のオプションを得るための定数。
     * getOption(int)に使用する。
     */
	public static final byte OPTION_FIRE = 0;
    /**
     * ミサイルの直撃したときのダメージ
     */
    public static final int DAMAGE = 20;
    /**
     * 爆風によるダメージ
     */
    public static final int FIRE_DAMAGE = 10;
    /**
     * 爆風の残る時間の係数
     */
    public static final int MISSILEFIRE_DERAY = 200;
    /**
     * 爆風の残る時間の増加係数
     */
    public static final int MISSILEFIRE_DERAY_INCREASE = 50;
    /**
     * ミサイルの飛翔時間を表す定数、単位はミリ秒。
     */
    public static final int LIFE_TIME = 10000;
    /**
     * ミサイルの最大射程距離（バラージは除く）
     */
    public static final float SHOOTING_RANGE = 15f;
    
    public static final byte ACTION_BOMB = 0;
    private Body owner;
	
	public BodyMissile()
	{
        super(BStageDM.TYPE_MISSILE1, 1);
//        setType(BStageThrowBom.TYPE_MISSILE1);
//		setOptions(new int[1]);
		setOption(BodyMissile.OPTION_FIRE, getOption(OPTION_FIRE));
		setShape(new BShape(BStageDM.MISSILE_WIDTH));
		//enableState(Body.COLLIDABLE);
		enableState(Body.STATE_LIFE);
	}
    public void drawCharacter(Graphics g, long currenttime, CharacterImageBundle cib, EffectBundle eb)
    {
        CharacterImage characterImage = getCharacterImage();
        if (characterImage == null)
            return;
        
        int offsetx = characterImage.getWidth() / 2;
        int offsety = characterImage.getHeight() / 2;
//        int x = -offset.width + (int) (getPosition().x * DataManager.BLOCKSIZE);
//        int y = -offset.height + (int) (getPosition().y * DataManager.BLOCKSIZE);
        int x = 0;
        int y = 0;
        int n = ((int)(currenttime - getLastStateChange()))/DataManager.ANIMINTERVAL;

        Image img = characterImage.getImages(CharacterImage.DOWN)[n%2];

        BVector2f v = new BVector2f(getDirection());
        v.normalize();
        float l = v.getLength();
        double rad;
        if (l > 0)
        {
            rad = Math.acos(v.y/l);
            if (v.x > 0)
                rad = 2 * Math.PI - rad;
        }else
        {
            rad = 0;
        }

        AffineTransform at = new AffineTransform();
        at.setToTranslation(x,y);
        at.rotate(rad);
        at.translate(-offsetx,-offsety);
        
        Graphics2D g2 = (Graphics2D)g;
        g2.drawImage(img, at, null);
    }
	public boolean hasName()
	{
		return false;
	}
	public boolean hasVariableBCircle()
	{
		return false;
	}
	public boolean hasScore()
	{
		return false;
	}
    @Override
    public boolean hasState()
    {
        return false;
    }
    @Override
    public boolean hasArmor()
    {
        return false;
    }
    @Override
    public boolean hasOption()
    {
        return false;
    }
    @Override
    public boolean hasArmorReport()
    {
        return false;
    }
	public void actionForClient(BodyEvent bodyEvent, ClientBody mybody, CharacterImageBundle cib, EffectBundle eb, BUserInterface ui, long currenttime, BVector2f bufferCenter)
	{
		if (bodyEvent instanceof BodyActionEvent)
		{
            BodyActionEvent bae = (BodyActionEvent)bodyEvent;
            if (bae.getAction() == ACTION_BOMB)
            {
                ui.getBAudio().playSE(BStageDM.SOUND_FIRE, 0, bufferCenter, getPosition());
            }
		}
	}
	public boolean eventAction(EventHandler eventHandler, BStage bstage, BVector2f position, BodyActionEvent event)
	{
		if (event.getAction() == ACTION_BOMB)
		{
			long currenttime = bstage.getCurrentTime();
			
			BVector2f p = new BVector2f(getPosition());
			//p.x = 0.5 + (int)(p.x);
			//p.y = 0.5 + (int)(p.y);
			
            int fire_deray = BodyMissile.MISSILEFIRE_DERAY + getOption(OPTION_FIRE) * BodyMissile.MISSILEFIRE_DERAY_INCREASE;
            BodyFire fire = new BodyFire();
            fire.setOwner(getOwner());
            //float length = BStageThrowBom.FIRE_WIDTH + BStageThrowBom.FIRE_WIDTH_INC * (float)getOption(OPTION_FIRE);
            //fire.setOption(BodyFire.OPTION_SIZE, getOption(OPTION_FIRE));
            fire.setOption(BodyFire.OPTION_SIZE, 0);
            fire.setShape(new BShape(BodyFire.calcRadius(0)));
			fire.setDamage(FIRE_DAMAGE);
            fire.setPosition(p);
			fire.setVelocity(getVelocity());
//			float speed = fire.getVelocity().getLength();
//            if (speed > 0 && !fire.getVelocity().isZeroVector())
//            {
//                fire.getVelocity().normalize();
//                fire.getVelocity().scale(speed/2);
//            }
            
			eventHandler.delete(currenttime, currenttime,this);
			eventHandler.add(currenttime, currenttime, fire);

            eventHandler.delete(currenttime, currenttime + fire_deray, fire);
            return true;
		}
        return false;
	}
	public void eventCollisition(EventHandler eventHandler, BStage bstage, Body target)
	{
		if (target.getType() == BStageDM.TYPE_MISSILE1)
		{
			BodyMissile bom = (BodyMissile)target;
			if (getOwner() == bom.getOwner())
				return;
		}
		if (target == owner)
            return;

        long currenttime = bstage.getCurrentTime();
        if (target.isState(Body.STATE_LIFE))
        {
            eventHandler.armor(currenttime, currenttime, target, this.getOwner(), BodyArmorEvent.SUB, DAMAGE);
        }
        //eventHandler.action(currenttime,currenttime,this,getPosition(),ACTION_BOMB);
        eventHandler.smallAction(currenttime,currenttime,this,ACTION_BOMB);
	}
	
	public void eventStateChange(EventHandler eventHandler, BStage bstage, byte type, int state)
	{
		if ((type == Body.SET || type == Body.ENABLE)
				&& (state & Body.STATE_DEAD) == Body.STATE_DEAD)
		{
			long currenttime = bstage.getCurrentTime();
			//eventHandler.action(currenttime,currenttime,this,getPosition(),ACTION_BOMB);
            eventHandler.smallAction(currenttime,currenttime,this, ACTION_BOMB);
		}
	}
	public void eventPush(EventHandler eventHandler, BStage bstage, Body target)
	{
		if (target.getType() == BStageDM.TYPE_PLAYER1
				&& (target.getOption(BodyPlayer.OPTION_WEAPON) & BodyPlayer.WEAPON_KICK) == BodyPlayer.WEAPON_KICK )
		{
			long currenttime = bstage.getCurrentTime();
			BVector2f v = new BVector2f(getPosition());
			v.sub(target.getPosition());
//			v.x -= target.getPosition().x;
//			v.y -= target.getPosition().y;
//			if (Math.abs(v.x) >= Math.abs(v.y))
//			{
//				v.y = 0;
//			}else
//			{
//				v.x = 0;
//			}
            if (!v.isZeroVector())
            {
                v.normalize();
                v.scale(target.getMaxSpeed());
                eventHandler.move(currenttime, currenttime, this, this.getPosition(), v, v);
            }
		}
	}
    @Override
    public byte[] getCheckCollidType()
    {
        return new byte[]{
                BStageDM.TYPE_WALL1,
                BStageDM.TYPE_SOFTWALL
                };
    }
    @Override
    public byte[] getCheckIntersectType()
    {
        return new byte[]{
                BStageDM.TYPE_WALL1,
                BStageDM.TYPE_PLAYER1,
                BStageDM.TYPE_MISSILE1,
                BStageDM.TYPE_FIRE1,
                BStageDM.TYPE_SOFTWALL,
                BStageDM.TYPE_BOMB1,
                BStageDM.TYPE_SLUG1,
                BStageDM.TYPE_HOMING1,
                BStageDM.TYPE_CLUSTERBOMB
                };
    }
	public boolean isCheckIntersectWith(Body target)
	{
		if (target.getType() == BStageDM.TYPE_ITEM)
			return false;
		return true;
	}
    @Override
    public boolean isCheckCollidWith(Body target)
    {
        return false;
    }
	public Body getOwner()
	{
		return owner;
	}
	public void setOwner(Body owner)
	{
		this.owner = owner;
	}
}
