/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/07
 */
package bakutan.game.deathmatch;

import java.util.HashSet;

import bakutan.common.BStage;
import bakutan.common.BUserInterface;
import bakutan.common.Body;
import bakutan.common.ClientBody;
import bakutan.common.EventHandler;
import bakutan.common.event.BodyActionEvent;
import bakutan.common.event.BodyEvent;
import bakutan.common.event.BodyArmorEvent;
import bakutan.common.graphics.CharacterImageBundle;
import bakutan.common.graphics.EffectBundle;
import bakutan.common.math.BMath;
import bakutan.common.math.BShape;
import bakutan.common.math.BVector2f;

/**
 * @author cattaka
 */
public class BodySlug extends Body
{
    /**
     * 火力のオプションを得るための定数。
     * getOption(int)に使用する。
     */
    public static final byte OPTION_FIRE = 0;
    /**
     * スラグの飛翔時間を表す定数、単位はミリ秒。
     */
    public static final int LIFE_TIME = 10000;
    /**
     * 衝突によるダメージ
     */
    public static final int SLUG_DAMAGE = 20;
    /**
     * 爆風のダメージ
     */
    public static final int FIRE_DAMAGE = 10;
    /**
     * 爆風の残る時間の係数
     */
    public static final int FIRE_DERAY = 200;
    /**
     * スラグの装甲
     */
    public static final int SLUG_ARMOR = 40;
    /**
     * スラグの装甲の増加係数
     */
    public static final int SLUG_ARMOR_INCREASE = 10;

    public static final byte ACTION_BOMB = 0;
    //public static final byte ACTION_REFLECT = 0;
    private Body owner;
    private HashSet<Body> hit = new HashSet<Body>();
	
	public BodySlug()
	{
        super(BStageDM.TYPE_SLUG1, 1);
//        setType(BStageThrowBom.TYPE_SLUG1);
//		setOptions(new int[1]);
		setOption(BodySlug.OPTION_FIRE, getOption(OPTION_FIRE));
		setShape(new BShape(BStageDM.SLUG_WIDTH));
		setArmor(SLUG_ARMOR);
        enableState(Body.STATE_LIFE);
	}
    @Override
	public boolean hasName()
	{
		return false;
	}
    @Override
	public boolean hasVariableBCircle()
	{
		return false;
	}
    @Override
	public boolean hasScore()
	{
		return false;
	}
    @Override
    public boolean hasState()
    {
        return false;
    }
    @Override
    public boolean hasOption()
    {
        return false;
    }
    @Override
    public boolean hasArmorReport()
    {
        return false;
    }
//    public void drawCharacter(Graphics g, Dimension offset, long currenttime)
//    {
//        if (!isState(COLLIDABLE))
//        {
//            int offsetx = getCharacterImage().getWidth() / 2;
//            int offsety = getCharacterImage().getHeight() / 2;
//            int x = -offset.width + (int) (getPosition().x * DataManager.CHARACTERSIZE - offsetx);
//            int y = -offset.height + (int) (getPosition().y * DataManager.CHARACTERSIZE - offsety);
//            int n = ((int)(currenttime - getLastStateChange()))/DataManager.ANIMINTERVAL;
//            Image[] img = getCharacterImage().getImages(CharacterImage.WIN);
//            g.drawImage(img[n % img.length], x, y, null);
//            return;
//        }
//        super.drawCharacter(g, offset, currenttime);
//    }
	
	public void actionForClient(BodyEvent bodyEvent, ClientBody mybody, CharacterImageBundle cib, EffectBundle eb, BUserInterface ui, long currenttime, BVector2f bufferCenter)
	{
        if (bodyEvent instanceof BodyActionEvent)
        {
            BodyActionEvent bae = (BodyActionEvent)bodyEvent;
            if (bae.getAction() == ACTION_BOMB)
            {
                ui.getBAudio().playSE(BStageDM.SOUND_FIRE, 0, bufferCenter, getPosition());
            }
        }
	}
	public boolean eventAction(EventHandler eventHandler, BStage bstage, BVector2f position, BodyActionEvent event)
	{
        if (event.getAction() == ACTION_BOMB)
        {
            long currenttime = bstage.getCurrentTime();
            
            BVector2f p = new BVector2f(getPosition());
            //p.x = 0.5 + (int)(p.x);
            //p.y = 0.5 + (int)(p.y);
            
            int fire_deray = (getOption(OPTION_FIRE)) * FIRE_DERAY;
            BodyFire fire = new BodyFire();
            fire.setOwner(getOwner());
//            float length = BStageThrowBom.FIRE_WIDTH + BStageThrowBom.FIRE_WIDTH_INC * (float)getOption(OPTION_FIRE);
            fire.setOption(BodyFire.OPTION_SIZE, 0);
//            fire.setPolygon(createBpolygon(length));
            fire.setShape(new BShape(BodyFire.calcRadius(0)));
            fire.setDamage(FIRE_DAMAGE);
            fire.setPosition(p);
            fire.setVelocity(getVelocity());
            float speed = fire.getVelocity().getLength();
            if (speed > 0 && !fire.getVelocity().isZeroVector())
            {
                fire.getVelocity().normalize();
                fire.getVelocity().scale(speed/4);
            }
            
            eventHandler.delete(currenttime, currenttime,this);
            eventHandler.add(currenttime, currenttime, fire);

            eventHandler.delete(currenttime, currenttime + fire_deray, fire);
            return true;
        }
        return false;
	}
	public void eventCollisition(EventHandler eventHandler, BStage bstage, Body target)
	{
        long currenttime = bstage.getCurrentTime();
		if (target.isState(Body.STATE_LIFE) && !hit.contains(target))
		{
            hit.add(target);
            eventHandler.armor(currenttime,currenttime,target,this.getOwner(),BodyArmorEvent.SUB, SLUG_DAMAGE);
		}
        if (target.isState(Body.STATE_COLLIDABLE) || target.getType() == getType())
        {
            hit.clear();
            hit.add(target);
            float speed = getVelocity().getLength();
            BVector2f vec = new BVector2f();
            BVector2f t = new BVector2f();
            if (BMath.calcNormal(t, vec, getPosition(), getShape(), target.getPosition(), target.getShape()))
            {
                vec.normalize();
                float r = BVector2f.getDot(getVelocity(),vec);
                if (r < 0)
                {
                    BVector2f vel = new BVector2f(getVelocity());
                    vel.scaleAdd(-r*2,vec);
                    BVector2f dir = new BVector2f(vel);
                    dir.normalize();
                    eventHandler.move(currenttime, currenttime, this, getPosition(),vel,dir);
                }
            }
        }
	}
	
	public void eventStateChange(EventHandler eventHandler, BStage bstage, byte type, int state)
	{
		if ((type == Body.SET || type == Body.ENABLE)
				&& (state & Body.STATE_DEAD) == Body.STATE_DEAD)
		{
			long currenttime = bstage.getCurrentTime();
            //eventHandler.action(currenttime,currenttime,this,getPosition(),ACTION_BOMB);
            eventHandler.smallAction(currenttime, currenttime, this, ACTION_BOMB);
            eventHandler.delete(currenttime,currenttime,this);
		}
	}
	public void eventPush(EventHandler eventHandler, BStage bstage, Body target)
	{
	}
    @Override
    public byte[] getCheckCollidType()
    {
        return new byte[]{
                BStageDM.TYPE_WALL1,
                BStageDM.TYPE_SOFTWALL
                };
    }
    @Override
    public byte[] getCheckIntersectType()
    {
        return new byte[]{
                BStageDM.TYPE_WALL1,
                BStageDM.TYPE_PLAYER1,
                BStageDM.TYPE_MISSILE1,
                BStageDM.TYPE_FIRE1,
                BStageDM.TYPE_SOFTWALL,
                BStageDM.TYPE_BOMB1,
                BStageDM.TYPE_SLUG1,
                BStageDM.TYPE_HOMING1,
                BStageDM.TYPE_CLUSTERBOMB
                };
    }

    public boolean isCheckCollidWith(Body target)
	{
	    return false;
	}
	public boolean isCheckIntersectWith(Body target)
	{
		if (target.getType() == BStageDM.TYPE_ITEM)
			return false;
		return true;
	}
	public void putHit(Body target)
	{
        hit.add(target);
	}
    public Body getOwner()
    {
        return owner;
    }
    public void setOwner(Body owner)
    {
        this.owner = owner;
    }
}
