/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/07
 */
package bakutan.game.deathmatch;



import bakutan.common.BStage;
import bakutan.common.Body;
import bakutan.common.EventHandler;
import bakutan.common.math.BShape;

/**
 * @author cattaka
 */
public class BodySoftWall extends Body
{
    /**
     * 柔らかい壁の標準の装甲
     */
    public static final int SOFTWALL_ARMOR = 50;
    
    
	private boolean respawnable = true;
	public BodySoftWall()
	{
        super(BStageDM.TYPE_SOFTWALL, 0);
//        setType(BStageThrowBom.TYPE_SOFTWALL);
        setArmor(SOFTWALL_ARMOR);
		enableState(Body.STATE_COLLIDABLE);
		enableState(Body.STATE_FIREWALL);
		enableState(Body.STATE_LIFE);
		setShape(new BShape(BStageDM.WALL_WIDTH));
	}
    @Override
	public boolean hasName()
	{
		return false;
	}
    @Override
	public boolean hasVariableBCircle()
	{
		return false;
	} 
    @Override
	public boolean hasScore()
	{
		return false;
	}
    @Override
    public boolean hasArmorReport()
    {
        return false;
    }

	public void eventStateChange(EventHandler eventHandler, BStage bstage, byte type, int state)
	{
		if ((type == Body.SET || type == Body.ENABLE)
				&& (state & Body.STATE_DEAD) == Body.STATE_DEAD)
		{
			long currenttime = bstage.getCurrentTime();
			eventHandler.delete(currenttime,currenttime+500,this);
			if (respawnable)
			{
				respawnable = false;
				BodySoftWall wall = new BodySoftWall();
				//wall.setID(bstage.getNextID());
				wall.setPosition(getPosition());
				wall.disableState(STATE_COLLIDABLE);
				wall.disableState(STATE_FIREWALL);
				eventHandler.add(currenttime, currenttime + BStageDM.WALL_SPAWN_INTERVAL, wall);
				eventHandler.state(currenttime, currenttime + BStageDM.WALL_SPAWN_INTERVAL+1000,wall,ENABLE,STATE_COLLIDABLE | STATE_FIREWALL);
				//eventHandler.state(currenttime, currenttime + BStageThrowBom.WALL_SPAWN_INTERVAL+1000,wall,ENABLE,STATE_FIREWALL);
			}
			
			if (Math.random() < BStageDM.ITEM_SPAWN_RATE)
			{
				BodyItem item = new BodyItem();
				//item.setID(bstage.getNextID());
				item.selectRundom();
				
				item.setPosition(getPosition());
				eventHandler.add(currenttime, currenttime + 500, item);
				eventHandler.state(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL, item, ENABLE, STATE_DEAD);
				eventHandler.delete(currenttime, currenttime + BStageDM.ITEM_SPAWN_INTERVAL + 500, item);
			}
		}

	}
	
//	public void drawCharacter(Graphics g ,long currenttime, CharacterImageBundle cib, EffectBundle eb)
//	{
//		CharacterImage characterImage = getCharacterImage();
//		if (characterImage == null)
//			return;
//		
//		int offsetx = characterImage.getWidth() / 2;
//		int offsety = characterImage.getHeight() / 2;
////		int x = -offset.width + (int) (getPosition().x * DataManager.BLOCKSIZE - offsetx);
////		int y = -offset.height + (int) (getPosition().y * DataManager.BLOCKSIZE - offsety);
//        int x = - offsetx;
//        int y = - offsety;
//		int n = ((int)(currenttime - getLastStateChange()))/DataManager.ANIMINTERVAL;
//
//
//		if (isState(Body.STATE_DEAD))
//		{
//			Image[] img = characterImage.getImages(CharacterImage.DEAD);
//			if (n < img.length)
//			{
//				g.drawImage(img[n], x, y, null);
//			}else
//			{
//				img = characterImage.getImages(CharacterImage.DEADSTOP);
//				g.drawImage(img[n%img.length], x, y, null);
//			}
//		}else
//		{
//            
//			Image[] img = characterImage.getImages(CharacterImage.WIN);
//			if (n < img.length)
//			{
//				g.drawImage(img[n], x, y, null);
//			}else
//			{
//                float alpha = 0.5f + 0.5f * ((float)getArmor()/(float)SOFTWALL_ARMOR);
//                if (alpha > 1f)
//                    alpha = 1f;
//                if (alpha < 0f)
//                    alpha = 0f;
//                Graphics2D g2 = (Graphics2D)g;
//                Composite oldac = g2.getComposite();
//                AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha);
//                g2.setComposite(ac);
//				img = characterImage.getImages(CharacterImage.DOWN);
//				g2.drawImage(img[n%img.length], x, y, null);
//                g2.setComposite(oldac);
//			}
//		}
//	}
	
	public boolean doPushReport()
	{
		return false;
	}
    @Override
    public byte[] getCheckCollidType()
    {
        return new byte[]{
                BStageDM.TYPE_PLAYER1,
                };
    }
    @Override
    public byte[] getCheckIntersectType()
    {
        return new byte[]{
                BStageDM.TYPE_PLAYER1,
                BStageDM.TYPE_MISSILE1,
                BStageDM.TYPE_FIRE1,
                BStageDM.TYPE_ITEM,
                BStageDM.TYPE_BOMB1,
                BStageDM.TYPE_SLUG1,
                BStageDM.TYPE_HOMING1,
                BStageDM.TYPE_CLUSTERBOMB
                };
    }
    public boolean isCheckIntersectWith(Body target)
    {
        if (target.getType() == BStageDM.TYPE_SOFTWALL
                || target.getType() == BStageDM.TYPE_WALL1)
            return false;
        else
            return true;
    }
	public boolean isCheckCollidWith(Body target)
	{
		if (target.getType() == BStageDM.TYPE_PLAYER1)
			return true;
		else
			return false;
	}
}
