/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/07
 */
package bakutan.game.deathmatch;

import java.awt.Graphics;
import java.awt.Image;

import bakutan.common.Body;
import bakutan.common.DataManager;
import bakutan.common.graphics.CharacterImage;
import bakutan.common.graphics.CharacterImageBundle;
import bakutan.common.graphics.EffectBundle;
import bakutan.common.math.BShape;
import bakutan.common.math.BVector2f;
import bakutan.common.math.BVector4f;

/**
 * @author cattaka
 */
public class BodyWall extends Body
{
	public static final byte ACTION_BOMB = 0;
	
	public BodyWall()
	{
        super(BStageDM.TYPE_WALL1, 0);
        //setType(BStageThrowBom.TYPE_WALL1);
		enableState(Body.STATE_COLLIDABLE);
		enableState(Body.STATE_FIREWALL);
		setShape(new BShape(BStageDM.WALL_WIDTH));
	}
    @Override
	public boolean hasName()
	{
		return false;
	}
    @Override
	public boolean hasVariableBCircle()
	{
		return true;
	}
    @Override
	public boolean hasScore()
	{
		return false;
	}
    @Override
    public boolean hasArmorReport()
    {
        return false;
    }
	public boolean doPushReport()
	{
		return false;
	}
    @Override
    public void drawCharacter(Graphics g, long currenttime, CharacterImageBundle cib, EffectBundle eb)
    {
        if (getShape().getType() == BShape.CIRCLE)
        {
            super.drawCharacter(g, currenttime, cib, eb);
        }
        else if (getShape().getType() == BShape.LINE)
        {
            CharacterImage characterImage = getCharacterImage();
            if (characterImage == null)
                return;
            
            int offsetx = characterImage.getWidth() / 2;
            int offsety = characterImage.getHeight() / 2;
            int n = ((int)(currenttime - getLastStateChange()))/DataManager.ANIMINTERVAL;
            Image img = characterImage.getImages(this, n);
            int x, y;
            
            BShape shape = getShape();
            BVector2f pos = new BVector2f();
            BVector2f move = new BVector2f();
            pos.set(shape.getPoint1());
            
            move.set(shape.getPoint2());
            move.sub(shape.getPoint1());
            float length = move.getLength();
            if (length > 0)
            {
                move.normalize();
                for (float p = 0;p<=length;p+=1.0f)
                {
                    x = (int) (pos.x * DataManager.BLOCKSIZE) - offsetx;
                    y = (int) (pos.y * DataManager.BLOCKSIZE) - offsety;
                    g.drawImage(img, x, y, null);
                    pos.add(move);
                }
            }
        }
        else if (getShape().getType() == BShape.RECTANGLE)
        {
            CharacterImage characterImage = getCharacterImage();
            if (characterImage == null)
                return;
            
            int offsetx = characterImage.getWidth() / 2;
            int offsety = characterImage.getHeight() / 2;
            int n = ((int)(currenttime - getLastStateChange()))/DataManager.ANIMINTERVAL;
            Image img = characterImage.getImages(this, n);
            int x, y;
            
            BShape shape = getShape();
            BVector2f pos = new BVector2f(getPosition());
            BVector4f rec = getShape().getRectangle();
            for (float fx = rec.a+0.5f; fx <= rec.b-0.5f;fx+=1.0f)
            {
                for (float fy = rec.c+0.5f; fy <= rec.d-0.5f;fy+=1.0f)
                {
                    x = (int) ((fx) * DataManager.BLOCKSIZE) - offsetx;
                    y = (int) ((fy) * DataManager.BLOCKSIZE) - offsety;
                    g.drawImage(img, x, y, null);
                }
            }
        }
    }
    @Override
    public byte[] getCheckCollidType()
    {
        return new byte[]{
                BStageDM.TYPE_PLAYER1
                };
    }
    @Override
    public byte[] getCheckIntersectType()
    {
        return new byte[]{
                BStageDM.TYPE_PLAYER1,
                BStageDM.TYPE_MISSILE1,
                BStageDM.TYPE_FIRE1,
                BStageDM.TYPE_ITEM,
                BStageDM.TYPE_BOMB1,
                BStageDM.TYPE_SLUG1,
                BStageDM.TYPE_HOMING1,
                BStageDM.TYPE_CLUSTERBOMB
                };
    }
    public boolean isCheckIntersectWith(Body target)
    {
        if (target.getType() == BStageDM.TYPE_SOFTWALL
                || target.getType() == BStageDM.TYPE_WALL1)
            return false;
        else
            return true;
    }
	public boolean isCheckCollidWith(Body target)
	{
		if (target.getType() == BStageDM.TYPE_PLAYER1)
			return true;
		else
			return false;
	}
}
