/*
 * 作成日: 2007/01/26
 */
package bakutan.server;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import bakutan.common.BUserInterface;
import bakutan.common.BUserInterfacePanel;
import bakutan.common.util.HelpPanel;
import bakutan.server.util.NetworkTool;

public class BakutanServerGUI extends JFrame implements ActionListener
{
    private BUserInterfacePanel userinterface;
    private ServerProcess serverProcess;
    private ServerConfigurationInfoDialog serverConfigurationInfoDialog;
    private UserDataHolder userDataHolder;
    private HelpPanel panelHelp;
    private HelpPanel upnplibHelp;
    private NetworkTool networkTool;
    
    public BakutanServerGUI()
    {
        super("BakutanServerGUI");
        setSize(400,400);
        this.userinterface = new BUserInterfacePanel();
        this.userinterface.setOutputDate(true);
        this.networkTool = NetworkTool.getAvailableNetworkTool();
        this.serverConfigurationInfoDialog = new ServerConfigurationInfoDialog(this, this.userinterface, this.networkTool.isSupportUPNP());
        this.userDataHolder = new NullUserDataHolder();
        makeLayout();
    }
    private void makeLayout()
    {
        URL helpURL = getClass().getClassLoader().getResource("help_server.utf8.txt");
        if (helpURL != null)
        {
            panelHelp = new HelpPanel("Bakutan server help", helpURL);
        }
        URL upnplibURL = getClass().getClassLoader().getResource("license-upnplib.txt");
        if (helpURL != null)
        {
            upnplibHelp = new HelpPanel("license-upnplib", upnplibURL);
        }
        
        JMenuBar menuBar = new JMenuBar();
        JMenu menuServer = new JMenu(userinterface.translate("server"));
        JMenuItem itemStart = new JMenuItem(userinterface.translate("start"));
        JMenuItem itemStop = new JMenuItem(userinterface.translate("stop"));
        itemStart.setActionCommand("start");
        itemStop.setActionCommand("stop");
        itemStart.addActionListener(this);
        itemStop.addActionListener(this);
        menuServer.add(itemStart);
        menuServer.add(itemStop);
        menuBar.add(menuServer);

        JMenu help = new JMenu(userinterface.translate("help"));
        JMenuItem aboutItem = new JMenuItem(userinterface.translate("help"));
        aboutItem.setActionCommand("help");
        aboutItem.addActionListener(this);
        JMenuItem upnplibItem = new JMenuItem("UPNPLib license");
        upnplibItem.setActionCommand("upnplib");
        upnplibItem.addActionListener(this);
        help.add(aboutItem);
        help.add(upnplibItem);
        menuBar.add(help);

        setJMenuBar(menuBar);

        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints(); 
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridwidth = 2;
        gbl.setConstraints(userinterface, gbc);
        
        gbc.anchor = GridBagConstraints.CENTER;
        getContentPane().setLayout(gbl);
        getContentPane().add(userinterface);
    }
    public void actionPerformed(ActionEvent e)
    {
        if (e.getActionCommand().equals("start"))
        {
            if (serverProcess != null)
            {
                serverProcess.stopServer();
                userinterface.outputMessage("Server stoped.");
                serverProcess = null;
            }
            
            ServerConfigurationInfo sci = serverConfigurationInfoDialog.inputServerConfigurationInfo();
            if (sci == null)
                return;
            
            BStageInfo bstageInfo = sci.getBStageInfo();
            if (bstageInfo == null)
                return;
            serverProcess = new ServerProcess(userinterface, networkTool);
            try
            {
                if (bstageInfo.getMapFileName() == null)
                {
                    URL mapFileURL = getClass().getClassLoader().getResource("bakutan.map");
                    serverProcess.init(bstageInfo, mapFileURL, userDataHolder);
                }
                else
                {
                    serverProcess.init(bstageInfo, userDataHolder);
                }
            }
            catch(Exception exc)
            {
                userinterface.outputMessage(exc.getMessage());
                exc.printStackTrace();
                return;
            }
            serverProcess.setRegisterServer(sci.isRegisterServer());
            serverProcess.setUseUPNP(sci.isUseUPNP());
            serverProcess.setServerInfoManagerAddress(sci.getServerInfoManagerAddress());
            serverProcess.start();
            userinterface.outputMessage("Server started.");
            userinterface.outputMessage("Port : " + serverProcess.getPort());
        }
        else if (e.getActionCommand().equals("stop"))
        {
            if (serverProcess != null)
            {
                serverProcess.stopServer();
                userinterface.outputMessage("Server stoped.");
                serverProcess = null;
            }
            else
            {
                userinterface.outputMessage("Server did not started.");
            }
        }
        else if (e.getActionCommand().equals("help"))
        {
            if (panelHelp != null)
            {
                panelHelp.setVisible(true);
            }
        }
        else if (e.getActionCommand().equals("upnplib"))
        {
            if (upnplibHelp != null)
            {
                upnplibHelp.setVisible(true);
            }
        }
    }
    public static void main(String[] args)
    {
        BakutanServerGUI bstageServerGUI = new BakutanServerGUI();
        bstageServerGUI.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        bstageServerGUI.setVisible(true);
    }
}
class ServerConfigurationInfoDialog extends JDialog implements ActionListener
{
    private ServerConfigurationInfoPanel serverConfigurationInfoPanel;
    private ServerConfigurationInfo serverConfigurationInfo;
    
    public ServerConfigurationInfoDialog(JFrame frame, BUserInterface userinterface, boolean supportUPNP)
    {
        super(frame, true);
        setTitle("Bakutan ServerInfoConfigurationInfoDialog");
        setSize(400,300);
        this.serverConfigurationInfoPanel = new ServerConfigurationInfoPanel(userinterface , supportUPNP);
        makeLayout();
    }
    private void makeLayout()
    {
        JButton buttonOK = this.serverConfigurationInfoPanel.getButtonOK();
        JButton buttonCancel = this.serverConfigurationInfoPanel.getButtonCancel();
        buttonOK.setActionCommand("ok");
        buttonCancel.setActionCommand("cancel");
        buttonOK.addActionListener(this);
        buttonCancel.addActionListener(this);
        
        getContentPane().add(serverConfigurationInfoPanel);
    }
    
    public void actionPerformed(ActionEvent e)
    {
        if (e.getActionCommand().equals("ok"))
        {
            ServerConfigurationInfo sci = serverConfigurationInfoPanel.getServerConfigurationInfo();
            if (sci != null)
            {
                this.serverConfigurationInfo = sci;
                setVisible(false);
            }
        }
        else if (e.getActionCommand().equals("cancel"))
        {
            this.serverConfigurationInfo = null;
            setVisible(false);
        }
    }
    public ServerConfigurationInfo inputServerConfigurationInfo()
    {
        setVisible(true);
        return serverConfigurationInfo;
    }
}
