/*
 * 作成日: 2006/12/15
 */
package bakutan.server; 

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;

import bakutan.common.DataManager;
import bakutan.common.util.SHA1Tool;


public class DBUserDataHolder implements UserDataHolder
{
    public static final String TABLE_USERS = "users";
    private DataBaseInfo dataBaseInfo;
    private int guestID = -1;
    public String tableScore;
    public String tableRelation;
    
    public DBUserDataHolder(DataBaseInfo dbi)
    {
        if (dbi == null)
            throw new NullPointerException();
        tableScore = dbi.getTablePrefix() + "_score";
        tableRelation = dbi.getTablePrefix() + "_relation";
        dataBaseInfo = dbi;
    }
    public boolean initialize()
    {
        try
        {
            Class.forName(dataBaseInfo.getDriver());
        }
        catch(ClassNotFoundException e)
        {
            throw new RuntimeException("DataBase Driver not found:" + dataBaseInfo.getDriver());
        }
        Connection conn = null;
        Statement stm = null;
        ResultSet rs = null;
        try
        {
            String str = dataBaseInfo.createDSN();
            conn = DriverManager.getConnection(str);
            stm = conn.createStatement();
            
            try
            {
                boolean f = stm.execute("CREATE TABLE "+
                        TABLE_USERS+"(id int NOT NULL AUTO_INCREMENT PRIMARY KEY, " +
                        "username CHAR(" + DataManager.USERNAME_LENGTH + ") NOT NULL UNIQUE, "+
                        "password CHAR(40) NOT NULL, " +
                        "nickname CHAR(" + DataManager.NICKNAME_LENGTH + ") NOT NULL UNIQUE, "+
                        //"score int NOT NULL DEFAULT 0, " +
                        "lastlogin DATETIME NOT NULL, " +
                        "gamecount int NOT NULL DEFAULT 0, " +
                        "options int NOT NULL DEFAULT 0)");
                stm.executeUpdate("INSERT IGNORE INTO "+TABLE_USERS+"(username,nickname,password) values ('guest','" + "guest" + "','guest')");
                System.out.println("table "+TABLE_USERS+" is created.");
            }
            catch(SQLException e)
            {
                //e.printStackTrace();
            }
            try
            {
                boolean f = stm.execute("CREATE TABLE " +
                        tableRelation+"(PRIMARY KEY(id,target), UNIQUE(id,target), " +
                        "id int NOT NULL, " +
                        "target int NOT NULL, " +
                        "score int NOT NULL DEFAULT 0)");
                System.out.println("table "+tableRelation+" is created.");
            }
            catch(SQLException e)
            {
                //e.printStackTrace();
            }
            try
            {
                boolean f = stm.execute("CREATE TABLE " +
                        tableScore+"("+
                        "id int NOT NULL PRIMARY KEY, " +
                        "gain int NOT NULL DEFAULT 0, " +
                        "lost int NOT NULL DEFAULT 0)");
                System.out.println("table "+tableScore+" is created.");
            }
            catch(SQLException e)
            {
                //e.printStackTrace();
            }
            try
            {
                rs = stm.executeQuery("SELECT id FROM "+TABLE_USERS+" WHERE username='guest'");
                if(rs.next())
                {
                    guestID = rs.getInt("id");
                }
                rs.close();
            }
            catch(SQLException e)
            {
            }
            if(guestID == -1)
            {
                try
                {
                    stm.executeUpdate("INSERT INTO "+TABLE_USERS+"(username,nickname,password) values ('guest','guest','guest')");
                    rs = stm.executeQuery("SELECT id FROM "+TABLE_USERS+" WHERE username='guest'");
                    if(rs.next())
                    {
                        guestID = rs.getInt("id");
                    }
                    rs.close();
                }
                catch(SQLException e)
                {
                }
            }
            if (guestID == -1)
            {
                throw new RuntimeException("can not create guest.");
            }
        }
        catch(SQLException e)
        {
            e.printStackTrace();
        }
        try
        {
            if (stm != null)
                stm.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (conn != null)
                conn.close();
        }
        catch(SQLException e2)
        {
        }
        return true;
    }
    
//    public static boolean isNeedSanitize(String arg)
//    {
//        return (arg.indexOf('%')!=-1 && arg.indexOf('_')!=-1);
//    }
//    public static String sanitize(String arg)
//    {
//        return arg.replace('%','-').replace('_','-');
//    }
    public UserInfo authorize(String username, String pass)
    {
        if (!username.matches("\\p{ASCII}+")
                || !pass.matches("\\p{ASCII}+"))
        {
            return null;
        }

        Connection conn = null;
        PreparedStatement pstm = null;
        ResultSet rs = null;
        UserInfo ui = null;
        try
        {
            String str = dataBaseInfo.createDSN();
            conn = DriverManager.getConnection(str);
            
            pstm = conn.prepareStatement("SELECT id,nickname FROM "+TABLE_USERS+" WHERE username=? AND password=?");
            pstm.setString(1,username);
            pstm.setString(2,SHA1Tool.makeHashString(pass));
            rs = pstm.executeQuery();
            if (rs.next())
            {
                ui = new UserInfo(rs.getInt("id"), rs.getString("nickname"));
            }
            else
            {
                return null;
            }
            pstm.close();
            pstm = conn.prepareStatement("UPDATE "+TABLE_USERS+" set lastlogin=?,gamecount=gamecount+1 WHERE id=" + ui.getDatabaseID());
            pstm.setString(1, getDate());
            pstm.executeUpdate();
            pstm.close();
        }
        catch(SQLException e)
        {
            e.printStackTrace();
        }
        try
        {
            if (rs != null)
                rs.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (pstm != null)
                pstm.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (conn != null)
                conn.close();
        }
        catch(SQLException e2)
        {
        }
        return ui;
    }
    public UserInfo checkGuest(String nickname)
    {
        boolean ret = false;;
        Connection conn = null;
        PreparedStatement pstm = null;
        ResultSet rs = null;
        UserInfo ui = null;
        try
        {
            String str = dataBaseInfo.createDSN();
            conn = DriverManager.getConnection(str);
            
            pstm = conn.prepareStatement("SELECT id FROM "+TABLE_USERS+" WHERE nickname=?");
            pstm.setString(1,nickname);
            rs = pstm.executeQuery();
            ret = !rs.next();
        }
        catch(SQLException e)
        {
            e.printStackTrace();
        }
        try
        {
            if (rs != null)
                rs.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (pstm != null)
                pstm.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (conn != null)
                conn.close();
        }
        catch(SQLException e2)
        {
        }
        return new UserInfo(guestID, "[G]"+nickname);
    }
    public boolean addUser(String username, String nickname, String pass)
    {
        if (!username.matches("\\p{ASCII}+")
                || username.length() > 40
                || !pass.matches("\\p{ASCII}+"))
        {
            return false;
        }
        if (nickname.length() == 0)
            return false;
        
        Connection conn = null;
        boolean ret = false;;
        PreparedStatement pstm = null;
        ResultSet rs = null;
        try
        {
            String str = dataBaseInfo.createDSN();
            conn = DriverManager.getConnection(str);
            
            pstm = conn.prepareStatement("SELECT id FROM "+TABLE_USERS+" WHERE username=? OR nickname=?");
            pstm.setString(1,username);
            pstm.setString(2,nickname);
            rs = pstm.executeQuery();
            boolean exist = rs.next();
            rs.close();
            pstm.close();
            if (!exist)
            {
                pstm = conn.prepareStatement("INSERT INTO "+TABLE_USERS+"(username,nickname,password) values (?,?,?)");
                pstm.setString(1,username);
                pstm.setString(2,nickname);
                pstm.setString(3,SHA1Tool.makeHashString(pass));
                ret = (pstm.executeUpdate() > 0);
            }
        }
        catch(SQLException e)
        {
            e.printStackTrace();
        }
        try
        {
            if (rs != null)
                rs.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (pstm != null)
                pstm.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (conn != null)
                conn.close();
        }
        catch(SQLException e2)
        {
        }
        return ret;
    }
    
    public boolean updateScore(int accounID, int total, HashMap<Integer, ScorePack> scoreMap)
    {
        //System.out.print(accounID);
        Iterator<Entry<Integer, ScorePack>> ite = scoreMap.entrySet().iterator();
        Connection conn = null;
        PreparedStatement pstm = null;
        ResultSet rs = null;
        String str = dataBaseInfo.createDSN();
        int gain = 0,lost = 0;
        boolean succeed = true;
        try
        {
            conn = DriverManager.getConnection(str);
            while(ite.hasNext())
            {
                Entry<Integer, ScorePack> entry = ite.next();
                if (entry.getValue().getScore() == 0)
                    continue;
                int s = entry.getValue().getScore();
                if (s >= 0)
                    gain += s;
                else
                    lost -= s;
                pstm = conn.prepareStatement("UPDATE "+tableRelation+" set score=score+? WHERE id=? AND target=?");
                pstm.setInt(1,entry.getValue().getScore());
                pstm.setInt(2,accounID);
                pstm.setInt(3,entry.getKey());
                boolean fail = (pstm.executeUpdate()==0);
                pstm.close();
                if (fail)
                {
                    try
                    {
                        pstm = conn.prepareStatement("INSERT INTO "+tableRelation+"(id,target,score) values(?,?,?)");
                        pstm.setInt(1,accounID);
                        pstm.setInt(2,entry.getKey());
                        pstm.setInt(3,entry.getValue().getScore());
                        fail = (pstm.executeUpdate()==0);
                        pstm.close();
                        if (fail)
                        {
                            System.out.println("INSERT failed:relation"+accounID+"<"+entry.getKey()+","+entry.getValue().getScore()+">");
                        }
                    }
                    catch(SQLException e)
                    {
                        e.printStackTrace();
                    }
                }
            }
            pstm = conn.prepareStatement("UPDATE "+tableScore+" set gain=gain+?,lost=lost+? WHERE id=?");
            pstm.setInt(1,gain);
            pstm.setInt(2,lost);
            pstm.setInt(3,accounID);
            boolean fail = (pstm.executeUpdate()==0);
            pstm.close();
            if (fail)
            {
                try
                {
                    pstm = conn.prepareStatement("INSERT INTO "+tableScore+"(id,gain,lost) values(?,?,?)");
                    pstm.setInt(1,accounID);
                    pstm.setInt(2,gain);
                    pstm.setInt(3,lost);
                    fail = (pstm.executeUpdate()==0);
                    pstm.close();
                    if (fail)
                    {
                        System.out.println("INSERT failed:score"+accounID+"<"+gain+","+total+">");
                    }
                }
                catch(SQLException e)
                {
                    e.printStackTrace();
                }
            }
        }
        catch(SQLException e)
        {
            e.printStackTrace();
            succeed = false;
        }
        try
        {
            if (pstm != null)
                pstm.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (conn != null)
                conn.close();
        }
        catch(SQLException e2)
        {
        }
        return succeed;
    }
    private String getDate()
    {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return format.format(new Date());
    }
    public int getGuestID()
    {
        return guestID;
    }
    
}
