/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/01/19
 *
 */
package bakutan.server;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.util.HashMap;

import bakutan.common.Body;
import bakutan.common.DataManager;
import bakutan.common.GameOption;
import bakutan.common.LoginInfo;
import bakutan.common.NetworkThread;
import bakutan.common.NullUserInterface;
import bakutan.common.event.BodyActionEvent;
import bakutan.common.event.BodyCommandEvent;
import bakutan.common.event.BodyEvent;
import bakutan.common.event.BodyKnock;
import bakutan.common.event.BodyMoveEvent;
import bakutan.common.event.BodyPushEvent;
import bakutan.common.event.GameRestartEvent;
import bakutan.common.event.StageEvent;
import bakutan.common.util.BakutanClock;
import bakutan.common.util.CompressedInputStream;
import bakutan.common.util.CompressedOutputStream;
import bakutan.common.util.DataFlusher;
import bakutan.common.util.ImplementDataFlusher;
import bakutan.common.util.NullDataFlusher;
import bakutan.common.util.PipedStream;

/**
 * @author cattaka
 *
 */
public class RemoteClient implements Runnable
{
    public static final int STATE_NEW = 0;
    public static final int STATE_AUTHORIZING = 1;
    public static final int STATE_READY = 2;
    public static final int STATE_GAME = 3;
    public static final int STATE_DISCONNECTED = 4;
    public static final int MODE_CLIENT = 0;
    public static final int MODE_DIRECT = 2;
	float rag = 0;
	long lastMove = 0;
	int state;

    private Thread thread;
	private UserInfo userInfo = null;
	private DataInputStream in;
	private DataOutputStream out;
	private Socket socket;
	private Body myBody;

    private BakutanServer server;
    private BakutanServer newServer;
	private boolean goflag;
	private DataFlusher dataFlusher;
	private UserDataHolder userDataHolder;
    private int accountID = 1;
    private HashMap<Integer, ScorePack> scoreMap = new HashMap<Integer, ScorePack>();
    private DataManager dataManager;
    private int mode = -1;
    
	public RemoteClient(DataManager dataManager, Socket _socket, UserDataHolder userDataHolder)
	{
        this.thread = new Thread(this,"RemoteClient Reader Thread");
        //this.thread.setPriority(Thread.NORM_PRIORITY+1);
        this.dataManager = dataManager;
		this.socket = _socket;
        this.userDataHolder = userDataHolder;
        this.state = STATE_NEW;
        this.mode = MODE_CLIENT;
		try
		{
			in = new DataInputStream(new CompressedInputStream(socket.getInputStream(),32768));
			out = new DataOutputStream(new CompressedOutputStream(socket.getOutputStream(),32768));
		}catch(IOException e)
		{
			try
			{
				socket.close();
			}catch(IOException ex)
			{
			}
			return;
		}
		goflag = true;
		dataFlusher = new ImplementDataFlusher("RemoteClient Writer Thread", out)
		{
			public void afterStop()
			{
				super.afterStop();
				try
				{
					socket.close();
				}catch(IOException e)
				{
					e.printStackTrace();
				}
			}
		};
		dataFlusher.start();
	}
    public RemoteClient(DataManager dataManager, GameOption go, LoginInfo li, UserDataHolder userDataHolder)
    {
        this.thread = new Thread(this,"RemoteClient Reader Thread");
        //this.thread.setPriority(Thread.NORM_PRIORITY+1);
        this.dataManager = dataManager;
        this.userDataHolder = userDataHolder;
        this.state = STATE_NEW;
        this.mode = MODE_DIRECT;
        this.userInfo = new UserInfo(userDataHolder.getGuestID(), li.getNickname());
        
        PipedStream ctor = new PipedStream(2<<14);
        PipedStream rtoc = new PipedStream(2<<14);
        in = new DataInputStream(ctor.getIn());
        out = new DataOutputStream(rtoc.getOut());
        NetworkThread nt = new NetworkThread(new NullUserInterface(), go);
        nt.startSession(rtoc.getIn(), ctor.getOut(), li);

//        PipedInputStream pin = new PipedInputStream()
//        {
//            int count = 0;
//            @Override
//            public synchronized int read() throws IOException
//            {
//                if (count == 59)
//                    System.out.println(">"+(count++)+":");
//                else
//                    count++;
//                int b = super.read();
//                return b;
//            }
//        };
//        PipedOutputStream pout = new PipedOutputStream();
//        try
//        {
//            in = new DataInputStream(new PipedInputStream(pout));
//            out = new DataOutputStream(new PipedOutputStream(pin));
//        }
//        catch(IOException e)
//        {
//            e.printStackTrace();
//        }
//        NetworkThread nt = new NetworkThread(new NullUserInterface(), go);
//        nt.startSession(pin, pout, li);
        
        goflag = true;
        dataFlusher = new NullDataFlusher(out)
        {
            @Override
            public void afterStop()
            {
                super.afterStop();
                try
                {
                    out.close();
                }
                catch(IOException e)
                {
                }
                try
                {
                    in.close();
                }
                catch(IOException e)
                {
                }
            }
        };
        //dataFlusher = new ImplementDataFlusher(out);
        dataFlusher.start();
    }
    
    
    public void start()
    {
        this.thread.start();
    }
	public void run()
	{
        this.state = STATE_AUTHORIZING;
        byte connectmode;
        try
        {
            out.writeUTF(DataManager.getCurrentProtocolVersion());
            out.flush();
            
            connectmode = in.readByte();
            if (connectmode != DataManager.REQUEST_LOGIN && connectmode != DataManager.REQUEST_BSTAGESTATE && connectmode != DataManager.REQUEST_SMALLBSTAGESTATE)
            {
                throw new IOException();
            }
        }catch(IOException e)
        {
            dispose();
            this.state = STATE_DISCONNECTED;
            return;
        }
        
		if (connectmode == DataManager.REQUEST_LOGIN)
		{
            try
            {
                byte loginmode = in.readByte();
                if (loginmode == DataManager.LOGINMODE_GUEST)
                {
                    String nickname = in.readUTF();
                    if (nickname.length() == 0 || nickname.length() > DataManager.NICKNAME_LENGTH)
                    {
                        out.writeByte(DataManager.AUTH_FAILED_INVALID);
                        out.flush();
                        throw new IOException();
                    }
                    else
                    {
                        ;
                        if ((userInfo = userDataHolder.checkGuest(nickname)) == null)
                        {
                            out.writeByte(DataManager.AUTH_FAILED_INVALID);
                            out.flush();
                            throw new IOException();
                        }
                    }
                    userInfo.setDatabaseID(userDataHolder.getGuestID());
                }
                else if (loginmode == DataManager.LOGINMODE_USER || loginmode == DataManager.LOGINMODE_NEWUSER)
                {
                    String username;
                    String password;
                    String nickname;
                    if (loginmode == DataManager.LOGINMODE_NEWUSER)
                    {
                        username = in.readUTF();
                        password = in.readUTF();
                        nickname = in.readUTF();
                        if (!userDataHolder.addUser(username,nickname,password))
                        {
                            out.writeByte(DataManager.AUTH_FAILED_INVALID);
                            out.flush();
                            throw new IOException();
                        }
                    }
                    else
                    {
                        username = in.readUTF();
                        password = in.readUTF();
                    }
                    if (username.length() > 0
                            && username.length() <= DataManager.USERNAME_LENGTH
                            && password.length() > 0
                            && password.length() <= DataManager.PASSWORD_LENGTH)
                    {
                        if ((userInfo = userDataHolder.authorize(username,password)) == null)
                        {
                            out.writeByte(DataManager.AUTH_FAILED);
                            out.flush();
                            throw new IOException();
                        }
                    }
                    else
                    {
                        out.writeByte(DataManager.AUTH_FAILED_INVALID);
                        out.flush();
                        throw new IOException();
                    }
                }
                
                if(userInfo == null)
                {
                    throw new IOException();
                }
                
                accountID = userInfo.getDatabaseID();
            }catch(IOException e)
            {
                //e.printStackTrace();
                try
                {
                    socket.close();
                }catch(IOException ex)
                {
                }
                this.state = STATE_DISCONNECTED;
                return;
            }
			
            //Start session
            outer:while(goflag)
            {
                this.state = STATE_READY;
                synchronized (this)
                {
                    if (newServer == null)
                    {
                        try
                        {
                            this.wait();
                        }catch(InterruptedException e)
                        {
                        }
                        if (newServer == null)
                        {
                            break outer;
                        }
                    }
                    server = newServer;
                    newServer = null;
                }
                this.state = STATE_GAME;
                long l;
                l = BakutanClock.getCurrenttime();
                StageEvent se = new StageEvent(new Body[0], new BodyClientAddEvent(this, BodyClientAddEvent.REQUEST_LOGIN),l,l);
                putStageEvent(se);
                try
                {
                    long lastReceive = l;
                    while(goflag)
                    {
                        BodyEvent be = server.inputBodyEvent(in);
                        l = BakutanClock.getCurrenttime();
                        if ( l == lastReceive)
                        {
                            l++;
                        }
                        lastReceive = l;

                        if(be instanceof GameRestartEvent)
                        {
                            break;
                        }

                        if(be instanceof BodyActionEvent)
                        {
                            BodyActionEvent bae = (BodyActionEvent)be;
                            BodyClientActionEvent bcae = new BodyClientActionEvent(bae, this);
                            putBodyEvent(bcae, l);
                        }
                        else if (be instanceof BodyPushEvent
                                || be instanceof BodyCommandEvent)
                        {
                            putBodyEvent(be, l);
                        }
                        else if(be instanceof BodyMoveEvent)
                        {
                            BodyMoveEvent bme = (BodyMoveEvent)be;
                            BodyClientMoveEvent bcme = new BodyClientMoveEvent(bme, this);
                            putBodyEvent(bcme, l);
                        }
                        else if(be instanceof BodyKnock)
                        {
                            putBodyEvent(be, l);
                        }
                        else
                        {
                            throw new IOException("restrect BodyEvent.");
                        }
                    }
                }catch(IOException e)
                {
                    goflag = false;
                }
                l = BakutanClock.getCurrenttime();
                StageEvent sse = new StageEvent(new Body[0], new BodyClientRemoveEvent(this),l,l);
                putStageEvent(sse);
                synchronized (this)
                {
                    if (myBody != null)
                    {
                        userDataHolder.updateScore(accountID, myBody.getScore(), scoreMap);
                    }
                }
            }
		}else if (connectmode == DataManager.REQUEST_BSTAGESTATE || connectmode == DataManager.REQUEST_SMALLBSTAGESTATE)
		{
            synchronized (this)
            {
                if (newServer == null)
                {
                    try
                    {
                        this.wait();
                    }catch(InterruptedException e)
                    {
                    }
                }
                server = newServer;
                newServer = null;
            }
			long l = BakutanClock.getCurrenttime();
            if (connectmode == DataManager.REQUEST_BSTAGESTATE)
            {
                StageEvent se = new StageEvent(new Body[0], new BodyClientAddEvent(this, BodyClientAddEvent.REQUEST_STATE),l,l);
                putStageEvent(se);
            }
            else
            {
                StageEvent se = new StageEvent(new Body[0], new BodyClientAddEvent(this, BodyClientAddEvent.REQUEST_SMALLSTATE),l,l);
                putStageEvent(se);
            }
			int i;
			try
			{
				while(goflag && (i = in.read()) != -1)
				{
				}
			}catch(IOException e)
			{
			}
		}
        
        this.state = STATE_DISCONNECTED;
        dispose();
	}
    
    public void removeServer()
    {
        synchronized (this)
        {
            this.server = null;
        }
    }
    public void joinGame(BakutanServer newserver)
    {
        synchronized (this)
        {
            this.newServer = newserver;
            this.notify();
        }
    }
    public void setBody(Body body)
    {
        synchronized (this)
        {
            this.myBody = body;
        }
    }
    public void sendPacket(byte[] packet)
	{
		if (packet == null)
		{
			throw new NullPointerException();
		}
		if (goflag)
		{
			dataFlusher.sendPacket(packet);
		}
	}
    public void sendPacket(byte[] packet, boolean flush)
    {
        if (packet == null)
        {
            throw new NullPointerException();
        }
        if (goflag)
        {
            dataFlusher.sendPacket(packet, flush);
        }
    }
	public Body getBody()
	{
        synchronized (this)
        {
            return myBody;
        }
	}
	public void dispose()
	{
		goflag = false;
		try
		{
			in.close();
		}catch(IOException e)
		{
		}
        //socket.close() is called by dataFlusher
        dataFlusher.dispose();
        dataFlusher.waitForStop();
	}
	public String getInetAddress()
	{
        if (socket != null)
            return socket.getInetAddress().toString();
        else
            return "localhost(bot)";
	}
    public void putScore(int fromAccountID, int change)
    {
        ScorePack sp = scoreMap.get(fromAccountID);
        if (sp == null)
        {
            sp = new ScorePack();
            scoreMap.put(fromAccountID,sp);
        }
        sp.addScore(change);
    }
	public int getAccountID()
    {
        return accountID;
    }
    private void putStageEvent(StageEvent be)
    {
        synchronized (this)
        {
            if (server != null)
            {
                server.putStageEvent(be);
            }
        }
    }
    private void putBodyEvent(BodyEvent be, long currenttime)
    {
        synchronized (this)
        {
            if (myBody == null || server == null)
                return;
            StageEvent se = new StageEvent(new Body[]{myBody}, be, currenttime, currenttime);
            server.putStageEvent(se);
        }
    }
    public int getRemoteClientState()
    {
        return state;
    }
    public UserInfo getUserInfo()
    {
        return userInfo;
    }
    
}
