/*
 * 作成日: 2007/02/13
 */
package bakutan.server;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JButton;
import javax.swing.JPanel;

import bakutan.common.BUserInterface;
import bakutan.common.BotInfo;
import bakutan.common.DataManager;
import bakutan.common.LoginInfo;
import bakutan.common.util.datainputpanel.DIPInfo;
import bakutan.common.util.datainputpanel.DIPInfoBoolean;
import bakutan.common.util.datainputpanel.DIPInfoFile;
import bakutan.common.util.datainputpanel.DIPInfoInteger;
import bakutan.common.util.datainputpanel.DIPInfoSelect;
import bakutan.common.util.datainputpanel.DIPInfoString;
import bakutan.common.util.datainputpanel.DIPInfoSwitch;
import bakutan.common.util.datainputpanel.DataInputPanel;
import bakutan.game.capturetheflag.BStageCTF;
import bakutan.game.deathmatch.BStageDM;

public class ServerConfigurationInfoPanel extends JPanel
{
    private DataInputPanel dataInputPanel;
    private DIPInfoSelect dipGame;
    private DIPInfoSwitch dipRegister;
    private DIPInfoBoolean dipUPNP;
    private DIPInfoString dipLabel;
    private DIPInfoInteger dipPort;
    private DIPInfoString dipLoginMessage;
    private DIPInfoInteger dipTimelimit;
    private DIPInfoInteger dipMaxClients;
    private DIPInfoInteger dipBotNum;
    private DIPInfoSwitch dipUseOuterMapFile;
    private DIPInfoSelect dipMapFile;
    private DIPInfoFile dipOuterMapFile;

    private JButton buttonOK;
    private JButton buttonCancel;

    private BUserInterface userInterface;
    private ServerConfigurationInfo serverConfigurationInfo = null;
    private String[] mapfiles = new String[]{"bakutan_std1.map","bakutan_std2.map","bakutan_clear1.map","bakutan_clear2.map", "bakutan_ctf1.map"};
    private boolean supportUPNP = false;
    
    public ServerConfigurationInfoPanel(BUserInterface userinterface, boolean supportUPNP)
    {
        super();
        this.userInterface = userinterface;
        this.supportUPNP = supportUPNP;
        makeLayout();
    }
    private void makeLayout()
    {
        dipUPNP = new DIPInfoBoolean(userInterface.translate("use_upnp"), false);
        dipRegister = new DIPInfoSwitch(userInterface.translate("register_to_server_list"), false, new DIPInfo[]{dipUPNP}, new DIPInfo[0]);
        dipLabel = new DIPInfoString(userInterface.translate("label"), "Bakutan Server");
        dipPort = new DIPInfoInteger(userInterface.translate("port_0_is_auto"), "0");
        dipLoginMessage = new DIPInfoString(userInterface.translate("loginmessage"), "Welcome to Bakutan");
        dipTimelimit = new DIPInfoInteger(userInterface.translate("timelimit_minute"), String.valueOf(DataManager.DEFAULT_TIMELIMIT/60000l));
        dipMaxClients = new DIPInfoInteger(userInterface.translate("max_clients"), String.valueOf(DataManager.DEFAULT_MAXCLIENTS));
        dipBotNum = new DIPInfoInteger(userInterface.translate("number_of_bots"), "0");
        dipMapFile = new DIPInfoSelect(userInterface.translate("map_file"),mapfiles,0);
        dipOuterMapFile = new DIPInfoFile(userInterface.translate("map_file"), "", DIPInfoFile.MODE_OPEN);
        dipUseOuterMapFile = new DIPInfoSwitch(userInterface.translate("use_outer_mapfile"), false, new DIPInfo[]{dipOuterMapFile}, new DIPInfo[]{dipMapFile});
        dipGame = new DIPInfoSelect(userInterface.translate("game"), new String[]{"DeathMatch","CTF"}, 0);
        
        DIPInfo[] dipis;
        if (supportUPNP)
        {
            dipis = new DIPInfo[]{
                    dipGame,
                    dipLabel, dipPort, dipLoginMessage, dipTimelimit, dipMaxClients, dipBotNum,
                    dipMapFile, dipUseOuterMapFile, dipOuterMapFile,
                    dipRegister, dipUPNP
            };
        }
        else
        {
            dipis = new DIPInfo[]{
                    dipGame,
                    dipLabel, dipPort, dipLoginMessage, dipTimelimit, dipMaxClients, dipBotNum,
                    dipMapFile, dipUseOuterMapFile, dipOuterMapFile
            };
        }
        dataInputPanel = new DataInputPanel(dipis);
        dataInputPanel.setShowButtons(false);
        
        buttonOK = new JButton(userInterface.translate("ok"));
        buttonCancel = new JButton(userInterface.translate("cancel"));
        
        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints(); 
        
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridwidth = 2;
        gbc.weightx = 1;
        gbc.weighty = 1;
        gbl.setConstraints(dataInputPanel, gbc);
        
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0;
        gbc.weighty = 0;
        gbc.gridwidth = 1;
        gbc.gridy++;
        gbl.setConstraints(buttonOK, gbc);
        gbc.gridx++;
        gbl.setConstraints(buttonCancel, gbc);
        
        setLayout(gbl);
        add(buttonOK);
        add(buttonCancel);
        add(dataInputPanel);
    }
    public ServerConfigurationInfo getServerConfigurationInfo()
    {
        if (!dataInputPanel.isValidData())
        {
            dataInputPanel.validateData();
            return null;
        }
        String loginMessage = (String)dipLoginMessage.getValue();;
        String lable = (String)dipLabel.getValue();
        int port = (Integer)dipPort.getValue();
        int timelimit = (Integer)dipTimelimit.getValue();;
        int maxclients = (Integer)dipMaxClients.getValue();
        int botNum = (Integer)dipBotNum.getValue();
        int selectedGame = (Integer)dipGame.getValue();
        String game = BStageDM.class.getName();
        if (selectedGame == 1)
            game = BStageCTF.class.getName();
        
        
        BStageInfo bstageInfo = new BStageInfo(game, null, port, "label", loginMessage, false);
        for (int i=0;i<botNum;i++)
        {
            BotInfo bo = new BotInfo();
            LoginInfo li = new LoginInfo();
            li.setMode(LoginInfo.MODE_GUEST);
            li.setNickname("Bot:"+i);
            bo.setLoginInfo(li);
            bstageInfo.getBotInfos().add(bo);
        }
        
        if ((Boolean)dipUseOuterMapFile.getValue())
        {
            bstageInfo.setMapFileName((String)dipOuterMapFile.getValue());
        }
        else
        {
            int index = (Integer)dipMapFile.getValue();
            String mapFile = mapfiles[index];
            bstageInfo.setMapFileName(mapFile);
        }
        bstageInfo.setLabel(lable);
        bstageInfo.setTimeLimit(timelimit*60000l);
        bstageInfo.setMaxClients(maxclients);
        ServerConfigurationInfo sci = new ServerConfigurationInfo();
        sci.setBStageInfo(bstageInfo);

        if ((Boolean)dipRegister.getValue())
        {
            sci.setRegisterServer(true);
            sci.setUseUPNP((Boolean)dipUPNP.getValue());
        }
        return sci;
    }
    public JButton getButtonCancel()
    {
        return buttonCancel;
    }
    public JButton getButtonOK()
    {
        return buttonOK;
    }
    
}