/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/07
 */
package bakutan.server;

//import bakusen.common.event.BodyStageActionEvent;
import bakutan.common.Body;
import bakutan.common.DataManager;
import bakutan.common.EventHandler;
import bakutan.common.EventThread;
import bakutan.common.event.BodyActionEvent;
import bakutan.common.event.BodyAddEvent;
import bakutan.common.event.BodyDeleteEvent;
import bakutan.common.event.BodyArmorEvent;
import bakutan.common.event.BodyMessageEvent;
import bakutan.common.event.BodyMoveEvent;
import bakutan.common.event.BodyMultiAddEvent;
import bakutan.common.event.BodyOptionEvent;
import bakutan.common.event.BodyRecheckIntersect;
import bakutan.common.event.BodyScoreChangeEvent;
import bakutan.common.event.BodyScoreEvent;
import bakutan.common.event.BodySetPotisionEvent;
import bakutan.common.event.BodySmallActionEvent;
import bakutan.common.event.BodySoundEvent;
import bakutan.common.event.BodyStageMessageEvent;
import bakutan.common.event.BodyStateEvent;
import bakutan.common.event.StageEvent;
import bakutan.common.math.BVector2f;

/**
 * @author cattaka
 */
public class ServerEventHandler implements EventHandler
{
	private EventThread eventThread = new EventThread();
	public ServerEventHandler()
	{
	}
	
	public EventThread getEventThread()
	{
		return eventThread;
	}
	
	public void add(long currentTime, long generationTime, Body target)
	{
		StageEvent se = new StageEvent(new Body[0], new BodyAddEvent(target), currentTime, generationTime);
		eventThread.putStageEvent(se);
	}
    public void multiAdd(long currentTime, long generationTime, Body[] targets)
    {
        StageEvent se = new StageEvent(new Body[0], new BodyMultiAddEvent(targets), currentTime, generationTime);
        eventThread.putStageEvent(se);
    }
	public void delete(long currentTime, long generationTime, Body target)
	{
		StageEvent se = new StageEvent(new Body[]{target}, new BodyDeleteEvent(), currentTime, generationTime);
		eventThread.putStageEvent(se);
	}
	public void armor(long currentTime, long generationTime, Body target, Body from, byte type, int armor)
	{
		StageEvent se = new StageEvent(new Body[]{target, from},new BodyArmorEvent(type, armor),currentTime, generationTime);
		eventThread.putStageEvent(se);
	}
	public void move(long currentTime, long generationTime, Body target, BVector2f position, BVector2f velocity, BVector2f direction)
	{
		StageEvent se = new StageEvent(new Body[]{target}, new BodyMoveEvent(position, velocity, direction), currentTime, generationTime);
		eventThread.putStageEvent(se);
	}
	public void setPosition(long currentTime, long generationTime, Body target, BVector2f position)
	{
		StageEvent se = new StageEvent(new Body[]{target}, new BodySetPotisionEvent(position), currentTime, generationTime);
		eventThread.putStageEvent(se);
	}
	public void action(long currentTime, long generationTime, Body target, BVector2f position, byte action)
	{
		StageEvent se = new StageEvent(new Body[]{target}, new BodyActionEvent(position,position,action), currentTime, generationTime);
		eventThread.putStageEvent(se);
	}
    public void action(long currentTime, long generationTime, Body target, BVector2f position,BVector2f actionPosition, byte action)
    {
        StageEvent se = new StageEvent(new Body[]{target}, new BodyActionEvent(position,actionPosition,action), currentTime, generationTime);
        eventThread.putStageEvent(se);
    }
    public void smallAction(long currentTime, long generationTime, Body target, byte action)
    {
        StageEvent se = new StageEvent(new Body[]{target}, new BodySmallActionEvent(action), currentTime, generationTime);
        eventThread.putStageEvent(se);
    }
	public void state(long currentTime, long generationTime, Body target, byte type, int state)
	{
		StageEvent se = new StageEvent(new Body[]{target}, new BodyStateEvent(type, state), currentTime, generationTime);
		eventThread.putStageEvent(se);
	}
	public void option(long currentTime, long generationTime, Body target, byte type, int option)
	{
		StageEvent se = new StageEvent(new Body[]{target}, new BodyOptionEvent(type, option), currentTime, generationTime);
		eventThread.putStageEvent(se);
	}
	public void sound(long currentTime, long generationTime, Body target, byte type)
	{
		StageEvent se = new StageEvent(new Body[]{target}, new BodySoundEvent(type), currentTime, generationTime);
		eventThread.putStageEvent(se);
	}
    public void sound(long currentTime, long generationTime, BVector2f pos, byte type)
    {
        StageEvent se = new StageEvent(new Body[]{}, new BodySoundEvent(type, pos), currentTime, generationTime);
        eventThread.putStageEvent(se);
    }
	public void sound(long currentTime, long generationTime, byte type)
	{
		StageEvent se = new StageEvent(new Body[]{}, new BodySoundEvent(type), currentTime, generationTime);
		eventThread.putStageEvent(se);
	}
	public void message(long currentTime, long generationTime, Body target, byte type, String message)
	{
		if (type == DataManager.MESSAGE_CHAT)
		{
			StageEvent se = new StageEvent(new Body[]{target}, new BodyMessageEvent(type, message), currentTime, generationTime);
			eventThread.putStageEvent(se);
		}else
		{
			StageEvent se = new StageEvent(new Body[0], new BodyMessageEvent(type, message), currentTime, generationTime);
			eventThread.putStageEvent(se);
		}
	}
	public void message(long currentTime, long generationTime,  byte type, String message)
	{
		if (type == DataManager.MESSAGE_CHAT)
		{
			throw new RuntimeException();
		}else
		{
			StageEvent se = new StageEvent(new Body[0], new BodyMessageEvent(type, message), currentTime, generationTime);
			eventThread.putStageEvent(se);
		}
	}
	public void score(long currentTime, long generationTime, Body[] targets, int[] scores)
	{
		if (targets.length != scores.length)
		{
			throw new ArrayIndexOutOfBoundsException("targets.length must equals to scores.length");
		}
		StageEvent se = new StageEvent(targets, new BodyScoreEvent(scores),currentTime, generationTime);
		eventThread.putStageEvent(se);
	}
    public void scoreChange(long currentTime, long generationTime, Body from, Body[] targets, int[] scores)
    {
        if (targets.length != scores.length)
        {
            throw new ArrayIndexOutOfBoundsException("targets.length must equals to scores.length");
        }
        Body[] bs2 = new Body[targets.length+1];
        bs2[0] = from;
        System.arraycopy(targets,0,bs2,1,targets.length);
        StageEvent se = new StageEvent(bs2, new BodyScoreChangeEvent(scores),currentTime, generationTime);
        eventThread.putStageEvent(se);
    }
	public void stageMessage(long currentTime, long generationTime, byte type, String message)
	{
		StageEvent se = new StageEvent(new Body[0], new BodyStageMessageEvent(type, message), currentTime, generationTime);
		eventThread.putStageEvent(se);
	}
    public void recheckIntersect(long currentTime, long generationTime, Body target)
    {
        StageEvent se = new StageEvent(new Body[]{target}, new BodyRecheckIntersect(), currentTime, generationTime);
        eventThread.putStageEvent(se);
    }
	/*
	public void stageAction(long currentTime, long generationTime, int action)
	{
		StageEvent se = new StageEvent(new Body[0], new BodyStageActionEvent(action), currentTime, generationTime);
		eventThread.putStageEvent(se);
	}
	*/
}