/*
 * 作成日: 2007/01/19
 */
package bakutan.server;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import bakutan.common.BUserInterface;
import bakutan.common.BotInfo;
import bakutan.common.DataManager;
import bakutan.common.GameOption;
import bakutan.common.ai.AIInterface;
import bakutan.common.serverinfo.ServerInfoManagerAddress;
import bakutan.common.serverinfo.SmallServerInfo;
import bakutan.server.util.NetworkTool;

public class ServerProcess extends Thread
{
    private BakutanServer server; 
    private UserDataHolder userDataHolder;
    private DataManager dataManager;
    private URL mapFileURL;
    private boolean goflag = false;
    private WaitingClientThread waitingClientThread;
    private BUserInterface userInterface;
    private ServerInfoManagerAddress serverInfoManagerAddress = null;
    private BStageInfo bstageInfo;
    private boolean registerServer = false;
    private boolean useUPNP = false;
    private NetworkTool networkTool;
    
    public ServerProcess(BUserInterface userInterface, NetworkTool networkTool)
    {
        this.userInterface = userInterface;
        this.networkTool = networkTool;
    }
    
    public ServerInfoManagerAddress getServerInfoManagerAddress()
    {
        return serverInfoManagerAddress;
    }

    public void setServerInfoManagerAddress(ServerInfoManagerAddress serverInfoManagerAddress)
    {
        this.serverInfoManagerAddress = serverInfoManagerAddress;
    }

    public void init(BStageInfo bstageInfo, UserDataHolder udh) throws IOException
    {
        URL mapFileURL;
        File mapFile = new File(bstageInfo.getMapFileName());
        if (mapFile.exists())
        {
            mapFileURL = mapFile.toURL();
        }
        else
        {
            mapFileURL = getClass().getClassLoader().getResource(bstageInfo.getMapFileName());
        }
        if (mapFileURL == null)
            throw new RuntimeException("map file not found.");
        init(bstageInfo, mapFileURL, udh);
    }
    
    public void init(BStageInfo bstageInfo, URL mapFileURL, UserDataHolder udh) throws IOException
    {
        this.bstageInfo = bstageInfo;
        this.userDataHolder = udh;
        this.dataManager = DataManager.getAvailableDataManager();
        this.mapFileURL = mapFileURL;
        
        setName("ServerProcess:" + bstageInfo.getBstageClassName());
        goflag = true;
        waitingClientThread = new WaitingClientThread(bstageInfo.getBstageClassName(), DataManager.DEFAULT_TIMEOUT, dataManager, userDataHolder);
        waitingClientThread.startWaitingClientThread(bstageInfo.getPort());
    }
    
    public void run()
    {
        if (!goflag)
            throw new RuntimeException("Not initiarized.");
        super.run();
        BotInfo[] bis = bstageInfo.getBotInfos().toArray(new BotInfo[bstageInfo.getBotInfos().size()]);
        RemoteClient[] rcs = new RemoteClient[bis.length];
        for(int i=0;i<bis.length;i++)
        {
            BotInfo bi = bis[i];
            GameOption gameOption = new GameOption();
            gameOption.setFpsLimit(bi.getFpsLimit());
            gameOption.setWorkAI(true);
            try
            {
                AIInterface aii;
                Class aiClass = Class.forName(bi.getAiClassName());
                aii = (AIInterface)aiClass.newInstance();
                gameOption.setAiInterface(aii);
            }
            catch(Exception e)
            {
            }
            
            RemoteClient rc = new RemoteClient(dataManager, gameOption, bi.getLoginInfo(), userDataHolder);
            rc.start();
            rcs[i] = rc;
        }
        
        SmallServerInfo ssi = new SmallServerInfo();
        ssi.setLabel(bstageInfo.getLabel());
        ssi.setHostName("");
        ssi.setPort(getPort());
        if (useUPNP && networkTool.isSupportUPNP())
        {
            int port = networkTool.addPortMapping(getPort(), userInterface);
            if (port != 0)
            {
                ssi.setPort(port);
            }
        }
        
        String noteMessage = null;
        boolean firstTime = true;
        
        while (goflag)
        {
            long sleep = System.currentTimeMillis();
            if (registerServer)
            {
                try
                {
                    DataManager.uploadSmallServerInfo(serverInfoManagerAddress.getInetAddress(), ssi, 5000);
                    userInterface.outputMessage("Server address is registered to ServerInfoManager.");
                }
                catch(IOException e)
                {
                    userInterface.outputMessage("Registering server address is failed : " + e.getMessage());
                    e.printStackTrace();
                }
            }
            sleep = System.currentTimeMillis() - sleep;
            sleep = 5000 - sleep;
            if (!firstTime && sleep > 0)
            {
                try
                {
                    Thread.sleep(sleep);
                }
                catch(InterruptedException e)
                {
                    e.printStackTrace();
                }
            }
            firstTime = false;
            System.gc();
            try
            {
                server = new BakutanServer(bstageInfo, mapFileURL, dataManager, userDataHolder, userInterface);
                if (noteMessage != null)
                    server.setNoteMessage(noteMessage);
                for (int i=0;i<rcs.length;i++)
                {
                    if (rcs[i].getRemoteClientState() != RemoteClient.STATE_DISCONNECTED)
                    {
                        server.joinRemoteClient(rcs[i]);
                    }
                }
                waitingClientThread.setServer(server);
                server.startServer();
                rcs = server.getRemoteClients();
                noteMessage = server.getNoteMessage();
            }
            catch(IOException e)
            {
                userInterface.outputMessage("error : " + e.getMessage());
                e.printStackTrace();
                break;
            }
        }
        if (useUPNP && networkTool.isSupportUPNP())
        {
            networkTool.deletePortMapping(ssi.getPort(), userInterface);
        }
        waitingClientThread.stopWaitingClientThread();
    }
    public void stopServer()
    {
        goflag = false;
        if (this.server != null)
            this.server.putGameSetEvent();
        try
        {
            this.join();
        }
        catch(InterruptedException e)
        {
        }
    }
    public int getPort()
    {
        return waitingClientThread.getPort();
    }

    public boolean isRegisterServer()
    {
        return registerServer;
    }

    public void setRegisterServer(boolean registerServer)
    {
        this.registerServer = registerServer;
    }

    public boolean isUseUPNP()
    {
        return useUPNP;
    }

    public void setUseUPNP(boolean useUPNP)
    {
        this.useUPNP = useUPNP;
    }
    
}

