/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/09
 */
package bakutan.server;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


import bakutan.common.BotInfo;
import bakutan.common.DataManager;
import bakutan.common.InvalidStructureException;
import bakutan.common.XMLConverter;
import bakutan.serverinfomanager.ServerInfoManagerInfo;

/**
 * @author cattaka
 */
public class ServerXMLConverter extends XMLConverter
{
	public static ServerConfigurationInfo createConfigurationInfo(Element element)
	throws InvalidStructureException
	{
		ServerConfigurationInfo sci = new ServerConfigurationInfo();
		if (!(element.getTagName().equals("ServerConfigurationInfo")))
			throw new InvalidStructureException("this is not ServerConfigurationInfo");
		NodeList nl = element.getElementsByTagName("BStageInfo");
		//for (int i=0;i<nl.getLength();i++)
        if (nl.getLength() > 0)
        {
			Element el = (Element)nl.item(0);
			sci.setBStageInfo(createBStageInfo(el));
		}
        else
        {
            throw new InvalidStructureException("BStageInfo is not found.");
        }
        
        nl = element.getElementsByTagName("DatabaseInfo");
        for (int i=0;i<nl.getLength();i++)
        {
            Element el = (Element)nl.item(0);
            if (el.getParentNode() != element)
                continue;
            sci.setDataBaseInfo(createDatabaseInfo(el));
            break;
        }

        nl = element.getElementsByTagName("ServerInfoManager");
        try
        {
            if (nl.getLength() > 0)
            {
                Element sim = (Element)nl.item(0);
                String arg;
                arg = sim.getAttribute("UseUPNP");
                sci.setUseUPNP(arg.toLowerCase().equals("true"));
                arg = sim.getAttribute("Register");
                sci.setRegisterServer(arg.toLowerCase().equals("true"));
                
                NodeList nl2 = element.getElementsByTagName("ServerInfoManagerAddress");
                try
                {
                    if (nl2.getLength() > 0)
                    {
                        Element sima = (Element)nl2.item(0);
                        sci.setServerInfoManagerAddress(createServerInfoManagerAddress(sima));
                    }
                }
                catch(Exception e)
                {
                    e.printStackTrace();
                }
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }

		return sci;
	}
	
	public static BStageInfo createBStageInfo(Element element)
	throws InvalidStructureException
	{
		if (!(element.getTagName().equals("BStageInfo")))
			throw new InvalidStructureException("this is not BStageInfo");

		String bstageClassName = element.getAttribute("BStageClassName");
		String mapFileName = element.getAttribute("MapFileName");
		String portStr = element.getAttribute("Port");
        String label = element.getAttribute("Label");
		String loginMessage = element.getAttribute("LoginMessage");
		int port;
		try
		{
			port = Integer.parseInt(portStr);
		}catch(NumberFormatException e)
		{
			port = DataManager.DEFAULT_PORT;
		}
		
		String tmp = element.getAttribute("CheckCheat");
		boolean cc = tmp.equals("true");
		
		BStageInfo bsi = new BStageInfo(bstageClassName, mapFileName, port, label, loginMessage, cc);
		
		tmp = element.getAttribute("TimeLimit");
		if (tmp.length() > 0)
		{
			try
			{
				long timelimit = Long.parseLong(tmp);
				timelimit *= 1000l;
				bsi.setTimeLimit(timelimit);
			}catch(NumberFormatException e)
			{
				e.printStackTrace();
			}
		}
		
		tmp = element.getAttribute("MaxClients");
		if (tmp.length() > 0)
		{
			try
			{
				int maxClients = Integer.parseInt(tmp);
				bsi.setMaxClients(maxClients);
			}catch(NumberFormatException e)
			{
				e.printStackTrace();
			}
		}
        
        NodeList nl = element.getElementsByTagName("BotInfo");
        for (int i=0;i<nl.getLength();i++)
        {
            Element botInfoElement = (Element)nl.item(i);
            BotInfo bi = creataBotInfo(botInfoElement);
            bsi.getBotInfos().add(bi);
        }
		
		return bsi;
	}
    public static DataBaseInfo createDatabaseInfo(Element element)
    throws InvalidStructureException
    {
        if (!(element.getTagName().equals("DatabaseInfo")))
            throw new InvalidStructureException("this is not DatabaseInfo");

        String driver = element.getAttribute("Driver");
        String db_program = element.getAttribute("DBProgram");
        String user = element.getAttribute("User");
        String password = element.getAttribute("Password");
        String hostname = element.getAttribute("Hostname");
        String database = element.getAttribute("Database");
        String charset = element.getAttribute("CharacterEncoding");
        String tablePrefix = element.getAttribute("TablePrefix");

        if (driver.length() == 0)
            throw new InvalidStructureException("Driver not found.");
        else if (driver.length() == 0)
            throw new InvalidStructureException("Driver not found.");
        else if (db_program.length() == 0)
            throw new InvalidStructureException("DBProgram not found.");
        else if (user.length() == 0)
            throw new InvalidStructureException("User not found.");
        else if (password.length() == 0)
            throw new InvalidStructureException("Password not found.");
        else if (hostname.length() == 0)
            throw new InvalidStructureException("Hostname not found.");
        else if (database.length() == 0)
            throw new InvalidStructureException("Database not found.");
        else if (tablePrefix.length() == 0)
            throw new InvalidStructureException("TablePrefix not found.");
        
        DataBaseInfo dbi = new DataBaseInfo(driver, db_program, user, password, hostname, database, charset, tablePrefix);
        return dbi;
    }
    public static ServerInfoManagerInfo createServerManagerInfo(Element element)
    throws InvalidStructureException
    {
        ServerInfoManagerInfo simi = new ServerInfoManagerInfo();
        if (!(element.getTagName().equals("ServerInfoManagerInfo")))
            throw new InvalidStructureException("this is not ServerInfoManagerInfo");
        String portStr = element.getAttribute("Port");
        int port;
        try
        {
            port = Integer.parseInt(portStr);
        }
        catch(NumberFormatException e)
        {
            throw new InvalidStructureException(e.getMessage());
        }
        simi.setPort(port);
        
        NodeList nl = element.getElementsByTagName("DatabaseInfo");
        for (int i=0;i<nl.getLength();i++)
        {
            Element el = (Element)nl.item(0);
            if (el.getParentNode() != element)
                continue;
            simi.setDataBaseInfo(createDatabaseInfo(el));
            break;
        }
        return simi;
    }
}
