/*
 * 作成日: 2007/01/22
 */
package bakutan.server;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import bakutan.common.DataManager;

public class WaitingClientThread extends Thread
{
    private BakutanServer server;
    private ServerSocket serverSocket;
    private int timeout;
    private int port;
    private UserDataHolder userDataHolder;
    private DataManager dataManager;;

    public WaitingClientThread(String game, int timeout, DataManager dataManager, UserDataHolder userDataHolder)
    {
        super("WaitingClientThread:" + game);
        this.timeout = timeout;
        this.dataManager = dataManager;
        this.userDataHolder = userDataHolder;
    }

    public synchronized void startWaitingClientThread(int port) throws IOException
    {
        if (serverSocket != null)
            throw new RuntimeException();
        serverSocket = new ServerSocket(port);
        this.start();
    }

    public void run()
    {
        while (true)
        {
            Socket socket;
            try
            {
                socket = serverSocket.accept();
            }
            catch (IOException e)
            {
                break;
            }
            try
            {
                socket.setSoTimeout(timeout);
                RemoteClient rc = new RemoteClient(dataManager, socket, userDataHolder);
                rc.start();
                synchronized (this)
                {
                    while (server == null || server.getServerState() == BakutanServer.STATE_GAMESET)
                    {
                        try
                        {
                            this.wait();
                        }
                        catch(InterruptedException e)
                        {
                        }
                    }
                    server.systemMessage("client connected from " + socket.getInetAddress(), 0);
                    server.joinRemoteClient(rc);
                }
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }
    }

    public void stopWaitingClientThread()
    {
        try
        {
            serverSocket.close();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        try
        {
            join();
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
    }
    
    public BakutanServer getServer()
    {
        synchronized (this)
        {
            return server;
        }
    }

    public void setServer(BakutanServer server)
    {
        synchronized (this)
        {
            this.server = server;
            this.notify();
        }
    }

    public int getPort()
    {
        return serverSocket.getLocalPort();
    }
}
