/*
 * 作成日: 2007/01/31
 */
package bakutan.server.util;

import java.io.IOException;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.util.Enumeration;

import bakutan.common.BUserInterface;
import bakutan.common.SimpleUserInterface;

import net.sbbi.upnp.impls.InternetGatewayDevice;
import net.sbbi.upnp.messages.ActionResponse;
import net.sbbi.upnp.messages.UPNPResponseException;

public class ImplementNetworkTool extends NetworkTool
{

    public ImplementNetworkTool()
    {
        super();
    }
    public boolean isSupportUPNP()
    {
        return true;
    }
    public String getLocalHostIP()
    {
        try
        {
            String localHostIP;
            {
                NetworkInterface ni = NetworkInterface.getByName("eth0");
                Enumeration<InetAddress> enu = ni.getInetAddresses();
                if (enu.hasMoreElements())
                {
                    localHostIP = ni.getInetAddresses().nextElement().getHostAddress();
                }
                else
                {
                    localHostIP = InetAddress.getLocalHost().getHostAddress();
                }
            }
            return localHostIP;
        }
        catch(IOException e)
        {
            e.printStackTrace();
        }
        return null;
    }
    public int addPortMapping(int internalPort, BUserInterface userInterface)
    {
        try
        {
            String localHostIP = getLocalHostIP();
            if (localHostIP == null)
            {
                userInterface.outputMessage("Can not get localhost IP");
            }
            
            int discoveryTimeout = 5000;
            InternetGatewayDevice[] igds = InternetGatewayDevice.getDevices( discoveryTimeout );
            if ( igds == null )
            {
                userInterface.outputMessage("UPNP device is not found.");
                return 0;
            }
            InternetGatewayDevice igd = igds[0];
            userInterface.outputMessage("Found UPNP device " + igd.getIGDRootDevice().getModelName());

            int externalPort = internalPort;
            int num = 0;
            {
                Integer numInt = igd.getNatTableSize();
                if (numInt != null)
                {
                    num = numInt.intValue();
                }
            }
            {
                for (int i=0;i<num;i++)
                {
                    ActionResponse ar = igd.getGenericPortMappingEntry(i);
//                    System.out.println(ar);
                    if (ar.getOutActionArgumentValue("NewInternalPort").equals(String.valueOf(internalPort))
                            && ar.getOutActionArgumentValue("NewInternalClient").equals(String.valueOf(localHostIP)))
                    {
                        String ePort = ar.getOutActionArgumentValue("NewExternalPort");
                        try
                        {
//                            System.out.println("Already exist.");
                            userInterface.outputMessage("NAT " + externalPort +" > " + internalPort + " mapped.");
                            return Integer.parseInt(ePort);
                        }
                        catch(NumberFormatException e)
                        {
                            e.printStackTrace();
                        }
                    }
                    else if (ar.getOutActionArgumentValue("NewExternalPort").equals(String.valueOf(externalPort)))
                    {
//                        System.out.println("Already used.");
                        externalPort = 0;
                    }
                }
            }
            
            boolean mapped = false;
            //mapped = testIGD.addPortMapping( "Bakutan", null, port, port, localHostIP, (int)(bstageInfo.getTimeLimit() + 60000), "TCP" );
            mapped = igd.addPortMapping( "Bakutan", null, internalPort, externalPort, localHostIP, 0, "TCP" );
            if ( mapped )
            {
                if (externalPort == 0)
                {
                    num = 0;
                    {
                        Integer numInt = igd.getNatTableSize();
                        if (numInt != null)
                        {
                            num = numInt.intValue();
                        }
                    }
                    for (int i=0;i<num;i++)
                    {
                        ActionResponse ar = igd.getGenericPortMappingEntry(i);
                        if (ar.getOutActionArgumentValue("NewInternalPort").equals(String.valueOf(internalPort))
                                && ar.getOutActionArgumentValue("NewInternalClient").equals(String.valueOf(localHostIP)))
                        {
                            String ePort = ar.getOutActionArgumentValue("NexExternalPort");
                            try
                            {
                                externalPort = Integer.parseInt(ePort);
                            }
                            catch(NumberFormatException e)
                            {
                                externalPort = 0;
                            }
                        }
                    }
                }
            }
            else
            {
                externalPort = 0;
            }
            if (externalPort > 0)
            {
                userInterface.outputMessage("NAT " + externalPort +" > " + internalPort + " mapped.");
                return externalPort;
            }
        } catch ( IOException ex ) {
            ex.printStackTrace();
        } catch( UPNPResponseException respEx ) {
            respEx.printStackTrace();
        }
        userInterface.outputMessage("NAT failed.");
        return 0;
    }
    public boolean deletePortMapping(int internalPort, BUserInterface userInterface)
    {
        try
        {
            String localHostIP = getLocalHostIP();
            if (localHostIP == null)
            {
                userInterface.outputMessage("Can not get localhost IP");
            }

            int discoveryTimeout = 5000;
            InternetGatewayDevice[] IGDs = InternetGatewayDevice.getDevices( discoveryTimeout );
            if ( IGDs == null )
            {
                userInterface.outputMessage("UPNP device is not found.");
                return false;
            }
            InternetGatewayDevice testIGD = IGDs[0];
            userInterface.outputMessage("Found UPNP device " + testIGD.getIGDRootDevice().getModelName());

            int externalPort = internalPort;
            int num = 0;
            {
                Integer numInt = testIGD.getNatTableSize();
                if (numInt != null)
                {
                    num = numInt.intValue();
                }
            }
            {
                for (int i=0;i<num;i++)
                {
                    ActionResponse ar = testIGD.getGenericPortMappingEntry(i);
//                    System.out.println(ar);
                    if (ar.getOutActionArgumentValue("NewInternalPort").equals(String.valueOf(internalPort))
                            && ar.getOutActionArgumentValue("NewInternalClient").equals(String.valueOf(localHostIP)))
                    {
                        String ePort = ar.getOutActionArgumentValue("NewExternalPort");
                        
                        try
                        {
                            externalPort = Integer.parseInt(ePort);
                            testIGD.deletePortMapping(null, externalPort, "TCP");
                            userInterface.outputMessage("NAT " + externalPort +" > " + internalPort + " deleted.");
                            return true;
                        }
                        catch(NumberFormatException e)
                        {
                            e.printStackTrace();
                        }
                    }
                }
            }
        } catch ( IOException ex ) {
            ex.printStackTrace();
        } catch( UPNPResponseException respEx ) {
            respEx.printStackTrace();
        }
        return false;
    }
    public static void main(String[] args)
    {
        SimpleUserInterface ui = new SimpleUserInterface();
        ImplementNetworkTool nt = new ImplementNetworkTool();
        ui.setOutputDate(true);
        int p = nt.addPortMapping(30000, ui);
        System.out.println(p);
        nt.deletePortMapping(p, ui);
    }
}
