/*
 * 作成日: 2007/01/27
 */
package bakutan.serverinfomanager;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import bakutan.common.DataManager;
import bakutan.common.SimpleUserInterface;
import bakutan.common.XMLConverter;
import bakutan.common.serverinfo.SmallBStageState;
import bakutan.common.serverinfo.SmallServerInfo;
import bakutan.common.util.CompressedInputStream;
import bakutan.common.util.CompressedOutputStream;
import bakutan.server.DataBaseInfo;
import bakutan.server.ServerXMLConverter;

public class ServerInfoManager
{
    public static final int TIMEOUT = 5000;
    
    public String tableServers;

    private SimpleUserInterface userinterface;
    private DataBaseInfo dataBaseInfo;
    private DataManager dataManager;
    private int port;
    private long updateInterval;
    private boolean goflag = false;
    
    public ServerInfoManager(ServerInfoManagerInfo simi)
    {
        super();
        this.userinterface = new SimpleUserInterface();
        this.userinterface.setOutputDate(true);
        this.dataBaseInfo = simi.getDataBaseInfo();
        this.tableServers = simi.getDataBaseInfo().getTablePrefix() + "_servers";
        this.dataManager = DataManager.getAvailableDataManager();
        this.port = simi.getPort();
        this.updateInterval = simi.getUpdateInterval();
    }
    public boolean initialize()
    {
        try
        {
            Class.forName(dataBaseInfo.getDriver());
        }
        catch(ClassNotFoundException e)
        {
            throw new RuntimeException("DataBase Driver not found:" + dataBaseInfo.getDriver());
        }
        Connection conn = null;
        Statement stm = null;
        ResultSet rs = null;
        try
        {
            String str = dataBaseInfo.createDSN();
            conn = DriverManager.getConnection(str);
            stm = conn.createStatement();
            
            try
            {
                boolean f = stm.execute("CREATE TABLE "+
                        tableServers+"(" +
                        "UNIQUE(hostname, port)," + 
                        "id int NOT NULL AUTO_INCREMENT PRIMARY KEY, " +
                        "label CHAR(40) NOT NULL, "+
                        "hostname CHAR(40) NOT NULL, " +
                        "port int NOT NULL DEFAULT " + DataManager.DEFAULT_PORT + ", " +
                        "protocol CHAR(40) NOT NULL, " +
                        "options int NOT NULL DEFAULT 0, "+
                        "static int NOT NULL DEFAULT 0"+
                        ")");
                userinterface.outputMessage("table "+tableServers+" is created.");
            }
            catch(SQLException e)
            {
                //e.printStackTrace();
            }
        }
        catch(SQLException e)
        {
            e.printStackTrace();
        }
        try
        {
            if (stm != null)
                stm.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (conn != null)
                conn.close();
        }
        catch(SQLException e2)
        {
        }
        goflag = true;
        return true;
    }
    public Collection<SmallServerInfo> getServerInfos()
    {
        ArrayList<SmallServerInfo> sies = new ArrayList<SmallServerInfo>();

        Connection conn = null;
        PreparedStatement pstm = null;
        ResultSet rs = null;
        try
        {
            String str = dataBaseInfo.createDSN();
            conn = DriverManager.getConnection(str);
            
            pstm = conn.prepareStatement("SELECT id,label,hostname,port,protocol,options FROM "+ tableServers);
            rs = pstm.executeQuery();
            while (rs.next())
            {
                SmallServerInfo sie = new SmallServerInfo();
                sie.setId(rs.getInt(1));
                sie.setLabel(rs.getString(2));
                sie.setHostName(rs.getString(3));
                sie.setPort(rs.getInt(4));
                SmallBStageState sbs = new SmallBStageState();
                sbs.setProtocol(rs.getString(5));
                sbs.setOption(rs.getInt(6));
                sie.setSmallBStageState(sbs);
                sies.add(sie);
            }
        }
        catch(SQLException e)
        {
            e.printStackTrace();
        }
        try
        {
            if (rs != null)
                rs.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (pstm != null)
                pstm.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (conn != null)
                conn.close();
        }
        catch(SQLException e2)
        {
        }
        return sies;
    }
    public void updateServerInfoEntry(SmallServerInfo sie)
    {
        SmallBStageState sbss = sie.getSmallBStageState();
        if (sbss == null)
            return;
        
        Connection conn = null;
        boolean ret = false;;
        PreparedStatement pstm = null;
        ResultSet rs = null;
        try
        {
            String str = dataBaseInfo.createDSN();
            conn = DriverManager.getConnection(str);

            pstm = conn.prepareStatement("SELECT id FROM "+tableServers+" WHERE hostname=? AND port=?");
            pstm.setString(1,sie.getHostName());
            pstm.setInt(2,sie.getPort());
            rs = pstm.executeQuery();
            int id = -1;
            if (rs.next())
                id = rs.getInt(1);
            rs.close();
            pstm.close();
            
            if (id == -1)
            {
                pstm = conn.prepareStatement("INSERT INTO "+tableServers+"(label,hostname,port,protocol,options) values (?,?,?,?,?)");
                pstm.setString(1,sie.getLabel());
                pstm.setString(2,sie.getHostName());
                pstm.setInt(3,sie.getPort());
                pstm.setString(4,sbss.getProtocol());
                pstm.setInt(5,sbss.getOption());
                ret = (pstm.executeUpdate() > 0);
            }
            else
            {
                pstm = conn.prepareStatement("UPDATE "+tableServers+" SET label=?,protocol=?,options=? WHERE id=?");
                pstm.setString(1,sie.getLabel());
                pstm.setString(2,sbss.getProtocol());
                pstm.setInt(3,sbss.getOption());
                pstm.setInt(4,id);
                ret = (pstm.executeUpdate() > 0);
            }
        }
        catch(SQLException e)
        {
            e.printStackTrace();
        }
        try
        {
            if (rs != null)
                rs.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (pstm != null)
                pstm.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (conn != null)
                conn.close();
        }
        catch(SQLException e2)
        {
        }
    }
    public boolean removeServerInfoEntry(int id)
    {
        Connection conn = null;
        boolean ret = false;;
        PreparedStatement pstm = null;
        ResultSet rs = null;
        try
        {
            String str = dataBaseInfo.createDSN();
            conn = DriverManager.getConnection(str);
            
            pstm = conn.prepareStatement("DELETE FROM "+tableServers+" WHERE ID=? AND static=0");
            pstm.setInt(1,id);
            ret = (pstm.executeUpdate() > 0);
        }
        catch(SQLException e)
        {
            e.printStackTrace();
        }
        try
        {
            if (rs != null)
                rs.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (pstm != null)
                pstm.close();
        }
        catch(SQLException e2)
        {
        }
        try
        {
            if (conn != null)
                conn.close();
        }
        catch(SQLException e2)
        {
        }
        return ret;
    }
    public void startServer()
    {
        if (!goflag)
            throw new RuntimeException("did not initiarized.");
        
        WaitingClient waitingClient = new WaitingClient();
        waitingClient.initiarize(port);
        Thread waitingClientThread = new Thread(waitingClient);
        waitingClientThread.start();
        
        userinterface.outputMessage("Server started : " + waitingClient.getPort());
        userinterface.outputMessage("using Database : " + dataBaseInfo.getDatabase() + "." + tableServers);
        
        while(true)
        {
            try
            {
                Thread.sleep(updateInterval);
            }
            catch(InterruptedException e)
            {
            }
            //userinterface.outputMessage("Updating");
            Collection<SmallServerInfo> sies = getServerInfos();
            Iterator<SmallServerInfo> ite = sies.iterator();
            while(ite.hasNext())
            {
                SmallServerInfo ssi = ite.next();
                SmallBStageState sbs = ssi.getSmallBStageState();
                //System.out.print("Checking:" +ssi.getHostName() + ":" + ssi.getPort()+":");
                try
                {
                    SmallBStageState nsbs = DataManager.getSmallBStageState(ssi.getHostName(), ssi.getPort(), TIMEOUT);
                    if (!sbs.equals(nsbs))
                    {
                        ssi.setSmallBStageState(nsbs);
                        updateServerInfoEntry(ssi);
                        userinterface.outputMessage(ssi.getLabel() + ":" + ssi.getHostName() + ":" + ssi.getPort() + " is updated.");
                    }
                }
                catch(IOException e)
                {
                    if (removeServerInfoEntry(ssi.getId()))
                        userinterface.outputMessage(ssi.getLabel() + ":" + ssi.getHostName() + ":" + ssi.getPort() + " is removed.");
                }
            }
        }
    }
    
    public static void main(String[] args)
    {
        ServerInfoManagerInfo simi;
        try
        {
            simi =  ServerXMLConverter.createServerManagerInfo(XMLConverter.readXML(new File("ServerInfoManagerInfo.xml")));
        }
        catch(Exception e)
        {
            e.printStackTrace();
            return;
        }
        ServerInfoManager sim = new ServerInfoManager(simi);
        sim.initialize();
        sim.startServer();
        
    }
    
    class WaitingClient implements Runnable
    {
        private ServerSocket ssocket;
        private int port;
        
        WaitingClient()
        {
        }
        public boolean initiarize(int port)
        {
            try
            {
                ssocket = new ServerSocket(port);
                this.port = ssocket.getLocalPort();
                return true;
            }
            catch(IOException e)
            {
                e.printStackTrace();
                return false;
            }
        }
        
        public void run()
        {
            if (ssocket == null)
                throw new RuntimeException("did not initiarized.");
            try
            {
                while(true)
                {
                    Socket socket = ssocket.accept();
                    ServerInfoClient sic = new ServerInfoClient(socket);
                    sic.run();
                }
            }
            catch(IOException e)
            {
                e.printStackTrace();
            }
        }
        public int getPort()
        {
            return port;
        }
        
    }
    
    class ServerInfoClient extends Thread
    {
        public static final int STATE_NEW = 0;
        public static final int STATE_AUTHORIZING = 1;
        public static final int STATE_READY = 2;
        public static final int STATE_GAME = 3;
        public static final int STATE_DISCONNECTED = 4;
        
        private DataInputStream in;
        private DataOutputStream out;
        private Socket socket;
        private boolean goflag;
        
        public ServerInfoClient(Socket _socket)
        {
            super("RemoteClient Reader Thread");
            setPriority(Thread.NORM_PRIORITY+1);
            this.socket = _socket;
            try
            {
                in = new DataInputStream(new CompressedInputStream(socket.getInputStream(),2<<12));
                out = new DataOutputStream(new CompressedOutputStream(socket.getOutputStream(),2<<12));
            }catch(IOException e)
            {
                try
                {
                    socket.close();
                }catch(IOException ex)
                {
                    //ex.printStackTrace();
                }
                return;
            }
            goflag = true;
        }
        public void run()
        {
            if (!goflag)
                return;
            byte mode;
            try
            {
                mode = in.readByte();
                if (mode == DataManager.SERVERMANAGE_ADD)
                {
                    SmallServerInfo sie = DataManager.inputSmallServerInfo(in);
                    sie.setHostName(socket.getInetAddress().getHostName());
                    SmallBStageState bss = DataManager.getSmallBStageState(sie.getHostName(), sie.getPort(), TIMEOUT);
                    sie.setSmallBStageState(bss);
                    updateServerInfoEntry(sie);
                    userinterface.outputMessage(sie.getLabel() + ":" + sie.getHostName() + ":" + sie.getPort() + " inserted from " + socket.getInetAddress());
                }
                else if (mode == DataManager.SERVERMANAGE_GET)
                {
                    Collection<SmallServerInfo> sies = getServerInfos();
                    DataManager.outputData(out, sies);
                }
                out.flush();
            }catch(IOException e)
            {
            }
            try
            {
                socket.close();
            }catch(IOException ex)
            {
                //ex.printStackTrace();
            }
        }
    }
}


