/*
 * Copyright 2005 Takao Sumitomo. All rights reserved.
 * 
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the 
 *      above copyright notice, this list of conditions
 *      and the following disclaimer.
 *   2. Redistributions in binary form must reproduce 
 *      the above copyright notice, this list of conditions
 *      and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software
 * and documentation are those of the authors and should not be
 * interpreted as representing official policies,
 * either expressed or implied.
 * 
 * @Author: Takao Sumitomo
 */

/*
 * 作成日: 2005/02/04
 */
package bakutan.common.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URL;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;

import bakutan.common.math.BVector2f;



/**
 * @author cattaka
 */
public class BSound
{
    public static final float DISTANCE = 5;

    private SEBundle soundBundle;
    private Player[] player; 
    private int next = 0;
    private int lineNum = 0;
    private int volume = 64;
    private float volumeF = 0.5f;

    public BSound(int lineNum)
    {
        this.lineNum = lineNum;
    }
    public void setEnable(boolean flag)
    {
        if (!flag)
        {
            stop();
            return;
        }
        
        if (player != null && player.length == lineNum)
            return;
        stop();
        try
        {
            AudioFormat format = new AudioFormat(22050.0f, 16, 1, true, false);
            DataLine.Info dataInfo = new DataLine.Info(SourceDataLine.class, format);
            
            stop();
            player = new Player[lineNum];
            for (int i=0;i<lineNum;i++)
            {
                SourceDataLine line;
                line = (SourceDataLine)AudioSystem.getLine(dataInfo);
                line.open(format);
                player[i] = new Player(line);
                player[i].start();
            }
            next = 0;
        }catch(Exception e)
        {
            e.printStackTrace();
            stop();
            return;
        }
    }
    public void stop()
    {
        if (player == null)
            return;
        for (int i=0;i<player.length;i++)
        {
            if (player[i] != null)
                player[i].myDestroy();
        }
        player = null;
    }
    public boolean isRunning()
    {
        return lineNum > 0;
    }
    /*
     private void createSoundData()
     {
     try
     {
     soundData = new byte[SOUNDTYPE_NUM][];
     
     ByteArrayOutputStream bao = new ByteArrayOutputStream(1024);
     for (int i=0;i<soundData.length;i++)
     {
     bao.reset();
     URL url;
     AudioInputStream ais;
     url = getClass().getClassLoader().getResource("sound"+i+".wav");
     ais = AudioSystem.getAudioInputStream(url);
     byte[] bs = new byte[(int)ais.getFrameLength()];
     while(true)
     {
     int s = ais.read(bs);
     if (s == -1)
     break;
     bao.write(bs,0,s);
     }
     soundData[i] = bao.toByteArray();
     ais.close();
     }
     bao.close();
     
     }catch(IOException e)
     {
     e.printStackTrace();
     }catch(UnsupportedAudioFileException e)
     {
     e.printStackTrace();
     }
     }
     */
    public static byte[] loadAudioFile(URL url)
    {
        try
        {
            ByteArrayOutputStream bao = new ByteArrayOutputStream(1024);
            AudioInputStream ais;
            ais = AudioSystem.getAudioInputStream(url);
            byte[] bs = new byte[(int)ais.getFrameLength()];
            while(true)
            {
                int s = ais.read(bs);
                if (s == -1)
                    break;
                bao.write(bs,0,s);
            }
            bs = bao.toByteArray();
            ais.close();
            bao.close();
            return bs;
        }catch(IOException e)
        {
            e.printStackTrace();
            return null;
        }catch(UnsupportedAudioFileException e)
        {
            e.printStackTrace();
            return null;
        }
    }
    
    public void play(byte type, int rag)
    {
        if (player == null || soundBundle == null)
            return;
        
        byte[] soundData = soundBundle.getAudioData(type);
        if (soundData == null)
            return;
        for (int i=0;i<player.length;i++)
        {
            if (player[i].isActive())
                continue;
            player[i].playSound(rag, soundData);
            break;
        }
    }
    public void play(byte type, int rag, BVector2f center, BVector2f pos)
    {
        if (player == null || soundBundle == null)
            return;
        float d = BSound.DISTANCE;
        float f;
        d = d*d;
        
        BVector2f p = new BVector2f();
        p.set(center);
//        p.x += BAudio.WIDTH / 2;
        p.sub(pos);
        f = p.getLengthSquare();
        float v = (float)Math.sqrt(d/(f+d)) * volumeF;
        
        byte[] soundData = soundBundle.getAudioData(type);
        if (soundData == null)
            return;
        for (int i=0;i<player.length;i++)
        {
            if (player[i].isActive())
                continue;
            player[i].playSound(rag, soundData, v);
            break;
        }
    }
    public SEBundle getSEBundle()
    {
        return soundBundle;
    }
    public void setSEBundle(SEBundle SoundBundle)
    {
        this.soundBundle = SoundBundle;
    }
    public int getVolume()
    {
        return volume;
    }
    public void setVolume(int volume)
    {
        this.volume = volume;
        if (this.volume > 127)
            this.volume = 127;
        if (this.volume < 0)
            this.volume = 0;
        volumeF = (float)volume/127f;
    }
    class Player extends Thread
    {
        private SourceDataLine line;
        private int rag = 0;
        private boolean active = true;
        private boolean destroy = false;
        private byte[] soundData;
        private float vl = 1.0f;
        
        Player(SourceDataLine line)
        {
            super("Sound Thread");
            setPriority(Thread.MIN_PRIORITY);
            this.line = line;
        }
        public boolean isActive()
        {
            synchronized(this)
            {
                return active;
            }
        }
        public void playSound(int rag, byte[] soundData)
        {
            playSound(rag, soundData, 1.0f);
        }
        public void playSound(int rag, byte[] soundData, float volume)
        {
            if (soundData == null)
                throw new NullPointerException();
            synchronized(this)
            {
                this.soundData = soundData;
                this.rag = rag;
                this.vl = volume;
                active = true;
                notify();
            }
        }
        public void run()
        {
            line.start();
            while (true)
            {
                int s;
                synchronized(this)
                {
                    try
                    {
                        active = false;
                        if (destroy)
                            break;
                        wait();
                        if (destroy)
                            break;
                    }catch(InterruptedException e)
                    {
                        e.printStackTrace();
                        continue;
                    }
                }
                try
                {
                    Thread.sleep(rag);
                }catch(InterruptedException e)
                {
                    e.printStackTrace();
                }
                byte[] bs = soundData;
                /*
                 int i;
                 for (i=0;i<bs.length/line.available();i++)
                 {
                 line.write(bs,i*line.available(),line.available());
                 line.drain();
                 }
                 */
//                line.flush();
//                line.start();
                byte[] sd = new byte[bs.length]; 
                for (int i=0;i<bs.length;i++)
                {
                    sd[i] = (byte)((float)bs[i] * vl);
                }
                line.write(sd,0,sd.length);
                line.drain();
//                line.stop();
            }
            line.stop();
            line.close();
        }
        public void myDestroy()
        {
            synchronized(this)
            {
                destroy = true;
                notify();
            }
        }
    }
}
